

######################################################################
# prototype
#

AlffGlobals := rec();


#######################################################################
# Initialisation


Alff := function(f)
   local F;
   F := _Alff(f);
   Extend(F);
   F.operations := rec(Print := function(a) Print(a.base); end );
   return F;
end;

_Extend := Extend;

Extend := function(A)
   _Extend(A);
   if IsAlffDivisor(A) then
      A.operations := rec( Print := function(a) Print(a.base); end );
   fi;
end;


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       AlffInit
#CALLING   AlffInit(k);\\
#          AlffInit(k, T);\\
#          AlffInit(k, T, y);
#SHORTDOC  Initializes some useful variables for an algebraic function 
#          field session.
#LONGDOC   This function is used for convenience to initialize 
#          some useful variables
#          for an algebraic function field session and is not
#          recommended for use in programming. It initializes:
#          \begin{itemize}
#          \item {\tt k}, the constant field,
#          \item {\tt w}, a generator of $k$ over its base field
#                         if finite,
#          \item {\tt kT}, the polynomial ring $k[T]$,
#          \item {\tt kTf}, the field $k(T)$,
#          \item {\tt kTy}, the polynomial ring $k[T][y]$,
#          \item {\tt T}, the variable $T$,
#          \item {\tt y}, the variable $y$,
#          \item {\tt AlffGlobals}, a record containing all the above values.
#          \end{itemize}
#          The optional two parameter strings define the names of
#          variables $T$ and $y$.
#
#PACKAGE   Alff
#
#ARGUS     k @ field 
#          T, y @ strings @ the variable names
#
#XREF      AlffOrders, Alff
#KASH      @
#          AlffInit(FF(5, 2));
#          k;
#          kTy;
#          y^4+T^3+2;
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  99xxxx  FH  changes
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffInit := function(arg)
   local E, T, y, kT, kTf, kTy;
   
   if Length(arg) = 1 then
      E := arg[1];
      T := "T";
      y := "y";
   elif Length(arg) = 2 then
      E := arg[1];
      T := arg[2];
      if not IsString(T) then
         return Error("usage: AlffInit( <field> [, <string>] [, <string>] )");
      fi;
      y := "y";
   elif Length(arg) = 3 then
      E := arg[1];
      T := arg[2];
      y := arg[3];
      if not IsString(T) or not IsString(y) then
         return Error("usage: AlffInit( <field> [, <string>] [, <string>] )");
      fi;
   else
      return Error("usage: AlffInit( <field> [, <string>] [, <string>] )");
   fi;
   
   if IsFF(E) then
      AlffGlobals.k := E;
      AlffGlobals.w := FFGenerator(E);
   elif E = Q then
      AlffGlobals.k := Q;
      AlffGlobals.w := 1;
   elif IsOrder(E) then
      AlffGlobals.k := E;
      AlffGlobals.w := OrderBasis(E)[2];
   else
      return Error("usage: AlffInit( <field> [, <string>] [, <string>] )");
   fi;
   
   kT := Concatenation("k", T);
   kTf := Concatenation(kT, "f");
   kTy := Concatenation(kT, y);
   
   AlffGlobals.(kT) := PolyAlg(AlffGlobals.k, T);
   AlffGlobals.(kTf) := QuotientField(AlffGlobals.(kT));
   AlffGlobals.(kTy) := PolyAlg(AlffGlobals.(kT), y);
   AlffGlobals.(T) := Poly(AlffGlobals.(kT), [1, 0]);
   AlffGlobals.(y) := Poly(AlffGlobals.(kTy), [1, 0]);
   
   Bind("w", AlffGlobals.w);
   Bind("k", AlffGlobals.k);
   Bind(kT, AlffGlobals.(kT));
   Bind(kTf, AlffGlobals.(kTf));
   Bind(kTy, AlffGlobals.(kTy));
   Bind(T, AlffGlobals.(T));
   Bind(y, AlffGlobals.(y));
   
   return Concatenation("Defining global variables: k, w, ", kT, ", ", kTf, ", ", kTy, ", ", T, ", ", y, ", AlffGlobals");
   
end;


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       AlffOrders
#CALLING   AlffOrders(f);
#ARGUS     f @ polynomial in $k[T][y]$
#SHORTDOC  Initializes variables for some structures
#          of an algebraic function field.
#LONGDOC   This function is used for convenience to initialize variables
#          for some structures of an algebraic function field and is
#          not recommended for use in programming. It initializes:
#          \begin{itemize}
#          \item {\tt F},  the algebraic function field generated by $f(T,y)=0$,
#          \item {\tt o}, the integral closure of $k[T]$ in $F$,
#          \item {\tt oi}, the integral closure of the degree valuation
#                          ring,
#          \item {\tt one}, the $1 \in F$.
#          \end{itemize}
#          The polynomial $f$ is checked to be irreducible and separable.
#
#PACKAGE   Alff
#
#XREF      AlffInit, Alff
#KASH      @
#          AlffInit(FF(5,2));
#          AlffOrders(y^3+T^3+2);
#          F;
#          o;
#          AlffGlobals.oi;
#          one;
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  99xxxx  FH  changes
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffOrders := function(f)
   if not AlffPolyIsIrrSep(f) then
      return Error("polynomial not irreducible and separable");
   fi;
   
   AlffGlobals.f := f;
   AlffGlobals.F := Alff(f);
   AlffGlobals.o := AlffOrderMaxFinite(AlffGlobals.F);
   AlffGlobals.oi := AlffOrderMaxInfty(AlffGlobals.F);
   
   Bind("F", AlffGlobals.F);
   Bind("o", AlffGlobals.o);
   Bind("oi", AlffGlobals.oi);
   Bind("one", AlffElt(AlffGlobals.o, 1));
   
   return "Defining global variables: F, o, oi, one";
end; 


##########################################################################
#
# Read other all files
#

Read("../lib/Alff/util.g");
Read("../lib/Alff/places.g");
Read("../lib/Alff/divisors.g");
Read("../lib/Alff/differentials.g");
Read("../lib/Alff/lpoly.g");
Read("../lib/Alff/classgroup.g");
Read("../lib/Alff/weierstrass.g");


##########################################################################
# unprototype
#
