
###########################################################################
#
#  Divisor class group computation.
#
#  See F. Hess: "Zur Divisorenklassengruppenberechnung in globalen 
#  Funktionenk"orpern", Dissertation, TU Berlin 1999.
#

GG := function(arg) return true; end;
#GG := GASMAN;

Info := function(arg) return true; end;
#Info := Print;


###########################################################################
#  Functions for smooth divisors
#


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDivisorsSmoothNum
#
#CALLING   N := AlffDivisorsSmoothNum(n, m, P);
#
#ARGUS     N @ integer @ number of $(n, m)$-smooth divisors 
#          n, m @ integers
#          P @ list @ of integers
#
#SHORTDOC  Compute the number of $(n, m)$-smooth divisors.
#     
#LONGDOC   Given two integers $n, m \geq 0$ and a list $P$ such that
#          $P[i]$ is the number of places of degree $1 \leq i \leq 
#          \min\{ n, m \}$ of a given
#          global function field, return the number of divisors $D$ which
#          are $(n,m)$-smooth, i.e. $D \geq 0$ and the support of $D$
#          consists of places of degree $\leq m$. A recursion from \cite{He2}
#          is used.
#
#XREF      AlffInit, Alff, AlffPlacesNum
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5, 2));;
#          AlffOrders(y^3+T^3+2);;
#          P := List([1..3], i->AlffPlacesNum(F, i));
#          AlffDivisorsSmoothNum(7, 3, P);
#          Zx1 := PolyAlg(Z, "x1");;
#          Zx1x2 := PolyAlg(Zx1, "x2");;
#          Zx1x2x3 := PolyAlg(Zx1x2, "x3");;
#          x1 := Poly(Zx1, [1,0]);;
#          x2 := Poly(Zx1x2, [1,0]);;
#          x3 := Poly(Zx1x2x3, [1,0]);;
#          N := AlffDivisorsSmoothNum(7, 3, [ x1, x2, x3 ]);
#          Eval(Eval(Eval(N, P[3]), P[2]), P[1]);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDivisorsSmoothNum := function(n, m, P)
#
#  Given the number of places of deg i in P[i],
#  this function computes the number of effective divisors 
#  of degree leq m, which are k-smooth. (P[i] can contain variables)
#
   local AlffSmoothN, AlffSmoothPsiSub, N, Psi;

   AlffSmoothN := function(n, m)
      local l;
      if IsBound(N[n]) then
         return N[n];
      fi;
      l := Filtered(IntDivisors(n), d->d <= m);
      N[n] := Sum(l, d->d*P[d]);
      return N[n];
   end;

   AlffSmoothPsiSub := function(n, m)
      local psi;
      if n = 0 or m = 0 then
         return 1;
      fi;
      if IsBound(Psi[n]) then
         return Psi[n];
      fi;
      psi := Sum([0..n-1], j->(1 + AlffSmoothN(n-j, m))*
                 AlffSmoothPsiSub(j, m)) / n;
      Psi[n] := psi;
      return psi;
   end;

   if Length(P) < Minimum(n, m) then
      return Error("list of number of places too short");
   fi;

   N := [];
   Psi := [];

   return AlffSmoothPsiSub(n, m);

end;


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDivisorsSmoothRatio
#
#CALLING   r := AlffDivisorsSmoothRatio(n, m, P);
#
#ARGUS     r @ real @ smoothness ratio
#          n, m @ integers
#          P @ list @ of integers
#
#SHORTDOC  Return a ratio of numbers of smooth divisors times a number
#          of places.
#
#LONGDOC   Given two integers $n, m \geq 0$ and a list $P$ such that
#          $P[i]$ is the number of places of degree $1 \leq i \leq 
#          \max\{ n, m \}$ of a given
#          global function field, return the ratio of the number of
#          $(n, m)$-smooth divisors and the number of $(n,n)$-smooth 
#          divisors times the number of places of degree $\leq m$.
#
#XREF      AlffDivisorsSmoothNum, AlffInit, Alff, AlffPlacesNum
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5, 2));;
#          AlffOrders(y^3+T^3+2);;
#          P := List([1..3], i->AlffPlacesNum(F, i));
#          P := Concatenation(P, [ (1/4)*5^8, (1/5)*5^10, (1/6)*5^12, 
#          (1/7)*5^14 ] );
#          AlffDivisorsSmoothRatio(7, 3, P);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDivisorsSmoothRatio := function(n, m, P)
   #
   #  P[i] contains the number of places of degree i
   #  Returns how many rel tries are approx. needed to complete a fb
   #
   if Length(P) < Maximum(n, m) then
      return Error("list of number of places too short");
   fi;

   return 1.0 * AlffDivisorsSmoothNum(n, n, P) / 
          AlffDivisorsSmoothNum(n, m, P) * Sum([1..m], i->P[i]);
end;




###########################################################################
#  generation bounds and approximation of class number
#


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffClassGroupGenBound
#
#CALLING   b := AlffClassGroupGenBound(q, g);\\
#          b := AlffClassGroupGenBound(F);
#
#ARGUS     b @ integer @ generation bound
#          q, g @ integers @ exact constant field size and genus
#          F @ alff @ global function field
#
#SHORTDOC  Return a bound for prime divisors which generate the divisor
#          class group.
#
#LONGDOC   Let $F/k$ be a global function field of genus $g$ over the exact
#          constant field of $q$ elements, let $A_1$ be a divisor of 
#          degree one and let $S$ be the set of prime
#          divisors of degree $\leq b$. This function computes 
#          $b \leq \lceil 2 \log_q(4g-2) \rceil$ 
#          without using further information on $F$ such that
#          the classes of the elements of 
#          $S \cup {\rm supp}(A_1)$ generate the divisor
#          class group of $F/k$. See \cite{He2} for details.
#
#XREF      AlffClassGroupGenBoundStrong, AlffClassGroupProdBound,
#          AlffClassGroup
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7, 2));;
#          AlffOrders(y^4 + T^7 + T);;
#          b := AlffClassGroupGenBound(F);
#          Sum([1..b], i->AlffPlacesNum(F, i));
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffClassGroupGenBound := function(arg)
   #
   #  Return bound for places which generate the class group 
   #
   local q, g, f, eps, F;
   if IsAlff(arg[1]) then
      F := arg[1];
      AlffIsGlobalAssert(F);
      q := AlffConstFieldSize(F);
      g := AlffGenus(F);
   else
      q := arg[1];
      g := arg[2];
   fi;
   eps := 10.0^-(Prec()/2);
   f := 1;
   while q^f + 1 - g*Floor(2*Sqrt(q^f) + eps) <=  (g-1)*2*Sqrt(q^f) + eps do
      f := f + 1;
   od;
   return f;
end;


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffClassGroupGenBoundStrong
#
#CALLING   b := AlffClassGroupGenBoundStrong(F);
#
#ARGUS     b @ integer @ generation bound
#          F @ alff @ global function field
#
#SHORTDOC  Return a bound for prime divisors which generate the divisor
#          class group.
#
#LONGDOC   Let $F/k$ be a global function field of genus $g$ over the exact
#          constant field of $q$ elements, let $A_1$ be a divisor of 
#          degree one and let $S$ be the set of prime
#          divisors of degree $\leq b$. This function computes 
#          $b \leq \lceil 2 \log_q(4g-2) \rceil$ 
#          using information on the number of places of $F$ of bounded degree 
#          such that
#          the classes of the elements of 
#          $S \cup {\rm supp}(A_1)$ generate the divisor
#          class group of $F/k$. See \cite{He2} for details.
#
#XREF      AlffClassGroupGenBound, AlffClassGroupProdBound,
#          AlffClassGroup
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7, 2));;
#          AlffOrders(y^4 + T^7 + T);;
#          b := AlffClassGroupGenBoundStrong(F);
#          Sum([1..b], i->AlffPlacesNum(F, i));
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffClassGroupGenBoundStrong := function(F)
   #
   #  Return stronger bound for places which generate the class group 
   #
   local q, g, f, eps;
   AlffIsGlobalAssert(F);
   q := AlffConstFieldSize(F);
   g := AlffGenus(F);
   eps := 10.0^-(Prec()/2);
   f := 1;
   while AlffPlacesDegOneNum(F, f) <= (g-1)*2*Sqrt(q^f) + eps do
      f := f + 1;
   od;
   return f;
end;


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffClassNumberApproxBound
#
#CALLING   b := AlffClassNumberApproxBound(q, g, a);\\
#          b := AlffClassNumberApproxBound(F, a);
#
#ARGUS     b @ integer @ approximation bound
#          q, g @ integers @ exact constant field size and genus
#          F @ alff @ global function field
#          a @ real @ $\in \R^{> 1}$
#
#SHORTDOC  Return a bound for the prime divisors to be considered
#          for the approximation of the class number of a global
#          function field.
#
#LONGDOC   Let $F/k$ be a global function field of genus $g$ over the exact
#          constant field of $q$ elements. This function computes $b \in \Z$ 
#          such that $$\frac{2g}{q^{1/2}-1} \cdot \frac{q^{-b/2}}{b+1} 
#          \;<\; \log(a) / 2$$ holds. 
#          See function {\tt AlffClassNumberApprox()}.
#
#XREF      AlffClassNumberApprox, AlffClassGroup, AlffLPoly
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7, 2));;
#          AlffOrders(y^4 + T^7 + T);;
#          b := AlffClassNumberApproxBound(F, 2);
#          b := AlffClassNumberApproxBound(F, 1.2);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffClassNumberApproxBound := function(arg)
   #
   #  Return bound up to which logarithmic series s has to be
   #  approximated such that multiplicative error is less than a, a >= 1.
   #
   local g, q, f, eps, a, F;
   if IsAlff(arg[1]) then
      F := arg[1];
      AlffIsGlobalAssert(F);
      q := AlffConstFieldSize(F);
      g := AlffGenus(F);
      a := arg[2];
   else
      q := arg[1];
      g := arg[2];
      a := arg[3];
   fi;
   eps := 10.0^-(Prec()/2);
   a := 0.5*Log(a);
   f := 0;
   while 2*g*q^-(f/2) >= a*(f+1)*(Sqrt(q)-1) - eps do
      f := f + 1;
   od;
   return f;
end;


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffClassNumberApprox
#
#CALLING   hbar := AlffClassNumberApprox(F, b);
#
#ARGUS     hbar @ real @ approximated class number
#          F @ alff @ global function field
#          b @ integer @ bound 
#
#SHORTDOC  Compute an approximation of the class number of a global
#          function field.
#
#LONGDOC   Let $F/k$ be a global function field of genus $g$ over the exact
#          constant field of $q$ elements. If $h$ is the class number of $F/k$
#          we have (see \cite{He2}) 
#          $$\biggl| \,\log \bigl( h \,/\, q^g \, \bigr) - 
#          \sum_{r=1}^{b} \frac{q^{-r}}{r} \bigl( N_r - (q^r + 1) \bigl)
#           \biggr| \,\leq\, \frac{2g}{q^{1/2}-1} \cdot
#           \frac{q^{-b/2}}{b+1}, $$ where $N_r$ is the number of places 
#          of degree one in the constant field extension of degree 
#          $r$ of $F/k$. By this formula an approximation of $h$ is
#          computed using the values of $N_r$ for $1 \leq r \leq b$.
#          The bound $b$ can be determined from a given multiplicative 
#          error bound $a \in \R^{>1}$ 
#          via {\tt AlffClassNumberApproxBound()}.
#
#XREF      AlffClassNumberApproxBound, AlffClassGroup, AlffLPoly
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7, 2));;
#          AlffOrders(y^4 + T^7 + T);;
#          b := AlffClassNumberApprox(F, 1);
#          b := AlffClassNumberApprox(F, 2);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffClassNumberApprox := function(F, m)
   #
   #  Approximation by a logarithmic series
   #
   local sum, q, g;
   AlffIsGlobalAssert(F);
   q := AlffConstFieldSize(F);
   g := AlffGenus(F);
   sum := Sum([1..m], k->(1/k)*q^(-k)*(AlffPlacesDegOneNum(F, k) - q^k - 1));
   return Exp(sum)*q^g;
end;



###########################################################################
#  Main class group functions
#
#  Used data structures:
#  
#  F.1x := 1/x;
#
#  F.cld:  Record, contains all class group data.
#
#  cld.A:      user defined reduction divisor.
#  cld.hbound:    heuristic bound (views only smooth prob)
#  cld.userbound: bound given by user
#  cld.P:         approx. number of places of degree 1, ..
#  cld.n:         the reduced divisors have degree <= n.
#  cld.smoothprob:  expected smoothness prob for current fbbound
#  cld.erv:         expected number of rel tries
#
#  cld.fb: The factor basis, containing recs fbp (p, i) p place i position.
#          Contains all infinite places, is sorted, increasing degrees.
#  cld.fbbound:  The fb contains all places of degree <= fbbound;
#  cld.A1:       used divisor of degree one
#  cld.A1prob:   whether fb had to be increased to contain supp(A1).
#  cld.fbdeg:    fb splitted into degree equal parts.
#  cld.fbpols:   Set of norms of all places in fb. 
#  cld.ffelts:   all elements of k
#  cld.happrox:  approx of class number (< sqrt(2) error).
#  cld.time:     compu time in seconds (after comp)
#
#  cld.fbboundenlarge: enlarged fbbound (if fb was too small)
#  cld.rels:           relation vectors on fb (row matrix)
#  cld.stat:           record of statistical values
#
#  cld.gens:           generators of the degree 0 part
#  cld.Cl:             clg structure.  
#  cld.proven:         whether result is proven or not.
#  cld.igensmat:       trafo fb -> gens    
#
#  cld.xinftyvec:  representation of (x)_infty on fb as vec
#  cld.Avec:       representation of cld.A (if exist) on fb as vec
#
#  fbp:           factor basis place record.
#  fbp.place:     the place 
#  fbp.i:         the position (for the rels).
#  fbp.n:         random weight
#  fbp.divisor:   random divisor (supp <= n, expos <= n) - place
#  fbp.basis:     basis of L(divisor)
#  fbp.bdim:      basis length
#  fbp.ffelts:    := F.cld.ffelts
#  fbp.ffeltsnum: := length(ffelts)
#  fbp.counter:   multicounter for enum of L(divisor)
#
#
###########################################################################


_AlffFactorBasisCreate := function(F, deg)
   local k, kx, x, fb, fbpols, infty, finite, L, fbdeg, bound, A1, A1deg,
         pe;

   Info("creating factor basis for bound ", deg, " ...\n");

   x := AlffVarT(F);
   kx := PolyAlg(x);
   k := PolyAlgCoef(kx);

   infty := AlffPlaceSplit(F, 1/x);
   F.1x := 1/x;

   A1 := AlffDivisorDegOne(F);
   F.cld.A1 := A1;
   A1 := List(AlffDivisorPlaces(A1), p->p[1]);
   A1 := Filtered(A1, AlffPlaceIsFinite);
   if Length(A1) > 0 then
      A1deg := Maximum(List(A1, AlffPlaceDeg));
   else
      A1deg := 0;
   fi;
   if deg < A1deg then
      Info("**** fb div deg one problem, increasing bound to ", A1deg, "\n");
      deg := A1deg;
      # for tex
      F.cld.A1prob := true;
   fi;

   fbpols := PolyPrimeList(kx, [1..deg]);
   finite := Flat(List(fbpols, p->AlffPlaceSplit(F, p)));
   finite := Filtered(finite, p->AlffPlaceDeg(p) <= deg);
   fb := Concatenation(infty, finite);
   Sort(fb, function(p1, p2) return AlffPlaceDeg(p1) < AlffPlaceDeg(p2); end);

   if IsBound(F.cld.A) then
      for pe in AlffDivisorPlaces(F.cld.A) do
         if not pe[1] in fb then
            return Error("reduction divisor not smooth");
         fi;
      od;
   fi;

   if not IsBound(F.cld) then
      F.cld := rec();
   fi;

   F.cld.fb := List([1..Length(fb)], j->rec( place := fb[j], i := j ));
   F.cld.fbbound := deg;

   fbdeg := Set(List(fb, p->AlffPlaceDeg(p)));
   fbdeg := List(fbdeg, j->Filtered(F.cld.fb,
                    fbp->AlffPlaceDeg(fbp.place) = j));
   F.cld.fbdeg := fbdeg;

   if Length(fbdeg) <= 1 then
      Info("**** all places of fb have the same degree !!\n");
   fi;

   F.cld.fbpols := Set(List(fb, p->AlffPlaceMin(p)));
   F.cld.ffelts := FFElts(k);

   L := [ Length(F.cld.fb), Length(F.cld.fbdeg[1]) ];
   Info("factor basis parameters ", L, "\n");

   if not IsBound(F.cld.happrox) then
      F.cld.happrox := AlffClassNumberApprox(F, AlffClassNumberApproxBound(F, 2));
   fi;

   GG("g");
   return L;
end;


_AlffFactorBasisEnlarge := function(F)
   local bound, kx, finite, fb, fbpols, i, v;

   bound := F.cld.fbbound + 1;
   # fuer beispiele tex
   F.cld.fbboundenlarge := bound;
   Info("enlarging factor basis to ", bound, "\n");

   kx := PolyAlg(AlffVarT(F));
   fbpols := PolyPrimeList(kx, [1..bound]);
   finite := Flat(List(fbpols, p->AlffPlaceSplit(F, p)));

   finite := Filtered(finite, p->AlffPlaceDeg(p) = bound);
   fb := finite;

   fb := List([1..Length(fb)], j->rec( place := fb[j], i := j + 
                 Length(F.cld.fb)));

   Append(F.cld.fb, fb);
   if IsBound(F.cld.fbdeg[bound]) then
      Append(F.cld.fbdeg[bound], fb);
   else
      Add(F.cld.fbdeg, fb);
   fi;
   F.cld.fbpols := Set(List(F.cld.fb, fbp->AlffPlaceMin(fbp.place)));
   if IsBound(F.cld.rels) then
      for i in [1..Length(F.cld.rels)] do
         v := F.cld.rels[i];
         v := Concatenation(v, 0*[1..Length(fb)]);
         IsMatCheck(v);
         F.cld.rels[i] := v;
      od;
      IsMatCheck(F.cld.rels);
   fi;
   F.cld.fbbound := bound;
   F.cld.smoothprob := 1.0 * AlffDivisorsSmoothNum(F.cld.n, bound, F.cld.P) / 
                       AlffDivisorsSmoothNum(F.cld.n, F.cld.n, F.cld.P);
   Info("new expected smoothness prob ", F.cld.smoothprob, "\n"); 

   GG("g");
   return fb;
end;


###########################################################################


_AlffEnumEnvDivisorLBasisReduced := function(F, D)
   local basis, expo, r;
   # simple strategy. for larger expos use AlffDivisorReduction()
   if not IsBound(F.cld.A) then # max infty reduction
      basis := AlffDivisorLBasisShort(D, "raw");
      expo := Maximum(List(basis, b->b[2]));
      basis := List(Filtered(basis, b->b[2]=expo), b->b[1]);
      return basis;
   else # degree reduction
      r := Floor((AlffDivisorDeg(D) - AlffGenus(F))/AlffDivisorDeg(F.cld.A));
      D := D - r*F.cld.A;
      basis := AlffDivisorLBasis(D);
      return basis;
   fi;
end;


_AlffEnumEnv := function(F, fbp, n)
   local deg, D, fbdeg, fbp1, i;

   D := - fbp.place;
   n := Minimum([n, Length(F.cld.fb), 10]);
   for i in [1..n] do
      fbp1 := RandomList(F.cld.fb);
      D := D + RandomList([1..n])*fbp1.place;
   od;

   fbp.n := n;
   fbp.divisor := D;
   fbp.basis := _AlffEnumEnvDivisorLBasisReduced(F, D);
   fbp.bdim := Length(fbp.basis);

   #   Info("new enum space of dim ", fbp.bdim, "\n");
   #   Info("expo size is ", fbp.n, "\n");

   Unbind(fbp.counter);
   fbp.ffelts := F.cld.ffelts;
   fbp.ffeltsnum := Length(fbp.ffelts);
   F.cld.stat.LDs := F.cld.stat.LDs + 1;
   GG("g");
   return fbp.bdim;
end;



_AlffEnumEnvNext := function(F, fbp)
   local next;
   F.cld.stat.reltries := F.cld.stat.reltries + 1;
   if not IsBound(fbp.counter) then
      fbp.counter := AlffMultiCounterInit(fbp.bdim-1);
      return true;
   fi;   
   next := AlffMultiCounterInc(fbp.counter, fbp.ffeltsnum);
   if next then
      return true;
   fi;
   _AlffEnumEnv(F, fbp, fbp.n);
   fbp.counter := AlffMultiCounterInit(fbp.bdim-1);
   return true;
end;


_AlffEnumEnvElt := function(F, fbp)
   local counter, ffelts, basis, dim;
   basis := fbp.basis;
   ffelts := fbp.ffelts;
   dim := fbp.bdim;
   counter := fbp.counter;
   return basis[dim] + Sum( [1..dim-1], i->ffelts[counter[i]] * basis[i]);
end;


###########################################################################



_AlffRelAdd := function(F, fbp, a, repcheck)
   # it is assumed that (a)_infty is smooth
   local dens, nums, fbpols, dnum, dden, flst, v, place, p, fbp1, i;
   fbpols := F.cld.fbpols;
   flst := List(Factor(AlffEltMin(a)), p->p[1]);
   if not ForAll(flst, p->PolyDeg(p) <= F.cld.fbbound ) then
      return false;
   fi;
   Add(flst, F.1x);
   dens := AlffDivisorSupp(AlffDivisorNum(fbp.divisor));
   v := 0 * [1..Length(F.cld.fb)];
   dden := 0;
   for place in dens do
      i := PositionProperty(F.cld.fb, fbp->fbp.place = place);
      v[i] := AlffEltValuation(place, a);
      dden := dden + v[i] * AlffPlaceDeg(place);
   od;
   dnum := 0;
   for p in flst do
      nums := Filtered(F.cld.fb, fbp->AlffPlaceMin(fbp.place) = p);
      nums := Filtered(nums, fbp->not fbp.place in dens);
      for fbp1 in nums do
         i := fbp1.i;
         v[i] := AlffEltValuation(fbp1.place, a);
         dnum := dnum + v[i] * AlffPlaceDeg(fbp1.place);
      od;
   od;
   if dnum + dden = 0 then
      if repcheck then
         return v[fbp.i] = 1;
      else
         IsMatCheck(v);
         if PositionProperty(F.cld.rels, x->x=v) = false then
            F.cld.rels := [ F.cld.rels, v ];
            IsMatCheck(F.cld.rels);
         fi;
         F.cld.stat.rels := F.cld.stat.rels + 1;
         return true;
      fi;
   else
      return false;
   fi;
end;


###########################################################################



_AlffCheckRelmat := function(F)
   local S, SS, det;
   GG("g");
   # if Length(F.cld.rels) <= Length(F.cld.fb) then
   #    return false;
   # fi;
   S := MatSmith(F.cld.rels);
   if S[1] = Length(F.cld.fb)-1 then
      SS := S[2];
      det := Product([1..MatCols(SS)-1], i->SS[i][i]);
      Info("the determinant is ", det, "\n");
      F.cld.stat.detlast := F.cld.stat.det;
      F.cld.stat.det := det;
      return rec( det := det );
   else   
      Info("the rank is ", S[1], "\n");
      F.cld.stat.ranklast := F.cld.stat.rank;
      F.cld.stat.rank := S[1];
      return rec( rank := S[1] );
   fi;   
end;

##########################################################################


_AlffRelSearch := function(F)
   local fbp, fb, a, i, relfound, stat, fbnewpart;

   stat := rec();
   F.cld.stat := stat;
   F.cld.stat.LDs := 0;
   stat.reltrieslast := 0;
   stat.reltries := 0;
   stat.relslast := 0;
   stat.rels := 0;
   stat.ranklast := 0;
   stat.rank := 0;
   stat.rankeq := 0;
   stat.det := 0;
   stat.deteq := 0;

   stat.reltriesnew := Ceil(Maximum(10 * Ceil(1/F.cld.smoothprob), 20));
   stat.relsnew := Ceil(Maximum(Length(F.cld.fb) / 10, 10));
   stat.rankrelratio := 1;
   stat.smoothness := 1;

   fb := F.cld.fb;
   # stupid:
   F.cld.rels := Mat(Z, [ 0*[1..Length(fb)] ]);

   Info("initializing enum envs ...\n");
   for fbp in fb do
      _AlffEnumEnv(F, fbp, 1);
   od;

   i := 1;
   Info("starting relation search ...\n");
   Info("******* at place ", i, "\n");

   while true do

      fbp := fb[i];
      _AlffEnumEnvNext(F, fbp);
      a := _AlffEnumEnvElt(F, fbp);
      relfound := _AlffRelAdd(F, fbp, a, false);

      if stat.reltries - stat.reltrieslast >= stat.reltriesnew then

         Info("%%%%%%% checking smoothness of reltries ...\n");
         stat.smoothness := 1.0 * stat.rels / stat.reltries; 
         Info("smoothness is ", stat.smoothness, "\n");
         Info("reltries are ", stat.reltries, "\n");

         if stat.smoothness <= F.cld.smoothprob / 100 then

            if IsBound(F.cld.userbound) then
               Info("bound low but user bound given, no enlargement\n");
            else
               Info("%%%% smoothness is bad, enlarging factorbasis\n");
               fbnewpart := _AlffFactorBasisEnlarge(F);
               Info("initializing enum envs ...\n");
               for fbp in fbnewpart do
                  _AlffEnumEnv(F, fbp, 1);
               od;
               stat.reltriesnew := Ceil(Maximum(10 * Ceil(1/F.cld.smoothprob), 
                                           20));
            fi;   
         else
            stat.reltriesnew := Ceil(Maximum(10 * Ceil(1/stat.smoothness), 
                                        20));
         fi;

         Info("redoing smoothness check after ", stat.reltriesnew);
         Info(" new reltries\n");

         stat.reltrieslast := stat.reltries;

      fi;

      if stat.rels - stat.relslast >= stat.relsnew then

         Info("####### checking value of rels ...\n");
         _AlffCheckRelmat(F);

         if IsBound(F.dlogp) and stat.det = true then
            return Length(F.cld.rels);
         fi;

         if IsBound(stat.detlast) then
            if stat.det <= Sqrt(0.5) * F.cld.happrox then
               return Error("approximation error !!!\n");
            fi;
            if stat.det < Sqrt(2) * F.cld.happrox - 0.1 then
               return Length(F.cld.rels);
            fi;

            if stat.det = stat.detlast then
               stat.deteq := stat.deteq + stat.relsnew;
            else
               stat.deteq := 0;
            fi;

            if stat.deteq >= Maximum(Length(F.cld.fb) / 10, 10) then
               Info("independence is bad, enlarging search interval to ", 
                    fbp.n+1, "\n");
               Info("initializing enum envs ...\n");
               for fbp in fb do
                  _AlffEnumEnv(F, fbp, fbp.n+1);
               od;
               stat.deteq := 0;
            fi;

            stat.relsnew := Ceil(stat.smoothness * 2 * Length(F.cld.fb) 
                                 / AlffDeg(F));
         else

            stat.rankrelratio := 1.0 * (
                                        (stat.rank - stat.ranklast) / (stat.rels - stat.relslast) +
                                        stat.rankrelratio ) / 2;

            if stat.rank = stat.ranklast then
               stat.rankeq := stat.rankeq + stat.relsnew;
            else 
               stat.rankeq := 0;
            fi;

            Info("rankrelratio is ", stat.rankrelratio, "\n");

            if stat.rankrelratio < 0.2 and 
               stat.rankeq >= Maximum(Length(F.cld.fb) / 10, 10) then
               Info("independence is bad, enlarging search interval to ", 
                    fbp.n+1, "\n");
               Info("initializing enum envs ...\n");
               for fbp in fb do
                  _AlffEnumEnv(F, fbp, fbp.n+1);
               od;
               stat.rankeq := 0;
               stat.relsnew := Ceil(Maximum(Length(F.cld.fb) / 10, 10));
            else
               stat.relsnew := Ceil( Minimum(3 * stat.rankrelratio 
                                       * stat.relsnew, 
                                       ( Length(fb) - 0.999 - stat.rank) 
                                       / stat.rankrelratio * 
                                       stat.smoothness * 2 * Length(fb) / 
                                       AlffDeg(F)));
            fi;   
         fi;

         Info("redoing check of relations after ", stat.relsnew);
         Info(" new relations\n");
         stat.relslast := stat.rels;

      fi;

      if relfound then
         i := i + 1;
         if i > Length(fb) then
            i := 1;
         fi;
         Info("******* at place ", i, "\n");
      fi;

      GG("g");   
   od;
   return Length(F.cld.rels);
end;


###########################################################################


_AlffEvalDataFast := function(F)
   local M;
   M := MatSmith(F.cld.rels)[2];
   M := Filtered(List([1..MatCols(M)-1], i->M[i][i]), x->not x = 1);
   F.cld.Cl := [ Product(M), M ];
   return F.cld.Cl;
end;


_AlffEvalDataSub := function(gens, M)
   local l, M, S, A, Ai, gensmat, igensmat, i, h;
   if Length(gens) = 0 then
      return [ 1, [], [], [] ];
   fi;
   # this Smith should be done modular
   M := MatSmithTrans(M);
   S := M[2];
   A := M[3];
   l := List([1..M[1]], i->S[i][i]);
   h := Product(l);
   Ai := MatToColList(A^-1);
   gensmat := [];
   igensmat := [];
   for i in [1..M[1]] do
      if not l[i] = 1 then
         Add(gensmat, Ai[i]);
         Add(igensmat, Reversed(A[i]));
      fi;
   od;
   #todo: at least gensmat should be reduced mod hnf of rels or so
   gens := List(gensmat, v->ListApplyListAdd(gens, v));
   l := Filtered(l, x->not x = 1);
   if Length(igensmat) > 0 then 
      igensmat := MatTrans(Z, igensmat);
   else # create matrix with zero columns
      igensmat := MatTrans(MatKernel(MatId(Z, Length(gens))));
   fi;
   return [ h, l, gens, igensmat ];
end;


_AlffEvalDataGens := function(F)
   local M, fb, rels;
   # reverse wegen groesse der stellen
   fb := Reversed(List(F.cld.fb, fb->fb.place));
   # todo: fuer groessere mats erm"oglichen
   rels := List(F.cld.rels, v->Reversed(v));
   M := _AlffEvalDataSub(fb, MatTrans(Z, rels));
   F.cld.Cl := M{[1,2]};
   F.cld.gens := M[3];
   F.cld.igensmat := M[4];
   return F.cld.Cl;
end;



###########################################################################


_AlffOverLattice := function(F)
   local l, p, i, gens, D, cnt;

   l := Factor(F.cld.Cl[1]);
   l := List(l, p->p[1]);

   Info("number of expo vectors to test ", Sum(List(l, 
           p->p^(Length(Filtered(F.cld.Cl[2], x->x mod p = 
                   0)) - 1))), " ...\n");

   for p in l do
      gens := [];
      for i in [1..Length(F.cld.Cl[2])] do
         if F.cld.Cl[2][i] mod p = 0 then
            Add(gens, (F.cld.Cl[2][i] / p) * F.cld.gens[i]);
         fi;
      od;
      cnt := AlffMultiCounterInit(Length(gens)-1);
      repeat
         D := ListApplyListAdd(gens, Concatenation([1], cnt));
         Info("testing expo vector ", List(AlffDivisorPlaces(D), p->p[2]),
              "\n");
         if not AlffDivisorLargeLDim(D) = 0 then
            return Error("yields a relation\n");
         fi;
      until not AlffMultiCounterInc(cnt, p); 
   od;
   return true;

end;


###########################################################################


_AlffHeuristicBound := function(F, fast)
   #
   #  Find heuristic bound for factor basis. 
   #  Assume that we find effective divisors
   #  randomly with degree leq c*g.
   #
   local AlffCostEstimation, 
         g, q, c, L, L1, P, n, maxbound, gbound, bound;


   AlffCostEstimation := function(F, n, b, P)
      # 
      #  P[i] contains the number of places of degree i
      #  We try to very roughly estimate the costs for rel tries 
      #  and SNF computations
      #
      return 1.0 * AlffDeg(F)^5 * AlffGenus(F)^2 * 
             AlffDivisorsSmoothNum(n, n, P) / 
             AlffDivisorsSmoothNum(n, b, P) * Sum([1..b], i->P[i])
             + Sum([1..b], i->P[i])^3 + 1;
   end;

   g := AlffGenus(F);
   q := AlffConstFieldSize(F);
   if g = 0 then
      g := 1;
   fi;
   if IsBound(F.cld.A) then
      c := 1.0 + (AlffDivisorDeg(F.cld.A) - 1) / g;
   else
      c := 1.0 + (AlffDeg(F) - 1) / g;
   fi;
   Info("computing heuristic bound ...\n");
   Info("const field size is ", AlffConstFieldSize(F), "\n");
   Info("genus is ", AlffGenus(F), "\n");
   Info("weak gen bound is ", AlffClassGroupGenBound(F), "\n");
   Info("prod bound for 2 is ", AlffClassNumberApproxBound(F, 2), "\n");
   n := Floor(c*g);
   maxbound := Ceil(n^0.7);
   gbound := Maximum([AlffClassGroupGenBound(F), 
                     AlffClassNumberApproxBound(F, 2)]);
   Info("computing number of places up to degree ", gbound, " ...\n");
   P := List([1..gbound], i->AlffPlacesNum(F, i));
   P := Concatenation(P, List([gbound+1..n], i->(1/i)*q^i));
   gbound := AlffClassGroupGenBoundStrong(F);
   Info("strong gen bound is ", gbound, "\n");
   L := List([1..maxbound], i->AlffCostEstimation(F, n, i, P));
   bound := PositionProperty(L, x->x > 1);
   if bound = false then
      bound := Length(L)+1;
   else
      L1 := L{[bound..Length(L)]};
      bound :=  Position(L1, Minimum(L1)) + bound - 1;
   fi;
   L := List(L, x->Round(10*Log(x))/10.0);
   Info("expected log costs are ", L, "\n");
   Info("heuristic bound ", bound, "\n");
   F.cld.hbound := bound; # for examples
   if not fast then
      bound := Maximum([gbound, bound]);
   fi;
   if IsBound(F.cld.userbound) then
      bound := F.cld.userbound;
   fi;
   F.cld.fbbound := bound;
   F.cld.P := P;
   F.cld.n := n;
   F.cld.smoothprob := 1.0 * AlffDivisorsSmoothNum(n, bound, P) / 
                       AlffDivisorsSmoothNum(n, n, P);
   Info("taking bound ", bound, "\n");
   F.cld.erv :=  AlffDivisorsSmoothRatio(n, bound, P); # for examples
   Info("expected reltries for this are ", F.cld.erv, "\n");
   Info("expected smooth prob is ", F.cld.smoothprob, "\n");
   return bound;
end;


###########################################################################


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffClassGroup
#
#CALLING   Cl := AlffClassGroup( F [, b ] [, A ] [, "fast" ] );
#
#ARGUS     Cl @ list @ of integer and list (class group structure)
#          F @ alff @ global function field
#          b @ integer @ bound for degree of places to be used
#          A @ alff divisor @ for reduction
#
#SHORTDOC  Compute the structure of the class group of a
#          global function field.
#
#LONGDOC   Let $F/k$ be a global function field.
#          This function returns a list {\tt Cl}:
#          {\tt Cl[1]} is the class number and {\tt Cl[2]} is
#          a list of integers $1 < c_1 \,|\, \dots \,|\, c_m$ 
#          such that the class group
#          of $F/k$ is isomorphic to $\Z/c_1\Z \times \dots \times \Z/c_m\Z$. 
#          The function requires that the constant field of definition is
#          the exact constant field.
#          \medskip 
#
#          The following additional parameters can be given:
#          \begin{itemize}
#          \item $b$ is a bound for the places to be considered in the
#           factor base. For large $q$ or $g$ it can be reasonable to take
#           smaller values for $b$ than returned by 
#           {\tt AlffClassGroupGenBoundStrong(F)}.
#          \item $A$ is a reduction divisor. It must factor over the
#           factor base (which is tested automatically) 
#           and should have small, positive degree. If the 
#           degree $[F : k(T)]$ is large than it can be reasonable to take
#           e.g. {\tt A1 := AlffDivisorDegOne(F)}.
#           \item If $b$ is chosen very small and if the class group 
#           is a product of many cyclic factors then option {\tt "fast"} gives 
#           a speedup. The result of the computation however is not proven be 
#           correct anymore.
#          \end{itemize}
#          The algorithm is described in \cite{He2}.
#XREF      AlffClassGroupGenBound, AlffClassNumberApprox, 
#          AlffDivisorDegOne, AlffLPoly, 
#          Alff, AlffInit
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7));;
#          AlffOrders(y^3 + T^7 + T);;
#          AlffClassGroup(F);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffClassGroup := function(arg)
   local F, bound, fast;
   
   if IsList(arg[1]) then 
      arg := arg[1];
   fi;
   
   F := arg[1];
   AlffIsGlobalAssert(F);
   
   if IsBound(F.cld) then
      return F.cld.Cl;
   else
      F.cld := rec();
   fi;
   
   F.cld.time := Runtime() / 100;
   
   if AlffDimExactConstField(F) > 1 then
      return "const field not exact";
   fi;
   fast := "fast" in arg; 
   
   if Length(arg) > 1 and IsInt(arg[2]) then
      bound := arg[2];
      F.cld.userbound := bound;
   fi;   

   if IsInt( PositionProperty(arg, IsAlffDivisor)) then
      F.cld.A := arg[ PositionProperty(arg, IsAlffDivisor) ];
      Info("reduction divisor is ", F.cld.A, "\n");
   fi;
   
   bound := _AlffHeuristicBound(F, fast);
   _AlffFactorBasisCreate(F, bound);
   GG("g");
   
   while Length(F.cld.fb) <= 1 do
      Info("fb too small, enlarge by one degree ...\n");
      _AlffFactorBasisEnlarge(F);
   od;
   GG("g");
   
   _AlffRelSearch(F);
   Info("total number of tried rels is ", F.cld.stat.reltries, "\n");
   GG("g");
   
   if IsBound(F.dlogp) then
      F.cld.Cl := [ F.dlogp ];
   else
      if bound < AlffClassGroupGenBoundStrong(F) then
         if not fast then
            _AlffEvalDataGens(F);
            _AlffOverLattice(F);
            Info("result proven by lattice\n");
            F.cld.proven := true;
         else
            _AlffEvalDataFast(F);
            F.cld.proven := false;
         fi;
      else
         Info("result proven by bounds\n");
         _AlffEvalDataFast(F);
         F.cld.proven := true;
      fi;   
   fi;
   
   F.cld.time := Round(Runtime() / 100 - F.cld.time) / 10.0;
   GASMAN("g");
   return F.cld.Cl;
end;


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffClassGroupGens
#
#CALLING   L := AlffClassGroupGens( F, [ b, ] [ A, ] [ "fast" ] );
#
#ARGUS     L @ list @ of generating divisors
#          F @ alff @ global function field
#          b @ integer @ bound for degree of places to be used
#          A @ alff divisor @ for reduction
#
#SHORTDOC  Compute generators of the divisor class group of a
#          global function field.
#
#LONGDOC   Let $F/k$ be a global function field.
#          This function returns a list $[ A_1, D_1, \dots, D_m ]$
#          of divisors of $F/k$ such that $\deg(A_1) = 1$ 
#          and $\deg(D_i) = 0$. The classes $[D_i]$ correspond to
#          generators of the cyclic factors of the class group of $F/k$
#          as returned by~{\tt AlffClassGroup(F)}. 
#          The calling sequence is identical to that 
#          of {\tt AlffClassGroup()}.
#
#XREF      AlffClassGroup, AlffClassGroupGenBound, 
#          AlffDivisorDegOne, Alff, AlffInit
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7));;
#          AlffOrders(y^3 + T^7 + T);;
#          AlffClassGroupGens(F);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffClassGroupGens := function(arg)
   local F;
   F := arg[1];
   if not IsBound(F.cld) then
      AlffClassGroup(arg);
   fi;
   if IsBound(F.cld.gens) then
      return Concatenation( [ F.cld.A1 ], F.cld.gens );
   fi;
   _AlffEvalDataGens(F);
   return Concatenation( [ F.cld.A1 ], F.cld.gens );
end;



#############################################################################
#  class representations
#


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDivisorClassRep
#
#CALLING   L := AlffDivisorClassRep(D);
#
#ARGUS     L @ list @  exponents
#          D @ alff divisor
#
#SHORTDOC  Find the class representation of a divisor class in given
#          generators.
#
#LONGDOC   Let $F/k$ be a global function field and $A_1, D_1, \dots, D_m$
#          be a generating system as returned by {\tt AlffClassGroupGens(F)}.
#          Let $d_i$ be the (finite) orders of $[D_i]$.
#          This function returns a list consisting 
#          of $a, r_1, \dots, r_m \in \Z$
#          with $0 \leq r_i < d_i$ such that $$ [D] = a [A_1] + 
#          r_1 [D_1] + \dots + r_m [D_m]$$ holds.
#
#XREF      AlffClassGroupGens, AlffClassGroup, Alff, AlffInit
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7));;
#          AlffOrders(y^3 + T^7 + T);;
#          L := AlffClassGroupGens(F);
#          D := -17135*L[1] + 15*L[4];
#          AlffDivisorClassRep(D);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDivisorClassRep := function(arg)
   
   local AlffClgDivisorFactorFb, F, v, D, Dred1, Dred2, Drand, vDrand, B, env,
         Dtilde, r;
   
   AlffClgDivisorFactorFb := function(F, D)
      local N, pe, pel, j, v;
      N := AlffDivisorNorm(D);
      N := Concatenation( Factor(QfeNum(N)), Factor(QfeDen(N)) );
      if Length(N) > 0 and
         Maximum(List(N, x->PolyDeg(x[1]))) > F.cld.fbbound then
         return false;
      fi;
      pel := AlffDivisorPlaces(D);
      v := 0*[1..Length(F.cld.fb)];
      for pe in pel do
         j := PositionProperty(F.cld.fb, x->x.place = pe[1]);
         if not IsInt(j) then
            return false;
         fi;
         v[F.cld.fb[j].i] := pe[2];
      od;
      return v;
   end;
   
   D := arg[1];
   F := AlffDivisorAlff(D);
   AlffIsGlobalAssert(F);
   if not IsBound(F.cld) or not IsBound(F.cld.igensmat) then
      AlffClassGroupGens(F);
   fi;
   
   if not IsBound(F.cld.xinftyvec) then
      F.cld.xinftyvec :=  AlffClgDivisorFactorFb(F, 
                                  AlffDivisorDen(AlffDivisor(AlffEltGenT(F))));
   fi;
   
   if IsBound(F.cld.A) and not IsBound(F.cld.Avec) then
      F.cld.Avec := AlffClgDivisorFactorFb(F, F.cld.A);
   fi;
   
   r := AlffDivisorDeg(D);
   D := D - r * F.cld.A1;  # gen 1
   
   Dred1 := AlffDivisorReduction(D); # (x)_infty is smooth!
   D := Dred1[1];
   v := AlffClgDivisorFactorFb(F, D);
   
   if not v = false then
      Info("first reduction ok\n");
      v := v + Dred1[2] * F.cld.xinftyvec;
   else
      Info("try several random reductions ...\n");
      while v = false do
         Drand := RandomList([1..3]) * RandomList(F.cld.fb).place;
         if IsBound(F.cld.A) then
            Dred2 := AlffDivisorReduction(D + Drand, F.cld.A);
         else
            Dred2 := AlffDivisorReduction(D + Drand);
         fi;
         B := AlffDivisorLBasis(Dred2[1]);
         Info("enum dim ", Length(B), "\n");
         env := AlffLinearSeriesEnumEnv(F, Dred2[1], B);
         while v = false and AlffLinearSeriesEnumNext(env) do
            Dtilde := AlffLinearSeriesEnumElt(env);
            v := AlffClgDivisorFactorFb(F, Dtilde);
         od;
      od;
      
      vDrand := AlffClgDivisorFactorFb(F, Drand);
      
      if IsBound(F.cld.A) then
         v := v - vDrand + Dred2[2] * F.cld.Avec + Dred1[2] * F.cld.xinftyvec;
      else
         v := v - vDrand + (Dred2[2] + Dred1[2]) * F.cld.xinftyvec;
      fi;
   fi;
   
   if "fb" in arg then
      return r * F.cld.A1 + 
             ListApplyListAdd(List(F.cld.fb, fbp->fbp.place), v);
   fi;
   
   IsMatCheck(v);
   v := v * F.cld.igensmat;
   v := List([ 1..Length(F.cld.Cl[2]) ], i->v[1][i] mod F.cld.Cl[2][i]);
   return Concatenation( [r], v );
end;


#############################################################################


#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffSUnits
#
#CALLING   L := AlffSUnits( S [, "raw" ] );
#
#ARGUS     L @ list @ the $S$-regulator and a basis of $S$-units
#          S @ list @ a list of places
#
#SHORTDOC  Find the $S$-regulator and a basis 
#          for the free part of the $S$-units.
#
#LONGDOC   Let $F/k$ be a global function field and $S$ be a set
#          of places of $F/k$. This function returns a list $L$ such that
#          $L[1]$ is the $S$-regulator and $L[2]$ is a basis for the
#          free part of the $S$-unit subgroup of $F^\times$, consisting
#          of principal divisors if the option {\tt "raw"} is given.
#
#XREF      AlffClassGroupGens, AlffClassGroup, Alff, AlffInit, 
#          AlffDivisorReduction, AlffDivisorLargeLDim
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7));;
#          AlffOrders(y^3 + T^3 + T);;
#          S := AlffPlacesDegOne(F);
#          AlffSUnits(S);
#          AlffSUnits(S, "raw");
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffSUnits := function(arg)
   local MatHermiteRowUpperTrans, S, F, h, Cl, L, im, imT, imS, i, a,
         RS, res1, res2, D;
   
   MatHermiteRowUpperTrans := function(M)
      M := MatTrans(M);
      M := MatHermiteColLowerTrans(M);
      return [ M[1], MatTrans(M[2]), MatTrans(M[3]) ];
   end;
   
   S := arg[1];
   if Length(S) <= 1 then
      return [];
   fi;
   F := AlffPlaceAlff(S[1]);
   AlffIsGlobalAssert(F);
   Cl := AlffClassGroup(F);
   h := Cl[1];
   im := List(S, p->AlffDivisorClassRep(1*p));
   L := Concatenation( [1], List([1..Length(Cl[2])], i->(h / Cl[2][i])) );
   im := List(im, v->List([1..Length(Cl[2])+1], i->L[i] * v[i]));
   IsMatCheck(im);
   im := MatHermiteRowUpperTrans(im);
   # cut off positive degrees 
   imT := im[3]{[2..Length(im[3])]};
   im := im[2]{[2..Length(im[2])]};
   IsMatCheck(imT);
   IsMatCheck(im);
   # make diagonal
   im := MatSmithTrans(im);
   imS := im[2];
   RS := Product([1..im[1]], i->IntLcm(imS[i][i], h) / imS[i][i]);
   imT := im[3] * imT;
   for i in [1..im[1]] do
      a := imS[i][i];
      imT[i] := (IntLcm(a, h) / a) * imT[i];
   od;
   IsMatCheck(imT);
   imT := MatHermiteColUpper(MatTrans(imT));
   
   res1 := ListApplyMatAdd(S, imT);
   if "raw" in arg then
      return [ RS, res1 ];
   else
      res2 := [];
      for D in res1 do
         a := Filtered(AlffDivisorLargeLBasisShort(D), x->x[2] >= 0)[1];
         if not a[2] = 0 then 
            return Error("something is wrong");
         fi;
         Add(res2, a[1]);
      od;
      return [ RS, res2 ];
   fi;
end;

#############################################################################

#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffIdealClassGroupUnitsInfty
#
#CALLING   L := AlffIdealClassGroupUnitsInfty(F);
#
#ARGUS     L @ list @ units and ideal class group info
#          F @ alff @ global function field
#
#SHORTDOC  Compute the unit group and the ideal class group of
#          the finite maximal order ${\rm Cl}( k[T], F )$. 
#
#LONGDOC   Let $F/k$ be a global function field and $\o$ be the finite
#          maximal order ${\rm Cl}( k[T], F )$. 
#          This function returns a list containing the regulator and
#          the structure of the ideal class group of $\o$ in the form 
#          group order, list of cyclic factors.
#
#XREF      AlffClassGroup, AlffSUnits, Alff, AlffInit
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7));;
#          AlffOrders(y^3 + T^7 + T);;
#          AlffPlaceSplitType(F, 1/T);
#          AlffIdealClassGroupUnitsInfty(F);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


AlffIdealClassGroupUnitsInfty := function(arg)
   local F, isLess, fb, rels, posf, col, H, RUnits, RClassgroup;
   F := arg[1];
   AlffIsGlobalAssert(F);
   if not IsBound(F.cld) then
      AlffClassGroup(F);
   fi;
   if IsBound(F.cld.ClS) then
      if "raw" in arg then
         return F.cld.ClS;
      else
         return [ F.cld.ClS[1][1], F.cld.ClS[2]{[1..2]} ];
      fi;
   fi;
   fb := List(F.cld.fb, fbp->fbp.place);
   #todo: fuer groessere mats erm"oglichen
   rels := F.cld.rels;
   rels := List([1..Length(fb)], i->List([1..Length(rels)], j->rels[j][i]));
   isLess := function(a, b)
      local af, bf;
      af := AlffPlaceIsFinite(a);
      bf := AlffPlaceIsFinite(b);
      if af = bf then
         return AlffPlaceDeg(a) < AlffPlaceDeg(b);
      elif af then
         return false;
      else 
         return true;
      fi;
   end;
   SortParallel(fb, rels, isLess);
   posf := PositionProperty(fb, AlffPlaceIsFinite);
   H := MatHermiteColUpper(Mat(Z, rels));
   RUnits := H{[1..posf-1]};
   IsMatCheck(RUnits);
   col := MatCols(H)-Length(fb)+1;
   RUnits := MatToColList(RUnits){[col..col+posf-2]};
   RUnits := MatTrans(Z, RUnits);
   RUnits := _AlffEvalDataSub( fb{[1..posf-1]}, RUnits ); 
   RClassgroup := H{[posf..Length(fb)]};
   IsMatCheck(RClassgroup);
   RClassgroup := _AlffEvalDataSub( fb{[posf..Length(fb)]}, RClassgroup);
   F.cld.ClS := [ RUnits, RClassgroup];
   if "raw" in arg then
      return F.cld.ClS;
   else
      return [ F.cld.ClS[1][1], F.cld.ClS[2]{[1..2]} ];
   fi;
   return F.cld.ClS;
end;
