
###############################################################
###############################################################
#
# differentials - basis functions
#
# representation by g(x, y)dx,  g(x, y) in OrderMaxFinite
#
#
#

#############################################################################
#
#

AlffDifferentialOps := rec();
AlffDifferentialLocals := rec();

AlffDifferentialLocals.New := function(K)
   return rec(
              Alff := K,
              name := "Alff Differential",
              operations := AlffDifferentialOps,
              lambda := 0
              );
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       IsAlffDiff
#
#SHORTDOC  Returns whether argument is an alff differential.
#
#CALLING   b := IsAlffDiff(a);
#
#ARGUS     b @ boolean
#          a @ alff differential
#
#CSRC      
#
#XREF      AlffDiff
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2 + T^3 + 1);
#          dT := AlffDiff(AlffEltGenT(F));
#          IsAlffDiff(dT);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

IsAlffDiff := function(fdx)
   if IsRec(fdx) then
      if IsBound(fdx.name) then
         return fdx.name = "Alff Differential";
      else
         return false;  
      fi;
   else
      return false;
   fi;
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDiffAlff
#
#SHORTDOC  Returns the algebraic function field of an alff differential.
#
#CALLING   F := AlffDiffAlff(hdx);
#
#ARGUS     F @ algebraic function field
#          hdx @ alff differential
#
#CSRC      
#
#XREF      AlffDiff
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2 + T^3 + 1);
#          dT := AlffDiff(AlffEltGenT(F));
#          AlffDiffAlff(dT);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDiffAlff := function(fdx)
   return fdx.Alff;
end;


AlffDifferentialOps.Print := function(fdx)
   Print(fdx.name, "\n");
   Print(fdx.lambda, " d", AlffEltGenT(fdx.Alff));
end;



AlffDifferentialOps.\* := function(f, gdx)
   local hdx;
   hdx := AlffDifferentialLocals.New(gdx.Alff);
   hdx.lambda := f * gdx.lambda;
   return hdx;
end;


AlffDifferentialOps.\= := function(fdx, gdx)
   local hdx;
   if not IsAlffDiff(fdx) or
      not IsAlffDiff(gdx) then
      return false;
   fi;
   return fdx.lambda = gdx.lambda;
end;


AlffDifferentialOps.\+ := function(fdx, gdx)
   local hdx;
   hdx := AlffDifferentialLocals.New(fdx.Alff);
   hdx.lambda := fdx.lambda + gdx.lambda;
   return hdx;
end;


AlffDifferentialOps.\- := function(fdx, gdx)
   local hdx;
   hdx := AlffDifferentialLocals.New(fdx.Alff);
   hdx.lambda := fdx.lambda - gdx.lambda;
   return hdx;
end;


AlffDifferentialOps.\/ := function(fdx, gdx)
   local hdx;
   if IsAlffElt(gdx) then
      hdx := AlffDifferentialLocals.New(fdx.Alff);
      hdx.lambda := 1/gdx * fdx.lambda;
      return hdx;
   else
      return fdx.lambda / gdx.lambda;
   fi;   
end;


AlffDifferentialLocals.DiffY := function(a)
#
#  Compute da/dy   
#  ( a must be represented in the finite eq order )
#
   local oe, K, n, b, den, i;
   if IsPolyOrQfe(a) then
      return 0*a;
   fi;
   K := AlffEltAlff(a);
   oe := AlffOrderEqFinite(K);
   b := AlffEltMove(a, oe);
   den := AlffEltDen(b);
   b := AlffEltToList(AlffEltNum(b))[1];
   n := Length(b);
   for i in [1..n-1] do
      b[i] := i*b[i+1];
   od;
   b[n] := 0;
   b := AlffElt(oe, b) / den;
   return b;
end;


AlffDifferentialLocals.DiffX := function(a)
#
#  Compute da/dx   
#  ( a must be represented in the finite eq order )  
#   
   local oe, K, b, den, c, d;
   if IsPolyOrQfe(a) then
      return QfeDeriv(a);
   fi;
   K := AlffEltAlff(a);
   oe := AlffOrderEqFinite(K);
   b := AlffEltMove(a, oe);
   den := AlffEltDen(b);
   b := AlffEltNum(b);
   c := AlffElt(oe, List(AlffEltToList(b)[1], j->PolyDeriv(j)));
   d := PolyDeriv(den);
   return (c*den - b*d) / den^2;
end;


AlffDifferentialLocals.dydx := function(K)
#   
#  Relate dy and dx by f(x,y) = 0.   
#   
   local oe, n, f, fy, fx, dy, y;
   if IsBound(K.dydx) then
      return K.dydx;
   fi;
   oe := AlffOrderEqFinite(K);
   n := AlffDeg(K);
   f := AlffOrderPoly(oe);
   fy := Reversed(PolyToList(PolyDeriv(f)));
   Append(fy, List( [ Length(fy)+1 .. n ], i->0));
   fy := AlffElt(oe, fy);
   fx := Reversed(PolyToList(PolySwapVars(PolyDeriv(PolySwapVars(f)))));
   Append(fx, List( [ Length(fx)+1 .. n ], i->0));
   fx := AlffElt(oe, fx);
   K.dydx := -fx/fy;
   return K.dydx;
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDiff
#
#SHORTDOC  Returns the exact differential $dh$ of an alff element $h$.
#
#CALLING   dh := AlffDiff(h);
#
#ARGUS     dh @ alff differential
#          h @ alff element
#
#CSRC      
#
#XREF      AlffDiffDivisor, Alff
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2 + T^3 + 1);
#          dT := AlffDiff(AlffEltGenT(F));
#          dy := AlffDiff(AlffEltGenY(F));
#          dy / dT;
#          dT / dy;
#          dT + dy;
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDiff := function(a)
#   
#  Return d(a) for a function a.
#   
   local fdx, K;
   K := AlffEltAlff(a);
   a := AlffEltMove(a, AlffOrderEqFinite(K));
   fdx := AlffDifferentialLocals.New(K);
   fdx.lambda := AlffDifferentialLocals.DiffX(a) + 
      AlffDifferentialLocals.DiffY(a) * AlffDifferentialLocals.dydx(K);
   fdx.lambda := AlffEltMove(fdx.lambda, AlffOrderMaxFinite(K));
   return fdx;
end;



###############################################################
#
# differentials - canonical divisors etc. 
# 



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDiffValuation
#
#SHORTDOC  Returns the order of a non zero alff differential at a place.
#
#CALLING   v := AlffDiffValuation(p, fdx);
#
#ARGUS     v @ integer
#          p @ alff place
#          fdx @ alff differential
#
#CSRC      
#
#XREF      AlffDiffDivisor, AlffDiff
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2 + T^3 + 1);
#          dT := AlffDiff(AlffEltGenT(F));
#          infty := AlffPlaceSplit(F, 1/T)[1];
#          AlffDiffValuation(infty, dT);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDiffValuation := function(place, fdx)
#
#  Compute the valuation of the differential at a place
#
   local t, dt;
   t := AlffPlacePrimeElt(place);
   dt := AlffDiff(t);
   return AlffEltValuation(place, fdx/dt); 
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDifferentDeg
#
#SHORTDOC  Computes the degree of the different of an algebraic function 
#          field extension $F/ k(T)$.
#
#CALLING   d := AlffDifferentDeg(F);
#
#ARGUS     d @ integer
#          F @ algebraic function field
#
#CSRC      
#
#XREF      AlffDifferent, AlffCanonicalDivisor, AlffDiffDivisor
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^7 + 1);
#          AlffDifferentDeg(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDifferentDeg := function(K)
#
#  Return the different degree of K / k(x)
#
   local of, oi, df, di;
   of := AlffOrderMaxFinite(K);
   oi := AlffOrderMaxInfty(K);
   df := AlffOrderDisc(of);
   di := AlffOrderDisc(oi);
   return (PolyDeg(df) + InftyVal(di)); # /AlffDimExactConstField(K);
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDifferent
#
#SHORTDOC  Computes the different of an algebraic function field 
#          extension $F/ k(T)$.
#
#CALLING   D := AlffDifferentDeg(F);
#
#ARGUS     D @ alff divisor
#          F @ algebraic function field
#
#CSRC      
#
#XREF      AlffDifferentDeg, AlffCanonicalDivisor, AlffDiffDivisor
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^7 + 1);
#          AlffDifferent(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDifferent := function(K)
#   
#  Compute the different divisor of K / k(x) (by the trace matrix) 
#   
   local o, oi, kx, kxf, b, trmat, p, pvf, x, pvi;
   
   if IsBound(K.different) then
      return K.different;
   fi;
   
   o := AlffOrderMaxFinite(K);
   oi := AlffOrderMaxInfty(K);
   kx := PolyAlg(AlffVarT(K));
   kxf := QuotientField(kx);
   
   b := AlffOrderBasis(o);
   trmat := Mat(kxf, List(b, x->List(b, y->AlffEltTrace(x*y))));
   b := ListApplyMatAdd(b, MatInv(trmat));
   p := Filtered(Factor(AlffOrderDisc(o)), x->PolyDeg(x[1]) > 0);
   p := Flat(List(p, x->AlffPlaceSplit(K, x[1])));
   pvf := List(p, x->[x,-Minimum(List(b, y->AlffEltValuation(x, y)))]);
   
   x := AlffVarT(K);
   b := AlffOrderBasis(oi);
   trmat := Mat(kxf, List(b, x->List(b, y->AlffEltTrace(x*y))));
   b := ListApplyMatAdd(b, MatInv(trmat));
   p := AlffPlaceSplit(K, 1/x); 
   pvi := List(p, x->[x,-Minimum(List(b, y->AlffEltValuation(x, y)))]);
   
   K.different := Sum(List(Concatenation(pvf, pvi), 
                          x->x[2]*AlffDivisor(x[1])));
   
   if not AlffDifferentDeg(K) = AlffDivisorDeg(K.different) then
      return Error("error in computation");
   fi;
   
   return K.different;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDiffDivisor
#
#SHORTDOC  Computes the divisor of an alff differential $h\,dT$.
#
#CALLING   D := AlffDifferentDeg(hdT);
#
#ARGUS     D @ alff divisor
#          hdT @ alff differential
#
#CSRC      
#
#XREF      AlffDivisor, AlffDiffSpace
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^7 + 1);
#          dT := AlffDiff(AlffEltGenT(F));
#          AlffDiffDivisor(dT);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDiffDivisor := function(fdx)
#
#  Compute the divisor of a differential
#
   local x;
   if IsBound(fdx.divisor) then
      return fdx.divisor;
   fi;
   x := AlffVarT(fdx.Alff);
   x := AlffElt(AlffOrderMaxFinite(fdx.Alff), x);
   fdx.divisor := AlffDivisor(fdx.lambda) - 2*AlffDivisorDen(AlffDivisor(x)) +
                  AlffDifferent(fdx.Alff);
   return fdx.divisor;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffCanonicalDivisor
#
#SHORTDOC  Computes a canonical divisor of an algebraic function field
#          $F/k$.
#
#CALLING   D := AlffCanonicalDivisor(F);
#
#ARGUS     D @ alff divisor
#          F @ algebraic function field
#
#CSRC      
#
#XREF      AlffDifferentDeg, AlffDifferent, AlffDiffDivisor
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^7 + 1);
#          AlffCanonicalDivisor(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffCanonicalDivisor := function(F)
#
#  Computes (dx) and stores it in F (practical for later use).
#
   local x, dx;
   if IsBound(F.Dx) then
      return F.Dx;
   fi;
   x := AlffVarT(F);
   dx := AlffDiff(AlffElt(AlffOrderEqFinite(F), x));
   F.Dx := AlffDiffDivisor(dx);
   return F.Dx;
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDiffResiduum
#
#SHORTDOC  Computes the residuum of a differential
#
#LONGDOC   Let $F/k$ be an algebraic function field and $\omega = hdT$ be
#          a differential of $F/k$. For a place $P$, which is currently 
#          required
#          to be of degree one, this function returns the residuum
#          of $\omega$ at $P$.
#
#CALLING   r := AlffDiffResiduum(P, hdT);
#
#ARGUS     r @ field element @ the residuum of $hdT$ at $P$
#          P @ alff place @
#          fdT @ alff differential @
#
#CSRC      
#
#XREF      AlffDiffDivisor, AlffDiffValuation, AlffEltResiduum
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^7 + 1);
#          P := AlffPlaceSplit(F, T+3)[1];
#          a := AlffEltGenY(F) + 1;
#          AlffEltValuation(P, a);
#          hdT := (1/a^2)*AlffDiff(a^2);
#          AlffDiffResiduum(P, hdT);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDiffResiduum := function(place, fdx)
#
#  Compute the residuum of a differential at a place.
#  ((The "residuum" is traced to the constant field))
#
   local t, dt;
   if AlffPlaceDeg(place) > 1 then
      return Error("only for degree one places");
   fi;
   t := AlffPlacePrimeElt(place);
   dt := AlffDiff(t);
   return Trace(AlffEltResiduum(place, t, fdx/dt), 
                AlffConstField(AlffPlaceAlff(place)));
end;


###############################################################
###############################################################
# differentials - specials
#
# 


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDiffFirstKind
#
#SHORTDOC  Computes a basis for the differentials of the first kind
#          (holomorphic differentials) of an algebraic function field
#          $F/k$.
#
#CALLING   B := AlffDiffFirstKind(F);
#
#ARGUS     B @ list @ of alff differentials
#          F @ algebraic function field
#
#CSRC      
#
#XREF      AlffDiffSpace, AlffDiffDivisor.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^7 + 1);
#          AlffDiffFirstKind(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


AlffDiffFirstKind := function(F)
#
#  Return basis of differentials of first kind (dimension g)
# 
   local x, dx, Dx, DxR;
   if not IsAlff(F) then
      return Error("usage: AlffDiffFirstKind ( <alff> )");
   fi;
   if IsBound(F.dfkbasis) then
      return F.dfkbasis;
   fi;
   x := AlffVarT(F);
   dx := AlffDiff(AlffElt(AlffOrderMaxFinite(F), x));
   Dx := AlffDiffDivisor(dx);
   DxR := AlffDivisorLBasis(Dx);
   F.dfkbasis := List(DxR, a->a*dx);
   if not Length(F.dfkbasis) = AlffGenus(F) then
      return Error("error in computation");
   fi;
   return F.dfkbasis;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDiffSpace
#
#SHORTDOC  Computes a basis for the space of differentials 
#          $$\Omega(D) := \{\; \omega \in \Omega(F/k) \;|\; 
#          (\omega) \geq D \;\}$$
#          for a divisor $D$ of an algebraic function field $F/k$.
#
#CALLING   B := AlffDiffSpace(D);
#
#ARGUS     B @ list @ of alff differentials
#          D @ alff divisor
#
#CSRC      
#
#XREF      AlffDiffFirstKind, AlffDiffDivisor.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^7 + 1);
#          D := AlffDivisor(F);
#          AlffDiffSpace(D);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDiffSpace := function(D)
#
#  Return basis of Omega(D): (omega) + D >= 0, omega differential
#
   local x, dx, Dx, basis;
   if not IsAlffDivisor(D) then
      return Error("usage: AlffDiffSpace ( <alff divisor> )");
   fi; 
   x := AlffVarT(AlffDivisorAlff(D));
   dx := AlffDiff(AlffElt(AlffOrderMaxFinite(AlffDivisorAlff(D)), x));
   Dx := AlffDiffDivisor(dx);
   basis := AlffDivisorLBasis(D+Dx);
   return List(basis, a->a*dx);
end;


#############################################################################
#############################################################################
#
#  char p stuff
#



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffEltPthRoot
#
#LONGDOC   Let $F/k$ be a global function field of characteristic $p$.
#          This function computes the unique $p^r$th root 
#          $b$ of an alff element
#          $a$ in $F$, or returns false if it does not exist. 
#
#CALLING   b := AlffEltPthRoot(F, r);
#
#ARGUS     b, a @ alff element
#
#CSRC      
#
#XREF      AlffDivisor, AlffDivisorLDim
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^7 + 1);
#          c := AlffEltGenY(F);
#          a := c^25;
#          AlffEltPthRoot(a, 1);
#          AlffEltPthRoot(a, 2);
#          AlffEltPthRoot(a, 3);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


AlffEltPthRoot := function(a, k)
#
#  Compute the k-fold pth root of a global function field element
#
   local B, p, F, M, v, i, j;

   F := AlffEltAlff(a);
   AlffIsGlobalAssert(F);
   p := FFSize(AlffConstField(F))[1];
   B := List(AlffOrderBasis(AlffEltOrder(a)), x->x^p);
   M := List(B, x->AlffEltToList(x)[1]);
   IsMatCheck(M);
   M := 1/M;
   for j in [1..k] do
      v := AlffEltToList(a);
      v := v[1] / v[2];
      IsMatCheck(v);
      v := v * M;
      v := MatToRowList(v)[1];
      for i in [1..Length(v)] do
         v[i] := QfePthRoot(v[i]);
         if v[i] = false then
            return false;
         fi;
      od;
      a := AlffElt(AlffEltOrder(a), v);
   od;
   return a;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDiffCartier
#
#SHORTDOC  Apply the Cartier operator.
#
#LONGDOC   Let $F/k$ be a global function field, $x \in F$ be a 
#          separating variable and $g\,dx\in \Omega(F/k)$
#          be a differential. 
#          The Cartier operator is defined by
#          $$ C(\omega) := \left( - d^{p-1} g 
#          / dx^{p-1} \right)^{1/p} dx.$$ This function computes the
#          the $r$th iterated application of $C$ to $g\,dx$.
#
#CALLING   hdx := AlffDiffCartier(gdx, r);
#
#ARGUS     hdx, gdx @ differentials
#          r @ integer @ number of iterated applications
#
#CSRC      
#
#XREF      AlffDiff.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^7 + 1);
#          a := AlffElt(o, [0, 1/T, 0, T+1]);
#          da := AlffDiff(a);
#          AlffDiffCartier(da, 1);
#          gdx := (1/a)*da;
#          AlffDiffCartier(gdx, 1);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


AlffDiffCartier := function(fdT, r)
#
#  Apply the Cartier operator r times to fdT.
#  
  local F, p, dT, f, i, j;
  F := AlffDiffAlff(fdT);
  AlffIsGlobalAssert(F);
  p := FFSize(AlffConstField(F))[1];
  dT := AlffDiff(AlffEltGenT(F));
  for j in [1..r] do
     f := fdT / dT;
     for i in [1..p-1] do
        f := AlffDiff(f) / dT;
     od;
     f := AlffEltPthRoot(-f, 1);
     fdT := f * dT;
  od;
  return fdT;
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDiffCartierMatrix
#
#SHORTDOC  Compute a representation matrix of the Cartier operator. 
#
#LONGDOC   Let $F/k$ be a global function field, $\omega_1, \dots, 
#          \omega_g \in \Omega(F/k)$ be a basis for the holomorphic 
#          differentials and $r \in \Z^{\geq 1}$.
#          Let $M = (\lambda_{i,j})_{i,j} \in 
#          k^{g \times g}$ be the matrix such that
#          $$ C^r(\omega_i) = \sum_{m=1}^g \lambda_{i, m} \omega_m$$
#          for all $1 \leq i \leq g$. This function returns $M$.
#
#CALLING   M := AlffDiffCartierMatrix(F, r);
#
#ARGUS     M @ matrix
#          F @ global function field
#          r @ integer @ $C$ is $r$ times applied.
# 
#CSRC      
#
#XREF      AlffDiff.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^7 + 1);
#          AlffDiffCartierMatrix(F, 1);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


AlffDiffCartierMatrix := function(F, r)
#
#  Compute a row representation matrix of the Cartier operator
#  applied r times on the space of holomorphic differentials.
#
   local AlffMat1p, B, C, M, Mi, i;
   
   AlffMat1p := function(M)
   #
   #  Compute M^(1/p).
   #
      local k, p, q, i, j, M1p;
      if MatRows(M) = 0 or MatCols(M) = 0 then
         return [];
      fi;
      M1p := 0*M;
      k := FFEltFF(M[1][1]);
      p := FFSize(k);
      q := p[1]^p[2];
      p := p[1];
      for i in [1..MatRows(M)] do
         for j in [1..MatCols(M)] do
            M1p[i][j] := M[i][j]^(q/p);
         od;
      od;
      IsMatCheck(M1p);
      return M1p;
   end;
   
   AlffIsGlobalAssert(F);
   if not IsBound(F.CartierMatrix) then
      B := AlffDiffFirstKind(F);
      C := List(B, x->AlffDiffCartier(x, 1));
      F.CartierMatrix := AlffLinearRepresent(B, C); 
   fi;
   
   M := F.CartierMatrix;
   Mi := M;
   for i in [2..r] do
      Mi := AlffMat1p(Mi);
      M := M * Mi;
   od;
   return M;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffClassGroupPRank
#
#SHORTDOC  Compute the $p$-rank of the class group of a 
#          global function field.
#
#LONGDOC   Let $F/k$ be a global function field of characteristic $p$.
#          Consider the subgroup $Cl^0(F/k)[p]$ of 
#          $p$-torsion elements of the group of divisor classes of
#          degree zero. This function returns its dimension as an
#          $\FF_p$-vector space. Possible values range from $0$ to $g$, 
#          where $g$ is the genus of $F/k$.
#
#CALLING   s := AlffClassGroupPRank(F);
#
#ARGUS     s @ integer
#          F @ global function field
#
#CSRC      
#
#XREF      AlffHasseWittInvariant
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2, 1));
#          F := Alff(T*y^3 + y + T^3);
#          AlffClassGroupPRank(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffClassGroupPRank := function(F)
#
#  Compute the p-rank of the class group of F.
#
   local AlffMat1pSolve, M;
   
   AlffMat1pSolve := function(M)
   #
   # Solve x^(1/p) M = x.  Return Fp-basis of solutions.
   #
      local k, k0, kx, r, p, q, a, M, MM1, MM2, v, i, I;
      
      if MatRows(M) = 0 or MatCols(M) = 0 then
         return [];
      fi;
      k := FFEltFF(M[1][1]);
      r := FFSize(k);
      p := r[1]; 
      r := r[2];
      q := p^r;
      a := FFGenerator(k);
      k0 := FFEltFF(FFEltToList(a)[1]);
      kx := PolyAlg(k);
      if FFSize(k0)[2] > 1 then
         return Error("double relative extension of ff not supported");
      fi;
      
      MM1 := [];
      for i in [1..Length(M)] do
         Add(MM1, List([0..r-1], j->a^(q/p*j) * M[i]));
      od;
      IsMatCheck(MM1);
      MM2 := [];
      for i in [1..Length(MM1)] do
         v := Flat(List(MM1[i], x->FFEltToList(x)));
         IsMatCheck(v);
         Add(MM2, v);
      od;
      IsMatCheck(MM2);
      
      I := MatId(k0, MatRows(MM2));
      MM1 := MatKernel(I - MM2);
      MM2 := [];
      for i in [1..Length(MM1)] do
         v := MM1[i];
         v := List([1..Length(v) / r], j->v{[(j-1)*r + 1 .. j*r ]});
         v := List(v, x->Eval(Poly(kx, Reversed(x)), a));
         Add(MM2, v);
      od;
      IsMatCheck(MM2);
      return MM2;
   end;
   
   AlffIsGlobalAssert(F);
   M := AlffDiffCartierMatrix(F, 1);
   M := AlffMat1pSolve(M);
   return Length(M);
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffHasseWittInvariant
#
#SHORTDOC  Compute the Hasse-Witt invariant of a global function field
#
#LONGDOC   Let $F/k$ be a global function field of characteristic $p$.
#          Let $F \bar{k} / \bar{k}$ be the constant field extension
#          by the algebraic closure $\bar{k}$ of $k$ within an algebraic 
#          closure $\bar{F}$ of $F$.
#          Consider the subgroup $Cl^0(F \bar{k} / \bar{k})[p]$ of 
#          $p$-torsion elements of the group of divisor classes of
#          degree zero. This function returns its dimension as an
#          $\FF_p$-vector space. Possible values range from $0$ to $g$, 
#          where $g$ is the genus of $F/k$.
#
#CALLING   s := AlffHasseWittInvariant(F);
#
#ARGUS     s @ integer
#          F @ global function field
#
#CSRC      
#
#XREF      AlffClassGroupPRank
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2, 1));
#          F := Alff(T*y^3 + y + T^3);
#          AlffHasseWittInvariant(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffHasseWittInvariant := function(F)
#
#  Return the Hasse-Witt invariant of F
#
   local g;
   AlffIsGlobalAssert(F);
   g := AlffGenus(F);
   return g - Length(MatKernel(AlffDiffCartierMatrix(F, g)));
end;




########################################################################
########################################################################
#
#  higher differentiations (a la Hasse)
#
#  Possible improvments of the implementation: use iterative
#  computation (in terms of D, D^p, D^(p^2) ... )
#  Or: Using function field over itself and usual series expansions.
#      (interpretation as Taylor coefficients).
#  OR: lift to char 0 do the diff divide by k! and reduce.
#


Multinomial := function(S)
   return Product([2..Sum(S)])/Product(S, s->Product([2..s]));
end;


AlffDifferentiationsLocals := rec();


AlffDifferentiationsLocals.PartialDiffXY := function(X, Y, mu, nu, A)
#
#  A is a function in X and Y (exactly: A is a list resp. powers of Y)
#  Computes (1/mu!)*(1/nu!)*d^(mu+nu)(A)/dx^mu*dy^nu.
#   
   local m, n, diffA, coef, diffcoef;
   diffA := 0;
   for n in [nu..Length(A)-1] do
      coef := Reversed(PolyToList(A[n+1]));
      diffcoef := 0;
      for m in [mu..Length(coef)-1] do
         diffcoef := diffcoef + Binomial(m, mu) * coef[m+1] * X^(m-mu);
      od;
      diffA := diffA + Binomial(n, nu) * diffcoef * Y^(n-nu);
   od;
   return diffA;
end;


AlffDifferentiationsLocals.PartitionsI := function(kappa)
#
#  Find all tuples (s_1, ... , s_kappa) of non-negative integers 
#  with s_1 + 2*s_2 + ... + kappa*s_kappa <= kappa (exclude the "last").
#
   local A, b, sim, s, S;
   A := [ [1..kappa] ];
   Append(A, MatToRowList(-MatId(Z, kappa)));
   A := Mat(R, A);
   b := 0*[1..kappa+1];
   b[1] := kappa + 0.5;
   b := MatTrans(R, [b]);
   sim := SimplexInit(A, b);
   S := [];
   SimplexNext(sim);
   s := SimplexElt(sim);
   while SimplexNext(sim) do
      Add(S, s);
      s := SimplexElt(sim);
   od;
   if not s[kappa] = 1 then
      return Error("indices by simplex method wrong");
   fi;
   return S;   
end;



AlffDifferentiationsLocals.PartitionsII := function(kappa, L)
#
#  Compute tuples (r_i | i in L) of non negative
#  integers such that Sum(i*r_i | i in L) = kappa.
#
   local len, A, b, sim, S, s;
   len := Length(L);
   A := [ L, -L ];
   Append(A, MatToRowList(-MatId(Z, len)));
   A := Mat(R, A);
   b := 0*[1..len+2];
   b[1] := kappa + 0.5;
   b[2] := -kappa + 0.5;
   b := MatTrans(R, [b]);
   sim := SimplexInit(A, b);
   S := [];
   while SimplexNext(sim) do
      Add(S, SimplexElt(sim));
   od;
   return S;   
end;



AlffDifferentiationsLocals.PartitionsIII := function(kappa, lR, lS)
#
#  Compute tuples (r_i | i in lR)(s_i | i in lS) of non negative
#  integers such that Sum(i*r_i | i in lR)+Sum(i*s_i | i in lS) = kappa.
#
   local len, A, b, sim, S, s, lr, lrs;
   lr := Length(lR);
   lrs := lr + Length(lS);
   A := Concatenation(lR, lS);
   len := Length(A);
   A := [ A, -A ];
   Append(A, MatToRowList(-MatId(Z, len)));
   A := Mat(R, A);
   b := 0*[1..len+2];
   b[1] := kappa + 0.5;
   b[2] := -kappa + 0.5;
   b := MatTrans(R, [b]);
   sim := SimplexInit(A, b);
   S := [];
   while SimplexNext(sim) do
      s := SimplexElt(sim);
      Add(S, [ s{[1..lr]}, s{[lr+1..lrs]} ]);
   od;
   return S;   
end;



AlffDifferentiationsLocals.ComputeDXkY := function(oe, X, Y, kappa, f, diffs)
#
#  Computes D_x^k(y) for k <= kappa by relation f(x,y)=0.
#  Results are stored in diffs.
#
   local maxkappa, p, sum, s, S, k, rho, ss, ssum, val, i;
   
   maxkappa := Length(diffs);
   f := Reversed(PolyToList(f));
  
   p := Characteristic(AlffOrderAlff(oe));
   
   for k in [maxkappa+1..kappa] do
   
      S := AlffDifferentiationsLocals.PartitionsI(k);
      sum := 0;
   
      for s in S do
         ssum := Sum(s);
         rho := k - Sum([1..k], i->i*s[i]);
         ss := Multinomial(s);
         if p > 0 then 
            ss := ss mod p;
         fi;
         if not ss = 0 then
            val := AlffDifferentiationsLocals.PartialDiffXY(X, Y, 
                           rho, ssum, f); 
            for i in [1..k-1] do
               if not s[i] = 0 then
                  val := val * diffs[i]^s[i];
               fi;
            od;
            val := val * ss;
            sum := sum + val;
         fi;
      od;
   
      diffs[k] := -sum / AlffDifferentiationsLocals.PartialDiffXY(X, Y, 
                          0, 1, f);
   od;
   
end;



AlffDifferentiationsLocals.ComputeDkf := function(oe, kappa, f, 
                                                 DkX, DkXweights, 
                                                 DkY, DkYweights)
#
# Computes Dkf for arbitrary function f(x,y) by DkX and DkY
# 
   local X, Y, Dkf, p, xlen, ylen, k, s, S, sum, sx, sy, sxsum,
         sysum, sxm, sym, val, i, sxbound, sybound;
   
   X := AlffVarT(AlffOrderAlff(oe));
   Y := AlffEltMove(AlffEltGenY(AlffOrderAlff(oe)), oe);
   
   Dkf := [];
   p := Characteristic(AlffOrderAlff(oe));
   xlen := Length(DkXweights);
   ylen := Length(DkYweights);
   sxbound := Maximum(List(f, x->PolyDeg(x)))+1;
   sybound := Length(f);
   
   for k in [1..kappa] do
   
      S := AlffDifferentiationsLocals.PartitionsIII(k, DkXweights, DkYweights);
      sum := 0;
   
      for s in S do
      
         sx := s[1];
         sy := s[2];
         sxsum := Sum(sx);
         sysum := Sum(sy);
         sxm := Multinomial(sx);
         sym := Multinomial(sy);
         if p > 0 then
            sxm := sxm mod p;
            sym := sym mod p;
         fi;
         
         if not sxm * sym = 0 and 
            sxsum < sxbound and 
            sysum < sybound then

            val := AlffDifferentiationsLocals.PartialDiffXY(X, Y, sxsum, 
                           sysum, f);
            
            for i in [1..xlen] do
               if not sx[i] = 0 then
                  val := val * DkX[i]^sx[i];
               fi;
            od;
            
            for i in [1..ylen] do
               if not sy[i] = 0 then
                  val := val * DkY[i]^sy[i];
               fi;
            od;
         
            val := val * sxm * sym;
            sum := sum + val;
         
         fi;
      od;
      
      Dkf[k] := sum;
   od;
      
   return Dkf;
end;



AlffDifferentiationsLocals.ComputeDk1g := function(oe, kappa, g, Dkg)
#
# Computes Dk(1/g) by Dkg using recursive product formula
#
   local ginv, Dk1g, k, sum, i;                                      
   
   ginv := 1/AlffElt(oe, Concatenation(g, 0*[1..AlffOrderDeg(oe)-Length(g)]));
   Dk1g := [];
   
   for k in [1..kappa] do
      sum := Dkg[k] * ginv;
      for i in [1..k-1] do
         sum := sum + Dkg[k-i] * Dk1g[i];
      od;
      Dk1g[k] := -sum * ginv; 
   od;
   
   return Dk1g;
end;



AlffDifferentiationsLocals.ComputeDkfg := function(oe, kappa, fg,
                                                 DkX, DkXweights,
                                                 DkY, DkYweights)
# 
# Computes Dk(f/g) by using the product formula and Dkf and Dk(1/g).
#
   local f, g, Dkf, Dkg, Dk1g, sum, i;                                      

   if kappa = 0 then 
      return fg;
   fi;
   
   f := AlffEltNum(fg);
   g := AlffEltDen(fg);
   
   Dkf := AlffDifferentiationsLocals.ComputeDkf(oe, 
                  kappa, AlffEltToList(f)[1], 
                  DkX, DkXweights, DkY, DkYweights);
   Dkg := AlffDifferentiationsLocals.ComputeDkf(oe, kappa, [ g ], 
                   DkX, DkXweights, DkY, DkYweights);
   Dk1g := AlffDifferentiationsLocals.ComputeDk1g(oe, kappa, [ g ], Dkg);
   
   sum := Dkf[kappa] * (1/g) + f * Dk1g[kappa];
   for i in [1..kappa-1] do
      sum := sum + Dkf[kappa-i] * Dk1g[i];
   od;
   
   return sum;
end;



AlffDifferentiationsLocals.ComputeDpkY := function(oe, kappa, DXkY, DpkX, DpkY)
#
# Computes DpkY by DXkY and DpkX (chain rule).
#
   local maxkappa, k, s, S, sum, sm, val, p;
   p := Characteristic(AlffOrderAlff(oe));
   maxkappa := Length(DpkY);
   for k in [maxkappa+1..kappa] do
      S := AlffDifferentiationsLocals.PartitionsII(k, [1..k]);
      sum := 0;
      for s in S do
         sm := Multinomial(s);
         if p > 0 then 
            sm := sm mod p;
         fi;
         if not sm = 0 then
            val := sm * DXkY[Sum(s)] * Product([1..k], i->DpkX[i]^s[i]);
            sum := sum + val;
         fi;
      od;
      DpkY[k] := sum;
   od;
end;



AlffDifferentiationsLocals.AssureDXkY := function(F, kappa)
#
#  Assures the computation of the DXkY with resp. to f(x,y) = 0.
#  Results are stored in F.DXkY.
#
   local oe, f, X, Y;
   if not IsBound(F.DXkY) then
      F.DXkY := [];
   fi;
   if Length(F.DXkY) >= kappa then
      return;
   fi;
   oe := AlffOrderEqFinite(F);
   f := AlffOrderPoly(oe);
   X := AlffVarT(F);
   Y := AlffEltMove(AlffEltGenY(F), oe);
   AlffDifferentiationsLocals.ComputeDXkY(oe, X, Y, kappa, f, F.DXkY);
end;



AlffDifferentiationsLocals.DiffIterative := function(p, kappa, a)
   local res, dres, dp, i;
   dp := AlffDiff(p);
   res := a;
   for i in [1..kappa] do
      dres := AlffDiff(res);
      res := dres/dp;
   od;
   return res/Product([1..kappa]);
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDifferentiation
#
#SHORTDOC  Compute higher differentiations of function field elements.
#
#CALLING   b := AlffDifferentiation(p, m, a);
#
#ARGUS     b @ alff element @ $m$-times differentiated element
#          m @ integer @ 
#          p @ alff place or element @ 
#          a @ alff element @ element to differentiate
#
#LONGDOC   Let $F/k$ be an algebraic function field. 
#          For a separating element $p$ and an integer $m$ this
#          function returns the $m$-th differentiation of $a \in F$.
#          If $p$ is a place, a local parameter of the place as returned
#          by {\tt AlffPlacePrimeElt()} is taken. Remark: The 
#          implementation
#          is currently not efficient if $m$ is larger than the positive 
#          characteristic.
#
#CSRC      
#
#XREF      AlffDiff, AlffWronskian, AlffRamDivisor
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2 + T^3 + 1);
#          T := AlffEltGenT(F);
#          AlffDifferentiation(T, 1, T);
#          AlffDifferentiation(T, 2, T);
#          AlffDifferentiation(T, 1, T^2);
#          AlffDifferentiation(T, 2, T^2);
#          AlffDifferentiation(T, 5, T^5);
#          AlffDifferentiation(T, 5, T^7);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


AlffDifferentiation := function(p, kappa, a)
#
#  Compute the Differentiation with resp. to prime elt 
#  p of order kappa for a. 
#
   local o, oe, F, kx, DkX, DkY, DkXweights, DkYweights, res, 
         mipo, DpkX, DpkY, x, b, doiter;
   o := AlffEltOrder(a);
   F := AlffOrderAlff(o);
   doiter := Characteristic(F) = 0 or kappa < Characteristic(F);
   oe := AlffOrderEqFinite(F);
   a := AlffEltMove(a, oe);
   kx := PolyAlg(AlffVarT(F));
   if not doiter then
      AlffDifferentiationsLocals.AssureDXkY(F, kappa);
   fi;
   if p = Poly(kx, [1,0]) then
      if doiter then
         res := AlffDifferentiationsLocals.DiffIterative(AlffElt(oe, p), 
                        kappa, a);
      else
         DkX := [ 1 ];
         DkXweights := [ 1 ];
         DkY := F.DXkY;
         DkYweights := [1..kappa];
         res := AlffDifferentiationsLocals.ComputeDkfg(oe, kappa, a,
                        DkX, DkXweights, DkY, DkYweights);
      fi;   
   elif IsAlffElt(p) then
      if doiter then
         res := AlffDifferentiationsLocals.DiffIterative(p, kappa, a);
      else
         b := AlffEltMove(p, oe);
         x := Poly(kx, [1,0]);
         mipo := PolySwapVars(AlffEltMinPoly(b));
         if not AlffPolyIrrIsSep(mipo) then
            return Error("non-separating element");
         fi;
         DpkX := [];
         DpkY := [];
         AlffDifferentiationsLocals.ComputeDXkY(oe, b, x, kappa, mipo, DpkX);
         AlffDifferentiationsLocals.ComputeDpkY(oe, kappa, F.DXkY, DpkX, DpkY);
         res := AlffDifferentiationsLocals.ComputeDkfg(oe, kappa, a,
                        DpkX, [1..kappa], DpkY, [1..kappa]);
      fi;   
   elif IsAlffPlace(p) then
      # almost the same like above
      if doiter then
         return AlffDifferentiationsLocals.DiffIterative(AlffPlacePrimeElt(p),
                        kappa, a);
      else
         b := AlffEltMove(AlffPlacePrimeElt(p), oe);
         x := Poly(kx, [1,0]);
         mipo := PolySwapVars(AlffEltMinPoly(b));
         if not IsBound(p.DpkX) then
            p.DpkX := [];
            p.DpkY := [];
         fi;
         AlffDifferentiationsLocals.ComputeDXkY(oe, b, x, kappa, mipo, p.DpkX);
         AlffDifferentiationsLocals.ComputeDpkY(oe, kappa, F.DXkY, p.DpkX, 
                 p.DpkY);
         res := AlffDifferentiationsLocals.ComputeDkfg(oe, kappa, a,
                        p.DpkX, [1..kappa], p.DpkY, [1..kappa]);
      fi;   
   else
      return Error("usage: AlffDifferentiation( <alff element> | <alff place>, <integer>, <alff element> ); ");
   fi;
   
   return AlffEltMove(res, o);
end;

