
############################################################################
#
# Some functions for divisors
#


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDivisorLBasis
#
#SHORTDOC  Computes a basis of a Riemann-Roch space
#
#LONGDOC   Given a divisor $D$ of an algebraic function field $F$ 
#          this function returns a basis for
#          the $k$-vector space ${\cal L}(D)$, where $k$ denotes the constant
#          field of definition of $F$. The basis elements are represented
#          in the finite maximal order.
#
#CALLING   B := AlffDivisorLBasis(D);
#
#ARGUS     B @ list @ of basis elements
#          D @ alff divisor
#
#CSRC      
#
#XREF      AlffDivisorLBasisShort, AlffDivisorLDim, AlffDivisorDeg, 
#          AlffDivisorLIndex, AlffGenus, Alff
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2+T^3+1);;
#          infty := AlffPlaceSplit(F, 1/T)[1];
#          l := AlffPlaceSplit(F, T+3);          
#          D := 2*infty + 3*l[1] - l[2];
#          AlffDivisorLBasis(D);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  980812  FH  installed.
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDivisorLBasis := function(D)
   local B, Bs, b, T;
   if not IsAlffDivisor(D) then
      return Error("usage: AlffDivisorLBasis( <alff divisor> )");
   fi;
   T := AlffVarT(AlffDivisorAlff(D));
   Bs := AlffDivisorLBasisShort(D);
   B := [];
   for b in Bs do
      Append(B, List([0..b[2]], j->T^j * b[1]));
   od;
   return B;
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDivisorSupp
#
#SHORTDOC  Returns the support of a divisor as a list of places.
#
#CALLING   L := AlffDivisorSupp(D);
#
#ARGUS     L @ list @ of places
#          D @ alff divisor
#
#XREF      AlffDivisorPlaces
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2+T^3+1);;
#          infty := AlffPlaceSplit(F, 1/T)[1];
#          l := AlffPlaceSplit(F, T+3);          
#          D := 2*infty + 3*l[1] - l[2];
#          AlffDivisorSupp(D);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDivisorSupp := function(D)
   return List(AlffDivisorPlaces(D), x->x[1]);
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDivisorNorm
#
#SHORTDOC  Computes the norm of a divisor.
#
#LONGDOC   Given a divisor $D$ of an algebraic function field $F$ 
#          this function returns the norm of $D$ down to the rational 
#          subfield $k(x)$ as a rational function.
#
#CALLING   N := AlffDivisorNorm(D);
#
#ARGUS     N @ qf element
#          D @ alff divisor
#
#XREF      AlffEltGenT, AlffIdealNorm.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2+T^3+1);;
#          infty := AlffPlaceSplit(F, 1/T)[1];
#          l := AlffPlaceSplit(F, T+3);          
#          D := 2*infty + 3*l[1] - l[2];
#          AlffDivisorNorm(D);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDivisorNorm := function(D)
   local L;
   L := AlffDivisorIdeals(D);
   return 1/(AlffIdealNorm(L[1])*AlffIdealNorm(L[2]));
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDivisorDegOne
#
#SHORTDOC  Computes a divisor of degree one for a global function field.
#
#LONGDOC   Given a global function field this function determines a
#          divisor of degree one.
#
#CALLING   D := AlffDivisorDegOne(F);
#
#ARGUS     D @ alff divisor @ of degree one
#          F @ global function field
#
#CSRC      
#
#XREF      AlffPlaces, AlffPlacesDegOne.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2+T^3+1);;
#          AlffDivisorDegOne(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDivisorDegOne := function(F)
#
#  Returns a divisor of degree one.
#
   local l1, l2, m;
   AlffIsGlobalAssert(F);
   if IsBound(F.Ddegone) then
      return F.Ddegone;
   fi;
   if AlffDimExactConstField(F) > 1 then
      return Error("dim exact const field > 1, no divisor of deg one");
   fi;
   l1 := AlffPlaces(F, 1);
   if Length(l1) > 0 then
      F.Ddegone := 1*l1[ Length(l1) ];
      return F.Ddegone;
   fi;
   m := 2;
   l2 := AlffPlaces(F, m);
   repeat
      l1 := l2;
      m := m + 1;
      l2 := AlffPlaces(F, m);
   until Length(l1) > 0 and Length(l2) > 0;
   F.Ddegone := l2[1] - l1[1];
   return F.Ddegone;
end;



#######################################################################
#  Linear enumeration for gffs
#


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffLinearSeriesEnumEnv
#
#SHORTDOC  Prepare for enumeration of a linear series over a finite field.
#
#LONGDOC   Let $F/k$ be a global function field and $D$ a divisor.
#          Let $v_1, \dots, v_l$ be $k$-linearly independent 
#          elements of $F$. This function
#          returns a record which is used by {\tt AlffLinearSeriesEnumNext()}
#          and {\tt AlffLinearSeriesEnumElt()} to enumerate successively
#          the divisors $D + \bigl( \sum_{i=1}^l \lambda_i v_i \bigr)$ 
#          (each once).
#
#CALLING   env := AlffLinearSeriesEnumEnv(D, B);
#
#ARGUS     env @ record @ of enumeration data
#          D @ alff divisor @ 
#          B @ list @ of $v_1, \dots, v_l$
#
#CSRC      
#
#XREF      AlffLinearSeriesEnumNext, AlffLinearSeriesEnumElt.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2+T^3+1);;
#          D := 1*AlffPlaceRandom(F, 3);
#          B := AlffDivisorLBasis(D);
#          env := AlffLinearSeriesEnumEnv(D, B);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffLinearSeriesEnumEnv := function(D, B)
#
#  For enumeration of linear series over finite fields:
#  D divisor ((, differential etc.)) 
#  B a k-basis of alff elements, k const field of F.
#  Returns env with which all D * k-span(B) multmod k^* can be
#  generated.
#
   local F, k, env;
   F := AlffDivisorAlff(D);
   AlffIsGlobalAssert(F);
   k := AlffConstField(F);
   env := rec();
   env.ffelts := FFElts(k);
   env.D := D;
   env.basis := ShallowCopy(B);
   env.dim := Length(B);
   return env;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffLinearSeriesEnumNext
#
#SHORTDOC  Whether there is a next divisor in the linear series enumeration.
#
#LONGDOC   Let $F/k$ be a global function field and $D$ a divisor.
#          Let $v_1, \dots, v_l$ be $k$-linearly independent 
#          elements of $F$. This function
#          returns {\tt true} or {\tt false} with respect to whether there
#          is a not yet considered divisor in a linear series enumeration
#          of the form $D + \bigl( \sum_{i=1}^l \lambda_i v_i \bigr)$ 
#          and increases 
#          an internal counter.
#
#CALLING   b := AlffLinearSeriesEnumNext(env);
#
#ARGUS     b @ bool @ whether there is a next divisor
#          env @ record @ of enumeration data
#
#CSRC      
#
#XREF      AlffLinearSeriesEnumEnv, AlffLinearSeriesEnumElt.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2+T^3+1);;
#          D := 1*AlffPlaceRandom(F, 3);
#          B := AlffDivisorLBasis(D);
#          env := AlffLinearSeriesEnumEnv(D, B);
#          AlffLinearSeriesEnumNext(env);
#          AlffLinearSeriesEnumNext(env);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffLinearSeriesEnumNext := function(env)
#
#  Whether there are not yet computed elements.
#
   if not IsBound(env.counter) then
      env.counter := AlffMultiCounterInit(env.dim-1);
      return true;
   fi;   
   return AlffMultiCounterInc(env.counter, Length(env.ffelts));
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffLinearSeriesEnumElt
#
#SHORTDOC  Return the current divisor in the series enumeration.
#
#LONGDOC   Let $F/k$ be a global function field and $D$ a divisor.
#          Let $v_1, \dots, v_l$ be $k$-linearly independent 
#          elements of $F$. This function
#          returns the (according to an internal counter) current divisor
#          $D + \bigl( \sum_{i=1}^l \lambda_i v_i \bigr)$ in the linear series
#          enumeration.
#
#CALLING   E := AlffLinearSeriesEnumElt(env);
#
#ARGUS     E @ alff divisor @ current divisor 
#          env @ record @ of enumeration data
#
#CSRC      
#
#XREF      AlffLinearSeriesEnumNext, AlffLinearSeriesEnumEnv.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^2+T^3+1);;
#          D := 1*AlffPlaceRandom(F, 3);
#          B := AlffDivisorLBasis(D);
#          env := AlffLinearSeriesEnumEnv(D, B);
#          AlffLinearSeriesEnumNext(env);
#          AlffLinearSeriesEnumElt(env);
#          AlffLinearSeriesEnumNext(env);
#          AlffLinearSeriesEnumElt(env);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffLinearSeriesEnumElt := function(env)
#
#  Returns a not yet computed element (after AlffLinearEnumNext()).
#
   local dim, basis, elt;
   if not IsBound(env.counter) then
      env.counter := AlffMultiCounterInit(env.dim-1);
   fi;   
   dim := env.dim;
   basis := env.basis;
   elt := basis[dim] + Sum( [1..dim-1], 
                  i->env.ffelts[env.counter[i]] * basis[i]);
   return env.D + AlffDivisor(elt, elt);
end;


#######################################################################
# Divisor Reduction 
#



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDivisorReduction
#
#SHORTDOC  Computes a reduced divisor.
#
#LONGDOC   Let $A, D$ be divisors of an algebraic function field $F$ 
#          of genus $g$ and
#          $\deg(A) \geq 1$. Let $r \in \Z$ be maximal such that
#          $D = \widetilde{D} + rA - (a)$ with a positive divisor
#          $\widetilde{D}$ of $F$ and $a \in F^\times$. 
#          Then $\deg(\widetilde{D}) < g + \deg(A)$ and
#          $\widetilde{D}$
#          is maximally reduced along $A$. This function returns on
#          input of $D$ and $A$ 
#          a list containing $\widetilde{D}$, $r$, $A$ and a list
#          $((a_{i,j})_j, e_i)_i$, such that $a = \prod_{i, j} a_{i, j}^{e_i}$.
#          If $A$ is omitted, $A := (T)_\infty$ is taken.
#          $\widetilde{D}$ is returned in ideal representation. The algorithm
#          is described in \cite{He2}.
#
#CALLING   L := AlffDivisorReduction(D);\\
#          L := AlffDivisorReduction(D, A);
#
#ARGUS     L @ list @ as above
#          D @ alff divisor @ to reduce
#          D @ alff divisor
#CSRC      
#
#XREF      AlffDivisorLargeLDim
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7));
#          AlffOrders(y^3 + T*y + T^7 + T^2 + 1);
#          p1 := AlffPlaceSplit(F, T)[1];
#          p2 := AlffPlaceSplit(F, T+1)[1];
#          D := 51*p1 + 37*p2;
#          A := 1*AlffPlaceSplit(F, T)[2];
#          L := AlffDivisorReduction(D, A);
#          D - ( L[1] + L[2]*A - 
#              AlffDivisor(Product(L[4], x->Product(x[1])^x[2])) );
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


AlffDivisorReduction := function(arg)
#
#  Maximal divisor reduction along divisor A 
#  using maximal or degree elementary reduction in each step.
#  Returns [ Dtilde, r, A, a ] where Dtilde is a positive divisor 
#  of degree < g + deg(A), r is integer, a is list of 
#  pairs [ [ alffoe, ...], int ], such that D = Dtilde + rA - PowerProduct(a).
#  Support reduction in block of length n (n = 0 means one block) (third parm)
#
   local D, F, n, p, B, S, s, Dprime, l, Dtilde, m, i, j, r, a, b, 
         T, E, E1, A, one, ElRedA, ElRedInfty, ElementaryReduction, 
         AlffDivisorHeight;
   
   AlffDivisorHeight := function(D)
      return Maximum(AlffDivisorDeg(AlffDivisorNum(D)), 
                     AlffDivisorDeg(AlffDivisorDen(D)));
   end;

   ElRedA := function(E, max)
      # Condition: E = Etilde + expo*A - (elt)
      local b, expo, uexpo, oexpo, elt, Etilde, EE, uEE;
      expo := Ceil((AlffDivisorDeg(E) - AlffGenus(F))/AlffDivisorDeg(A));
      if max then  # find biggest possible expo
         EE := E - (expo+1)*A;
         if AlffDivisorLDim(EE) = 0 then
            uEE := EE + A;
            b := AlffDivisorLBasis(uEE);
            elt := b[1];
         else
            # convert E into ideal rep exclusively
            E := AlffDivisorIdeals(-E);
            E := AlffDivisor(E[1], E[2]);
            uexpo := expo+1;
            uEE := EE;
            oexpo := Ceil((AlffDivisorDeg(E) + 1) / AlffDivisorDeg(A));
            while oexpo - uexpo > 1 do
               expo := Floor((uexpo + oexpo)/2);
               EE := E - expo*A;
               if AlffDivisorLDim(EE) = 0 then
                  oexpo := expo;
               else
                  uexpo := expo;
                  uEE := EE;
               fi;
            od;
            expo := uexpo;
            b := AlffDivisorLBasis(uEE);
            elt := b[1];
         fi;
      else
         uEE := E - expo*A;
         b := AlffDivisorLBasis(uEE);
         elt := b[1];
      fi;
      Etilde := uEE + AlffDivisor(elt, elt);
      return [ Etilde, expo, elt ];
   end;
   
   ElRedInfty := function(E, dummy)
      # Condition: E = Etilde + expo*(x)_\infty - (elt)
      local b, expo, elt, Etilde;
      b := AlffDivisorLBasisShort(E, "raw");
      expo := Maximum(List(b, a->a[2]));
      elt := First(b, a->a[2]=expo)[1];
      Etilde := E + AlffDivisor(elt, T^expo*elt);
      return [ Etilde, expo, elt ];   
   end;
   
   D := arg[1];
   if Length(arg) = 2 then
      if IsInt(arg[2]) then
         n := arg[2];
      else
         A := arg[2];
      fi;
   elif Length(arg) = 3 then
      A := arg[2];
      n := arg[3];
   fi;
      
   F := AlffDivisorAlff(D);
   T := AlffEltGenT(F);
   
   # take reduction
   if IsBound(A) then
      ElementaryReduction := ElRedA;
   else
      ElementaryReduction := ElRedInfty;
      one := 0*AlffEltGenT(F) + 1;
      A := AlffDivisor(one, one / T);
   fi;
   
   # compute "padic representation"
   p := 2;
   B := [];
   m := -1;
   while not _IsZero(D) do
      m := m + 1;
      Add(B, D mod p);
      D := D / p;
   od;
   
   # some simple heuristic
   if not IsBound(n) then
      if Length(B) > 0 and Maximum(List(B, AlffDivisorHeight)) > 100 then
         n := 50;
      else
         n := 0;
      fi;
   fi;
   
   if n > 0 then
    # do support reductions
      Dprime := [];
      l := [];
      b := [];
      for i in [0..m] do
         S := List(ListSplit(AlffDivisorPlaces(B[i+1]), n), 
                   pe->Sum(pe, a->a[2] * a[1]));
         Dprime[i+1] := 0*D;
         l[i+1] := 0;
         b[i+1] := [];
         for s in S do
            E1 := Dprime[i+1] + s;
            E := ElementaryReduction(E1);
            Dprime[i+1] := E[1];
            l[i+1] := l[i+1] + E[2];
            Add(b[i+1], E[3]);
         od;
         b[i+1] := Filtered(b[i+1], a->not _IsOne(a));
      od;
   else
      Dprime := B;
      l := 0*[0..m];
      b := List([0..m], i->[]);
   fi;
   
   # do exponent reductions
   Dtilde := [];
   Dtilde[m+2] := 0*D;
   r := [];
   a := [];
   for j in [-1 .. m-2] do
      E := ElementaryReduction(p * Dtilde[m-j+1] + Dprime[m-j], false);
      Dtilde[m-j] := E[1];
      r[m-j] := E[2];
      a[m-j] := E[3];
   od;
   if m-1 >= -1 then
      j := m-1;
      E := ElementaryReduction(p * Dtilde[m-j+1] + Dprime[m-j], true);
      Dtilde[m-j] := E[1];
      r[m-j] := E[2];
      a[m-j] := E[3];
   fi;
   
   # conclude
   r := Sum([0..m], i->p^i * (r[i+1] + l[i+1]));
   for i in [0..m] do
      if not _IsOne(a[i+1]) then
         Add(b[i+1], a[i+1]);
      fi;
   od;
   a := Filtered(List([0..m], i->[ b[i+1], p^i ]), b->Length(b[1]) > 0);   
   
   return [ Dtilde[1], r, A, a ];
end;



#######################################################################
# Riemann Roch spaces II (for large divisors)
#

#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDivisorLargeLDim
#
#SHORTDOC  Computes the dimension of a large divisor.
#
#LONGDOC   Given a divisor $D$ of an algebraic function field $F$ 
#          which exponents are very large this function returns the
#          dimension of ${\cal L}(D)$ much more faster than 
#          {\tt AlffDivisorLDim()}.
#
#CALLING   d := AlffDivisorLargeLDim(D);
#
#ARGUS     d @ integer @ the dimension
#          D @ alff divisor
#
#CSRC      
#
#XREF      AlffDivisorLargeLBasisShort, 
#          AlffDivisorLBasisShort, AlffDivisorReduction, 
#          AlffDivisorLDim, AlffDivisorDeg, 
#          AlffDivisorLIndex, AlffGenus, Alff
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7));
#          AlffOrders(y^3 + T*y + T^7 + T + 1);
#          p1 := AlffPlaceSplit(F, T)[1];
#          p2 := AlffPlaceSplit(F, T)[2];
#          p3 := AlffPlaceSplit(F, 1/T)[1];
#          D := -11107286187918025*p1 -271773078445978235700*p2 +
#               271784185732166153725*p3;
#          AlffDivisorDeg(D);
#          AlffDivisorLargeLDim(D);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDivisorLargeLDim := function(D)
   local F, invs, L, Dtilde, r;
   F := AlffDivisorAlff(D);
   if AlffDivisorDeg(D) > 2*AlffGenus(F) - 2 then
      return AlffDivisorDeg(D) - AlffGenus(F) + 1;
   fi;
   L := AlffDivisorReduction(D);
   Dtilde := L[1];
   r := L[2];
   invs := List(AlffDivisorLBasisShort(Dtilde, "raw"), a->a[2]);
   return Sum(Filtered(invs + r, x->x >= 0) + 1);
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffDivisorLargeLBasisShort
#
#SHORTDOC  Computes a basis of a large divisor.
#
#LONGDOC   Given a divisor $D$ of an algebraic function field $F$ 
#          which exponents are very large this function returns 
#          a basis of ${\cal L}(D)$ similar to
#          {\tt AlffDivisorLBasisShort()} with option {\tt "raw"}.
#          If the option {\tt "raw"} is given to 
#          {\tt AlffDivisorLargeLBasisShort()} the basis elements
#          are represented as power product as in 
#          {\tt AlffDivisorReduction()}.
#
#CALLING   B := AlffDivisorLargeLBasis(D, "raw");
#
#ARGUS     B @ list @ the short basis
#          D @ alff divisor
#
#CSRC      
#
#XREF      AlffDivisorLargeLDim, 
#          AlffDivisorLBasisShort, AlffDivisorReduction, 
#          AlffDivisorLDim, AlffDivisorDeg, 
#          AlffDivisorLIndex, AlffGenus, Alff
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(7));
#          AlffOrders(y^3 + T*y + T^7 + T + 1);
#          p1 := AlffPlaceSplit(F, T)[1];
#          p2 := AlffPlaceSplit(F, T)[2];
#          p3 := AlffPlaceSplit(F, 1/T)[1];
#          D := -p1 - p2 + 2*p3;
#          AlffDivisorDeg(D);
#          AlffDivisorLargeLBasisShort(D, "raw");
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffDivisorLargeLBasisShort := function(arg)
   local F, D, L, Dtilde, r, a, B;
   D := arg[1];
   F := AlffDivisorAlff(D);
   L := AlffDivisorReduction(D);
   Dtilde := L[1];
   r := L[2];
   a := L[4];
   B := List(AlffDivisorLBasisShort(Dtilde, "raw"), x->[ x[1], x[2] + r ]);
   B := List(B, x->[ Concatenation( [ [ [x[1]], 1 ] ], a ), x[2] ]); 
   if "raw" in arg then
      return B;
   else
      B := List(B, x->[ Product(x[1], y->Product(y[1])^y[2]), x[2] ]);
      return B;
   fi;
end;

