
##################################################################
#
# Functions for the L polynomial of a global function field
#
#


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffLPoly
#
#SHORTDOC  Computes the $L$-polynomial of a global function field.
#
#LONGDOC   This function computes the $L$-polynomial of a global
#          function field $F/k$. The algorithm used is exponential in the
#          genus of $F/k$. The constant field of definition of $F/k$
#          has to be exact.
#
#CALLING   L := AlffLPoly(F);
#
#ARGUS     L @ polynomial
#          F @ global function field
#
#CSRC      
#
#XREF      AlffPlacesDegOneNum, AlffPlacesDegOne 
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^4 + 1);
#          AlffLPoly(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffLPoly := function(arg)
#
#  Compute the L-polynomial of a global function field.
#  With the second parameter gg only the part of the L-polynomial
#  which can be deduced from the first N1, ..., Nd is returned.
#
   local g, q, gg, K, S, a, i;

   K := arg[1];
   AlffIsGlobalAssert(K);
   
   g := AlffGenus(K);
   q := AlffConstFieldSize(K);
   
   if Length(arg) = 1 then
      gg := g;
   else
      gg := arg[2];
   fi;
   
   if not AlffDimExactConstField(K) = 1 then
      return Error("constant field must be exact");
   fi;
   
   S := List( [1..gg], i->AlffPlacesDegOneNum(K, i) - (q^i+1) );
   
   a := [];
   a[1] := 1;
   
   for i in [1..gg] do
      a[1+i] := 1/i * Sum( [0..i-1], j->a[1+j]*S[i-j] );
   od;
   
   for i in [0..gg-1] do
      a[2*g-i+1] := q^(g-i) * a[i+1];
   od;
   
   for i in [gg+1..2*g-gg-1] do
      a[1 + i] := 0;
   od;
   
   return Poly(Zx, Reversed(a));
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffLPolyLift
#
#SHORTDOC  Lift the $L$-polynomial of a global function field to
#          constant field extensions.
#
#LONGDOC   This function computes the $L$-polynomial of 
#          a constant field extension of a global
#          function field $F/k$ of degree $r$ from the $L$-polynomial $L$ of
#          $F/k$.
#
#CALLING   Lr := AlffLPolyLift(L, r);
#
#ARGUS     Lr, L @ polynomial
#          r @ integer
#
#CSRC      
#
#XREF      AlffLPoly, AlffPlacesDegOne 
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^4 + 1);
#          L := AlffLPoly(F);
#          AlffLPolyLift(L, 2);
#          AlffInit(FF(5,2));;
#          AlffOrders(y^4 + T^4 + 1);
#          AlffLPoly(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffLPolyLift := function(L, r)
#
#  Lift the L polynomial to constant field extensions
#
   local P, l, prod, fac, x;
   P := Poly(Zx, Reversed(PolyToList(L)));
   l := Factor(P);
   prod := 1;
   for x in l do
      if PolyDeg(x[1]) = 1 then
         fac := Poly(Zx, [1, -(-Eval(x[1], 0))^r]);
      else
         fac := EltCharPoly( OrderBasis(Order(x[1]))[2]^r, Z);
      fi;
      prod := prod * fac ^ x[2];   
   od;
   return Poly(Zx, Reversed(PolyToList(prod)));
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffLPolyRed
#
#
#LONGDOC   Computes the $L$-polynomial of 
#          a global function field of 
#          characteristic $p$ modulo $p$. The constant field of definition
#          of $F/k$ has to be exact and $F/k$ must contain a non
#          special system of places. If the parameter {\tt "nocheck"}
#          is given, the existence of the non special system is not
#          checked (hence the resulting $L$-polynomial may be false).
#         
#CALLING   L := AlffLPolyRed(F);\\
#          L := AlffLPolyRed(F, "nocheck");
#
#ARGUS     L @ polynomial
#          F @ F global function field
#
#CSRC      
#
#XREF      AlffLPolyRed 
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          AlffOrders(y^4 + T^4 + T + 1);
#          AlffLPolyRed(F);
#          AlffLPoly(F) mod 5;
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffLPolyRed := function(arg)
#
#  Return the L polynomial of F reduced mod p by Hasse-Witt-Manin.
#
   local F, r, g, P, x;
   F := arg[1];
   AlffIsGlobalAssert(F);
   if Length(arg) > 1 and not arg[2] = "nocheck" then
      return Error("wrong second parameter");
   fi;
   if not AlffDimExactConstField(F) = 1 then
      return Error("constant field must be exact");
   fi;
   if not "nocheck" in arg and AlffPlacesNonSpecial(F, 1) = false then
      return Error("non special system of places does not exist");
   fi;
   g := AlffGenus(F);
   r := FFSize(AlffConstField(F))[2];
   P := MatCharPoly(AlffDiffCartierMatrix(F, r));
   x := Poly(PolyAlg(P), [1, 0]);
   return x^g * Eval(P, 1/x);
end;


