

#######################################################################
# Initialisation


AlffPlaceLocals := rec();

AlffPlaceSplit := function(F, p)
   local l, a;
   l := _AlffPlaceSplit(F, p);
   if not IsRec(l[1]) then
      for a in l do
         Extend(a);
         a.operations := rec(Print := function(b) Print(b.base); end);
      od;
   fi;
   return l;
end;


AlffDivisorPlaces := function(D)
   local l, a;
   l := _AlffDivisorPlaces(D);
   if Length(l) > 0 and not IsRec(l[1][1]) then
      for a in l do
         Extend(a[1]);
         a[1].operations := rec(Print := function(b) Print(b.base); end);
      od;
   fi;
   return l;
end;


AlffEltValuation := function(place, a)
   if IsAlffPlace(place) then
      return _AlffEltValuation(place, a);
   elif IsPoly(place) or IsQfe(place) then
      return QfeVal(place, a);   
   else
      return Error("usage: AlffEltValuation( <alff place>, <alff element> )");
   fi;
end;


#######################################################################
#


#/*TEX*****I***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TE 
#FXN       
# 
#SHORTDOC  Returns a list containing all monic prime polynomials of 
#          degree $d$ in $k[x]$ for a finite field $k$. 
# 
#CALLING   L := PolyPrimeList(kx, d);
# 
#ARGUS     L @ list
#          kx @ polynomial algebra @ over finite field $k$
#          d @ integer
#    
#XREF      PolyPrimeNum, PolyPrimeRandom.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5));
#          PolyPrimeList(kT, 2);
#         
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTO
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


PolyPrimeList := function(kx, l)
   local q, n, m, k, ka, L, B, x, i, w, len, qs, qq, qqq, pows, f;
   k := PolyAlgCoef(kx);
   if not IsFF(k) then
      return Error("constant field is infinite");
   fi;
   if not IsList(l) then
      l := [ l ];
   fi;
   Sort(l);
   L := [];
   x := Poly(kx, [1,0]);
   q := FFSize(k);
   q := q[1]^q[2];
   for n in l do
      if n = 1 then
         Add(L, x);
         w := FFPrimitiveElt(k);
         Append(L, List([1..q-1], i->x + w^i));
      else
         qq := q^n-1;
         m := List(Factor(n), p->n/p[1]);
         qqq := List(m, x->qq/(q^x-1));
         qs := List( [0..n-1], r->q^r );
         pows := Set([]);
         len := PolyPrimeNum(kx, n);
         B := [];
         i := 1;
         while not Length(B) = len do
            if ForAll(qqq, x->not i mod x = 0) and not i in pows then
               SetUnite(pows, List(i*qs, x->x mod qq));
               Add(B, i);
            fi;
            i := i + 1;
         od;
         f := PolyPrimeRandom(kx, n);
         ka := FF(f);
         w := FFPrimitiveElt(ka);
         B := List(B, r->FFEltMinPoly(w^r, kx));
         Append(L, B);
      fi;
   od;
   return L;
end;


#######################################################################
#


#/*TEX*****I***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TE 
#FXN       AlffPlaceSplitType
# 
#SHORTDOC  Returns a list with ramification indices and relative degrees
#          of all pairwise distinct places lying over a rational
#          function field place.
# 
#CALLING   L := AlffPlaceSplitType(F, p);
# 
#ARGUS     L @ list
#          F @ algebraic function field
#          p @ polynomial or $1/T$
#    
#XREF      AlffPlaceSplit, AlffPlaceRam, AlffPlaceResDeg, 
#          AlffSignature, AlffIdealFactor.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,2));
#          AlffOrders(y^3+T^4+1);
#          AlffPlaceSplitType(F, 1/T);
#          AlffPlaceSplitType(F, T+1);
#         
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTO
#
#  98xxxx FH changed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
  
AlffPlaceSplitType := function(F, p)
   local l;
   l := AlffPlaceSplit(F, p);
   return List(l, a->[ AlffPlaceRam(a), AlffPlaceResDeg(a) ]);
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffPlacesDegOneNum
#
#SHORTDOC  Computes the number of places of degree one of a global
#          function field's constant field extension. 
#
#LONGDOC   Let $N_1(F')$ be the  number of places of degree one of the 
#          constant field extension $F'$ of degree $m$ of a global function
#          field $F$. This function calculates
#          $N_1(F')$ (using a simple algorithm).
#
#CALLING   N := AlffPlacesDegOneNum(F,m);
#
#ARGUS     F @ global function field
#          N, m @ integer
#
#CSRC      alff\_num\_primes\_deg\_one 
#
#XREF      AlffPlacesDegOneNumBound, AlffPlacesDegOne
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2, 3));
#          AlffOrders(y^3+T^3*y+T);
#          AlffPlacesDegOneNum(F,1);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  980406 FH/jm written
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffPlacesDegOneNum := function(F, m)
   local l, s, o;
   AlffIsGlobalAssert(F);
   if m < 1 then
      return Error("wrong degree\n");
   fi;
   if not IsBound(F.N) then
      F.N := [];
   fi;
   if not IsBound(F.N[m]) then
      o := AlffOrderEqFinite(F);
      l := Filtered(AlffPlaceSplitType(F, 1/AlffVarT(F)), a->m mod a[2] = 0);
      s := Sum(l, a->a[2]);
      # factoring not fast, may be use equal degree fact later
      l := Filtered(Factor(AlffOrderDisc(o)), a->m mod PolyDeg(a[1]) = 0);
      l := Concatenation(List(l, a->AlffPlaceSplit(F, a[1])));
      l := Filtered(l, a->m mod AlffPlaceDeg(a) = 0);
      s := s + Sum(l, a->AlffPlaceDeg(a));
      s := s + AlffPlacesDegOneNonSingFiniteNum(F, m);
      F.N[m] := s;
   fi;
   return F.N[m];
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffPlacesNum
#
#SHORTDOC  Computes the number of places of a given degree 
#          of a global function field. 
#
#CALLING   N := AlffPlacesNum(F, m);
#
#ARGUS     F @ global function field
#          N, m @ integers
#
#XREF      AlffPlacesDegOneNum, AlffPlaces
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2, 3));
#          AlffOrders(y^3+T^3*y+T);
#          AlffPlacesNum(F, 2);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  99xxxx FH changed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffPlacesNum := function(F, m)
   AlffIsGlobalAssert(F);
   if m < 1 then
      return 0;
   fi;
   return (1/m)*Sum(IntDivisors(m), 
                  d->IntMoebiusMy(m/d)*AlffPlacesDegOneNum(F, d));
end;


##########################################################################

#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffPlacesDegOne
#
#SHORTDOC  Compute all places of degree one of a global function field. 
#
#LONGDOC   This function returns a list of all places of degree one
#          of a global function field $F$. The constant field 
#          of definition should be the 
#          exact constant field of $F$ since the degree is taken over the
#          former.
#
#CALLING   L := AlffPlacesDegOne(F);
#
#ARGUS     F @ global function field
#          L @ list @ of alff places
#
#CSRC      alff\_num\_primes\_deg\_one 
#
#XREF      AlffPlacesNum
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2, 3));
#          AlffOrders(y^3+T^3*y+T);
#          AlffPlacesDegOne(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  99xxxx FH changed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffPlacesDegOne := function(F)
   local T, w, q, k, i, pl;
   AlffIsGlobalAssert(F);
   T := AlffVarT(F);
   k := AlffConstField(F);
   q := FFSize(k);
   q := q[1]^q[2];
   w := FFPrimitiveElt(k);
   pl := [ 1/T, T ];
   for i in [1..q-1] do
      Add(pl, T + w^i);
   od;
   Apply(pl, p->AlffPlaceSplit(F, p));
   pl := Filtered( Flat(pl), p->AlffPlaceDeg(p) = 1 );
   return pl;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffPlaces
#
#SHORTDOC  Compute places of a global function field. 
#
#LONGDOC   This function returns a list of all places of a degree $d$
#          of a global function field $F$. The constant field 
#          of definition should be the 
#          exact constant field of $F$.
#
#CALLING   L := AlffPlacesDegOne(F);
#
#ARGUS     F @ global function field
#          L @ list @ of alff places
#
#CSRC      alff\_num\_primes\_deg\_one 
#
#XREF      AlffPlacesNum
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2, 3));
#          AlffOrders(y^3+T^3*y+T);
#          AlffPlaces(F, 2);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  99xxxx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


AlffPlaces := function(F, m)
#
#  Return all places of degree m
#
   local l;
   AlffIsGlobalAssert(F);
   if m < 1 then
      return [];
   fi;
   l := Concatenation( [1/AlffVarT(F)], 
                PolyPrimeList(PolyAlg(AlffVarT(F)), IntDivisors(m)) );
   l := Flat(List(l, p->AlffPlaceSplit(F, p)));
   l := Filtered(l, p->AlffPlaceDeg(p) = m);
   return l;
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffPlaceRandom
#
#LONGDOC   This function returns a random place of degree $d$ of a
#          global function field $F$. It may return false though there
#          exist such a place (try calling again). 
#
#CALLING   p := AlffPlaceRandom(F);
#
#ARGUS     p @ alff place @ of degree $d$
#          F @ global function field
#
#CSRC      alff\_num\_primes\_deg\_one 
#
#XREF      AlffPlaces
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2, 3));
#          AlffOrders(y^3+T^3*y+T);
#          AlffPlaceRandom(F, 2);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  99xxxx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffPlaceRandom := function(F, d)
#
#  Try to return a random place of degree d for 
#  a global function field F.
#
   local kT, l, p, maxtries, tries;
   AlffIsGlobalAssert(F);
   if d < 1 then
      return false;
   fi;
   kT := PolyAlg(AlffVarT(F));
   maxtries := d*(AlffGenus(F)+1);
   tries := 0;
   repeat
      p := PolyPrimeRandom(kT, d);
      l := Filtered(AlffPlaceSplit(F, p), x->AlffPlaceDeg(x) = d);
      tries := tries + 1;
   until Length(l) > 0 or tries > maxtries;
   if tries > maxtries then
      return false;
   else
      return RandomList(l);
   fi;   
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffPlacesNonSpecial
#
#SHORTDOC  Computes a non special system of $g$ distinct places
#          for a global function field $F/k$ of genus $g$.
#
#LONGDOC   Let $g$ be the genus of the global function field $F/k$ and
#          $c$ be the dimension of the exact constant field over the
#          constant field of definition.
#          This function returns a list of distinct places $\p_1, \dots, \p_m$
#          of $F/k$ such that $\sum_{i=1}^m \deg(\p_i) = cg$ and 
#          $\max_{i=1}^m \deg(\p_i)$ is minimal among any such places.
#          If a second parameter $d \in \Z^{\geq 1}$ is given, the maximum
#          is forced to be $\leq d$. If there do not exist such places in
#          either case, false is returned.
#
#CALLING   S := AlffPlacesNonSpecial(F);\\
#          S := AlffPlacesNonSpecial(F, d);
#
#ARGUS     S @ list @ of alff places
#          F @ global function field
#          d @ integer @ $\geq 1$
#
#XREF      AlffPlaces, AlffDivisorLDim
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5));
#          AlffOrders(y^2 + T^17 + T + 1);
#          AlffPlacesNonSpecial(F);
#          AlffPlacesNonSpecial(F, 1);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  99xxxx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffPlacesNonSpecial := function(arg)
#
#  Return g distinct places of smallest maximal degree (or bounded as 
#  specified) such that their sum has dimension 1 over exact const field. 
#  false is returned if they do not exist.
#
   local check, L, F, d, g, R, i, ecfd;
   
   check := function(D, Q)
      # construct recursively subsets of places of degree sum c.
      local d, R, i, len;
      d := AlffDivisorDeg(D) / ecfd;
      if d > g or AlffDivisorLDim(D) / ecfd > 1 then
         return false;
      elif d = g then
         return D;
      else
         len := Length(Q);
         for i in [1..len] do
            R := check( D + Q[i], Q{[i+1..len]} ); 
            if not R = false then
               return R;
            fi;
         od;
         return false;
      fi;   
   end;
   
   F := arg[1];
   AlffIsGlobalAssert(F);
   ecfd := AlffDimExactConstField(F);
   g := AlffGenus(F);
   
   # F.plnonspec contains either a list of non spec places of smallest
   # maximal degree or is an int dd s.t. there are no non spec places of
   # maximal degree <= dd.
   
   if not IsBound(F.plnonspec) then
      F.plnonspec := -1;
   fi;
   if Length(arg) = 2 then
      d := Minimum(arg[2], ecfd*g);
   else
      d := ecfd*g;
   fi;
   if IsList(F.plnonspec) then
      if ForAll(F.plnonspec, p->AlffPlaceDeg(p) <= d) then
         return F.plnonspec;
      else
         return false;
      fi;
   fi;
   if IsInt(F.plnonspec) and F.plnonspec >= d then
      return false;
   fi;
   for i in [0..d] do
      L := Concatenation(List([0..i], x->AlffPlaces(F, x)));
      R := check(AlffDivisor(F), L);
      if not R = false then
         F.plnonspec := AlffDivisorSupp(R);
         return F.plnonspec;
      fi;
   od;   
   F.plnonspec := d;
   return false;
end;



#######################################################################
#
#  Residue fields for general alffs
#

#######################################################################

#  Internal implementation remarks:

#  problems are due to i) make AlffResidueField relative.
#                     ii) FF(poly) != FF(poly) (in Magma)

#  The residue field is a bit hacky. Since we cannot recover from
#  the prime poly the residue ring, we do store it with
#  a place as residue field of its subplace (in C) ...
#  The situation for the residue field of a place is similar:
#  Once computed and stored within the place.
#  Additionally: Evaluation of a function at a place is the res hom
#  for the assoc. valuation ring, we do not have this ...

#  the _-functions are intended to work with the internal residue field
#  of subplaces. We could extend prime polys to have their own residue
#  fields as created by AlffPolyResField, to be 'subplaces' ...

#######################################################################


AlffPrimeResField := function(p)
#
#  prime p from k1x or kx.
#  an rf is an extension of k (or quotient of k1x or kx) defined by p.
#  for this there are also AlffPrimeResFieldHom, AlffPrimeResFieldLift.
#
   local l, rf;   
   l := _AlffPrimeResField(p);
   rf := rec();
   rf.base := l[2];
   rf.ring := l[1];
   rf.info := l[3];
   rf.prime := p;
   rf.operations := rec();
   rf.operations.Print := function(a) 
      Print("Alff prime residue field for prime ", p, "\n", a.base); 
   end;
   return rf;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffResidueField
#
#SHORTDOC  Return the residue field of a place.
#
#LONGDOC   Let $F/k$ be an algebraic function field and $P$ be a place
#          of $F/k$. If $\o_P$ is the ring of algebraic functions defined
#          at $P$ then $k(P) := \o_P / P$ is the residue class field
#          of $P$. It is returned by this function.
#
#CALLING   K := AlffResidueField(P);
#
#ARGUS     K @ field @ residue field of $P$
#          P @ alff place @ 
#
#XREF      AlffEltEval, AlffEltLift, AlffEltValuation
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(Q);
#          AlffOrders(y^2 - T^4 + T + 1);
#          P := AlffPlaceSplit(F, T+2)[1];
#          AlffResidueField(P);
#          P := AlffPlaceSplit(F, 1/T)[1];
#          AlffResidueField(P);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffResidueField := function(place)
   local i, j, k, o, n, one, zero, H, ind, B, dim,
         M, MM, A, a, ma, KK, found, subplace, subplace_resfield;
   
   if not IsAlffPlace(place) then
      return Error("usage: AlffResidueField( <alff place> )");
   fi;
   
   if IsBound(place.resfield) then
      return place.resfield;
   fi;
   
   o := AlffPlaceOrder(place);
   n := AlffOrderDeg(o);
   zero := 0*AlffVarT(AlffOrderAlff(o));
   one := zero + 1;
   
   subplace := place;   # for readability
   subplace_resfield := _AlffPlaceSubResField(place); 
   
   H := MatToColList(AlffIdealBasisUpperHNF(AlffPlaceIdeal(place))[1]);
   
   # compute indices of residue field basis elts
   ind := Filtered([1..n], i->not _IsOne(H[i][i]));
   
   # set res field basis and dimension
   B := AlffOrderBasis(o){ind};
   dim := Length(ind);
   
   # make trafo matrix: res field basis -> order basis
   M := [];
   for i in [1..n] do
      if i in ind then
         M[i] := List([1..n], x->zero);
         M[i][i] := one;
      else
         M[i] := -H[i];
         M[i][i] := zero;
      fi;
   od;
   # move coefficients into residue field 
   MM := [];
   for i in ind  do
      Add(MM, List([1..n], j->_AlffEltEval(subplace, M[j][i])));
   od;
   M := Mat(subplace_resfield, MM);

   # find generator of residue field
   found := false;
   i := 1;
   while not found and i <= dim do
      a := B[i];
      A := List(a*B, k->AlffEltToList(k)[1]);
      for j in [1..dim] do
         Apply(A[j], x->_AlffEltEval(subplace, x));
      od;
      A := M * MatTrans(subplace_resfield, A);
      ma := MatMinPoly(A);
      if PolyDeg(ma) = dim then
         found := true;
      fi;
      i := i+1;   
   od;
   if not found then
      return Error("could not find primitive element of residue class field");
   fi;
   
   # and relate its power basis to B
   A := List([0..dim-1], k->AlffEltToList(a^k)[1]);
   for j in [1..dim] do
      Apply(A[j], x->_AlffEltEval(subplace, x));
   od;
   A := M * MatTrans(subplace_resfield, A);
   
   # create residue field extension, polynomial variable is x.
   KK := AlffPrimeResField(ma);
   
   # store information
   place.ind := ind;
   place.A := A;
   place.iAM := MatInv(A) * M;
   place.resfield := KK;
   KK.operations.Print := function(a) 
      Print("Alff place residue field\n", a.base); 
   end;
   
   return KK;
end;




#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffEltEval
#
#SHORTDOC  Evaluate an algebraic function at a place.
#
#LONGDOC   Let $F/k$ be an algebraic function field and $P$ be a place
#          of $F/k$. If $\o_P$ is the ring of algebraic functions defined
#          at $P$ then $k(P) := \o_P / P$ is the residue class field
#          of $P$. The image $a(P)$ of an algebraic function $a \in F$ in 
#          $k(P)$ or {\tt false} for $a \notin \o_P$ is returned.
#
#CALLING   b := AlffEltEval(P, a);
#
#ARGUS     b @ field element @ value of $a$ at $P$
#          P @ alff place @
#          a @ alff element @ 
#
#XREF      AlffResidueField, AlffEltLift, AlffEltValuation
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(Q);
#          AlffOrders(y^2 - T^4 + T + 1);
#          P := AlffPlaceSplit(F, T+2)[1];
#          K := AlffResidueField(P);
#          a := AlffEltGenT(F);
#          b := AlffEltEval(P, a);
#          b := AlffEltEval(P, 1/(a+2));
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffEltEval := function(place, a)
#
#  Return the value of an algebraic function at a place, i.e.
#  return its image in the residue class field of the place
#
   local o, K, d, dn, dd, coefs, subplace, subplace_resfield,
         p, a1, denval, j, beta, rho;
   
   # the first two cases apply for 'subplaces'
   
   if IsPoly(place.base) or IsQfe(place.base) then
      return _AlffEltEval(place, a);
   elif IsAlffPlace(place) then
      if not IsBound(place.resfield) then
         AlffResidueField(place);
      fi;   
      a := AlffEltMove(a, AlffPlaceOrder(place));
      subplace := place;                           #for readability
      subplace_resfield := _AlffPlaceSubResField(place);
      if _IsOne(AlffEltDen(a)) then
         coefs := List(AlffEltToList(a)[1], x->_AlffEltEval(subplace, x));
         coefs := place.iAM * MatTrans(subplace_resfield, [coefs]);
         coefs := MatToColList(coefs)[1];
         return AlffPrimeResFieldHom(place.resfield,
                        Poly(PolyAlg(subplace_resfield),
                             Reversed(coefs)));
         #remark: polynomial variable in x as in generation of resfield.   
      fi;
      p := AlffPlaceMin(place);
      beta := AlffPlaceBeta(place) / p;
      denval := QfeVal(p, AlffEltDen(a))*AlffPlaceRam(place);
      a1 := AlffEltNum(a);
      j := 0;
      while j < denval and _IsOne(AlffEltDen(a1)) do
         a1 := beta * a1;
         j := j + 1;
      od;
      if not _IsOne(AlffEltDen(a1)) then
         return false;
      fi;
      a1 := AlffEltEval(place, a1);
      if _IsZero(a1) then
         return a1;
      fi;
      d := beta^denval * AlffEltDen(a);
      d := AlffEltEval(place, d);
      return a1 / d;
   else
      return Error("usage: AlffEltEval( <place>, <elt> )");
   fi;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffEltLift
#
#SHORTDOC  Lift a residue field element.
#
#LONGDOC   Let $F/k$ be an algebraic function field and $P$ be a place
#          of $F/k$. If $\o_P$ is the ring of algebraic functions defined
#          at $P$ then $k(P) := \o_P / P$ is the residue class field
#          of $P$. For $b \in k(P)$ this function returns $a \in \o_P$
#          such that $a(P) = b$. Constants are lifted to constants.
#
#CALLING   a := AlffEltLift(P, b);
#
#ARGUS     a @ alff element @ in $\o_P$
#          P @ alff place @
#          b @ field element @ value of $a$ at $P$ to be lifted
#
#XREF      AlffResidueField, AlffEltEval, AlffEltValuation
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(Q);
#          AlffOrders(y^2 - T^4 + T + 1);
#          P := AlffPlaceSplit(F, T+2)[1];
#          K := AlffResidueField(P);
#          a := AlffEltGenT(F);
#          b := AlffEltEval(P, a);
#          AlffEltLift(P, b);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffEltLift := function(place, a)
#
#  a is an element of the residue class field
#  of the given place. Return a lift of a to the function field (defined
#  in the order of place)
#
   local k, subplace, subplace_resfield, coefs, l, i, o, oi, b, F,
         AlffPrimeResFieldLiftList;
   
   AlffPrimeResFieldLiftList := function(rf, a)
      local n, l;
      a := AlffPrimeResFieldLift(rf, a);
      if IsPoly(rf.prime) then
         n := PolyDeg(rf.prime);
      else
         n := 1;
      fi;
      l := Reversed(PolyToList(a));
      Append(l, 0*[1..n-Length(l)]);
      return l;
   end;
   
   if not IsAlffPlace(place) then
      return Error("usage: AlffEltLift( <alff place>, <elt> )");
   fi;
   if not IsBound(place.resfield) then
      return Error("residue field not calculated");
   fi;
   F := AlffPlaceAlff(place);
   k := AlffConstField(F);
   subplace := place;                            #for readability
   subplace_resfield := _AlffPlaceSubResField(place);
   coefs := AlffPrimeResFieldLiftList(place.resfield, a);
   #   coefs := FFEltToList(a, subplace.resfield);
   coefs := place.A * MatTrans(subplace_resfield, [coefs]);
   coefs := MatTrans(coefs);
   l := List([1..AlffDeg(F)], x->0);
   for i in [1..Length(place.ind)] do
      l[place.ind[i]] := _AlffEltLift(subplace, coefs[1][i]);
   od;
   return AlffElt(AlffPlaceOrder(place), l);
end;


#######################################################################
#
# Simple series expansion at a place
#


AlffPlaceLocals.EltExpansion := function(place, t, a, prec, lift)
#
#  t a local parameter,
#  a an element, both defined in the maximal order of place
#  result given mod O(t^(prec+1))
#  (For places of degree > 1 the expansion is not canonical)
#
   local p, val, zero, b, l, j, c1, c2;
   p := AlffPlaceMin(place);
   val := AlffPlaceRam(place)*QfeVal(p, AlffEltDen(a));   
   b := t^val * a;
   l := [];
   j := -val;
   while not _IsZero(b) and j <= prec do
      c1 := AlffEltEval(place, b);
      c2 := AlffEltLift(place, c1);
      b := (b - c2) / t;
      if not _IsZero(c1) then
         if lift then
            Add(l, [c2, j]); 
         else
            Add(l, [c1, j]); 
         fi;
      fi;
      j := j + 1;
   od;
   return l;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffEltResiduum
#
#SHORTDOC  Compute the residuum of an algebraic function.
#
#LONGDOC   Let $F/k$ be an algebraic function field and $P$ be a place
#          of $F/k$. It is currently required that $\deg(P) = 1$ holds.
#          For a local parameter $t \in F$ at $P$ this function computes
#          the residuum of an algebraic function $a \in F$.
#
#CALLING   r := AlffEltResiduum(P, t, a);
#
#ARGUS     r @ field element @ the residuum of $a$ at $P$ for $t$
#          P @ alff place @
#          t @ alff element @ a local paramter at $P$
#          a @ alff element @ an algebraic function
#
#XREF      AlffPlacePrimeElt, 
#          AlffResidueField, AlffEltEval, AlffEltLift, AlffEltValuation,
#          AlffDiffResiduum.
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(Q);
#          AlffOrders(y^2 - T^4 + T + 1);
#          P := AlffPlaceSplit(F, T+1)[1];
#          t := AlffPlacePrimeElt(P);
#          AlffEltResiduum(P, t, 1/t^2 + 3 + t);
#          AlffEltResiduum(P, t, 1/t^2 + 2/t + 3 + t);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffEltResiduum := function(place, t, a)
#
#  Compute the residuum of an algebraic function a
#  at a place with respect to a prime element t.
#  It is an element of the residue class field.   
#
   local l, n, res, o, Zero;
   Zero := function(R)
      if IsFF(R) then
         return FFElt(R, 0);
      elif IsOrder(R) then
         return Elt(R, 0);
      else
         return 0;
      fi;
   end;
   if AlffPlaceDeg(place) > 1 then
      return Error("only for degree one places");
   fi;
   o := AlffPlaceOrder(place);
   t := AlffEltMove(t, o);
   a := AlffEltMove(a, o);
   l := AlffPlaceLocals.EltExpansion(place, t, a, -1, false);
   n := Length(l);
   if n = 0 or l[n][2] < -1 then
      res := Zero(AlffConstField(AlffPlaceAlff(place)));
   else
      res := l[n][1];
   fi;
   return res;
end;

