

#############################################################################
#
# Some qfe functions
#


IsPolyOrQfe := function(a)
   if IsPoly(a) or IsQfe(a) then
      return true;
   else
      return false;
   fi;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       QfeDeriv
#
#SHORTDOC  Computes the derivation of an rational function.
#
#LONGDOC   This function computes $dh/dT$ for $h \in k(T)$.
#
#CALLING   dhdT := QfeDeriv(h);
#
#ARGUS     dhdT, h @ qf elements
#
#CSRC      
#
#XREF      AlffDiff, PolyDeriv
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(5,1));;
#          h := 1/T;
#          QfeDeriv(h);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xxxx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

QfeDeriv := function(a)
   local num, den;
   num := QfeNum(a);
   den := QfeDen(a);
   return (PolyDeriv(num)*den - num*PolyDeriv(den))/den^2;
end;


#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       QfePthRoot
#
#SHORTDOC  Returns the $p$-th root of a rational function
#          with scalars in a finite field of characteristic $p$,
#          if existent.
#
#CALLING   r := QfePthRoot(a);
#
#
#ARGUS     r @ qf element or bool @ $r$ $p$-th root of $a$ or {\tt false}
#          a  @ qf element
#
#PACKAGE   Alff
#
#XREF      AlffEltPthRoot
#
#KASH       @
#           k := FF(3);
#           kx := PolyAlg(k);
#           kxy := PolyAlg(kx);
#           X := Poly(kx, [1,0]);
#           y := Poly(kxy, [1,0]);
#           QfePthRoot(1/X^3);
#           QfePthRoot(y);
#           QfePthRoot(1/X + y^3);
#           QfePthRoot(1/X^3 + y^3);
#           QfePthRoot(1/y^3);
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**
#
#  9909xxxx FH installed
# 
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

QfePthRoot := function(c)
   local kx, l, ll, num, den, i, p, pinv;
   if not IsPolyOrQfe(c) and not IsFFElt(c) then
      return Error("usage: QfePthRoot( <qf element> )");
   fi;
   if IsFFElt(c) then
      p := FFSize(FFEltFF(c));
      pinv := p[1]^(p[2]-1);
      return c^pinv;
   else
      kx := PolyAlg(QfeNum(c));
      if not IsFF(QfScalarRing(kx)) then
         return Error("qf not defined over finite field");
      fi;
      p := FFSize(QfScalarRing(kx))[1];
      l := Reversed(PolyToList(QfeNum(c)));
      ll := 0*l;
      for i in [0..Length(l)-1] do
         if not _IsZero(l[i+1]) then
            if not i mod p = 0 then
               return false;
            fi;
            ll[i/p+1] := QfePthRoot(l[i+1]);
            if ll[i/p+1] = false then
               return false;
            fi;
         fi;
      od;
      num := Poly(kx, Reversed(ll));
      if _IsOne(QfeDen(c)) then
         return num;
      fi;
      den := QfePthRoot(QfeDen(c));
      if den = false then
         return false;
      fi;
      return num / den;
   fi;   
end;


#############################################################################
#
# Function field definitions
#



#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffEllipticFunField
#
#SHORTDOC  Creates an elliptic algebraic function field $F/k$.
#
#LONGDOC   Let $k$ be a field. The function creates the algebraic elliptic 
#          function field $F$.
#          The field is defined by  the polynomial
#          $y^2 + a_1Ty + a_3y-( T^3 + a_2T^2 + a_4T + a_6)$ with $a_i \in k$.
#           
#CALLING   F := AlffEllipticFunField(k, L);
#
#ARGUS     F @ algebraic function field
#          k @ field
#          L @ list
#
#PACKAGE   Alff
#
#KASH      @
#          F := AlffEllipticFunField(FF(2,5), [1,1,1,1,1]);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
# 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffEllipticFunField := function ( k, L )
   local p, kT, kTy, T, y;
   kT := PolyAlg(k, "T");
   kTy := PolyAlg(kT, "y");
   T := Poly(kT, [1,0]);
   y := Poly(kTy, [1,0]);
   if not (Length(L) = 5) then
    return Error("usage: AlffEllipticFunField( <field>, <list> )");
   fi;     
   p:= y^2 + L[1]*T*y + L[3]*y - (T^3 + L[2]*T^2 + L[4]*T + L[5]);
   if not AlffPolyIsIrrSep(p) then
      return Error("polynomial not irreducible and separable");
   else
      return Alff(p);
   fi;   
end;



#############################################################################
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffHermitianFunField
#
#SHORTDOC  Creates a Hermitian global function field $F/\FF_{q^2}(T)$.
#
#LONGDOC   Creates a Hermitian global function field defined by the polynomial
#          $y^q+y=T^{q+1} \in \FF_{q^2}[T,y]$ with $q = p^d$. 
#
#
#CALLING   F := AlffHermitianFunField(p, d);
#
#ARGUS     F @ global function field
#          p, d @ integer
#         
#XREF      AlffInit
#
#PACKAGE   Alff
#
#KASH      The definition of a Hermitian global function field:@
#          F := AlffHermitianFunField(2, 3);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


AlffHermitianFunField := function ( p, d )
   local  q0, k, kT, kTy, T, y;
   q0 := p ^ d;
   k := FF(p, 2 * d);
   kT := PolyAlg(k, "T");
   kTy := PolyAlg(kT, "y");
   T := Poly(kT, [1,0]);
   y := Poly(kTy, [1,0]);
   return Alff( y ^ q0 + y - T ^ (q0 + 1) );
end;


#############################################################################
#
#


#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX
#
#FXN       AlffEltAlff
#
#SHORTDOC  Returns the algebraic function field of an alff element.
#
#CALLING   F := AlffEltAlff(a);
#
#ARGUS     F @ alff 
#          a  @ alff element
#
#XREF      AlffEltOrder
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2,1));
#          AlffOrders(y^3+T^3*y+T);
#          a := AlffElt(o, [0,1,0] / (T+1));
#          AlffEltAlff(a);
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
# 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffEltAlff := function(a)
   return AlffOrderAlff(AlffEltOrder(a));
end;

#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX
#
#FXN       AlffOrderDeg
#
#SHORTDOC  Returns the degree of an alff order over its coefficient ring.
#
#CALLING   d := AlffOrderDeg(o);
#
#ARGUS     d @ integer @ degree 
#          o @ alff order
#
#XREF      AlffDeg
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2,1));
#          AlffOrders(y^3+T^3*y+T);
#          AlffOrderDeg(o);
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
# 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffOrderDeg := function(o)
   return PolyDeg(AlffOrderPoly(o));
end;

#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX
#
#FXN       AlffOrderBasis
#
#SHORTDOC  Returns a list containing the basis elements of an alff order.
#
#CALLING   B := AlffOrderBasis(o);
#
#ARGUS     B @ list @ of basis elements
#          o @ alff order
#
#XREF      AlffIdealBasis
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2,1));
#          AlffOrders(y^3+T^3*y+T);
#          Bf := AlffOrderBasis(o);
#          Bi := AlffOrderBasis(oi);
#          Bf = Bi;
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
# 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffOrderBasis := function(o)
   local L;
   L := MatToRowList(MatId(Z, AlffOrderDeg(o)));
   Apply(L, x->AlffElt(o, x));
   return L;
end;


#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX
#
#FXN       AlffIdealAlff
#
#SHORTDOC  Returns the algebraic function field of an alff order ideal.
#
#CALLING   F := AlffIdealAlff(I);
#
#ARGUS     F @ alff 
#          I @ alff order ideal
#
#XREF      AlffIdealOrder
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2,1));
#          AlffOrders(y^3+T^3*y+T);
#          I := AlffElt(o, [0,1,0] / (T+1)) * o;
#          AlffIdealAlff(I);
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
# 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffIdealAlff := function(id)
   return AlffOrderAlff(AlffIdealOrder(id));
end;


#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX
#
#FXN       AlffIdealBasis
#
#SHORTDOC  Returns a list containing the basis elements of an alff order
#          ideal.
#
#CALLING   B := AlffIdealBasis(I);
#
#ARGUS     B @ list @ of basis elements
#          I @ alff order ideal
#
#XREF      AlffOrderBasis, AlffIdealBasisUpperHNF
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2,1));
#          AlffOrders(y^3+T^3*y+T);
#          I := AlffElt(o, [0,1,0] / (T+1)) * o;
#          AlffIdealBasis(I);
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
# 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffIdealBasis := function(a)
   local L, o, hnf;
   o := AlffIdealOrder(a);
   hnf := AlffIdealBasisUpperHNF(a);
   L := MatToColList(hnf[1]);
   Apply(L, x->AlffElt(o, x)/hnf[2]);
   return L;
end;

#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX
#
#FXN       AlffEltRepMat
#
#SHORTDOC  Returns a representation matrix of an algebraic function field
#          element.
#
#LONGDOC   Given an alff element $a$ represented in its order $\o$ 
#          this function computes the representation matrix of 
#          $a$ on the basis of $\o$. 
#
#CALLING   M := AlffEltRepMat(a);
#
#ARGUS     M @ matrix
#          a  @ alff element
#
#XREF      AlffEltCharPoly
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2,1));
#          AlffOrders(y^3+T^3*y+T);
#          a := AlffElt(o, [0,1,0] / (T+1));
#          AlffEltRepMat(a);
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
#
#  9909xxxx FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffEltRepMat := function(a)
   return _AlffEltRepMat(AlffEltDen(a)*a)/AlffEltDen(a);
end;


#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffEltMinPoly
#
#SHORTDOC  Returns the minimal polynomial of an algebraic function field 
#          element.
#
#LONGDOC   Given an alff element $a$ of $F$ this function 
#          computes the minimal polynomial of $a$ over $k(T)$.
#
#CALLING   p := AlffEltMinPoly(a);
#
#
#PACKAGE   Alff
#
#ARGUS     p  @ polynomial
#          a  @ alff element
#
#XREF      AlffEltCharPoly
#
#CSRC      alffo\_elt\_min\_char\_poly
#
#KASH       @
#          AlffInit(FF(2,1));
#          AlffOrders(y^3+T^3*y+T);
#          a := AlffElt(o, [0,1,0]);
#          p := AlffEltMinPoly(a);
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**
#
#  9909xxxx FH installed 
# 
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffEltMinPoly := function(a)
   local F, p, f, g, h, d;
   if not IsAlffElt(a) then
      return Error("usage: AlffEltMinPoly( <alff elt> )");
   fi;
   F := AlffEltAlff(a);
   p := Characteristic(F);
   f := AlffEltCharPoly(a);
   if  p > 0 then
      AlffIsGlobalAssert(F); # otherwise we cannot take pth roots
      g := f;
      repeat
         f := g;
         g := QfePthRoot(f);
      until g = false;
   fi;
   g := PolyGcd(f, PolyDeriv(f));
   h := f / g;
   d := Gcd(PolyToList(h));
   return h / d;
   # remark: f should has integral coeffs and is primitive
   #         g is monic but maybe not integral
   #         therefore h is integral but maybe not primitive
end;



#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX
#
#FXN       AlffConstFieldSize
#
#SHORTDOC  Returns the size of the constant field of definition of
#          a global function field.
#
#CALLING   q := AlffConstFieldSize(F);
#
#ARGUS     q @ integer @ size of constant field
#          F @ algebraic function field
#
#XREF      AlffConstField, AlffIsGlobal, Characteristic
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2,3));
#          F := Alff(y^3+T^3*y+T);
#          AlffConstFieldSize(F);
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
# 
#  9909xxxx FH installed 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffConstFieldSize := function(F)
   local l, k;
   k := AlffConstField(F);
   if not IsFF(k) then
      return Error("constant field is infinite");
   fi;
   l := FFSize(k);
   return l[1]^l[2];
end;


AlffMultiCounterInit := function(len)
   return List([1..len], i->1);
end;


AlffMultiCounterInc := function ( L, Bound )
   local  testcoeff, len, i, ende;
   i := 1;
   len := Length( L );
   ende := false;
   while not ende and i <= len  do
      L[i] := L[i] + 1;
      if not L[i] <= Bound then
         L[i] := 1;
         i := i + 1;
      else
         ende := true;
      fi;
   od;
   return i <= len;
end;


#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX
#
#FXN       AlffIsGlobal
#
#SHORTDOC  Returns whether an algebraic function field $F$ is global.
#
#CALLING   b := AlffIsGlobal(F);
#
#ARGUS     b @ boolean @ whether global or not
#          F @ algebraic function field
#
#XREF      AlffIsGlobalAssert, AlffConstField, Characteristic
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(Q);
#          F := Alff(y^3+T^3*y+T);
#          AlffIsGlobal(F);
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
#
#  9909xxxx FH installed 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffIsGlobal := function(F)
   return IsFF(AlffConstField(F));
end;


#############################################################################
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX
#
#FXN       AlffIsGlobalAssert
#
#SHORTDOC  Signals an error if an algebraic function field $F$ is not global.
#
#CALLING   AlffIsGlobalAssert(F);
#
#ARGUS     F @ algebraic function field
#
#XREF      AlffIsGlobal, AlffConstField, Characteristic
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(FF(2,1));
#          F := Alff(y^3+T^3*y+T);
#          AlffIsGlobalAssert(F);
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
# 
#  9909xxxx FH installed 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffIsGlobalAssert := function(F)
   if not AlffIsGlobal(F) then
      Error("assertion failed: alff not global");
   fi;
end;


##############################################################################
#
#

#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       IntMoebiusMy
#
#SHORTDOC  Computes the Moebius $\mu$-function.
#
#CALLING   a := IntMoebiusMy(n);
#
#ARGUS     a, n @ integers
#
#PACKAGE   Alff
#
#XREF      IntDivisors
#
#KASH      @
#          IntMoebiusMy(1);
#          IntMoebiusMy(2*3);
#          IntMoebiusMy(2*3*5);
#          IntMoebiusMy(2^2*3*5);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
# 
#  9909xxxx FH installed 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

IntMoebiusMy := function(n)
   local flst;
   if n = 1 then
      return 1;
   elif n < 1 then
      return false;
   else
      flst := Factor(n);
      if ForAll(flst, p->p[2] = 1) then
         return (-1)^Length(flst);
      else
         return 0;   
      fi;
   fi;
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       ListSplit
#
#SHORTDOC  Split a list into equal parts. All parts but the last 
#          have equal length
#          as specified by the parameter $n \geq 1$.
#
#CALLING   S := ListSplit(L, n);
#
#ARGUS     S @ list @ of lists of length $\leq n$
#          L @ list @ to be split
#          n @ integer @ block length
#
#PACKAGE   Alff
#
#XREF      IntDivisors
#
#KASH      @
#          L := [1, 2, 3, 4, 5];
#          ListSplit(L, 1);
#          ListSplit(L, 2);
#          ListSplit(L, 101);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
# 
#  9909xxxx FH installed 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

ListSplit := function(L, n)
   local S, r;
   r := IntQuo(Length(L), n);
   S := List([0..r-1], i->L{[i*n+1..(i+1)*n]});
   if Length(L) > r*n then
      Add(S, L{[r*n+1..Length(L)]});
   fi;
   return S;
end;


FFElts := function(k)
   local i, w, l, q;
   w := FFPrimitiveElt(k);
   q := FFSize(k);
   q := q[1]^q[2];
   l := [0*w];
   for i in [1..q-1] do
      Add(l, w^i);
   od;
   return l;
end;	


IsMatCheck := function(M)
   if not IsMat(M) then
      return Error("cannot convert into matrix");
   fi;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       ListApplyListAdd
#
#SHORTDOC  Compute a linear combination.
#
#LONGDOC   Given lists $L := [ a_1, \dots, a_n ]$ and 
#          $S := [ \lambda_1, \dots, \lambda_n ]$ this 
#          function returns 
#          $ a := \sum_{i=1}^n \lambda_{i} a_i$.
#
#CALLING   a := ListApplyListAdd(L, S);
#
#ARGUS     a @ element
#          L, S @ lists @ of elements
#
#PACKAGE   Alff
#
#XREF      ListApplyMatAdd
#
#KASH      @
#          L := [1, 2, 3];
#          S := [1, 3, 4];
#          ListApplyListAdd(L, S);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
# 
#  9909xxxx FH installed 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

ListApplyListAdd := function(L, v)
   if not Length(L) = Length(v) then
      return Error("wrong dimensions");   
   fi;
   return Sum([1..Length(L)], i->v[i]*L[i]);
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       ListApplyMatAdd
#
#SHORTDOC  Compute linear combinations.
#
#LONGDOC   Given a list $L := [ a_1, \dots, a_n ]$ and 
#          an $n \times m$ matrix $M = (\lambda_{i,j})_{i,j}$ this 
#          function returns a list
#          $ S := \bigl[ \, \sum_{i=1}^n \lambda_{i, j} a_i \, \bigr]_j $.
#
#CALLING   S := ListApplyMatAdd(L, M);
#
#ARGUS     S, L @ lists @ of elements
#          M @ matrix @ for linear combinations
#
#PACKAGE   Alff
#
#XREF      ListApplyListAdd
#
#KASH      @
#          M := MatTrans(Z, [ [ 1, 0 ], [1, 1], [0, 2] ]);
#          L := [ 1, 2 ];
#          ListApplyMatAdd(L, M);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
# 
#  9909xxxx FH installed 
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

ListApplyMatAdd := function(L, M)
   local col, N, n;
   n := MatRows(M);
   if not Length(L) = n then
      return Error("wrong dimensions");   
   fi;
   M := MatToColList(M);
   N := [];
   for col in M do
      Add(N, Sum([1..n], i->col[i]*L[i]));
   od;
   return N;
end;


AlffMakeLinear := function(L)
#
#  Map objects in list L into a linear row space over k
#  (works not over k1x)
#  Returns matrix
#
   local F, dT, cden, n, l, i, j, S, v, v1, max;
   if Length(L) = 0 then
      return [];
   fi;
   if IsAlffDiff(L[1]) then
      F := AlffDiffAlff(L[1]);
      dT := AlffDiff(AlffEltGenT(F));
      L := List(L, x->x / dT);
      return AlffMakeLinear(L);
   elif IsAlffElt(L[1]) then
      cden := Iterated(List(L, x->AlffEltDen(x)), Lcm);
      L := List(L, x->AlffEltNum(x) * (cden / AlffEltDen(x)));
      L := List(L, x->AlffEltToList(x)[1]);
      n := Length(L[1]);
      l := Length(L);
      L := List([1..l], i->List([1..n], j->PolyToList(L[i][j])));
      max := List([1..n], j->Maximum(List([1..l], i->Length(L[i][j]))));
      S := [];
      for i in [1..l] do
         v := [];
         for j in [1..n] do
            v1 := Reversed(L[i][j]);
            Append(v, v1);
            Append(v, 0*[ 1 .. max[j] - Length(v1)]);
         od;
         IsMatCheck(v);
         Add(S, v);
      od;
      IsMatCheck(S);
      return S;
   else
      return Error("objects not supported");
   fi;
end;


AlffLinearRepresent := function(B, L)
#
#  Represent all elements of L rowwise in basis B.
#  Return false if not possible.
#
   local v, S, s;
   S := AlffMakeLinear(Concatenation(B, L));
   B := S{[1..Length(B)]};
   IsMatCheck(B);
   L := S{[Length(B)+1..Length(S)]};
   IsMatCheck(L);
   B := MatTrans(B);
   S := [];
   for v in L do
      IsMatCheck(v);
      s := MatSolve(B, MatTrans(v));
      if s = false then
         return false;
      fi;
      Add(S, MatTrans(s[1]));
   od;
   IsMatCheck(S);
   return S;
end;


AlffInftyDeg := function(F)
   return Gcd(List(AlffPlaceSplit(F, 1/AlffVarT(F)), AlffPlaceDeg));
end;
