

# proto

IsAlffPseudoMat := function(M)
   if IsRec(M) and IsBound(M.type) and
      M.type = "Alff Matrix" then
      return true;
   else
      return false;
   fi;
end;



###########################################################################
#########################################################################
#
#  Some temporary functions for ordinary matrices (over function fields),
#  since this is not yet covered by the usual Mat functions.
#

AlffMatMult := function(M1, M2)
   local M, dim;
   dim := Length(M1[1]);
   if not dim = Length(M2) then
      return Error("wrong matrix dimensions");
   fi;
   M := List(M1, v->List([1..Length(M2[1])], 
                j->Sum([1..dim], k->v[k]*M2[k][j])));
   return M;
end;



First := function ( list, func )
#
#  Return first element x of list for which func(x) is true.
#
   local  l;
   for l  in list  do
      if func( l )  then
         return l;
      fi;
   od;
   return false;
end;


AlffMatEchelon := function(M)
#
#  M is a list of rows. row transformations done.
#
   local rows, cols, i0, j0, i, m, ipiv;
   M := Copy(M);
   rows := Length(M);
   cols := Length(M[1]);
   for m in M do
      if not Length(m) = cols then
         return Error("wrong row");
      fi;
   od;
   i0 := 1;
   j0 := 1;
   while j0 <= cols and i0 <= rows do
      ipiv := First([i0..rows], i->not _IsZero(M[i][j0]));
      if not ipiv = false then
         m := M[i0];
         M[i0] := M[ipiv];
         M[ipiv] := m;
         for i in [ipiv+1..rows] do
            M[i] := M[i] - M[i][j0]/M[i0][j0] * M[i0];
         od;
         i0 := i0 + 1;
      fi;
      j0 := j0 + 1;   
   od;
   return M;
end;


AlffMatEchelonTrans := function(M)
#
#  M is a list of rows. row transformations done.
#
   local T, rows, cols, i0, j0, i, m, ipiv;
   M := Copy(M);
   rows := Length(M);
   cols := Length(M[1]);
   T := MatToRowList(MatId(Z, rows));
   for m in M do
      if not Length(m) = cols then
         return Error("wrong row");
      fi;
   od;
   i0 := 1;
   j0 := 1;
   while j0 <= cols and i0 <= rows do
      ipiv := First([i0..rows], i->not _IsZero(M[i][j0]));
      if not ipiv = false then
         m := T[i0];
         T[i0] := T[ipiv];
         T[ipiv] := m;
         m := M[i0];
         M[i0] := M[ipiv];
         M[ipiv] := m;
         for i in [ipiv+1..rows] do
            T[i] := T[i] - M[i][j0]/M[i0][j0] * T[i0];
            M[i] := M[i] - M[i][j0]/M[i0][j0] * M[i0];
         od;
         i0 := i0 + 1;
      fi;
      j0 := j0 + 1;   
   od;
   return [M, T];
end;


AlffMatRank := function(M)
   local rank;
   if IsAlffPseudoMat(M) then
      M := M.mat;
   fi;
   M := AlffMatEchelon(M);
   rank := First([1..Length(M)], i->ForAll(M[i], x->_IsZero(x)));
   if rank = false then
      return Length(M);
   else
      return rank-1;
   fi;
end;


AlffMatDet := function(M)
   if IsAlffPseudoMat(M) then
      M := M.mat;
   fi;
   M := AlffMatEchelon(M);
   return Product([1..Length(M)], i->M[i][i]);
end;



AlffMatDetDiag := function(M)
   M := AlffMatEchelon(M);
   return List([1..Length(M)], i->M[i][i]);
end;



###########################################################################
###########################################################################
#
# Compute the Wronskian of a Riemann Roch space
#


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffWronskian
#
#SHORTDOC  Return the Wronski matrix of a Riemann-Roch space.
#
#CALLING   W := AlffWronskian(D);\\
#          W := AlffWronskian(F);
#
#ARGUS     W @ list @ list of rows
#          D @ alff divisor @ ${\cal L}(D)$ is computed
#          F @ alff @ equivalent to taking $D = 0$
#
#LONGDOC   Let $F/k$ be an algebraic function field with separating 
#          element $x$ and let $v_1, \dots v_l$
#          be a basis of ${\cal L}(D)$. For the differentiation $D_x$ with 
#          respect to $x$ consider the successively 
#          smallest $\nu_1 \leq \dots \leq 
#          \nu_l \in \Z^{\geq 0}$ such that the rows $D_x^{(\nu_i)}(v_1), 
#          \dots, D_x^{(\nu_i)}(v_l)$, $1 \leq i \leq l$ 
#          are $F$-linearly independent. These rows form the Wronski 
#          matrix of $D$ with respect to $x$ and are returned as 
#          a list of lists $W$. If $D$ has dimension zero, {\tt false} is
#          returned. The constant field $k$ is required to be exact.
# 
#CSRC      
#
#XREF      AlffWronskianOrders, 
#          AlffDiff, AlffDifferentiation, AlffRamDivisor
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(Q);;
#          AlffOrders(y^2 - (T-1)*(T-2)*(T-3)*(T-4)*(T-5)*(T-6)*(T-7));
#          AlffWronskian(AlffCanonicalDivisor(F));
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffWronskian := function(D)
#
#  Compute the Wronskian of L(D) with resp. to x
#
   local D1, W, orders, basis, dim, len, k, x;
   Extend(D);
   if IsBound(D.Wronskian) then
      return D.Wronskian;
   fi;
   if IsAlff(D) then
      D1 := AlffDivisor(D);
   elif IsAlffDivisor(D) then
      D1 := D;
   else
      return Error("usage: AlffWronskian( <alff | alff divisor> )");
   fi;
   if AlffDimExactConstField(AlffDivisorAlff(D1)) > 1 then
      return Error("constant field must be exact");
   fi;
   x := AlffVarT(AlffDivisorAlff(D1));
   basis := AlffDivisorLBasis(D1);
   dim := Length(basis);
   if dim > 0 then 
      W := [ basis ];
      orders := [ 0 ];
      len := 1;
      k := 1;
      while len < dim do
         W[len+1] := List(basis, f->AlffDifferentiation(x, k, f));
         if AlffMatRank(W) > len then
            len := len + 1;
            orders[len] := k;
         fi;
         k := k + 1;
      od;
      D.Wronskian := W;
      D.WronskianOrders := orders;
   else
      D.Wronskian := false;
      D.WronskianOrders := [];
   fi;
   return D.Wronskian;
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffWronskianOrders
#
#SHORTDOC  Return the Wronski orders of a Riemann-Roch space.
#
#CALLING   W := AlffWronskianOrders(D);\\
#          W := AlffWronskianOrders(F);
#
#ARGUS     W @ list @ list of rows
#          D @ alff divisor @ ${\cal L}(D)$ is computed
#          F @ alff @ equivalent to taking $D = 0$
#
#LONGDOC   Let $F/k$ be an algebraic function field with separating 
#          element $x$ and let $v_1, \dots v_l$
#          be a basis of ${\cal L}(D)$. For the differentiation $D_x$ with 
#          respect to $x$ consider the successively 
#          smallest $\nu_1 \leq \dots \leq 
#          \nu_l \in \Z^{\geq 0}$ such that the rows $D_x^{(\nu_i)}(v_1), 
#          \dots, D_x^{(\nu_i)}(v_l)$, $1 \leq i \leq l$ 
#          are $F$-linearly independent. The numbers $\nu_1, \dots, \nu_l$
#          are the Wronski orders of $D$ with respect to $x$ and are 
#          returned. If $D$ has dimension zero, the empty list is
#          returned. The constant field $k$ is required to be exact.
# 
#CSRC      
#
#XREF      AlffWronskian, 
#          AlffDiff, AlffDifferentiation, AlffRamDivisor
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(Q);;
#          AlffOrders(y^2 - (T-1)*(T-2)*(T-3)*(T-4)*(T-5)*(T-6)*(T-7));
#          AlffWronskianOrders(AlffCanonicalDivisor(F));
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffWronskianOrders := function(D)
   AlffWronskian(D);
   return D.WronskianOrders;
end;


###########################################################################
###########################################################################
#
# Compute the ramification divisor of a divisor
#


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffRamDivisor
#
#SHORTDOC  Return the ramification divisor of a divisor.
#
#CALLING   A := AlffRamDivisor(D);\\
#          A := AlffRamDivisor(F);
#
#ARGUS     A @ alff divisor @ the ramification divisor of $D$
#          D @ alff divisor @ 
#          F @ alff @ equivalent to taking $D = 0$
#
#LONGDOC   Let $F/k$ be an algebraic function field, $x$ a separating 
#          variable and $D$ a divisor. The ramification divisor
#          of $D$ is defined to be $$ i(D) \, ( W - D ) + \bigl( 
#          W_x(D) \bigr) + \nu \, (dx),$$
#          where $W$ is a canonical divisor of $F/k$, $W_x(D)$ is 
#          the determinant
#          of the Wronksi matrix of $D$ with respect to $x$ and $\nu$ is the
#          sum of the Wronski orders of $D$ with respect to $x$. It is positive
#          and consists of the $D$-Weierstra\ss{} places. 
#          The constant field $k$ is required to be exact.
# 
#CSRC      
#
#XREF      AlffWronskian, AlffWronskianOrders, AlffWeierstrassPlaces,
#          AlffGapNumbers, AlffDiff, AlffDifferentiation
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(Q);;
#          AlffOrders(y^2 - (T-1)*(T-2)*(T-3)*(T-4)*(T-5)*(T-6)*(T-7));
#          AlffRamDivisor(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffRamDivisor := function(D)
#
#  Computes the ramification divisor of a divisor or the alff itself
#  It consists of the D-Weierstrass points of the curve
#
   local D1, F, Dx, DD, eps, n, Wdet;
   Extend(D);
   if IsBound(D.RamDivisor) then
      return D.RamDivisor;
   fi;
   if IsAlff(D) then
      D1 := AlffDivisor(D);
   elif IsAlffDivisor(D) then
      D1 := D;
   else
      return Error("usage: AlffRamDivisor( <alff | alff divisor> )");
   fi;
   F := AlffDivisorAlff(D1);
   if AlffDimExactConstField(F) > 1 then
      return Error("constant field must be exact");
   fi;
   Dx := AlffCanonicalDivisor(F);
   DD := Dx - D1;
   n := AlffDivisorLDim(DD);
   if n > 0 then
      Wdet := AlffMatDet(AlffWronskian(DD));
      eps := Sum(AlffWronskianOrders(DD));
      D.RamDivisor := n*DD + AlffDivisor(Wdet) + eps*Dx;
      D.GapNumbers := List(AlffWronskianOrders(DD), x->x+1);
   else
      D.RamDivisor := AlffDivisor(F);
      D.GapNumbers := [];
   fi;
   return D.RamDivisor;
end;



#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffWeierstrassPlaces
#
#SHORTDOC  Return the Weierstra\ss{} places of a divisor.
#
#CALLING   L := AlffWeierstrassPlaces( D );\\
#          L := AlffWeierstrassPlaces( F );
#
#ARGUS     L @ list @ containing the Weierstra\ss{} places
#          D @ alff divisor @ 
#          F @ alff @ equivalent to taking $D = 0$
#
#LONGDOC   Let $F/k$ be an algebraic function field,
#          $D$ a divisor and $P$ a place of degree one. An integer
#          $m \geq 1$ is a $D$-gap number of $P$ if $\dim \bigl( D + (m-1)P 
#          \bigr) = 
#          \dim(D + mP)$ holds. The $D$-gap numbers $m$ of 
#          $P$ satisfy $1 \leq m 
#          \leq 2g-1-\deg(D)$ and their cardinality equals the index of 
#          speciality $i(D)$. 
#          The sequences of 
#          $D$-gap numbers are independent of constant field extensions for
#          perfect $k$ and are the same for all but a
#          finite number of places $P$ of degree one
#          (consider e.g. $k$ algebraically closed). The places $P$
#          of degree one 
#          which have different sequences of $D$-gap numbers are called 
#          $D$-Weierstra\ss{} places. \par This function returns a list
#          of all places of $F/k$ (having not necessarily degree 
#          one) which are lying below $D$-Weierstra\ss{} 
#          places of 
#          $F \bar{k} / \bar{k}$ ($k$ perfect).
#          The constant field $k$ is required to be exact. Remark:
#          If the characteristic of $F$ is positive this function
#          is currently very slow for large genus 
#          because of {\tt AlffDifferentiation()}.
# 
#CSRC      
#
#XREF      AlffGapNumbers, AlffRamDivisor, AlffWronskian, AlffWronskianOrders,
#          AlffDiff, AlffDifferentiation
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(Q);;
#          AlffOrders(y^2 - (T-1)*(T-2)*(T-3)*(T-4)*(T-5)*(T-6)*(T-7));
#          AlffWeierstrassPlaces(F);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffWeierstrassPlaces := function(D)
   return AlffDivisorSupp(AlffRamDivisor(D));
end;


#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       AlffGapNumbers
#
#SHORTDOC  Return the gap numbers of a divisor.
#
#CALLING   L := AlffGapNumbers( D [, P] );\\
#          L := AlffGapNumbers( F [, P] );
#
#ARGUS     L @ list @ containing the gap numbers
#          D @ alff divisor @ 
#          F @ alff @ equivalent to taking $D = 0$
#
#LONGDOC   Let $F/k$ be an algebraic function field,
#          $D$ a divisor and $P$ a place of degree one. An integer
#          $m \geq 1$ is a $D$-gap number of $P$ if $\dim \bigl( D + (m-1)P 
#          \bigr) = 
#          \dim(D + mP)$ holds. The $D$-gap numbers $m$ satisfy $1 \leq m 
#          \leq 2g-1-\deg(D)$ and their cardinality equals the index of 
#          speciality $i(D)$. If $P$ of degree one 
#          is given as argument this function
#          returns the sequence of its $D$-gap numbers. The sequences of 
#          $D$-gap numbers are independent of constant field extensions for
#          perfect $k$ and are the same for all but a
#          finite number of places $P$ of degree one
#          (consider e.g. $k$ algebraically closed). If $P$ is omitted
#          in the function call, this uniform sequence 
#          is returned. The places $P$
#          which have different sequences of $D$-gap numbers are called 
#          $D$-Weierstra\ss{} places. 
#          The constant field $k$ is required to be exact.
# 
#CSRC      
#
#XREF      AlffWeierstrassPlaces, AlffWronskian, AlffWronskianOrders,
#          AlffDiff, AlffDifferentiation
#
#PACKAGE   Alff
#
#KASH      @
#          AlffInit(Q);;
#          AlffOrders(y^2 - (T-1)*(T-2)*(T-3)*(T-4)*(T-5)*(T-6)*(T-7));
#          P := AlffPlaceSplit(F, T-1)[1];
#          AlffGapNumbers(F);
#          AlffGapNumbers(F, P);
#         
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  9909xx  FH installed
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

AlffGapNumbers := function(arg)
#
#  Computes the gap numbers of a divisor or a alff (with or
#  without respect to place of degree one)
#
   local D, p, gaps, num, dim0, dim1, n;
   if Length(arg) = 1 then
      D := arg[1];
      AlffRamDivisor(D);
      return D.GapNumbers;
   elif Length(arg) = 2 then
      D := arg[1];
      p := arg[2];
      if AlffPlaceDeg(p) > 1 then
         return Error("place must be of degree one");
      fi;
      if IsAlff(D) then
         D := AlffDivisor(D);
      fi;
      gaps := [];
      num := AlffDivisorLIndex(D);
      dim0 := AlffDivisorLDim(D);
      n := 1;
      while Length(gaps) < num do
         dim1 := AlffDivisorLDim(D + n*p);
         if dim1 = dim0 then
            Add(gaps, n);
         fi;
         dim0 := dim1;
         n := n + 1;
      od;
      return gaps;
   else
      return Error("usage: AlffGapNumbers( <alff | alff divisor> [, <alff place> ] )");
   fi;
end;
