#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#FXN       IntEulerPhi
#
#SHORTDOC  Computes the  Euler-$\phi$ function, i.e. the number of coprime integers less than $n$,  of $n$.
#
#LONGDOC   
#
#CALLING   a := IntEulerPhi(n);
#
#ARGUS     a @ integer
#          n @ integer
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#EXAMPLE   
#
#KASH      @IntEulerPhi(3);
#          IntEulerPhi(81);
#          IntEulerPhi(134);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
IntEulerPhi := function(n)
  local L, phi, a;

  if not IsInt(n) or n < 1 then
    Error("usage: EulerPhi(<positive integer>)\n");
  fi;

  if n = 1 then return 1; fi;

  L:=Factor(n); 
  phi:=1;
  for a in L do
    phi:=phi*(a[1]-1)*a[1]^(a[2]-1);
  od;
  return phi;
end;


#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#FXN       OrderCyclotomic
#
#SHORTDOC  Computes the $n$'th cyclotomic field.
#
#LONGDOC   
#
#CALLING   o := OrderCyclotomic(n);
#
#ARGUS     o @ order
#          n @ integer 
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      @o := OrderCyclotomic(15);
#          OrderIsMaximal(o);
#          OrderDisc(o);
#          OrderAutomorphisms(o, []);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
OrderCyclotomic := function(n)
  local i, l, d, sig, j, nn, o, f, num, den, x, L, zeta;

  nn := n;
  if n mod 4 = 2 then n := n/2; fi;

  l := Factor(n);
  num := 1;
  den := 1;
  x := Poly(Zx, [1,0]);
  i := MultiCounterInit(Length(l));
  while MultiCounterInc(i, 1) do
    sig := 1;
    d := 1;
    for j in [1.. Length(i)] do
      if i[j]=0 then
        d := d*l[j][1]^(l[j][2] -1);
        sig := -sig;
      else
        d := d* l[j][1]^l[j][2];
      fi;
    od;
    if sig>0 then
       num := num * (x^d-1);
    else
       den := den * (x^d-1);
    fi;
  od;
  o := Order(num/den);
  OrderSetMaximal(o);
  HEAP(o,"CYCLO_FIELD",nn);

  zeta := OrderBasis(o)[2];
  L := [];
  for i in [1..nn-1] do
    if IntGcd(i,nn) = 1 then
      Add(L,zeta^i);
    fi;
  od;
  OrderAutomorphisms(o,L);

  return o;
end;

eps := 10.0^(-Prec()/2);
EltIsTorsionUnit := function(elt)
  local i, m, s;
  m := EltCon(elt);
  s := 0;
  for i in [1..MatCols(m)] do
    s := s + Abs(MatElt(m, 1, i));
  od;
  if s < OrderDeg(EltOrder(elt)) + eps then return true; else return false; fi;
end;

OrderCyclotomicUnits := function(o)
  local l, i, zeta, u, n, j, f, ub, noUnits;

  n := HEAP(o,"CYCLO_FIELD");

  zeta := OrderBasis(o)[2];
  noUnits := 0;

  if n mod 4 = 2 then
    if zeta^n = Elt(o, 1) then zeta := - zeta; fi;
  fi;

  for i in [1..n-1] do
    if Length(Factor(n/IntGcd(i, n))) >1 then
       u := 1 - zeta^i;
       if not EltIsTorsionUnit(u) then OrderUnitsMerge(o, u); fi;
    fi;
  od;

  Print("First case finished...\n");

  l := Factor(n);
  for f in l do
    ub := f[1] ^f[2];
    for i in [1..ub] do
      if i mod f[1] <> 0 then
        for j in [1..ub] do
          if i<>j and j mod f[1] <> 0 then
            u :=  (1 - zeta ^(i*n/ub)) / (1 - zeta ^(j*n/ub));
            if not EltIsTorsionUnit(u) then
              if OrderUnitsMerge(o, u) then
                noUnits:= noUnits+1;
                Print("+", noUnits, "\n");
              else
                Print("-", noUnits, "\n");
              fi;
            fi;
          fi;
        od;
      fi;
    od;
  od;
end;


#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#FXN       OrderCyclotomicRealSubfield
#
#SHORTDOC  Computes the maximal real subfield of the $n$'th cyclotimic field.
#
#LONGDOC   
#
#CALLING   o := OrderCyclotomicRealSubfield(n);
#
#ARGUS     o @ order
#          n @ integer
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      @o := OrderCyclotomicRealSubfield(11);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
OrderCyclotomicRealSubfield := function(n)
  local F,f,O,a;
  F := OrderCyclotomic(n);
  a := OrderBasis(F)[2];
  f := EltMinPoly(a+1/a);
  O := Order(f);
  OrderSetMaximal(O);
#  SubfieldAdd(F, O, a+1/a);
  return O;
end;





