
#############################################################################
##
#  GaloisSymb( <order> | <algebraic function field> | <poly> ); 
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       GaloisSymb
#SHORTDOC  Computation of the Galois group.
#CALLING   G := GaloisSymb(o|F|f);
#LONGDOC   This function computes the Galois group of an irreducible 
#          polynomial with coefficients in $\Q$ or $\Q(x)$ and simple 
#          relative orders with degree 
#          up to 7. This function uses factorization of absolute resolvents 
#          as described in the articles \cite{MK1} resp. \cite{MK2}, 
#          so the results are unconditional. The name of the Galois group 
#          is returned.
#ARGUS     G @ string
#          o @ order
#          F @ algebraic function field
#          f @ polynomial
#XREF      Galois
#PACKAGE   lib
#DEBUG     
#KASH      @
#          Zxy := PolyAlg(Zx);;
#          y := Poly(Zxy, [1,0]);;
#          f := y^6 - 9/(x^2 + 1)*y^2 + 12/(x^2 + 1)*y - 4/(x^2 + 1);
#          GaloisSymb(f);
#          @ @
#          E1 := OrderMaximal(Z, 2, 2);;
#          E1x:= PolyAlg(E1);;
#          F1:= Order(Poly(E1x,[Elt(E1,1),0,-4,0, Elt(E1,[2,-1])])); 
#          GaloisSymb(F1);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


#############################################################################
##
#F  GaloisSymbT( <order> | <algebraic function field> | <poly> ); 
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       GaloisSymbT
#SHORTDOC  Computation of the Galois group. 
#CALLING   G := GaloisSymbT(o|F|f);
#LONGDOC   This function computes the Galois group of an irreducible 
#          polynomial with coefficients in $\Q$ or $Q(x)$ and simple 
#          relative orders with degree 
#          up to 7. This function uses factorization of absolute resolvents 
#          as described in the articles \cite{MK1} resp. \cite{MK2}, so the 
#          results are unconditional. The number of the Galois group is 
#          returned.
#ARGUS     G @ int
#          o @ order
#          F @ algebraic function field
#          f @ polynomial
#XREF      Galois
#PACKAGE   lib
#DEBUG     
#KASH      @
#          Qx := PolyAlg(Q);
#          Qxy := PolyAlg(Qx);
#          y:= Poly(Qxy, [1,0]);;
#          f := y^7-3*y^6-y^5+3*y^4+(-x + 1)*y^3+(x + 1)*y^2-5*y+4;
#          F := Alff(f);
#          GaloisSymbT(F);
#         
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


PRINTLEVEL_GALOISSYMB := 0;

GalPrintSymb := function(arg)
   local a;
   if PRINTLEVEL_GALOISSYMB >= arg[1] then
      for a in arg{[2..Length(arg)]} do 
         Print(a);
      od;   
   fi;   
end;


PolyAlgMakeEuclidian := function(Rx)
   return PolyAlg(QuotientField(PolyAlgCoef(Rx)), QfName(Rx));
end;


FieldPoly := function(F)
   if IsOrder(F) then
      return OrderPoly(F);
   elif IsAlff(F) then
      return AlffPoly(F);
   fi;
end;


Field := function(f)
   local R, cr, QR;
   R := PolyAlg(f);
   if QfRank(R) = 1 then
      cr:= PolyAlgCoef(R);
      if IsOrder(cr) and IsOrder(OrderCoefOrder(cr)) then
         return Error("usage: only simple relative orders allowed");         
      fi;
      return Order(f);
   elif QfRank(R) = 2 then
      QR := PolyAlgMakeEuclidian(R);
      f := PolyMove(f, QR);
      f := PolyMoveIntegral(f);
      return Alff(f);
   fi;
   return false;
end;


FieldDeg := function(F)
   if IsOrder(F) then
      return OrderDeg(F);
   elif IsAlff(F) then
      return AlffDeg(F);
   fi;
   return false;
end;


IsField1 := function(F)
   return IsOrder(F) or IsAlff(F);
end;


special5 := function(f)
   
   local lf, re, e1, e2, e3, e4, e5, r0, r1, r2, r3, r4, r5, z, R, QR;
   
   lf:= PolyToList(f);
   e5:=lf[6];e4:=lf[5];e3:=lf[4];e2:=lf[3];e1:=lf[2];
   r5:=-2*e1*e3 + 8*e4;
   r4:=-8*e1^3*e5+(2*e2*e4+e3^2)*e1^2+(-14*e3*e4+30*e2*e5)*e1+
       40*e4^2-6*e2^2*e4+2*e2*e3^2-50*e3*e5;
   r3:= (-2*e4^2+16*e3*e5)*e1^4+(-44*e4*e5-2*e2^2*e5-2*e2*e3*e4)*e1^3+
        ((-66*e3*e5+21*e4^2)*e2-50*e5^2+6*e3^2*e4)*e1^2+
        (9*e2^3*e5+5*e2^2*e3*e4+(-2*e3^3+190*e4*e5)*e2-80*e3*e4^2+
         120*e3^2*e5)*e1+(-40*e4^2-15*e3*e5)*e2^2+(21*e3^2*e4+125*e5^2)*e2+
         160*e4^3-2*e3^4-400*e3*e4*e5;
   r2:= 16*e1^6*e5^2+(-8*e3^2*e5+2*e3*e4^2-8*e2*e4*e5)*e1^5+
        ((2*e3*e5+e4^2)*e2^2-120*e2*e5^2+68*e3*e4*e5-8*e4^3)*e1^4+
        (46*e2^2*e4*e5+(-19*e3*e4^2+28*e3^2*e5)*e2-144*e4^2*e5+
         250*e3*e5^2)*e1^3+((-9*e3*e5-6*e4^2)*e2^3+(225*e5^2+3*e3^2*e4)*e2^2
         +(-354*e3*e4*e5+76*e4^3)*e2-200*e4*e5^2+41*e3^2*e4^2-70*e3^3*e5)*e1^2+
        (-54*e2^3*e4*e5+(30*e3*e4^2+45*e3^2*e5)*e2^2+
         (-19*e3^3*e4+640*e4^2*e5-875*e3*e5^2)*e2-264*e3*e4^3+2*e3^5+
         630*e3^2*e4*e5)*e1+9*e2^4*e4^2-6*e2^3*e3^2*e4+(90*e3*e4*e5+e3^4-
         136*e4^3)*e2^2+(500*e4*e5^2+76*e3^2*e4^2-50*e3^3*e5)*e2+400*e4^4-
        1400*e3*e4^2*e5+625*e3^2*e5^2-8*e3^4*e4;
   r1:=(-32*e3*e5^2+8*e4^2*e5)*e1^7+(8*e2^2*e5^2+(-2*e4^3+8*e3*e4*e5)*e2+
        48*e4*e5^2)*e1^6+(-2*e2^3*e4*e5+(264*e3*e5^2-94*e4^2*e5)*e2-
        24*e3^2*e4*e5-56*e5^3+6*e3*e4^3)*e1^5+(-66*e2^3*e5^2+
        (-50*e3*e4*e5+19*e4^3)*e2^2+(-2*e3^2*e4^2+8*e3^3*e5-318*e4*e5^2)*e2+
        3*e4^4-352*e3^2*e5^2+166*e3*e4^2*e5)*e1^4+(15*e2^4*e4*e5+(-2*e3^2*e5-
        e3*e4^2)*e2^3+(-574*e3*e5^2+347*e4^2*e5)*e2^2+(350*e5^3+
        194*e3^2*e4*e5-89*e3*e4^3)*e2+1090*e3*e4*e5^2-8*e3^4*e5+4*e3^3*e4^2-
        364*e4^3*e5)*e1^3+(162*e2^4*e5^2+(-51*e4^3+33*e3*e4*e5)*e2^3+
        (305*e4*e5^2+28*e3^2*e4^2-32*e3^3*e5)*e2^2+(-901*e3*e4^2*e5+76*e4^4+
         1340*e3^2*e5^2-2*e3^4*e4)*e2-234*e3^3*e4*e5-550*e4^2*e5^2-750*e3*e5^3+
                102*e3^2*e4^3)*e1^2+(-27*e2^5*e4*e5+(9*e3^2*e5+3*e3*e4^2)*e2^4+(-366*e4^2*e5+180*e3*e5^2-e3^3*e4)*e2^3+(212*e3*e4^3-375*e5^3-231*e3^2*e4*e5)*e2^2+(-89*e3^3*e4^2+112*e3^4*e5-3075*e3*e4*e5^2+1640*e4^3*e5)*e2-850*e3^3*e5^2-384*e3*e4^4+1220*e3^2*e4^2*e5+2500*e4*e5^3+6*e3^5*e4)*e1-108*e2^5*e5^2+(32*e4^3+117*e3*e4*e5)*e2^4+(-31*e3^3*e5+525*e4*e5^2-51*e3^2*e4^2)*e2^3+(-256*e4^4+19*e3^4*e4+260*e3*e4^2*e5-325*e3^2*e5^2)*e2^2+(105*e3^3*e4*e5-500*e4^2*e5^2-2*e3^6+76*e3^2*e4^3+625*e3*e5^3)*e2-3125*e5^4-58*e3^5*e5+2750*e3^2*e4*e5^2-2400*e3*e4^3*e5+3*e3^4*e4^2+512*e4^5;
   
   
   r0:=(16*e3^2*e5^2+e4^4-8*e3*e4^2*e5)*e1^8+((2*e4^2*e5-8*e3*e5^2)*e2^2+12*e4^3*e5-48*e3*e4*e5^2)*e1^7+(e2^4*e5^2+12*e2^2*e4*e5^2+(-144*e3^2*e5^2+88*e3*e4^2*e5-13*e4^4)*e2+56*e3*e5^3+86*e4^2*e5^2)*e1^6+((72*e3*e5^2-22*e4^2*e5)*e2^3+(-14*e5^3-4*e3^2*e4*e5+e3*e4^3)*e2^2+(304*e3*e4*e5^2-148*e4^3*e5)*e2-54*e3^2*e4^2*e5+152*e3^3*e5^2+5*e3*e4^4-468*e4*e5^3)*e1^5+(-9*e2^5*e5^2+e2^4*e3*e4*e5-76*e2^3*e4*e5^2+(370*e3^2*e5^2+65*e4^4-287*e3*e4^2*e5)*e2^2+(-28*e3^3*e4*e5-294*e4^2*e5^2+5*e3^2*e4^3-200*e3*e5^3)*e2+8*e3^5*e5+17*e4^5+180*e3*e4^3*e5+625*e5^4-676*e3^2*e4*e5^2-2*e3^4*e4^2)*e1^4+((-210*e3*e5^2+76*e4^2*e5)*e2^4+(50*e5^3+43*e3^2*e4*e5-15*e3*e4^3)*e2^3+(514*e4^3*e5-6*e3^4*e5+2*e3^3*e4^2-397*e3*e4*e5^2)*e2^2+(2300*e4*e5^3+447*e3^2*e4^2*e5-118*e3*e4^4-700*e3^3*e5^2)*e2+1470*e3*e4^2*e5^2-12*e3^4*e4*e5+6*e3^3*e4^3+250*e3^2*e5^3-276*e4^4*e5)*e1^3+(27*e2^6*e5^2+(e4^3-9*e3*e4*e5)*e2^5+(141*e4*e5^2+e3^3*e5)*e2^4+(-128*e4^4-185*e3^2*e5^2+168*e3*e4^2*e5)*e2^3+(93*e3^3*e4*e5+19*e3^2*e4^3-125*e3*e5^3-610*e4^2*e5^2)*e2^2+(-36*e3^5*e5-1174*e3*e4^3*e5+5*e3^4*e4^2-16*e4^5+1995*e3^2*e4*e5^2-3125*e5^4)*e2-1450*e4^3*e5^2-172*e3^3*e4^2*e5-3500*e3*e4*e5^3+82*e3^2*e4^4+375*e3^4*e5^2)*e1^2+((-78*e4^2*e5+198*e3*e5^2)*e2^5+(44*e3*e4^3-95*e3^2*e4*e5)*e2^4+(-15*e3^3*e4^2-384*e4^3*e5+15*e3*e4*e5^2+25*e3^4*e5)*e2^3+(e3^5*e4+525*e3^3*e5^2-528*e3^2*e4^2*e5-1750*e4*e5^3+384*e3*e4^4)*e2^2+(-850*e3*e4^2*e5^2-29*e3^4*e4*e5+1760*e4^4*e5+625*e3^2*e5^3-118*e3^3*e4^3)*e2+38*e3^6*e5+5*e3^5*e4^2-2050*e3^3*e4*e5^2+780*e3^2*e4^3*e5+(-192*e4^5+3125*e5^4)*e3+7500*e4^2*e5^3)*e1-27*e2^7*e5^2+(-4*e4^3+18*e3*e4*e5)*e2^6+(-99*e4*e5^2+e3^2*e4^2-4*e3^3*e5)*e2^5+(48*e4^4-150*e3^2*e5^2+196*e3*e4^2*e5)*e2^4+(1200*e4^2*e5^2+12*e3^3*e4*e5-128*e3^2*e4^3)*e2^3+(-160*e3*e4^3*e5+65*e3^4*e4^2-725*e3^2*e4*e5^2+3125*e5^4-12*e3^5*e5-192*e4^5)*e2^2+(-2000*e4^3*e5^2+590*e3^3*e4^2*e5-1250*e3*e4*e5^3-16*e3^2*e4^4-13*e3^6*e4-125*e3^4*e5^2)*e2-1600*e3*e4^4*e5+17*e3^4*e4^3-9375*e4*e5^4+3250*e3^2*e4^2*e5^2+256*e4^6-124*e3^5*e4*e5+e3^8;
   z := Poly(PolyAlg(f), [1,0]);
   re :=z^6+r5*z^5+r4*z^4+r3*z^3+r2*z^2+r1*z+r0;
   R := PolyAlg(re);
   if QfRank(R) > 1 then
      QR := PolyAlgMakeEuclidian(R);
      re := PolyMove(re, QR);
   fi;
   return re;
end;


special6_deg6:= function(f)
   
   local R, QR, lf, re, a1, a2, a3, a4, a5, a6, r0, r1, r2, r3, r4, r5, r6, z;
   lf:= PolyToList(f);
   a6:=lf[7];a5:=lf[6];a4:=lf[5];a3:=lf[4];a2:=lf[3];a1:=lf[2];
   z:= Poly(PolyAlg(f), [1,0]);
   
   re:= z^6 + (8*a1*a5 - 2*a2*a4 - 18*a6)*z^5 + 
        (-50*a1^2*a4*a6 + 40*a1^2*a5^2 + 
         30*a1*a2*a3*a6 - 14*a1*a2*a4*a5 - 6*a1*a3^2*a5 + 2*a1*a3*a4^2 - 
         30*a1*a5*a6 
    - 8*a2^3*a6 + 2*a2^2*a3*a5 + a2^2*a4^2 + 114*a2*a4*a6 - 50*a2*a5^2 - 
    54*a3^2*a6 + 30*a3*a4*a5 - 8*a4^3 - 135*a6^2)*z^4 + (125*a1^3*a3*a6^2 - 
                400*a1^3*a4*a5*a6 + 160*a1^3*a5^3 - 50*a1^2*a2^2*a6^2 + 
                190*a1^2*a2*a3*a5*a6
    + 120*a1^2*a2*a4^2*a6 - 80*a1^2*a2*a4*a5^2 - 15*a1^2*a3^2*a4*a6 - 
    40*a1^2*a3^2*a5^2 + 21*a1^2*a3*a4^2*a5 - 2*a1^2*a4^4 + 900*a1^2*a4*a6^2 - 
    80*a1^2*a5^2*a6 - 44*a1*a2^3*a5*a6 - 66*a1*a2^2*a3*a4*a6 + 
    21*a1*a2^2*a3*a5^2 + 6*a1*a2^2*a4^2*a5 + 9*a1*a2*a3^3*a6 + 
    5*a1*a2*a3^2*a4*a5 - 2*a1*a2*a3*a4^3 - 990*a1*a2*a3*a6^2 + 
                920*a1*a2*a4*a5*a6 - 400*a1*a2*a5^3 - 135*a1*a3^2*a5*a6 - 
                126*a1*a3*a4^2*a6 
    + 190*a1*a3*a4*a5^2 - 44*a1*a4^3*a5 - 2070*a1*a5*a6^2 + 16*a2^4*a4*a6 - 
    2*a2^4*a5^2 - 2*a2^3*a3^2*a6 - 2*a2^3*a3*a4*a5 + 304*a2^3*a6^2 - 
                126*a2^2*a3*a5*a6 - 232*a2^2*a4^2*a6 + 120*a2^2*a4*a5^2 + 
                198*a2*a3^2*a4*a6 
    - 15*a2*a3^2*a5^2 - 66*a2*a3*a4^2*a5 + 16*a2*a4^4 - 1440*a2*a4*a6^2 + 
                900*a2*a5^2*a6 - 27*a3^4*a6 + 9*a3^3*a4*a5 - 2*a3^2*a4^3 + 
                1350*a3^2*a6^2 - 
                990*a3*a4*a5*a6 + 125*a3*a5^3 + 304*a4^3*a6 - 50*a4^2*a5^2 + 
                3240*a6^3)*z^3 
    + (500*a1^4*a3*a5*a6^2 + 625*a1^4*a4^2*a6^2 - 1400*a1^4*a4*a5^2*a6 + 
    400*a1^4*a5^4 - 200*a1^3*a2^2*a5*a6^2 - 875*a1^3*a2*a3*a4*a6^2 + 
    640*a1^3*a2*a3*a5^2*a6 + 630*a1^3*a2*a4^2*a5*a6 - 264*a1^3*a2*a4*a5^3 + 
    90*a1^3*a3^2*a4*a5*a6 - 136*a1^3*a3^2*a5^3 - 50*a1^3*a3*a4^3*a6 + 
    76*a1^3*a3*a4^2*a5^2 - 1125*a1^3*a3*a6^3 - 8*a1^3*a4^4*a5 + 
    2550*a1^3*a4*a5*a6^2 - 200*a1^3*a5^3*a6 + 250*a1^2*a2^3*a4*a6^2 - 
        144*a1^2*a2^3*a5^2*a6 + 225*a1^2*a2^2*a3^2*a6^2 - 
        354*a1^2*a2^2*a3*a4*a5*a6 
    + 76*a1^2*a2^2*a3*a5^3 - 70*a1^2*a2^2*a4^3*a6 + 41*a1^2*a2^2*a4^2*a5^2 + 
    450*a1^2*a2^2*a6^3 - 54*a1^2*a2*a3^3*a5*a6 + 45*a1^2*a2*a3^2*a4^2*a6 + 
        30*a1^2*a2*a3^2*a4*a5^2 - 19*a1^2*a2*a3*a4^3*a5 - 
        2880*a1^2*a2*a3*a5*a6^2 + 
    2*a1^2*a2*a4^5 - 3480*a1^2*a2*a4^2*a6^2 + 4692*a1^2*a2*a4*a5^2*a6 - 
        1400*a1^2*a2*a5^4 + 9*a1^2*a3^4*a5^2 - 6*a1^2*a3^3*a4^2*a5 + 
        a1^2*a3^2*a4^4 
        + 1485*a1^2*a3^2*a4*a6^2 - 522*a1^2*a3^2*a5^2*a6 - 
        1257*a1^2*a3*a4^2*a5*a6 +
    640*a1^2*a3*a4*a5^3 + 218*a1^2*a4^4*a6 - 144*a1^2*a4^3*a5^2 + 
    1350*a1^2*a4*a6^3 - 5175*a1^2*a5^2*a6^2 - 120*a1*a2^4*a3*a6^2 + 
    68*a1*a2^4*a4*a5*a6 - 8*a1*a2^4*a5^3 + 46*a1*a2^3*a3^2*a5*a6 + 
    28*a1*a2^3*a3*a4^2*a6 - 19*a1*a2^3*a3*a4*a5^2 + 868*a1*a2^3*a5*a6^2 - 
    9*a1*a2^2*a3^3*a4*a6 - 6*a1*a2^2*a3^3*a5^2 + 3*a1*a2^2*a3^2*a4^2*a5 + 
        2484*a1*a2^2*a3*a4*a6^2 - 1257*a1*a2^2*a3*a5^2*a6 - 
        1356*a1*a2^2*a4^2*a5*a6 
    + 630*a1*a2^2*a4*a5^3 - 891*a1*a2*a3^3*a6^2 + 882*a1*a2*a3^2*a4*a5*a6 + 
    90*a1*a2*a3^2*a5^3 + 84*a1*a2*a3*a4^3*a6 - 354*a1*a2*a3*a4^2*a5^2 + 
    3240*a1*a2*a3*a6^3 + 68*a1*a2*a4^4*a5 - 4392*a1*a2*a4*a5*a6^2 + 
    2550*a1*a2*a5^3*a6 + 54*a1*a3^4*a5*a6 - 54*a1*a3^3*a4^2*a6 - 
    54*a1*a3^3*a4*a5^2 + 46*a1*a3^2*a4^3*a5 + 2727*a1*a3^2*a5*a6^2 - 
        8*a1*a3*a4^5 + 756*a1*a3*a4^2*a6^2 - 2880*a1*a3*a4*a5^2*a6 + 
        500*a1*a3*a5^4 
    + 868*a1*a4^3*a5*a6 - 200*a1*a4^2*a5^3 + 8100*a1*a5*a6^3 + 16*a2^6*a6^2 - 
    8*a2^5*a3*a5*a6 - 8*a2^5*a4^2*a6 + 2*a2^5*a4*a5^2 + 2*a2^4*a3^2*a4*a6 + 
        a2^4*a3^2*a5^2 - 688*a2^4*a4*a6^2 + 218*a2^4*a5^2*a6 + 
        234*a2^3*a3^2*a6^2 + 
        84*a2^3*a3*a4*a5*a6 - 50*a2^3*a3*a5^3 + 168*a2^3*a4^3*a6 - 
        70*a2^3*a4^2*a5^2
    - 1224*a2^3*a6^3 - 54*a2^2*a3^3*a5*a6 - 144*a2^2*a3^2*a4^2*a6 + 
    45*a2^2*a3^2*a4*a5^2 + 28*a2^2*a3*a4^3*a5 + 756*a2^2*a3*a5*a6^2 - 
    8*a2^2*a4^5 + 4320*a2^2*a4^2*a6^2 - 3480*a2^2*a4*a5^2*a6 + 625*a2^2*a5^4 + 
        27*a2*a3^4*a4*a6 - 9*a2*a3^3*a4^2*a5 + 2*a2*a3^2*a4^4 - 
        4752*a2*a3^2*a4*a6^2
    + 1485*a2*a3^2*a5^2*a6 + 2484*a2*a3*a4^2*a5*a6 - 875*a2*a3*a4*a5^3 - 
    688*a2*a4^4*a6 + 250*a2*a4^3*a5^2 - 4536*a2*a4*a6^3 + 1350*a2*a5^2*a6^2 + 
        972*a3^4*a6^2 - 891*a3^3*a4*a5*a6 + 234*a3^2*a4^3*a6 + 
        225*a3^2*a4^2*a5^2 - 
    1944*a3^2*a6^3 - 120*a3*a4^4*a5 + 3240*a3*a4*a5*a6^2 - 1125*a3*a5^3*a6 + 
    16*a4^6 - 1224*a4^3*a6^2 + 450*a4^2*a5^2*a6)*z^2 + (-3125*a1^6*a6^4 + 
    2500*a1^5*a2*a5*a6^3 + 625*a1^5*a3*a4*a6^3 - 500*a1^5*a3*a5^2*a6^2 + 
    2750*a1^5*a4^2*a5*a6^2 - 2400*a1^5*a4*a5^3*a6 + 512*a1^5*a5^5 - 
    750*a1^4*a2^2*a4*a6^3 - 550*a1^4*a2^2*a5^2*a6^2 - 375*a1^4*a2*a3^2*a6^3 - 
                3075*a1^4*a2*a3*a4*a5*a6^2 + 1640*a1^4*a2*a3*a5^3*a6 - 
                850*a1^4*a2*a4^3*a6^2
    + 1220*a1^4*a2*a4^2*a5^2*a6 - 384*a1^4*a2*a4*a5^4 + 22500*a1^4*a2*a6^4 + 
                525*a1^4*a3^3*a5*a6^2 - 325*a1^4*a3^2*a4^2*a6^2 + 
                260*a1^4*a3^2*a4*a5^2*a6 -
    256*a1^4*a3^2*a5^4 + 105*a1^4*a3*a4^3*a5*a6 + 76*a1^4*a3*a4^2*a5^3 + 
    375*a1^4*a3*a5*a6^3 - 58*a1^4*a4^5*a6 + 3*a1^4*a4^4*a5^2 - 
    12750*a1^4*a4^2*a6^3 + 3700*a1^4*a4*a5^2*a6^2 + 640*a1^4*a5^4*a6 + 
                350*a1^3*a2^3*a3*a6^3 + 1090*a1^3*a2^3*a4*a5*a6^2 - 
                364*a1^3*a2^3*a5^3*a6 + 
    305*a1^3*a2^2*a3^2*a5*a6^2 + 1340*a1^3*a2^2*a3*a4^2*a6^2 - 
    901*a1^3*a2^2*a3*a4*a5^2*a6 + 76*a1^3*a2^2*a3*a5^4 - 
                234*a1^3*a2^2*a4^3*a5*a6 + 102*a1^3*a2^2*a4^2*a5^3 - 
                16650*a1^3*a2^2*a5*a6^3
    + 180*a1^3*a2*a3^3*a4*a6^2 - 366*a1^3*a2*a3^3*a5^2*a6 - 
    231*a1^3*a2*a3^2*a4^2*a5*a6 + 212*a1^3*a2*a3^2*a4*a5^3 + 
                112*a1^3*a2*a3*a4^4*a6 - 89*a1^3*a2*a3*a4^3*a5^2 + 
                10950*a1^3*a2*a3*a4*a6^3 
                + 1555*a1^3*a2*a3*a5^2*a6^2 + 6*a1^3*a2*a4^5*a5 - 
                9540*a1^3*a2*a4^2*a5*a6^2 
    + 9016*a1^3*a2*a4*a5^3*a6 - 2400*a1^3*a2*a5^5 - 108*a1^3*a3^5*a6^2 + 
    117*a1^3*a3^4*a4*a5*a6 + 32*a1^3*a3^4*a5^3 - 31*a1^3*a3^3*a4^3*a6 - 
    51*a1^3*a3^3*a4^2*a5^2 - 2025*a1^3*a3^3*a6^3 + 19*a1^3*a3^2*a4^4*a5 + 
    2955*a1^3*a3^2*a4*a5*a6^2 - 1436*a1^3*a3^2*a5^3*a6 - 2*a1^3*a3*a4^6 + 
                2770*a1^3*a3*a4^3*a6^2 - 5123*a1^3*a3*a4^2*a5^2*a6 + 
                1640*a1^3*a3*a4*a5^4 - 
    40500*a1^3*a3*a6^4 + 914*a1^3*a4^4*a5*a6 - 364*a1^3*a4^3*a5^3 + 
    53550*a1^3*a4*a5*a6^3 - 17930*a1^3*a5^3*a6^2 - 56*a1^2*a2^5*a6^3 - 
    318*a1^2*a2^4*a3*a5*a6^2 - 352*a1^2*a2^4*a4^2*a6^2 + 
    166*a1^2*a2^4*a4*a5^2*a6 + 3*a1^2*a2^4*a5^4 - 574*a1^2*a2^3*a3^2*a4*a6^2 + 
    347*a1^2*a2^3*a3^2*a5^2*a6 + 194*a1^2*a2^3*a3*a4^2*a5*a6 - 
    89*a1^2*a2^3*a3*a4*a5^3 - 8*a1^2*a2^3*a4^4*a6 + 4*a1^2*a2^3*a4^3*a5^2 + 
                560*a1^2*a2^3*a4*a6^3 + 3662*a1^2*a2^3*a5^2*a6^2 + 
                162*a1^2*a2^2*a3^4*a6^2 +
    33*a1^2*a2^2*a3^3*a4*a5*a6 - 51*a1^2*a2^2*a3^3*a5^3 - 
    32*a1^2*a2^2*a3^2*a4^3*a6 + 28*a1^2*a2^2*a3^2*a4^2*a5^2 + 
    270*a1^2*a2^2*a3^2*a6^3 - 2*a1^2*a2^2*a3*a4^4*a5 + 
    4872*a1^2*a2^2*a3*a4*a5*a6^2 - 5123*a1^2*a2^2*a3*a5^3*a6 + 
    2144*a1^2*a2^2*a4^3*a6^2 - 2812*a1^2*a2^2*a4^2*a5^2*a6 + 
    1220*a1^2*a2^2*a4*a5^4 - 37800*a1^2*a2^2*a6^4 - 27*a1^2*a2*a3^5*a5*a6 + 
    9*a1^2*a2*a3^4*a4^2*a6 + 3*a1^2*a2*a3^4*a4*a5^2 - a1^2*a2*a3^3*a4^3*a5 - 
    3078*a1^2*a2*a3^3*a5*a6^2 - 4014*a1^2*a2*a3^2*a4^2*a6^2 + 
    5412*a1^2*a2*a3^2*a4*a5^2*a6 + 260*a1^2*a2*a3^2*a5^4 - 
    310*a1^2*a2*a3*a4^3*a5*a6 - 901*a1^2*a2*a3*a4^2*a5^3 - 
                3780*a1^2*a2*a3*a5*a6^3 + 166*a1^2*a2*a4^4*a5^2 + 
                40320*a1^2*a2*a4^2*a6^3 - 
                25344*a1^2*a2*a4*a5^2*a6^2 + 3700*a1^2*a2*a5^4*a6 + 
                918*a1^2*a3^4*a4*a6^2 + 
    27*a1^2*a3^4*a5^2*a6 - 342*a1^2*a3^3*a4^2*a5*a6 - 366*a1^2*a3^3*a4*a5^3 + 
    32*a1^2*a3^2*a4^4*a6 + 347*a1^2*a3^2*a4^3*a5^2 - 4590*a1^2*a3^2*a4*a6^3 + 
    594*a1^2*a3^2*a5^2*a6^2 - 94*a1^2*a3*a4^5*a5 + 3618*a1^2*a3*a4^2*a5*a6^2 + 
    1555*a1^2*a3*a4*a5^3*a6 - 500*a1^2*a3*a5^5 + 8*a1^2*a4^7 - 
    7192*a1^2*a4^4*a6^2 + 3662*a1^2*a4^3*a5^2*a6 - 550*a1^2*a4^2*a5^4 - 
    48600*a1^2*a4*a6^4 + 1080*a1^2*a5^2*a6^3 + 48*a1*a2^6*a5*a6^2 + 
    264*a1*a2^5*a3*a4*a6^2 - 94*a1*a2^5*a3*a5^2*a6 - 24*a1*a2^5*a4^2*a5*a6 + 
    6*a1*a2^5*a4*a5^3 - 66*a1*a2^4*a3^3*a6^2 - 50*a1*a2^4*a3^2*a4*a5*a6 + 
    19*a1*a2^4*a3^2*a5^3 + 8*a1*a2^4*a3*a4^3*a6 - 2*a1*a2^4*a3*a4^2*a5^2 - 
    552*a1*a2^4*a3*a6^3 - 2560*a1*a2^4*a4*a5*a6^2 + 914*a1*a2^4*a5^3*a6 + 
    15*a1*a2^3*a3^4*a5*a6 - 2*a1*a2^3*a3^3*a4^2*a6 - a1*a2^3*a3^3*a4*a5^2 + 
    1602*a1*a2^3*a3^2*a5*a6^2 - 608*a1*a2^3*a3*a4^2*a6^2 - 
    310*a1*a2^3*a3*a4*a5^2*a6 + 105*a1*a2^3*a3*a5^4 + 600*a1*a2^3*a4^3*a5*a6 - 
    234*a1*a2^3*a4^2*a5^3 + 31368*a1*a2^3*a5*a6^3 + 756*a1*a2^2*a3^3*a4*a6^2 - 
    342*a1*a2^2*a3^3*a5^2*a6 + 216*a1*a2^2*a3^2*a4^2*a5*a6 - 
                231*a1*a2^2*a3^2*a4*a5^3 - 192*a1*a2^2*a3*a4^4*a6 + 
                194*a1*a2^2*a3*a4^3*a5^2
                - 39096*a1*a2^2*a3*a4*a6^3 + 3618*a1*a2^2*a3*a5^2*a6^2 - 
                24*a1*a2^2*a4^5*a5 
                + 9408*a1*a2^2*a4^2*a5*a6^2 - 9540*a1*a2^2*a4*a5^3*a6 + 
                2750*a1*a2^2*a5^5 - 
    162*a1*a2*a3^5*a6^2 - 378*a1*a2*a3^4*a4*a5*a6 + 117*a1*a2*a3^4*a5^3 + 
    150*a1*a2*a3^3*a4^3*a6 + 33*a1*a2*a3^3*a4^2*a5^2 + 10044*a1*a2*a3^3*a6^3 - 
                50*a1*a2*a3^2*a4^4*a5 - 8640*a1*a2*a3^2*a4*a5*a6^2 + 
                2955*a1*a2*a3^2*a5^3*a6
    + 8*a1*a2*a3*a4^6 + 6144*a1*a2*a3*a4^3*a6^2 + 4872*a1*a2*a3*a4^2*a5^2*a6 - 
    3075*a1*a2*a3*a4*a5^4 + 174960*a1*a2*a3*a6^4 - 2560*a1*a2*a4^4*a5*a6 + 
    1090*a1*a2*a4^3*a5^3 - 148824*a1*a2*a4*a5*a6^3 + 53550*a1*a2*a5^3*a6^2 + 
    81*a1*a3^6*a5*a6 - 27*a1*a3^5*a4^2*a6 - 27*a1*a3^5*a4*a5^2 + 
    15*a1*a3^4*a4^3*a5 + 2430*a1*a3^4*a5*a6^2 - 2*a1*a3^3*a4^5 - 
    2052*a1*a3^3*a4^2*a6^2 - 3078*a1*a3^3*a4*a5^2*a6 + 525*a1*a3^3*a5^4 + 
    1602*a1*a3^2*a4^3*a5*a6 + 305*a1*a3^2*a4^2*a5^3 + 18144*a1*a3^2*a5*a6^3 - 
    104*a1*a3*a4^5*a6 - 318*a1*a3*a4^4*a5^2 - 33696*a1*a3*a4^2*a6^3 - 
    3780*a1*a3*a4*a5^2*a6^2 + 375*a1*a3*a5^4*a6 + 48*a1*a4^6*a5 + 
    31368*a1*a4^3*a5*a6^2 - 16650*a1*a4^2*a5^3*a6 + 2500*a1*a4*a5^5 + 
    77760*a1*a5*a6^4 - 32*a2^7*a4*a6^2 + 8*a2^7*a5^2*a6 + 8*a2^6*a3^2*a6^2 + 
    8*a2^6*a3*a4*a5*a6 - 2*a2^6*a3*a5^3 + 96*a2^6*a6^3 - 2*a2^5*a3^3*a5*a6 - 
    104*a2^5*a3*a5*a6^2 + 416*a2^5*a4^2*a6^2 - 58*a2^5*a5^4 - 
    312*a2^4*a3^2*a4*a6^2 + 32*a2^4*a3^2*a5^2*a6 - 192*a2^4*a3*a4^2*a5*a6 + 
    112*a2^4*a3*a4*a5^3 - 8*a2^4*a4^3*a5^2 + 4224*a2^4*a4*a6^3 - 
    7192*a2^4*a5^2*a6^2 + 54*a2^3*a3^4*a6^2 + 150*a2^3*a3^3*a4*a5*a6 - 
    31*a2^3*a3^3*a5^3 - 32*a2^3*a3^2*a4^2*a5^2 - 864*a2^3*a3^2*a6^3 + 
    8*a2^3*a3*a4^4*a5 + 6144*a2^3*a3*a4*a5*a6^2 + 2770*a2^3*a3*a5^3*a6 - 
    4032*a2^3*a4^3*a6^2 + 2144*a2^3*a4^2*a5^2*a6 - 850*a2^3*a4*a5^4 - 
    16416*a2^3*a6^4 - 27*a2^2*a3^5*a5*a6 + 9*a2^2*a3^4*a4*a5^2 - 
    2*a2^2*a3^3*a4^3*a5 - 2052*a2^2*a3^3*a5*a6^2 + 2376*a2^2*a3^2*a4^2*a6^2 - 
    4014*a2^2*a3^2*a4*a5^2*a6 - 325*a2^2*a3^2*a5^4 - 608*a2^2*a3*a4^3*a5*a6 + 
    1340*a2^2*a3*a4^2*a5^3 - 33696*a2^2*a3*a5*a6^3 + 416*a2^2*a4^5*a6 - 
    352*a2^2*a4^4*a5^2 - 6048*a2^2*a4^2*a6^3 + 40320*a2^2*a4*a5^2*a6^2 - 
    12750*a2^2*a5^4*a6 - 324*a2*a3^4*a4*a6^2 + 918*a2*a3^4*a5^2*a6 + 
    756*a2*a3^3*a4^2*a5*a6 + 180*a2*a3^3*a4*a5^3 - 312*a2*a3^2*a4^4*a6 - 
    574*a2*a3^2*a4^3*a5^2 + 43416*a2*a3^2*a4*a6^3 - 4590*a2*a3^2*a5^2*a6^2 + 
    264*a2*a3*a4^5*a5 - 39096*a2*a3*a4^2*a5*a6^2 + 10950*a2*a3*a4*a5^3*a6 + 
    625*a2*a3*a5^5 - 32*a2*a4^7 + 4224*a2*a4^4*a6^2 + 560*a2*a4^3*a5^2*a6 - 
                750*a2*a4^2*a5^4 + 85536*a2*a4*a6^4 - 48600*a2*a5^2*a6^3 - 
                162*a3^5*a4*a5*a6
    - 108*a3^5*a5^3 + 54*a3^4*a4^3*a6 + 162*a3^4*a4^2*a5^2 - 11664*a3^4*a6^3 - 
                66*a3^3*a4^4*a5 + 10044*a3^3*a4*a5*a6^2 - 2025*a3^3*a5^3*a6 + 
                8*a3^2*a4^6 - 
    864*a3^2*a4^3*a6^2 + 270*a3^2*a4^2*a5^2*a6 - 375*a3^2*a4*a5^4 - 
    163296*a3^2*a6^4 - 552*a3*a4^4*a5*a6 + 350*a3*a4^3*a5^3 + 
    174960*a3*a4*a5*a6^3 - 40500*a3*a5^3*a6^2 + 96*a4^6*a6 - 56*a4^5*a5^2 - 
    16416*a4^3*a6^3 - 37800*a4^2*a5^2*a6^2 + 22500*a4*a5^4*a6 - 3125*a5^6 - 
    93312*a6^5)*z + -9375*a1^7*a5*a6^4 + 3125*a1^6*a2*a4*a6^4 + 
    7500*a1^6*a2*a5^2*a6^3 + 3125*a1^6*a3^2*a6^4 - 1250*a1^6*a3*a4*a5*a6^3 - 
    2000*a1^6*a3*a5^3*a6^2 + 3250*a1^6*a4^2*a5^2*a6^2 - 1600*a1^6*a4*a5^4*a6 + 
    256*a1^6*a5^6 + 40625*a1^6*a6^5 - 3125*a1^5*a2^2*a3*a6^4 - 
    3500*a1^5*a2^2*a4*a5*a6^3 - 1450*a1^5*a2^2*a5^3*a6^2 - 
    1750*a1^5*a2*a3^2*a5*a6^3 + 625*a1^5*a2*a3*a4^2*a6^3 - 
    850*a1^5*a2*a3*a4*a5^2*a6^2 + 1760*a1^5*a2*a3*a5^4*a6 - 
        2050*a1^5*a2*a4^3*a5*a6^2 + 780*a1^5*a2*a4^2*a5^3*a6 - 
        192*a1^5*a2*a4*a5^5 +
    35000*a1^5*a2*a5*a6^4 + 1200*a1^5*a3^3*a5^2*a6^2 - 
        725*a1^5*a3^2*a4^2*a5*a6^2 - 160*a1^5*a3^2*a4*a5^3*a6 - 
        192*a1^5*a3^2*a5^5 -
    125*a1^5*a3*a4^4*a6^2 + 590*a1^5*a3*a4^3*a5^2*a6 - 16*a1^5*a3*a4^2*a5^4 - 
    20625*a1^5*a3*a4*a6^4 + 17250*a1^5*a3*a5^2*a6^3 - 124*a1^5*a4^5*a5*a6 + 
    17*a1^5*a4^4*a5^3 - 20250*a1^5*a4^2*a5*a6^3 + 1900*a1^5*a4*a5^3*a6^2 + 
    1344*a1^5*a5^5*a6 + 625*a1^4*a2^4*a6^4 + 2300*a1^4*a2^3*a3*a5*a6^3 + 
    250*a1^4*a2^3*a4^2*a6^3 + 1470*a1^4*a2^3*a4*a5^2*a6^2 - 
    276*a1^4*a2^3*a5^4*a6 - 125*a1^4*a2^2*a3^2*a4*a6^3 - 
    610*a1^4*a2^2*a3^2*a5^2*a6^2 + 1995*a1^4*a2^2*a3*a4^2*a5*a6^2 - 
    1174*a1^4*a2^2*a3*a4*a5^3*a6 - 16*a1^4*a2^2*a3*a5^5 + 
    375*a1^4*a2^2*a4^4*a6^2 - 172*a1^4*a2^2*a4^3*a5^2*a6 + 
        82*a1^4*a2^2*a4^2*a5^4 - 7750*a1^4*a2^2*a4*a6^4 - 
        46650*a1^4*a2^2*a5^2*a6^3 
    + 15*a1^4*a2*a3^3*a4*a5*a6^2 - 384*a1^4*a2*a3^3*a5^3*a6 + 
    525*a1^4*a2*a3^2*a4^3*a6^2 - 528*a1^4*a2*a3^2*a4^2*a5^2*a6 + 
    384*a1^4*a2*a3^2*a4*a5^4 - 10125*a1^4*a2*a3^2*a6^4 - 
    29*a1^4*a2*a3*a4^4*a5*a6 - 118*a1^4*a2*a3*a4^3*a5^3 + 
        36700*a1^4*a2*a3*a4*a5*a6^3 + 2410*a1^4*a2*a3*a5^3*a6^2 + 
        38*a1^4*a2*a4^6*a6
    + 5*a1^4*a2*a4^5*a5^2 + 5550*a1^4*a2*a4^3*a6^3 - 
        10040*a1^4*a2*a4^2*a5^2*a6^2 + 5800*a1^4*a2*a4*a5^4*a6 - 
        1600*a1^4*a2*a5^6 -
    292500*a1^4*a2*a6^5 - 99*a1^4*a3^5*a5*a6^2 - 150*a1^4*a3^4*a4^2*a6^2 + 
    196*a1^4*a3^4*a4*a5^2*a6 + 48*a1^4*a3^4*a5^4 + 12*a1^4*a3^3*a4^3*a5*a6 - 
    128*a1^4*a3^3*a4^2*a5^3 - 6525*a1^4*a3^3*a5*a6^3 - 12*a1^4*a3^2*a4^5*a6 + 
        65*a1^4*a3^2*a4^4*a5^2 + 225*a1^4*a3^2*a4^2*a6^3 + 
        80*a1^4*a3^2*a4*a5^2*a6^2
        - 13*a1^4*a3*a4^6*a5 + 5145*a1^4*a3*a4^3*a5*a6^2 - 
        6746*a1^4*a3*a4^2*a5^3*a6
    + 1760*a1^4*a3*a4*a5^5 - 103500*a1^4*a3*a5*a6^4 + a1^4*a4^8 + 
    954*a1^4*a4^5*a6^2 + 449*a1^4*a4^4*a5^2*a6 - 276*a1^4*a4^3*a5^4 + 
    70125*a1^4*a4^2*a6^4 + 58900*a1^4*a4*a5^2*a6^3 - 23310*a1^4*a5^4*a6^2 - 
    468*a1^3*a2^5*a5*a6^3 - 200*a1^3*a2^4*a3*a4*a6^3 - 
    294*a1^3*a2^4*a3*a5^2*a6^2 - 676*a1^3*a2^4*a4^2*a5*a6^2 + 
    180*a1^3*a2^4*a4*a5^3*a6 + 17*a1^3*a2^4*a5^5 + 50*a1^3*a2^3*a3^3*a6^3 - 
    397*a1^3*a2^3*a3^2*a4*a5*a6^2 + 514*a1^3*a2^3*a3^2*a5^3*a6 - 
    700*a1^3*a2^3*a3*a4^3*a6^2 + 447*a1^3*a2^3*a3*a4^2*a5^2*a6 - 
        118*a1^3*a2^3*a3*a4*a5^4 + 11700*a1^3*a2^3*a3*a6^4 - 
        12*a1^3*a2^3*a4^4*a5*a6
    + 6*a1^3*a2^3*a4^3*a5^3 + 10360*a1^3*a2^3*a4*a5*a6^3 + 
    11404*a1^3*a2^3*a5^3*a6^2 + 141*a1^3*a2^2*a3^4*a5*a6^2 - 
    185*a1^3*a2^2*a3^3*a4^2*a6^2 + 168*a1^3*a2^2*a3^3*a4*a5^2*a6 - 
    128*a1^3*a2^2*a3^3*a5^4 + 93*a1^3*a2^2*a3^2*a4^3*a5*a6 + 
    19*a1^3*a2^2*a3^2*a4^2*a5^3 + 5895*a1^3*a2^2*a3^2*a5*a6^3 - 
    36*a1^3*a2^2*a3*a4^5*a6 + 5*a1^3*a2^2*a3*a4^4*a5^2 - 
    12020*a1^3*a2^2*a3*a4^2*a6^3 - 5698*a1^3*a2^2*a3*a4*a5^2*a6^2 - 
    6746*a1^3*a2^2*a3*a5^4*a6 + 5064*a1^3*a2^2*a4^3*a5*a6^2 - 
        762*a1^3*a2^2*a4^2*a5^3*a6 + 780*a1^3*a2^2*a4*a5^5 + 
        93900*a1^3*a2^2*a5*a6^4
    + 198*a1^3*a2*a3^5*a4*a6^2 - 78*a1^3*a2*a3^5*a5^2*a6 - 
    95*a1^3*a2*a3^4*a4^2*a5*a6 + 44*a1^3*a2*a3^4*a4*a5^3 + 
    25*a1^3*a2*a3^3*a4^4*a6 - 15*a1^3*a2*a3^3*a4^3*a5^2 + 
    1935*a1^3*a2*a3^3*a4*a6^3 - 2808*a1^3*a2*a3^3*a5^2*a6^2 + 
    a1^3*a2*a3^2*a4^5*a5 - 4844*a1^3*a2*a3^2*a4^2*a5*a6^2 + 
    8996*a1^3*a2*a3^2*a4*a5^3*a6 - 160*a1^3*a2*a3^2*a5^5 - 
    3616*a1^3*a2*a3*a4^4*a6^2 + 500*a1^3*a2*a3*a4^3*a5^2*a6 - 
    1174*a1^3*a2*a3*a4^2*a5^4 + 72900*a1^3*a2*a3*a4*a6^4 - 
        55665*a1^3*a2*a3*a5^2*a6^3 + 128*a1^3*a2*a4^5*a5*a6 + 
        180*a1^3*a2*a4^4*a5^3 
    + 16240*a1^3*a2*a4^2*a5*a6^3 - 9330*a1^3*a2*a4*a5^3*a6^2 + 
    1900*a1^3*a2*a5^5*a6 - 27*a1^3*a3^7*a6^2 + 18*a1^3*a3^6*a4*a5*a6 - 
    4*a1^3*a3^6*a5^3 - 4*a1^3*a3^5*a4^3*a6 + a1^3*a3^5*a4^2*a5^2 + 
    54*a1^3*a3^5*a6^3 + 1143*a1^3*a3^4*a4*a5*a6^2 - 820*a1^3*a3^4*a5^3*a6 + 
        923*a1^3*a3^3*a4^3*a6^2 + 57*a1^3*a3^3*a4^2*a5^2*a6 - 
        384*a1^3*a3^3*a4*a5^4 
        + 29700*a1^3*a3^3*a6^4 - 547*a1^3*a3^2*a4^4*a5*a6 + 
        514*a1^3*a3^2*a4^3*a5^3 
    - 10305*a1^3*a3^2*a4*a5*a6^3 - 7405*a1^3*a3^2*a5^3*a6^2 + 
    108*a1^3*a3*a4^6*a6 - 148*a1^3*a3*a4^5*a5^2 - 11360*a1^3*a3*a4^3*a6^3 + 
        22209*a1^3*a3*a4^2*a5^2*a6^2 + 2410*a1^3*a3*a4*a5^4*a6 - 
        2000*a1^3*a3*a5^6 +
    432000*a1^3*a3*a6^5 + 12*a1^3*a4^7*a5 - 22624*a1^3*a4^4*a5*a6^2 + 
    11404*a1^3*a4^3*a5^3*a6 - 1450*a1^3*a4^2*a5^5 - 242100*a1^3*a4*a5*a6^4 + 
    58430*a1^3*a5^3*a6^3 + 56*a1^2*a2^6*a4*a6^3 + 86*a1^2*a2^6*a5^2*a6^2 - 
    14*a1^2*a2^5*a3^2*a6^3 + 304*a1^2*a2^5*a3*a4*a5*a6^2 - 
    148*a1^2*a2^5*a3*a5^3*a6 + 152*a1^2*a2^5*a4^3*a6^2 - 
    54*a1^2*a2^5*a4^2*a5^2*a6 + 5*a1^2*a2^5*a4*a5^4 - 2472*a1^2*a2^5*a6^4 - 
    76*a1^2*a2^4*a3^3*a5*a6^2 + 370*a1^2*a2^4*a3^2*a4^2*a6^2 - 
    287*a1^2*a2^4*a3^2*a4*a5^2*a6 + 65*a1^2*a2^4*a3^2*a5^4 - 
    28*a1^2*a2^4*a3*a4^3*a5*a6 + 5*a1^2*a2^4*a3*a4^2*a5^3 - 
    8092*a1^2*a2^4*a3*a5*a6^3 + 8*a1^2*a2^4*a4^5*a6 - 2*a1^2*a2^4*a4^4*a5^2 + 
    1096*a1^2*a2^4*a4^2*a6^3 - 5144*a1^2*a2^4*a4*a5^2*a6^2 + 
    449*a1^2*a2^4*a5^4*a6 - 210*a1^2*a2^3*a3^4*a4*a6^2 + 
    76*a1^2*a2^3*a3^4*a5^2*a6 + 43*a1^2*a2^3*a3^3*a4^2*a5*a6 - 
    15*a1^2*a2^3*a3^3*a4*a5^3 - 6*a1^2*a2^3*a3^2*a4^4*a6 + 
    2*a1^2*a2^3*a3^2*a4^3*a5^2 + 1962*a1^2*a2^3*a3^2*a4*a6^3 + 
    3181*a1^2*a2^3*a3^2*a5^2*a6^2 + 1684*a1^2*a2^3*a3*a4^2*a5*a6^2 + 
    500*a1^2*a2^3*a3*a4*a5^3*a6 + 590*a1^2*a2^3*a3*a5^5 - 
    168*a1^2*a2^3*a4^4*a6^2 - 494*a1^2*a2^3*a4^3*a5^2*a6 - 
    172*a1^2*a2^3*a4^2*a5^4 - 22080*a1^2*a2^3*a4*a6^4 + 
    58894*a1^2*a2^3*a5^2*a6^3 + 27*a1^2*a2^2*a3^6*a6^2 - 
    9*a1^2*a2^2*a3^5*a4*a5*a6 + a1^2*a2^2*a3^5*a5^3 + a1^2*a2^2*a3^4*a4^3*a6 - 
    486*a1^2*a2^2*a3^4*a6^3 + 1071*a1^2*a2^2*a3^3*a4*a5*a6^2 + 
    57*a1^2*a2^2*a3^3*a5^3*a6 + 2262*a1^2*a2^2*a3^2*a4^3*a6^2 - 
    2742*a1^2*a2^2*a3^2*a4^2*a5^2*a6 - 528*a1^2*a2^2*a3^2*a4*a5^4 - 
    29160*a1^2*a2^2*a3^2*a6^4 + 772*a1^2*a2^2*a3*a4^4*a5*a6 + 
    447*a1^2*a2^2*a3*a4^3*a5^3 - 96732*a1^2*a2^2*a3*a4*a5*a6^3 + 
    22209*a1^2*a2^2*a3*a5^3*a6^2 - 160*a1^2*a2^2*a4^6*a6 - 
    54*a1^2*a2^2*a4^5*a5^2 - 7992*a1^2*a2^2*a4^3*a6^3 + 
    8634*a1^2*a2^2*a4^2*a5^2*a6^2 - 10040*a1^2*a2^2*a4*a5^4*a6 + 
    3250*a1^2*a2^2*a5^6 + 529200*a1^2*a2^2*a6^5 - 351*a1^2*a2*a3^5*a5*a6^2 - 
    1215*a1^2*a2*a3^4*a4^2*a6^2 - 360*a1^2*a2*a3^4*a4*a5^2*a6 + 
    196*a1^2*a2*a3^4*a5^4 + 741*a1^2*a2*a3^3*a4^3*a5*a6 + 
    168*a1^2*a2*a3^3*a4^2*a5^3 + 11718*a1^2*a2*a3^3*a5*a6^3 - 
    106*a1^2*a2*a3^2*a4^5*a6 - 287*a1^2*a2*a3^2*a4^4*a5^2 + 
    22572*a1^2*a2*a3^2*a4^2*a6^3 - 8892*a1^2*a2*a3^2*a4*a5^2*a6^2 + 
    80*a1^2*a2*a3^2*a5^4*a6 + 88*a1^2*a2*a3*a4^6*a5 + 
    22144*a1^2*a2*a3*a4^3*a5*a6^2 - 5698*a1^2*a2*a3*a4^2*a5^3*a6 - 
    850*a1^2*a2*a3*a4*a5^5 + 169560*a1^2*a2*a3*a5*a6^4 - 8*a1^2*a2*a4^8 + 
        3032*a1^2*a2*a4^5*a6^2 - 5144*a1^2*a2*a4^4*a5^2*a6 + 
        1470*a1^2*a2*a4^3*a5^4 
    - 249480*a1^2*a2*a4^2*a6^4 - 105390*a1^2*a2*a4*a5^2*a6^3 + 
    58900*a1^2*a2*a5^4*a6^2 + 162*a1^2*a3^6*a4*a6^2 + 216*a1^2*a3^6*a5^2*a6 - 
    216*a1^2*a3^5*a4^2*a5*a6 - 78*a1^2*a3^5*a4*a5^3 + 36*a1^2*a3^4*a4^4*a6 + 
        76*a1^2*a3^4*a4^3*a5^2 - 3564*a1^2*a3^4*a4*a6^3 + 
        8802*a1^2*a3^4*a5^2*a6^2 -
    22*a1^2*a3^3*a4^5*a5 - 11475*a1^2*a3^3*a4^2*a5*a6^2 - 
    2808*a1^2*a3^3*a4*a5^3*a6 + 1200*a1^2*a3^3*a5^5 + 2*a1^2*a3^2*a4^7 + 
    222*a1^2*a3^2*a4^4*a6^2 + 3181*a1^2*a3^2*a4^3*a5^2*a6 - 
    610*a1^2*a3^2*a4^2*a5^4 - 165240*a1^2*a3^2*a4*a6^4 + 
        118260*a1^2*a3^2*a5^2*a6^3 + 572*a1^2*a3*a4^5*a5*a6 - 
        294*a1^2*a3*a4^4*a5^3 
    - 32616*a1^2*a3*a4^2*a5*a6^3 - 55665*a1^2*a3*a4*a5^3*a6^2 + 
    17250*a1^2*a3*a5^5*a6 - 232*a1^2*a4^7*a6 + 86*a1^2*a4^6*a5^2 + 
        48408*a1^2*a4^4*a6^3 + 58894*a1^2*a4^3*a5^2*a6^2 - 
        46650*a1^2*a4^2*a5^4*a6 +
    7500*a1^2*a4*a5^6 - 129600*a1^2*a4*a6^5 + 41040*a1^2*a5^2*a6^4 - 
    48*a1*a2^7*a4*a5*a6^2 + 12*a1*a2^7*a5^3*a6 + 12*a1*a2^6*a3^2*a5*a6^2 - 
    144*a1*a2^6*a3*a4^2*a6^2 + 88*a1*a2^6*a3*a4*a5^2*a6 - 13*a1*a2^6*a3*a5^4 + 
    1680*a1*a2^6*a5*a6^3 + 72*a1*a2^5*a3^3*a4*a6^2 - 22*a1*a2^5*a3^3*a5^2*a6 - 
        4*a1*a2^5*a3^2*a4^2*a5*a6 + a1*a2^5*a3^2*a4*a5^3 - 
        144*a1*a2^5*a3*a4*a6^3 + 
        572*a1*a2^5*a3*a5^2*a6^2 + 736*a1*a2^5*a4^2*a5*a6^2 + 
        128*a1*a2^5*a4*a5^3*a6
    - 124*a1*a2^5*a5^5 - 9*a1*a2^4*a3^5*a6^2 + a1*a2^4*a3^4*a4*a5*a6 + 
    36*a1*a2^4*a3^3*a6^3 - 2028*a1*a2^4*a3^2*a4*a5*a6^2 - 
    547*a1*a2^4*a3^2*a5^3*a6 - 480*a1*a2^4*a3*a4^3*a6^2 + 
        772*a1*a2^4*a3*a4^2*a5^2*a6 - 29*a1*a2^4*a3*a4*a5^4 + 
        6336*a1*a2^4*a3*a6^4 -
    12*a1*a2^4*a4^3*a5^3 + 4368*a1*a2^4*a4*a5*a6^3 - 22624*a1*a2^4*a5^3*a6^2 + 
    441*a1*a2^3*a3^4*a5*a6^2 + 336*a1*a2^3*a3^3*a4^2*a6^2 + 
    741*a1*a2^3*a3^3*a4*a5^2*a6 + 12*a1*a2^3*a3^3*a5^4 - 
    868*a1*a2^3*a3^2*a4^3*a5*a6 + 93*a1*a2^3*a3^2*a4^2*a5^3 + 
    11016*a1*a2^3*a3^2*a5*a6^3 + 176*a1*a2^3*a3*a4^5*a6 - 
    28*a1*a2^3*a3*a4^4*a5^2 + 14784*a1*a2^3*a3*a4^2*a6^3 + 
    22144*a1*a2^3*a3*a4*a5^2*a6^2 + 5145*a1*a2^3*a3*a5^4*a6 - 
    11344*a1*a2^3*a4^3*a5*a6^2 + 5064*a1*a2^3*a4^2*a5^3*a6 - 
    2050*a1*a2^3*a4*a5^5 - 346896*a1*a2^3*a5*a6^4 - 54*a1*a2^2*a3^5*a4*a6^2 - 
    216*a1*a2^2*a3^5*a5^2*a6 + 324*a1*a2^2*a3^4*a4^2*a5*a6 - 
    95*a1*a2^2*a3^4*a4*a5^3 - 80*a1*a2^2*a3^3*a4^4*a6 + 
    43*a1*a2^2*a3^3*a4^3*a5^2 - 12204*a1*a2^2*a3^3*a4*a6^3 - 
    11475*a1*a2^2*a3^3*a5^2*a6^2 - 4*a1*a2^2*a3^2*a4^5*a5 - 
    3888*a1*a2^2*a3^2*a4^2*a5*a6^2 - 4844*a1*a2^2*a3^2*a4*a5^3*a6 - 
    725*a1*a2^2*a3^2*a5^5 - 1312*a1*a2^2*a3*a4^4*a6^2 + 
    1684*a1*a2^2*a3*a4^3*a5^2*a6 + 1995*a1*a2^2*a3*a4^2*a5^4 + 
    139104*a1*a2^2*a3*a4*a6^4 - 32616*a1*a2^2*a3*a5^2*a6^3 + 
        736*a1*a2^2*a4^5*a5*a6 - 676*a1*a2^2*a4^4*a5^3 + 
        131040*a1*a2^2*a4^2*a5*a6^3
    + 16240*a1*a2^2*a4*a5^3*a6^2 - 20250*a1*a2^2*a5^5*a6 - 
    27*a1*a2*a3^6*a4*a5*a6 + 18*a1*a2*a3^6*a5^3 + 9*a1*a2*a3^5*a4^3*a6 - 
    9*a1*a2*a3^5*a4^2*a5^2 + 1944*a1*a2*a3^5*a6^3 + a1*a2*a3^4*a4^4*a5 + 
    6156*a1*a2*a3^4*a4*a5*a6^2 + 1143*a1*a2*a3^4*a5^3*a6 + 
    324*a1*a2*a3^3*a4^3*a6^2 + 1071*a1*a2*a3^3*a4^2*a5^2*a6 + 
        15*a1*a2*a3^3*a4*a5^4 - 7776*a1*a2*a3^3*a6^4 - 
        2028*a1*a2*a3^2*a4^4*a5*a6 - 
    397*a1*a2*a3^2*a4^3*a5^3 + 112860*a1*a2*a3^2*a4*a5*a6^3 - 
        10305*a1*a2*a3^2*a5^3*a6^2 + 336*a1*a2*a3*a4^6*a6 + 
        304*a1*a2*a3*a4^5*a5^2 -
    68976*a1*a2*a3*a4^3*a6^3 - 96732*a1*a2*a3*a4^2*a5^2*a6^2 + 
    36700*a1*a2*a3*a4*a5^4*a6 - 1250*a1*a2*a3*a5^6 - 1477440*a1*a2*a3*a6^5 - 
    48*a1*a2*a4^7*a5 + 4368*a1*a2*a4^4*a5*a6^2 + 10360*a1*a2*a4^3*a5^3*a6 - 
    3500*a1*a2*a4^2*a5^5 + 935280*a1*a2*a4*a5*a6^4 - 242100*a1*a2*a5^3*a6^3 - 
    972*a1*a3^6*a5*a6^2 - 351*a1*a3^5*a4*a5^2*a6 - 99*a1*a3^5*a5^4 + 
    441*a1*a3^4*a4^3*a5*a6 + 141*a1*a3^4*a4^2*a5^3 - 36936*a1*a3^4*a5*a6^3 - 
    84*a1*a3^3*a4^5*a6 - 76*a1*a3^3*a4^4*a5^2 + 17496*a1*a3^3*a4^2*a6^3 + 
    11718*a1*a3^3*a4*a5^2*a6^2 - 6525*a1*a3^3*a5^4*a6 + 12*a1*a3^2*a4^6*a5 + 
    11016*a1*a3^2*a4^3*a5*a6^2 + 5895*a1*a3^2*a4^2*a5^3*a6 - 
    1750*a1*a3^2*a4*a5^5 - 252720*a1*a3^2*a5*a6^4 - 2544*a1*a3*a4^5*a6^2 - 
    8092*a1*a3*a4^4*a5^2*a6 + 2300*a1*a3*a4^3*a5^4 + 536544*a1*a3*a4^2*a6^4 + 
    169560*a1*a3*a4*a5^2*a6^3 - 103500*a1*a3*a5^4*a6^2 + 1680*a1*a4^6*a5*a6 - 
    468*a1*a4^5*a5^3 - 346896*a1*a4^3*a5*a6^3 + 93900*a1*a4^2*a5^3*a6^2 + 
        35000*a1*a4*a5^5*a6 - 9375*a1*a5^7 + 108864*a1*a5*a6^5 + 
        16*a2^8*a4^2*a6^2 -
        8*a2^8*a4*a5^2*a6 + a2^8*a5^4 - 8*a2^7*a3^2*a4*a6^2 + 
        2*a2^7*a3^2*a5^2*a6 - 
        96*a2^7*a4*a6^3 - 232*a2^7*a5^2*a6^2 + a2^6*a3^4*a6^2 + 
        24*a2^6*a3^2*a6^3 + 
    336*a2^6*a3*a4*a5*a6^2 + 108*a2^6*a3*a5^3*a6 - 32*a2^6*a4^3*a6^2 - 
    160*a2^6*a4^2*a5^2*a6 + 38*a2^6*a4*a5^4 + 144*a2^6*a6^4 - 
    84*a2^5*a3^3*a5*a6^2 + 8*a2^5*a3^2*a4^2*a6^2 - 106*a2^5*a3^2*a4*a5^2*a6 - 
    12*a2^5*a3^2*a5^4 + 176*a2^5*a3*a4^3*a5*a6 - 36*a2^5*a3*a4^2*a5^3 - 
    2544*a2^5*a3*a5*a6^3 - 32*a2^5*a4^5*a6 + 8*a2^5*a4^4*a5^2 - 
    3072*a2^5*a4^2*a6^3 + 3032*a2^5*a4*a5^2*a6^2 + 954*a2^5*a5^4*a6 + 
    36*a2^4*a3^4*a5^2*a6 - 80*a2^4*a3^3*a4^2*a5*a6 + 25*a2^4*a3^3*a4*a5^3 + 
    16*a2^4*a3^2*a4^4*a6 - 6*a2^4*a3^2*a4^3*a5^2 + 2520*a2^4*a3^2*a4*a6^3 + 
    222*a2^4*a3^2*a5^2*a6^2 - 1312*a2^4*a3*a4^2*a5*a6^2 - 
    3616*a2^4*a3*a4*a5^3*a6 - 125*a2^4*a3*a5^5 + 1296*a2^4*a4^4*a6^2 - 
    168*a2^4*a4^3*a5^2*a6 + 375*a2^4*a4^2*a5^4 + 19296*a2^4*a4*a6^4 + 
    48408*a2^4*a5^2*a6^3 + 9*a2^3*a3^5*a4*a5*a6 - 4*a2^3*a3^5*a5^3 - 
    2*a2^3*a3^4*a4^3*a6 + a2^3*a3^4*a4^2*a5^2 - 432*a2^3*a3^4*a6^3 + 
        324*a2^3*a3^3*a4*a5*a6^2 + 923*a2^3*a3^3*a5^3*a6 - 
        752*a2^3*a3^2*a4^3*a6^2 +
        2262*a2^3*a3^2*a4^2*a5^2*a6 + 525*a2^3*a3^2*a4*a5^4 - 
        9936*a2^3*a3^2*a6^4 - 
        480*a2^3*a3*a4^4*a5*a6 - 700*a2^3*a3*a4^3*a5^3 - 
        68976*a2^3*a3*a4*a5*a6^3 - 
    11360*a2^3*a3*a5^3*a6^2 - 32*a2^3*a4^6*a6 + 152*a2^3*a4^5*a5^2 + 
    6912*a2^3*a4^3*a6^3 - 7992*a2^3*a4^2*a5^2*a6^2 + 5550*a2^3*a4*a5^4*a6 - 
    29376*a2^3*a6^5 + 108*a2^2*a3^4*a4^2*a6^2 - 1215*a2^2*a3^4*a4*a5^2*a6 - 
    150*a2^2*a3^4*a5^4 + 336*a2^2*a3^3*a4^3*a5*a6 - 185*a2^2*a3^3*a4^2*a5^3 + 
    17496*a2^2*a3^3*a5*a6^3 + 8*a2^2*a3^2*a4^5*a6 + 370*a2^2*a3^2*a4^4*a5^2 - 
    864*a2^2*a3^2*a4^2*a6^3 + 22572*a2^2*a3^2*a4*a5^2*a6^2 + 
    225*a2^2*a3^2*a5^4*a6 - 144*a2^2*a3*a4^6*a5 + 14784*a2^2*a3*a4^3*a5*a6^2 - 
        12020*a2^2*a3*a4^2*a5^3*a6 + 625*a2^2*a3*a4*a5^5 + 
        536544*a2^2*a3*a5*a6^4 + 
    16*a2^2*a4^8 - 3072*a2^2*a4^5*a6^2 + 1096*a2^2*a4^4*a5^2*a6 + 
    250*a2^2*a4^3*a5^4 - 93744*a2^2*a4^2*a6^4 - 249480*a2^2*a4*a5^2*a6^3 + 
    70125*a2^2*a5^4*a6^2 + 162*a2*a3^6*a5^2*a6 - 54*a2*a3^5*a4^2*a5*a6 + 
    198*a2*a3^5*a4*a5^3 - 210*a2*a3^4*a4^3*a5^2 - 3564*a2*a3^4*a5^2*a6^2 + 
        72*a2*a3^3*a4^5*a5 - 12204*a2*a3^3*a4^2*a5*a6^2 + 
        1935*a2*a3^3*a4*a5^3*a6 - 
    8*a2*a3^2*a4^7 + 2520*a2*a3^2*a4^4*a6^2 + 1962*a2*a3^2*a4^3*a5^2*a6 - 
        125*a2*a3^2*a4^2*a5^4 - 178848*a2*a3^2*a4*a6^4 - 
        165240*a2*a3^2*a5^2*a6^3 - 
    144*a2*a3*a4^5*a5*a6 - 200*a2*a3*a4^4*a5^3 + 139104*a2*a3*a4^2*a5*a6^3 + 
    72900*a2*a3*a4*a5^3*a6^2 - 20625*a2*a3*a5^5*a6 - 96*a2*a4^7*a6 + 
    56*a2*a4^6*a5^2 + 19296*a2*a4^4*a6^3 - 22080*a2*a4^3*a5^2*a6^2 - 
    7750*a2*a4^2*a5^4*a6 + 3125*a2*a4*a5^6 + 248832*a2*a4*a6^5 - 
    129600*a2*a5^2*a6^4 - 27*a3^7*a5^3 + 27*a3^6*a4^2*a5^2 - 9*a3^5*a4^4*a5 + 
    1944*a3^5*a4*a5*a6^2 + 54*a3^5*a5^3*a6 + a3^4*a4^6 - 432*a3^4*a4^3*a6^2 - 
    486*a3^4*a4^2*a5^2*a6 + 46656*a3^4*a6^4 + 36*a3^3*a4^4*a5*a6 + 
    50*a3^3*a4^3*a5^3 - 7776*a3^3*a4*a5*a6^3 + 29700*a3^3*a5^3*a6^2 + 
    24*a3^2*a4^6*a6 - 14*a3^2*a4^5*a5^2 - 9936*a3^2*a4^3*a6^3 - 
    29160*a3^2*a4^2*a5^2*a6^2 - 10125*a3^2*a4*a5^4*a6 + 3125*a3^2*a5^6 + 
    1026432*a3^2*a6^5 + 6336*a3*a4^4*a5*a6^2 + 11700*a3*a4^3*a5^3*a6 - 
    3125*a3*a4^2*a5^5 - 1477440*a3*a4*a5*a6^4 + 432000*a3*a5^3*a6^3 + 
    144*a4^6*a6^2 - 2472*a4^5*a5^2*a6 + 625*a4^4*a5^4 - 29376*a4^3*a6^4 + 
    529200*a4^2*a5^2*a6^3 - 292500*a4*a5^4*a6^2 + 40625*a5^6*a6 - 186624*a6^6;

   R := PolyAlg(re);
   if QfRank(R) > 1 then
      QR := PolyAlgMakeEuclidian(R);
      re := PolyMove(re, QR);
   fi;
   
   return re;
   
end;
   

special6_deg10:= function(f)
   
   local R, QR, lf, re, a1, a2, a3, a4, a5, a6, r0, r1, r2, r3, r4, r5, r6, z;
   lf:= PolyToList(f);
   a6:=lf[7];a5:=lf[6];a4:=lf[5];a3:=lf[4];a2:=lf[3];a1:=lf[2];
   z:= Poly(PolyAlg(f), [1,0]);
   
   re:= z^10 - 4*a2*z^9 + (3*a1*a3 + 6*a2^2 - 6*a4)*z^8 + (-a1^2*a4 - 9*a1*a2*a3 - 
    11*a1*a5 - 4*a2^3 + 26*a2*a4 - 3*a3^2 + 66*a6)*z^7 + (-3*a1^3*a5 + 
    4*a1^2*a2*a4 + 3*a1^2*a3^2 + 58*a1^2*a6 + 9*a1*a2^2*a3 + 30*a1*a2*a5 - 
    13*a1*a3*a4 + a2^4 - 42*a2^2*a4 + 9*a2*a3^2 - 324*a2*a6 + 36*a3*a5 + 
    a4^2)*z^6 + (11*a1^4*a6 + 5*a1^3*a2*a5 - 2*a1^3*a3*a4 - 5*a1^2*a2^2*a4 - 
    6*a1^2*a2*a3^2 - 192*a1^2*a2*a6 + 3*a1^2*a3*a5 - 4*a1^2*a4^2 - 3*a1*a2^3*a3 
    - 27*a1*a2^2*a5 + 42*a1*a2*a3*a4 - 6*a1*a3^3 - 57*a1*a3*a6 + 63*a1*a4*a5 + 
    30*a2^3*a4 - 9*a2^2*a3^2 + 642*a2^2*a6 - 120*a2*a3*a5 - 18*a2*a4^2 + 
    12*a3^2*a4 + 114*a4*a6 - 123*a5^2)*z^5 + (-22*a1^4*a2*a6 - 2*a1^4*a3*a5 - 
    a1^4*a4^2 - 2*a1^3*a2^2*a5 + 6*a1^3*a2*a3*a4 + a1^3*a3^3 - 3*a1^3*a3*a6 + 
    32*a1^3*a4*a5 + 2*a1^2*a2^3*a4 + 3*a1^2*a2^2*a3^2 + 246*a1^2*a2^2*a6 - 
    15*a1^2*a2*a3*a5 - 6*a1^2*a3^2*a4 + 113*a1^2*a4*a6 - 111*a1^2*a5^2 + 
    8*a1*a2^3*a5 - 45*a1*a2^2*a3*a4 + 12*a1*a2*a3^3 + 57*a1*a2*a3*a6 - 
    175*a1*a2*a4*a5 + 46*a1*a3^2*a5 + 10*a1*a3*a4^2 - 43*a1*a5*a6 - 8*a2^4*a4 + 
    3*a2^3*a3^2 - 640*a2^3*a6 + 148*a2^2*a3*a5 + 49*a2^2*a4^2 - 40*a2*a3^2*a4 - 
    482*a2*a4*a6 + 521*a2*a5^2 + 3*a3^4 + 138*a3^2*a6 - 138*a3*a4*a5 + 24*a4^3 +
    129*a6^2)*z^4 + (4*a1^5*a3*a6 + 4*a1^5*a4*a5 + 16*a1^4*a2^2*a6 - 
    a1^4*a3^2*a4 + 48*a1^4*a4*a6 - 36*a1^4*a5^2 - 4*a1^3*a2^2*a3*a4 - 
    a1^3*a2*a3^3 - 48*a1^3*a2*a3*a6 - 56*a1^3*a2*a4*a5 + 17*a1^3*a3^2*a5 - 
    2*a1^3*a3*a4^2 - 88*a1^3*a5*a6 - 144*a1^2*a2^3*a6 + 20*a1^2*a2^2*a3*a5 + 
    12*a1^2*a2^2*a4^2 + 10*a1^2*a2*a3^2*a4 - 376*a1^2*a2*a4*a6 + 
    368*a1^2*a2*a5^2 - 3*a1^2*a3^4 + 52*a1^2*a3^2*a6 - 72*a1^2*a3*a4*a5 + 
    30*a1^2*a4^3 + 74*a1^2*a6^2 + 16*a1*a2^3*a3*a4 - 6*a1*a2^2*a3^3 + 
    112*a1*a2^2*a3*a6 + 160*a1*a2^2*a4*a5 - 102*a1*a2*a3^2*a5 - 34*a1*a2*a3*a4^2    + 356*a1*a2*a5*a6 + 14*a1*a3^3*a4 + 34*a1*a3*a4*a6 + 10*a1*a3*a5^2 - 
    76*a1*a4^2*a5 + 320*a2^4*a6 - 80*a2^3*a3*a5 - 48*a2^3*a4^2 + 44*a2^2*a3^2*a4
    + 752*a2^2*a4*a6 - 898*a2^2*a5^2 - 6*a2*a3^4 - 366*a2*a3^2*a6 + 
    470*a2*a3*a4*a5 - 80*a2*a4^3 - 384*a2*a6^2 - 36*a3^3*a5 - 2*a3^2*a4^2 - 
    342*a3*a5*a6 + 80*a4^2*a6 + 94*a4*a5^2)*z^3 + (7*a1^6*a4*a6 - 4*a1^6*a5^2 - 
    9*a1^5*a2*a3*a6 - 3*a1^5*a2*a4*a5 + a1^5*a3^2*a5 - a1^5*a3*a4^2 - 
    35*a1^5*a5*a6 - 4*a1^4*a2^3*a6 + a1^4*a2^2*a3*a5 + a1^4*a2^2*a4^2 + 
    2*a1^4*a2*a3^2*a4 - 94*a1^4*a2*a4*a6 + 79*a1^4*a2*a5^2 + 5*a1^4*a3^2*a6 - 
    9*a1^4*a3*a4*a5 + 10*a1^4*a4^3 + 4*a1^4*a6^2 + 80*a1^3*a2^2*a3*a6 + 
    24*a1^3*a2^2*a4*a5 - 22*a1^3*a2*a3^2*a5 - 6*a1^3*a2*a3*a4^2 + 
    314*a1^3*a2*a5*a6 + a1^3*a3^3*a4 - 9*a1^3*a3*a4*a6 + 18*a1^3*a3*a5^2 - 
    55*a1^3*a4^2*a5 + 32*a1^2*a2^4*a6 - 8*a1^2*a2^3*a3*a5 - 8*a1^2*a2^3*a4^2 - 
    4*a1^2*a2^2*a3^2*a4 + 392*a1^2*a2^2*a4*a6 - 449*a1^2*a2^2*a5^2 + 
    3*a1^2*a2*a3^4 - 81*a1^2*a2*a3^2*a6 + 183*a1^2*a2*a3*a4*a5 - 62*a1^2*a2*a4^3
    - 108*a1^2*a2*a6^2 - 15*a1^2*a3^3*a5 + 13*a1^2*a3^2*a4^2 - 153*a1^2*a3*a5*a6
    + 74*a1^2*a4^2*a6 + 68*a1^2*a4*a5^2 - 176*a1*a2^3*a3*a6 - 48*a1*a2^3*a4*a5 +
    72*a1*a2^2*a3^2*a5 + 40*a1*a2^2*a3*a4^2 - 696*a1*a2^2*a5*a6 - 
    30*a1*a2*a3^3*a4 - 36*a1*a2*a3*a4*a6 - 87*a1*a2*a3*a5^2 + 220*a1*a2*a4^2*a5 
    + 3*a1*a3^5 + 3*a1*a3^3*a6 - 9*a1*a3^2*a4*a5 - 24*a1*a3*a4^3 - 60*a1*a3*a6^2
    - 84*a1*a4*a5*a6 - 11*a1*a5^3 - 64*a2^5*a6 + 16*a2^4*a3*a5 + 16*a2^4*a4^2 - 
    16*a2^3*a3^2*a4 - 512*a2^3*a4*a6 + 788*a2^3*a5^2 + 3*a2^2*a3^4 + 
    324*a2^2*a3^2*a6 - 588*a2^2*a3*a4*a5 + 88*a2^2*a4^3 + 384*a2^2*a6^2 + 
    84*a2*a3^3*a5 + 18*a2*a3^2*a4^2 + 852*a2*a3*a5*a6 - 240*a2*a4^2*a6 - 
    246*a2*a4*a5^2 - 6*a3^4*a4 - 6*a3^2*a4*a6 + 51*a3^2*a5^2 - 24*a3*a4^2*a5 + 
    16*a4^4 + 120*a4*a6^2 + 66*a5^2*a6)*z^2 + (-4*a1^7*a5*a6 - 4*a1^6*a2*a4*a6 +
    4*a1^6*a2*a5^2 + a1^6*a3^2*a6 + a1^6*a4^3 - 6*a1^6*a6^2 + 4*a1^5*a2^2*a3*a6 
    - a1^5*a2*a3^2*a5 - a1^5*a2*a3*a4^2 + 64*a1^5*a2*a5*a6 - 2*a1^5*a3*a4*a6 + 
    4*a1^5*a3*a5^2 - 13*a1^5*a4^2*a5 + 40*a1^4*a2^2*a4*a6 - 54*a1^4*a2^2*a5^2 - 
    10*a1^4*a2*a3^2*a6 + 20*a1^4*a2*a3*a4*a5 - 10*a1^4*a2*a4^3 + 40*a1^4*a2*a6^2
    - a1^4*a3^3*a5 + 3*a1^4*a3^2*a4^2 - 14*a1^4*a3*a5*a6 + 14*a1^4*a4^2*a6 + 
    19*a1^4*a4*a5^2 - 32*a1^3*a2^3*a3*a6 + 8*a1^3*a2^2*a3^2*a5 + 
    8*a1^3*a2^2*a3*a4^2 - 320*a1^3*a2^2*a5*a6 - 4*a1^3*a2*a3^3*a4 + 
    8*a1^3*a2*a3*a4*a6 - 47*a1^3*a2*a3*a5^2 + 104*a1^3*a2*a4^2*a5 + 
    5*a1^3*a3^3*a6 - 7*a1^3*a3^2*a4*a5 - 10*a1^3*a3*a4^3 - 52*a1^3*a3*a6^2 - 
    36*a1^3*a4*a5*a6 - 7*a1^3*a5^3 - 128*a1^2*a2^3*a4*a6 + 240*a1^2*a2^3*a5^2 + 
    48*a1^2*a2^2*a3^2*a6 - 160*a1^2*a2^2*a3*a4*a5 + 32*a1^2*a2^2*a4^3 - 
    32*a1^2*a2^2*a6^2 + 20*a1^2*a2*a3^3*a5 - 4*a1^2*a2*a3^2*a4^2 + 
    232*a1^2*a2*a3*a5*a6 - 112*a1^2*a2*a4^2*a6 - 130*a1^2*a2*a4*a5^2 + 
    a1^2*a3^4*a4 + 26*a1^2*a3^2*a4*a6 + 14*a1^2*a3^2*a5^2 - 10*a1^2*a3*a4^2*a5 +
    8*a1^2*a4^4 + 72*a1^2*a4*a6^2 + 30*a1^2*a5^2*a6 + 64*a1*a2^4*a3*a6 - 
    16*a1*a2^3*a3^2*a5 - 16*a1*a2^3*a3*a4^2 + 512*a1*a2^3*a5*a6 + 
    16*a1*a2^2*a3^3*a4 + 124*a1*a2^2*a3*a5^2 - 208*a1*a2^2*a4^2*a5 - 
    3*a1*a2*a3^5 - 36*a1*a2*a3^3*a6 + 28*a1*a2*a3^2*a4*a5 + 40*a1*a2*a3*a4^3 + 
    192*a1*a2*a3*a6^2 + 144*a1*a2*a4*a5*a6 + 28*a1*a2*a5^3 + a1*a3^4*a5 - 
    10*a1*a3^3*a4^2 + 4*a1*a3^2*a5*a6 - 56*a1*a3*a4^2*a6 + 18*a1*a3*a4*a5^2 - 
    32*a1*a5*a6^2 + 128*a2^4*a4*a6 - 352*a2^4*a5^2 - 96*a2^3*a3^2*a6 + 
    320*a2^3*a3*a4*a5 - 32*a2^3*a4^3 - 128*a2^3*a6^2 - 64*a2^2*a3^3*a5 - 
    32*a2^2*a3^2*a4^2 - 704*a2^2*a3*a5*a6 + 224*a2^2*a4^2*a6 + 216*a2^2*a4*a5^2 
    + 14*a2*a3^4*a4 + 8*a2*a3^2*a4*a6 - 86*a2*a3^2*a5^2 + 40*a2*a3*a4^2*a5 - 
    32*a2*a4^4 - 256*a2*a4*a6^2 - 112*a2*a5^2*a6 - a3^6 + 12*a3^4*a6 - 
    6*a3^3*a4*a5 + 8*a3^2*a4^3 - 48*a3^2*a6^2 + 24*a3*a4*a5*a6 - 14*a3*a5^3 + 
    32*a4^3*a6 - 8*a4^2*a5^2 + 64*a6^3)*z + -a1^8*a6^2 + 2*a1^7*a2*a5*a6 + 
    a1^7*a3*a4*a6 - a1^7*a4^2*a5 - a1^6*a2^2*a5^2 - a1^6*a2*a3^2*a6 + 
    a1^6*a2*a3*a4*a5 + 12*a1^6*a2*a6^2 + a1^6*a3*a5*a6 + 2*a1^6*a4*a5^2 - 
    24*a1^5*a2^2*a5*a6 - 8*a1^5*a2*a3*a4*a6 - 3*a1^5*a2*a3*a5^2 + 
    12*a1^5*a2*a4^2*a5 - a1^5*a3^2*a4*a5 - a1^5*a3*a4^3 - 8*a1^5*a3*a6^2 - 
    4*a1^5*a4*a5*a6 - a1^5*a5^3 + 12*a1^4*a2^3*a5^2 + 8*a1^4*a2^2*a3^2*a6 - 
    12*a1^4*a2^2*a3*a4*a5 - 48*a1^4*a2^2*a6^2 + a1^4*a2*a3^3*a5 + 
    a1^4*a2*a3^2*a4^2 + 4*a1^4*a2*a3*a5*a6 - 4*a1^4*a2*a4^2*a6 - 
    20*a1^4*a2*a4*a5^2 + 10*a1^4*a3^2*a4*a6 - a1^4*a3*a4^2*a5 + a1^4*a4^4 + 
    8*a1^4*a4*a6^2 + 4*a1^4*a5^2*a6 + 96*a1^3*a2^3*a5*a6 + 16*a1^3*a2^2*a3*a4*a6
    + 24*a1^3*a2^2*a3*a5^2 - 48*a1^3*a2^2*a4^2*a5 - 8*a1^3*a2*a3^3*a6 + 
    8*a1^3*a2*a3^2*a4*a5 + 8*a1^3*a2*a3*a4^3 + 64*a1^3*a2*a3*a6^2 + 
    32*a1^3*a2*a4*a5*a6 + 8*a1^3*a2*a5^3 - 2*a1^3*a3^3*a4^2 + 2*a1^3*a3^2*a5*a6 
    - 16*a1^3*a3*a4^2*a6 + 5*a1^3*a3*a4*a5^2 - 8*a1^3*a5*a6^2 - 
    48*a1^2*a2^4*a5^2 - 16*a1^2*a2^3*a3^2*a6 + 48*a1^2*a2^3*a3*a4*a5 + 
    64*a1^2*a2^3*a6^2 - 8*a1^2*a2^2*a3^3*a5 - 8*a1^2*a2^2*a3^2*a4^2 - 
    80*a1^2*a2^2*a3*a5*a6 + 32*a1^2*a2^2*a4^2*a6 + 64*a1^2*a2^2*a4*a5^2 + 
    2*a1^2*a2*a3^4*a4 - 40*a1^2*a2*a3^2*a4*a6 - 9*a1^2*a2*a3^2*a5^2 + 
    8*a1^2*a2*a3*a4^2*a5 - 8*a1^2*a2*a4^4 - 64*a1^2*a2*a4*a6^2 - 
    28*a1^2*a2*a5^2*a6 + a1^2*a3^4*a6 - a1^2*a3^3*a4*a5 + 2*a1^2*a3^2*a4^3 - 
    8*a1^2*a3^2*a6^2 + 4*a1^2*a3*a4*a5*a6 - 3*a1^2*a3*a5^3 + 8*a1^2*a4^3*a6 - 
    2*a1^2*a4^2*a5^2 + 16*a1^2*a6^3 - 128*a1*a2^4*a5*a6 - 48*a1*a2^3*a3*a5^2 + 
    64*a1*a2^3*a4^2*a5 + 32*a1*a2^2*a3^3*a6 - 16*a1*a2^2*a3^2*a4*a5 - 
    16*a1*a2^2*a3*a4^3 - 128*a1*a2^2*a3*a6^2 - 64*a1*a2^2*a4*a5*a6 - 
    16*a1*a2^2*a5^3 + 8*a1*a2*a3^3*a4^2 - 8*a1*a2*a3^2*a5*a6 + 
    64*a1*a2*a3*a4^2*a6 - 20*a1*a2*a3*a4*a5^2 + 32*a1*a2*a5*a6^2 - a1*a3^5*a4 + 
    8*a1*a3^3*a4*a6 - a1*a3^3*a5^2 - 16*a1*a3*a4*a6^2 + 4*a1*a3*a5^2*a6 + 
    64*a2^5*a5^2 - 64*a2^4*a3*a4*a5 + 16*a2^3*a3^3*a5 + 16*a2^3*a3^2*a4^2 + 
    192*a2^3*a3*a5*a6 - 64*a2^3*a4^2*a6 - 64*a2^3*a4*a5^2 - 8*a2^2*a3^4*a4 + 
    36*a2^2*a3^2*a5^2 - 16*a2^2*a3*a4^2*a5 + 16*a2^2*a4^4 + 128*a2^2*a4*a6^2 + 
    48*a2^2*a5^2*a6 + a2*a3^6 - 12*a2*a3^4*a6 + 4*a2*a3^3*a4*a5 - 8*a2*a3^2*a4^3
    + 48*a2*a3^2*a6^2 - 16*a2*a3*a4*a5*a6 + 12*a2*a3*a5^3 - 32*a2*a4^3*a6 + 
    8*a2*a4^2*a5^2 - 64*a2*a6^3 + a3^4*a4^2 - 8*a3^2*a4^2*a6 + 2*a3^2*a4*a5^2 + 
    16*a4^2*a6^2 - 8*a4*a5^2*a6 + a5^4;

   R := PolyAlg(re);

   if QfRank(R) > 1 then
      QR := PolyAlgMakeEuclidian(R);
      re := PolyMove(re, QR);
   fi;

   return re;
   
end;





IntegralSplit := function(f)
   local l, p, pp, ppp, pppp;
   
   if PolyAlgCoef(PolyAlg(f)) = Z then
      return f;
   elif PolyAlgCoef(PolyAlg(f)) = Q then
      l := Lcm(List(PolyToList(f), g->Denominator(g)));
      return l*f;
   elif IsOrder(PolyAlgCoef(PolyAlg(f))) then
      l := Lcm(List(PolyToList(f), g->Den(g)));
      return l*f;
   elif QfRank(PolyAlgCoef(PolyAlg(f))) = 1 then
      l := Lcm(List(PolyToList(f), g->Den(g)));
      p := List(PolyToList(f), g->g*l);
      pp:= List(p, g->PolyToList(g));
      ppp := Lcm(List(pp, g->Lcm(List(g, gg->Denominator(gg)))));
      pppp := Poly(PolyAlg(f), List(p, g->g*ppp));
      pppp:= PolyMoveIntegral(pppp); 
      return pppp;
   else   
      return Error("not supported");
   fi;   
end;


galoisgroup_deg3 := function(f)
   local disc, square;
   disc := PolyDisc(IntegralSplit(f));
   if IsInt(disc) then
      square := IntIsSquare(disc);
   elif IsElt(disc) then
      disc := EltMove(disc, OrderMaximal(EltOrder(disc)));
      square := EltRoot(disc, 2) <> false;
   else
      return GaloisModulo(Field(f), 50)[1];
      #square := ForAll(Factor(disc), b->b[2] mod 2 = 0);
   fi;
   if square then
      return 1;
   else 
      return 2; 
   fi;
end;


gcd := function(re)
   local deriv, gcd, Ry, QRy;
   
   Ry := PolyAlg(re);
   deriv := PolyDeriv(re);
   
   if QfRank(Ry) > 1 then
      QRy := PolyAlgMakeEuclidian(Ry);
      re := PolyMove(re, QRy);
      deriv := PolyMove(deriv, QRy);
   fi;
   
   GASMAN("g");
   GASMAN("C");
   
   gcd := PolyGcd(re, deriv);
   return gcd;
end;



vpoly_tschirn:= function(o, ox, f, re, func, m)
   local deriv, o, ox, O, ocoef, random, alpha, t, ff, x;
   
   GalPrintSymb(9, "we are in vpoly_tschirn : \n");
   t := PolyDeg(gcd(re));
   GalPrintSymb(9, "PolyDeg(gcd(resolvent)) : ", t, "\n");
   x := Poly(ox, [1,0]);
   
   if t = 0*t then
      return re;
   else
      while t > 0 do
         if IsOrder(o) then
            #random:= RandomElt(o);
            #alpha:= OrderBasis(o)[2]+random*OrderBasis(o)[3];
            #f:= EltMinPoly(alpha, ox, o);
            if not IsOrder(PolyAlgCoef(ox)) then
               f:= Eval(f, x+1);  
               GalPrintSymb(9, "f :", f, "\n");
            else
               ocoef:= OrderBasis(OrderCoefOrder(o), OrderCoefOrder(o))[2];
               f:= Eval(f,x+ocoef);
               GalPrintSymb(9, "f :", f, "\n");
            fi;
            if not m = 0 then
               re:= func(f,m);
               GalPrintSymb(9, "resolvent :", re, "\n");
            else
               re:= func(f);
               GalPrintSymb(9, "resolvent :", re, "\n");
            fi;
            t:= PolyDeg(gcd(re));   
            GalPrintSymb(9, "PolyDeg(gcd(resolvent)) :", t, "\n");
         else
            f := Eval(f, x+1);
            ff := PolyMoveIntegral(f);
            GalPrintSymb(9, "f :", ff, "\n");
            if ff = false then
               ff:= f;
            fi;
            if not m = 0 then
               re := func(ff, m);
               GalPrintSymb(9, "resolvent :", re, "\n");   
            else
               re := func(ff);
               GalPrintSymb(9, "resolvent :", re, "\n");
            fi;   
            t := PolyDeg(gcd(re));
            GalPrintSymb(9, "PolyDeg(gcd(resolvent)) :", t, "\n");   
         fi;
      od;
      return re;
   fi;
end;


     
squarefreepart := function(f)
   
   local t,tt, p, l, ff, s, ss, g;
   
   t := PolyDisc(IntegralSplit(f));
   if QfRank(PolyAlg(f)) > 1 then
      ff:= PolyToList(t)[1];
      s:= Factor(ff);
      ss := Filtered(s, g->not g[2] mod 2 = 0);
      ss:= Product(List(ss,g->g[1]));
      ff:= ff/Abs(ff);
      if not ss = 0 then
            ff:= ss;
      fi;
   fi;
   t := Factor(t);
   tt := Filtered(t, g->not g[2] mod 2 = 0);
   if tt = [] then
      p:=1;
   else
      p := Product(List(tt,g->g[1]));
   fi;
   if QfRank(PolyAlg(f)) > 1 then
      if not tt = [] then
         l:= PolyToList(p);
         t:= Lcm(List(l, g->Denominator(g)));
         g:= Gcd(ff,t);
         p:= p*t/g;   
      fi;
      p:= p*ff;
   fi;

   return p;
end;
         

special6:= function(factors,f,fold,deg)
   local y, fdeg, fdegyz, resultant, i, fact, orbitlength, p, 
         Ry, Ryz, z, QRy;
   
   i:= 1;
   
   Ry := PolyAlg(f);
   y := Poly(Ry, [1,0]);
   Ryz := PolyAlg(Ry, "z");
   z := Poly(Ryz, [1,0]);
   
   fdeg:= Filtered(factors, g->PolyDeg(g[1])=deg)[1][1]; 
   GalPrintSymb(9, "fdeg: ", fdeg, "\n");
   
   if QfRank(Ry) > 1 then 
      QRy := PolyAlgMakeEuclidian(Ry);
      fdeg := PolyMove(fdeg, QRy);
      GalPrintSymb(9, "fold: ", fold, "\n");
      p:= squarefreepart(fold);
      GalPrintSymb(9, "squarefreepart: ", p, "\n");
   elif QfRank(Ry) = 1 and not Ry = Zx then
      p:= PolyDisc(fold);
   else    
      GalPrintSymb(9, "fold: ", fold, "\n");
      p:= squarefreepart(fold);   
      GalPrintSymb(9, "squarefreepart: ", p, "\n");
   fi;
   
   fdegyz := Eval(fdeg, (y-z));
   GalPrintSymb(9, "fdegyz: ", fdegyz, "\n");
   resultant:= PolyResultant(fdegyz, z^2-1*p);
   GalPrintSymb(9, "resultant: ", resultant, "\n");
   
   while PolyDeg(gcd(resultant))>0 do
      i:=i+1; 
      resultant := PolyResultant(fdegyz, z^2-i^2*p);  
   od;
   
   if QfRank(Ry) > 1 then 
      resultant := PolyMove(resultant, QRy);
   fi;
   fact:= Factor(resultant);
   orbitlength:= List(fact, g->PolyDeg(g[1]));
   Sort(orbitlength);
   GalPrintSymb(9, "orbitlength: ", orbitlength, "\n");
   return orbitlength;
end;


special8:= function(factors, fold)
   local factor_deg4, o, t, t_old, t_new, factors_new, Rx;
   
   factor_deg4:= Filtered(factors, g->PolyDeg(g[1]) = 4)[1][1];
   factor_deg4:= IntegralSplit(factor_deg4);
   Rx := PolyAlg(factor_deg4);
   if not QfRank(Rx) > 1 then 
      o:= OrderShort(Field(factor_deg4));
      if not o = false then
         factor_deg4:= OrderPoly(o);
      fi;   
   fi;
   t:= GaloisMSetPol(factor_deg4/PolyToList(factor_deg4)[1], 2);
   t_old:= t;
   t:= vpoly_tschirn(Field(factor_deg4), 
               PolyAlg(factor_deg4),
               factor_deg4/PolyToList(factor_deg4)[1],t,GaloisMSetPol,2);
   t:= IntegralSplit(t);
   t_new:= GaloisMSetPol(t/PolyToList(t)[1],3);
   t_new:= vpoly_tschirn(Field(t), 
                   PolyAlg(t),
                   t,t_new,GaloisMSetPol,3);
   factors_new:= Factor(t_new);
   return special6(factors_new,t,fold,12);
end;


special_msumpol8:= function(f, o, ox)
   local f,oldf,l,t,p,factor_deg35, orbitlength, ff, Ry, QRy;
   
   GalPrintSymb(9, "We are in special_msumpol8: \n");
   oldf:= f;
   GalPrintSymb(9, "oldf: ", oldf, "\n");
   x := Poly(ox, [1,0]);
   f:= Eval(f, x-(PolyToList(f)[2]/8*PolyToList(f)[1]));
   GalPrintSymb(9, "f : ", f, "\n");
   t:= GaloisMSumPol(f,4);
   GalPrintSymb(9, "resolv: ", t, "\n");
   f:= IntegralSplit(f);
   while PolyDeg(gcd(t)) > 0 do
      f:= Eval(oldf, 1/(x+1));
      f:= QfeNum(f);
      f:= f/PolyToList(f)[1];
      oldf:= f;
      f:= Eval(f, x-(PolyToList(f)[2]/8/PolyToList(f)[1]));
      f:= IntegralSplit(f/(PolyToList(f)[1]));
      GalPrintSymb(9, "f: ", f, "\n");
      t:= GaloisMSumPol(f/PolyToList(f)[1],4);
   od;
   GalPrintSymb(9, "resolv: ", t, "\n");
   p:= PolyToList(t);
   f:= Sublist(p,[1,3,5,7,9,11,13,15,17,19,21,23,25,
               27,29,31,33,35,37,39,41,43,45,47,49,51,
               53,55,57,59,61,63,65,67,69,71]);
   Ry := ox;
   QRy := PolyAlgMakeEuclidian(Ry);
   ff:= Poly(QRy, f);

   factor_deg35:= List(Factor(ff),g->g[1]);
   GalPrintSymb(9, "factor_deg35: ", factor_deg35, "\n");
   p:= List(factor_deg35, g->Eval(g,x^2));
   orbitlength:= List(p, g->PolyDeg(g));
   Sort(orbitlength);
   return [orbitlength, factor_deg35];
end;            
            
            
Orbitlength:= function(o,ox,f,t,func,m)
   local resolv, fact, orbitlength, Ry, QRy;
   
   resolv:= vpoly_tschirn(o,ox, f, t, func, m);
   Ry := PolyAlg(resolv);
   if QfRank(Ry) > 1 then
      QRy := PolyAlgMakeEuclidian(Ry);
      resolv := PolyMove(resolv, QRy);
   fi;
   GalPrintSymb(9, "Factorization of the resolvent in Orbitlength: \n");
   fact:= Factor(resolv);
   orbitlength:= List(fact, g->PolyDeg(g[1]));
   GalPrintSymb(9, "orbitlength:", orbitlength, "\n");      

   Sort(orbitlength);
   
   return orbitlength;
end;


Orbitlength_factors:= function(o,ox,f,t,func,m)
   local resolv, fact, orbitlength, Ry, QRy;
   
   resolv:= vpoly_tschirn(o,ox, f, t, func, m);
   GalPrintSymb(9, "resolvent:", resolv, "\n");
   Ry := PolyAlg(resolv);
   if QfRank(Ry) > 1 then
      QRy := PolyAlgMakeEuclidian(Ry);
      resolv := PolyMove(resolv, QRy);
   fi;
   GalPrintSymb(9, "Factorization of the resolvent in Orbitlength_factors:\n");
   fact:= Factor(resolv);
   GalPrintSymb(9, "fact: ", fact, "\n");
   return fact;
end;


galoisgroup:= function(deg, o, ox, f, groupspossible)
   
   local t, resultant, f, p, orbitlength, G, factors, factors2, factor_deg4, 
         factor_deg6, factor_deg7, fold,
         factor_deg12, factor_deg35, orbitlength_special6, t_new, 
         group, l, liste;

   if Length(groupspossible)=1 then
      return groupspossible[1];
   elif deg = 4 then 
      if groupspossible = [2,4] then
         t:= GaloisMSetPol(f,2);
         orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
         if orbitlength = [6] then
            G:= 4;
         elif orbitlength = [2,2,2] then
            G:= 2;
         fi;      
      elif groupspossible = [3,5] then   
         t:= GaloisMSetPol(f,2);
         orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
         if orbitlength = [6] then
            G:= 5;
         elif orbitlength = [2,4] then
            G:= 3;
         fi; 
      elif groupspossible[Length(groupspossible)] = 5 then 
         t:= GaloisTwoSequencePol(f);
         orbitlength:= Orbitlength(o,ox,f,t,GaloisTwoSequencePol,0);
         if orbitlength = [12] then
            G:= 5;
         elif orbitlength = [4,8] then
            G:= 3;
         elif orbitlength = [4,4,4] then
            G:= 1;      
         fi;   
      else
         Print("Error in deg4 !!!\n\n");
         G:= false;   
      fi;
   elif deg = 5 then   
      if groupspossible[Length(groupspossible)] = 4 then
         if groupspossible = [ 1, 2, 4 ] then   
            if IsOrder(o) and OrderAutomorphismsAbel(o) = true then
               G:= 1;
            else
               t:= GaloisTwoSequencePol(f); 
               orbitlength:= Orbitlength(o,ox,f,t,GaloisTwoSequencePol,0);
               if orbitlength = [20] then   
                  G:= 4;
               elif orbitlength = [10,10] then 
                  G:= 2;
               elif orbitlength = [5,5,5,5] then 
                  G:= 1;
               fi;
            fi;
         elif groupspossible = [ 2, 4 ] then
            t:= special5(f);
            orbitlength:= Orbitlength(o,ox,f,t,special5,0);
            if orbitlength = [1,5] then   
               G:= 2;
            elif orbitlength = [6] then 
               G:= 4;
            fi;  
         else
            t:= GaloisTwoSequencePol(f); 
            orbitlength:= Orbitlength(o,ox,f,t,GaloisTwoSequencePol,0);
            if orbitlength = [20] then   
               G:= 4;
            elif orbitlength = [10,10] then 
               G:= 2;
            elif orbitlength = [5,5,5,5] then 
               G:= 1;
            fi; 
         fi;
      elif groupspossible[Length(groupspossible)] = 5 then
         t:= special5(f);
         orbitlength:= Orbitlength(o,ox,f,t,special5,0);
         if orbitlength = [1,5] then   
            G:= 3;
         elif orbitlength = [6] then 
            G:= 5;
         fi; 
      else
         Print("Error in deg5 !!!\n\n");
         G:= false;   
      fi;
   elif deg = 6 then
      if groupspossible[Length(groupspossible)] = 15 then
         if groupspossible = [10,15] or groupspossible = [7,10,15] then
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [3,12] then   
               G:= 7;
            elif orbitlength = [6,9] then   
               G:= 10;
            elif orbitlength = [15] then 
               G:= 15;
            else
               Print("Error in deg6 !!!\n\n");
               G:= false;   
            fi;  
         else
            t:= GaloisMSetPol(f,3);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);
            if orbitlength = [20] then   
               G:= 15;
            elif orbitlength = [10,10] then 
               G:= 12;
            elif orbitlength = [2,18] then 
               G:= 10;
            elif orbitlength = [4,4,12] then 
               G:= 7;
            elif orbitlength = [4,4,6,6] then 
               G:= 4;
            else
               Print("Error in deg6 !!!\n\n");
               G:= false;   
            fi;
         fi;   
      #if groupspossible[Length(groupspossible)] = 15 then
      #   if groupspossible = [10,15] then
      #      t:= special6_deg10(f);
      #      orbitlength:= Orbitlength(o,ox,f,t,special6_deg10,0);
      #      if orbitlength = [1,9] then   
      #         G:= 10;
      #      elif orbitlength = [10] then   
      #         G:= 15;
      #      else
      #         Print("Error in deg6 !!!\n\n");
      #         G:= false;   
      #      fi;  
      #   else
      #      t:= special6_deg6(f);
      #      orbitlength:= Orbitlength(o,ox,f,t,special6_deg6,0);
      #      if orbitlength = [6] then   
      #         t:= special6_deg10(f);
      #         orbitlength:= Orbitlength(o,ox,f,t,special6_deg10,0);
      #         if orbitlength = [1,9] then   
      #            G:= 10;
      #         elif orbitlength = [10] then   
      #            G:= 15;
      #         else
      #            Print("Error in deg6 !!!\n\n");
      #            G:= false;   
      #         fi;  
      #      elif orbitlength = [1,5] then 
      #         G:= 12;
      #      elif orbitlength = [2,4] then 
      #         G:= 7;
      #      elif orbitlength = [1,1,4] then 
      #         G:= 4;
      #      else
      #         Print("Error in deg6 !!!\n\n");
      #         G:= false;   
      #      fi;
      #   fi;    
      elif groupspossible[Length(groupspossible)] = 16 then   
         if groupspossible = [6,11,13,16] or 
            groupspossible = [1,3,5,6,9,11,13,14,16] then
            t:= GaloisMSetPol(f,3);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3); 
            if orbitlength = [6,6,8] then   
               G:= 6;
            elif orbitlength = [2,6,6,6] then   
               G:= 1;   
            else
               Print("Error in deg6 !!!\n\n");
               G:= false;   
            fi;  
         elif groupspossible = [5,9,13,16] then   
            t:= GaloisTwoSequencePol(f);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisTwoSequencePol,0); 
            if orbitlength = [6,6,18] then   
               G:= 5;
            else
               Print("Error in deg6 !!!\n\n");
               G:= false;   
            fi;   
         elif groupspossible = [13,16] or groupspossible = [11,16] or 
           groupspossible = [11,13,16] or
           groupspossible = [1,2,3,5,6,8,9,11,13,14,16] or 
           groupspossible = [3,6,9,11,13,14,16] then 
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [15] then   
               G:= 16;
            elif orbitlength = [6,9] then 
               G:= 13;
            elif orbitlength = [3,12] then 
               G:= 11;   
            elif orbitlength = [3,6,6] then 
               G:= 3;   
            elif orbitlength = [3,3,3,6] then 
               G:= 2;      
            else
               Print("Error in deg6 !!!\n\n");
               G:= false;   
            fi;      
         elif groupspossible = [9,13,16] then
            fold:= f;
            t:= GaloisMSetPol(f,3);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,3);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [20] then   
               G:= 16;
            elif orbitlength = [2,18] then 
               orbitlength_special6:= special6(factors,f,fold,2);
               if orbitlength_special6 = [2,2] then
                  G:=9;
               else
                  G:=13;   
               fi;   
            fi;
         elif groupspossible = [8,11,14,16] then
            fold:= f;
            t:= GaloisMSetPol(f,3);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,3);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [15] then   
               Print("The group is T_6_14 or T_6_16!\n");
               G:= 16;
               #or G:=14;
            elif orbitlength = [8,12] then 
               orbitlength_special6:= special6(factors,f,fold,12);
               if orbitlength_special6 = [12,12] then
                  G:=8;
               else
                  G:=11;   
               fi;   
            fi;   
         elif groupspossible = [14,16] then
            t:= special6_deg6(f);
            orbitlength:= Orbitlength(o,ox,f,t,special6_deg6,0);
            if orbitlength= [1,5] then
               G:=14;
            else
               G:=16;   
            fi;  
         fi;   
      else
         Print("Error in deg6 !!!\n\n");
         G:= false;   
      fi;
   elif deg = 7 then
      if groupspossible[Length(groupspossible)] = 7 then
         if groupspossible = [ 2, 4, 7 ] then
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [7,7,7] then   
               G:= 2;
            else
               Print("Error in deg7 !!!\n\n");
               G:= false;   
            fi;   
         else
            t:= GaloisMSetPol(f,3); 
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);
            if orbitlength = [35] then
               G:= 7;
            elif orbitlength = [14,21] then
               G:= 4;
            elif orbitlength = [7,7,7,14] then
               G:= 2;      
            fi;   
         fi;
      elif groupspossible[Length(groupspossible)] = 6 then
         if groupspossible = [ 1, 3, 5, 6 ] then
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [7,7,7] then   
               G:= 1;
            else
               Print("Error in deg7 !!!\n\n");
               G:= false;   
            fi;   
         else
            t:= GaloisMSetPol(f,3); 
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);
            if orbitlength = [35] then
               G:= 6;
            elif orbitlength = [7,28] then
               G:= 5;
            elif orbitlength = [7,7,21] then
               G:= 3;      
            elif orbitlength = [7,7,7,7,7] then
               G:= 1;   
            fi;   
         fi;   
      else
         Print("Error in deg7 !!!\n\n");
         G:= false;   
      fi;   
   elif deg = 8 then
      if groupspossible[Length(groupspossible)] = 49 then
         if groupspossible = [48,49] or 
            groupspossible = [37,48,49] then
            if QfRank(ox) > 1 then
               t:= GaloisMSetPol(f,4);
               orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,4);
            else
               orbitlength:= special_msumpol8(f, o, ox)[1];
            fi;
            if orbitlength = [70] then   
               G:= 49;
            elif orbitlength = [14,56] then   
               G:= 48;   
            elif orbitlength = [14,14,42] then   
               G:= 37;   
            elif orbitlength = [2,12,12,12,32] then   
               G:= 34;      
            else
               Print("Error in deg8 !!!\n\n");
            fi;
         elif groupspossible = [39,41,45,48,49] then
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [4,24] then   
               G:= 39;
            elif orbitlength = [12,16] then   
               t:= GaloisMSetPol(f,4);
               orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,4);
               if orbitlength = [2,32,36] then  
                  G:= 45;   
               elif orbitlength = [2,12,24,32] then  
                  G:= 41;      
               fi;
            fi;   
         elif groupspossible = 
           [2,4,5,9,10,11,12,14,18,19,20,22,24,29,32,33,34,37,39,41,42,45,
            48,49 ] then
            if ox = Zx and OrderAutomorphismsAbel(o) then
               G:= 2;
            else
               t:= GaloisMSetPol(f,2);
               orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
               if orbitlength = [4,4,4,8,8] then
                  G:= 2;
               elif orbitlength = [4,4,4,4,4,8] then
                  G:= 4;
               elif orbitlength = [4,8,8,8] then
                  t:= GaloisMSetPol(f,3);
                  GalPrintSymb(9, "resolvent:", t, "\n");
                  orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);
                  if orbitlength = [8,8,8,8,8,8,8] then  
                     G:= 5;
                  else
                     Print("Error in deg8 !!!\n\n");  
                     G:= false;  
                  fi;
               else
                  Print("Error in deg8 !!!\n\n");  
                  G:= false;
               fi;
            fi;
         elif groupspossible = 
           [9,10,11,18,19,20,22,24,29,32,33,34,39,41,42,45,48,49] then   
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [4,4,4,8,8] then
               G:= 9;
            elif orbitlength = [4,8,8,8] then
               t:= GaloisMSetPol(f,3);
               orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);
               if orbitlength = [8,8,8,16,16] then
                  G := 11;   
               elif orbitlength = [8,8,8,32] then
                  G:= 22;
               fi;
            elif orbitlength = [4,4,4,16] then
               t:= GaloisMSetPol(f,3);
               GalPrintSymb(9, "resolvent:", t, "\n");
               orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);
               if orbitlength = [8,8,8,16,16] then
                  G:=10;   
               elif orbitlength = [8,16,16,16] then
                  G:=18;
               fi;
            elif orbitlength = [4,8,16] then
               t:= GaloisTwoSequencePol(f);
               orbitlength:= Orbitlength(o,ox,f,t,GaloisTwoSequencePol,0);
               if orbitlength = [8,16,16,16] then
                  G:=20;   
               elif orbitlength = [8,16,32] then
                  G:=1929;
               fi;
            else
               Print("Error in deg8 !!!\n\n");  
               G:= false;
            fi;   
         elif groupspossible = [12,13,24,32,33,36,39,41,42,45,48,49] then   
            t:= GaloisMSetPol(f,2);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [4,12,12] then
               factor_deg4:= Filtered(factors, g->PolyDeg(g[1]) = 4)[1][1]; 
               factor_deg4:= IntegralSplit(factor_deg4);
               t:= Field(factor_deg4);
               GaloisModulo(t,100);
               if GaloisGroupsPossible(t)=[4] then
                  G:= 13;
               else
                  G:= 24;
               fi;
            else
               Print("Error in deg8 !!!\n\n");  
               G:= false; 
            fi;
         elif groupspossible = [12,14,24,32,33,34,37,39,41,42,45,48,49] then   
            t:= GaloisMSetPol(f,2);   
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [4,12,12] then
               t:= GaloisMSetPol(f,4);
               orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,4);
               if orbitlength = [2,6,6,8,12,12,24] then
                  G:= 14;
               elif orbitlength = [2,6,6,8,24,24] then
                  G:= 24;
               else
                  Print("Error in deg8 !!!\n\n");  
                  G:= false;   
               fi;
            else
               Print("Error in deg8 !!!\n\n");  
               G:= false;   
            fi;
         elif groupspossible = 
           [2,3,4,5,9,10,11,12,13,14,18,19,20,22,24,25,29,32,33,34,36,37,39,
            41,42,45,48,49] then     
            t:= GaloisMSetPol(f,2);
             orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
             if orbitlength = [4,4,4,4,4,4,4] then      
                G:= 3;
             else
                Print("Error in deg8 !!!\n\n");
                G:= false;
             fi;
          elif groupspossible = [45,49] then   
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [12,16] then   
               G:= 45;
            elif orbitlength = [28] then   
               G:= 49;   
            else
               Print("Error in deg8 !!!\n\n");
            fi;
         elif groupspossible = [42,45,49] then   
            t:= GaloisMSetPol(f,2);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [12,16] then   
               factor_deg12:= Filtered(factors, g->PolyDeg(g[1]) = 12)[1][1];
               t:= GaloisMSetPol(factor_deg12/PolyToList(factor_deg12)[1],2);
               factor_deg12:= IntegralSplit(factor_deg12);
               t:= vpoly_tschirn(Field(factor_deg12),
                           PolyAlg(factor_deg12), 
                           factor_deg12/PolyToList(factor_deg12)[1],
                           t,GaloisMSetPol,2);
               factors:= Factor(t);
               factor_deg6:= Filtered(factors, g->PolyDeg(g[1]) = 6)[1][1];
               t:= GaloisTwoSequencePol(factor_deg6);
               factor_deg12:= IntegralSplit(factor_deg6);
               t:= vpoly_tschirn(Field(factor_deg6),
                           PolyAlg(factor_deg6), 
                           factor_deg6/PolyToList(factor_deg6)[1],
                           t,GaloisTwoSequencePol,0);
               factors:= Factor(t);
               orbitlength:= List(factors, g->PolyDeg(g[1]));
               if orbitlength = [6,6,18] then
                  G:= 42;
               elif orbitlength = [12,18] then
                  G:= 45;
               fi;
            elif orbitlength = [28] then   
               G:= 49;   
            else
               Print("Error in deg8 !!!\n\n");
            fi;
         elif groupspossible = [12,24,32,33,39,41,42,45,48,49] then    
            t:= GaloisMSetPol(f,2);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [14,12,12] then
               G:=24;
            elif orbitlength = [4,24] then
               t:= GaloisTwoSequencePol(f);
               factors2:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
               orbitlength:= List(factors2, g->PolyDeg(g[1]));
               Sort(orbitlength);
               if orbitlength = [8,24,24] then
                  G:= 12;
               elif orbitlength = [8,48] then
                  factor_deg4:= Filtered(factors, g->PolyDeg(g[1]) = 4)[1][1];
                  factor_deg4:= IntegralSplit(factor_deg4);
                  t:= Field(factor_deg4);
                  GaloisModulo(t, 100);
                  if GaloisGroupsPossible(t) = [4] then
                     G:= 32;
                  else
                     G:= 39;
                  fi;   
               fi;
            elif orbitlength = [12,16] or 
              orbitlength = [28] then
               liste:= special_msumpol8(f, o, ox);
               GalPrintSymb(9, "We are out of special_msum\n");
               orbitlength:= liste[1];
               factor_deg35:= liste[2];
               if orbitlength = [2,12,24,32] then
                  factor_deg6:= Filtered(factor_deg35, g->PolyDeg(g)=6)[1];
                  GalPrintSymb(9, "factor_deg6:", factor_deg6, "\n");
                  t:= GaloisMSetPol(factor_deg6/PolyToList(factor_deg6)[1],3);
                  factors:= Orbitlength_factors(Field(
                                    IntegralSplit(factor_deg6)),
                                    PolyAlg(factor_deg6),factor_deg6,t,
                                    GaloisMSetPol,3);
                  orbitlength:= List(factors, g->PolyDeg(g[1]));
                  Sort(orbitlength);
                  if orbitlength = [4,4,6,6] then
                     G:= 33;
                  elif orbitlength = [4,4,12] then
                     G:= 41;
                  else
                     Print("Error in deg8 !!!\n\n");
                     G:= false;
                  fi;
               elif orbitlength = [2,32,36] then
                  G:= 4245;
               elif orbitlength = [14,56] then
                  G:= 48;   
               elif orbitlength = [70] then
                  G:= 49;   
               else
                  Print("Error in deg8 !!!\n\n");
               fi;
            else
               Print("Error in deg8 !!!\n\n");
            fi;
         elif groupspossible = [24,32,33,39,41,42,45,48,49] then   
            t:= GaloisMSetPol(f,2);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [4,12,12] then
               G:=24;
            elif orbitlength = [4,24] then
               factor_deg4:= Filtered(factors, g->PolyDeg(g[1]) = 4)[1][1];
               factor_deg4:= IntegralSplit(factor_deg4);
               t:= Field(factor_deg4);
               GaloisModulo(t, 100);
               if GaloisGroupsPossible(t) = [4] then
                  G:= 32;
               else
                  G:=39;
               fi;   
            elif orbitlength = [12,16] or 
              orbitlength = [28] then
               liste:= special_msumpol8(f, o, ox);
               orbitlength:= liste[1];
               factor_deg35:= liste[2];
               if orbitlength = [2,12,24,32] then
                  factor_deg6:= Filtered(factor_deg35, g->PolyDeg(g)=6)[1];
                  t:= GaloisMSetPol(factor_deg6/PolyToList(factor_deg6)[1],3);
                  factors:= Orbitlength_factors(Field(IntegralSplit(
                                    factor_deg6)),
                                    PolyAlg(factor_deg6),factor_deg6,t,
                                    GaloisMSetPol,3);
                  orbitlength:= List(factors, g->PolyDeg(g[1]));
                  Sort(orbitlength);
                  if orbitlength = [4,4,6,6] then
                     G:= 33;
                  elif orbitlength = [4,4,12] then
                     G:= 41;
                  else
                     Print("Error in deg8 !!!\n\n");
                     G:= false;
                  fi;
               elif orbitlength = [2,32,36] then
                  G:= 4245;
               elif orbitlength = [14,56] then
                  G:= 48;   
               elif orbitlength = [70] then
                  G:= 49;   
               else
                  Print("Error in deg8 !!!\n\n");
               fi;
            else
               Print("Error in deg8 !!!\n\n");
            fi;
         elif groupspossible = [36,48,49] or 
           groupspossible = [25,36,37,48,49] then
            liste:= special_msumpol8(f, o, ox);
            orbitlength:= liste[1];
            factor_deg35:= liste[2];
            if orbitlength = [14,56] then      
               factor_deg7:=Filtered(factor_deg35, 
                                    g->PolyDeg(g)=7)[1];
               if groupspossible[1] = 25 then
                  t:= GaloisMSetPol(factor_deg7/PolyToList(factor_deg7)[1],2);
                  t:= IntegralSplit(t);
                  t:= vpoly_tschirn(Field(IntegralSplit(factor_deg7)),
                              PolyAlg(factor_deg7),
                              f,t,GaloisMSetPol,2);
               else
                  t:= GaloisMSetPol(factor_deg7/PolyToList(factor_deg7)[1],3);
                  t:= vpoly_tschirn(Field(IntegralSplit(factor_deg7)),
                              PolyAlg(factor_deg7),
                              f,t,GaloisMSetPol,3);
               fi;
               factors:= Factor(t);
               orbitlength:= List(factors, g->PolyDeg(g[1]));
               if orbitlength = [7,7,21] then
                  G:= 36;
               elif orbitlength = [7,28] then
                  G:= 48;
               elif orbitlength = [7,7,7] then   
                  G:= 25;   
               else
                  Print("Error in deg8 !!!\n\n");
                  G:= false;   
               fi;
            elif orbitlength = [70] then
               G:= 49;
            fi;   
         elif groupspossible = [24,32,33,34,39,41,42,45,48,49] then
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [12,16] then
               t:= GaloisMSetPol(f,4);
               orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,4);
               if orbitlength = [2,12,12,12,32] then
                  G:= 34;
               else
                  Print("Error in deg8 !!!\n\n");
                  G:= false;   
               fi;
            else
               Print("Error in deg8 !!!\n\n");
               G:= false;   
            fi;
         elif groupspossible = [19,29,39,41,45,48,49] then   
            GalPrintSymb(9, "f : ", f, "\n");
            t:= GaloisMSetPol(f,2);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            GalPrintSymb(9, "orbitlength : ", orbitlength, "\n");
            if orbitlength = [4,8,16] then
               f:= Filtered(factors,g->PolyDeg(g[1])=8)[1][1];   
               fold:= IntegralSplit(f);
               t:= GaloisMSetPol(f,2);
               factors:=Orbitlength_factors(Field(fold),PolyAlg(fold),f,t,
                                GaloisMSetPol,2);
               orbitlength_special6:= special6(factors,f,fold,4);
               if orbitlength_special6 = [4,4] then
                  G:= 19;   
               elif orbitlength_special6 = [8] then
                  G:= 29;
               else
                  Print("Error in deg8 !!!\n\n");
                  G:=false;
               fi;
            elif orbitlength = [4,24] then
               G:= 39;   
            elif orbitlength = [12,16] then
               G:=4541;   
            elif orbitlength = [28] then
               G:= 4948;
            fi;   
         fi;   
      elif groupspossible[Length(groupspossible)] = 50 then   
         if groupspossible = 
            [1,6,7,8,15,16,17,23,26,27,28,35,40,43,44,46,47,50] then   
            if ox = Zx then
               if OrderAutomorphismsAbel(o) then   
                  G:= 1;   
               else
                  Print("Error in deg8 !!!\n\n");
               fi; 
            else
               t:= GaloisTwoSequencePol(f);
               orbitlength:= Orbitlength(o,ox,f,t,GaloisTwoSequencePol,0);
               if orbitlength = [8,8,8,8,8,8,8] then
                  G:= 1;
               else
                 Print("Error in deg8 !!!\n\n");
                  G:=false;
               fi;
            fi;
         elif groupspossible = [6,8,15,23,26,27,35,40,43,44,47,50] then
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [4,8,8,8] then
               G:= 6;
            elif orbitlength = [4,8,16] then
               t:= GaloisTwoSequencePol(f);
               orbitlength:= Orbitlength(o,ox,f,t,GaloisTwoSequencePol,0);
               if orbitlength = [8,16,16,16] then
                  t:= GaloisMSetPol(f,3);
                  orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);   
                  if orbitlength = [8,8,8,16,16] then
                     G:= 8;
                  elif orbitlength = [8,8,8,32] then
                     G:= 27;
                  else
                     Print("Error in deg8 !!!\n\n");
                     G:= false;
                  fi;   
               elif orbitlength = [8,16,32] then
                  t:= GaloisMSetPol(f,3);
                  orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);
                  if orbitlength = [8,16,16,16] then
                     G:= 15;
                  else
                     Print("Error in deg8 !!!\n\n");
                     G:= false;
                  fi;
               else
                  Print("Error in deg8 !!!\n\n");
                  G:= false;
               fi;
            else
               Print("Error in deg8 !!!\n\n");
               G:= false;
            fi;
         elif groupspossible = [15,26,27,35,40,44,47,50] then
            t:= GaloisMSetPol(f,4);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,4);
            if orbitlength = [2,4,8,8,16,32] then
               G:= 15;
            fi;
         elif groupspossible = [7,15,16,17,26,27,28,35,40,44,46,47,50] then
            fold:= f;
            t:= GaloisTwoSequencePol(f);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisTwoSequencePol,0);
            if orbitlength = [8,8,8,16,16] then
               G:= 7;
            elif orbitlength = [8,16,16,16] then   
               t:= GaloisMSetPol(f,2);
               factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
               orbitlength:= List(factors, g->PolyDeg(g[1]));
               Sort(orbitlength);
               orbitlength_special6:= special6(factors,f,fold,4);
               if orbitlength_special6 = [4,4] then
                  G:=16;
               elif orbitlength_special6 = [8] then
                  G:= 27;
               fi;   
            else
               Print("Error in deg8 !!!\n\n");
               G:= false;
            fi;
         elif groupspossible = [17,26,35,40,44,47,50] then
            t:= GaloisTwoSequencePol(f);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisTwoSequencePol,0);
            if orbitlength = [8,8,8,32] then
               G:= 17;
            else
               Print("Error in deg8 !!!\n\n");
               G:= false;
            fi; 
         elif groupspossible=[17,21,26,27,28,30,31,35,38,40,44,46,47,50] then
            fold:= f;
            t:= GaloisMSetPol(f,2);  
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [4,8,8,8] then
               orbitlength_special6:= special6(factors,f,fold, 4);
               if orbitlength_special6 = [4,4] then
                  G:= 21;
               elif orbitlength_special6 = [8] then
                  G:= 31;
               fi;   
            else
               Print("Error in deg8 !!!\n\n");
               G:= false;
            fi;
         elif groupspossible=[28,35,44,46,47,50] then
            fold:= f;
            t:= GaloisMSetPol(f,2);  
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [4,8,16] then
               orbitlength_special6:= special6(factors,f,fold,4);
               if orbitlength_special6 = [4,4] then
                  G:= 28;
               elif orbitlength_special6 = [8] then
                  G:= 35;
               fi;   
            else
               Print("Error in deg8 !!!\n\n");
               G:= false;
            fi;
         elif groupspossible = [23,40,44,47,50] then
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [4,24] then
               t:= GaloisMSetPol(f,3);
               orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);
               if orbitlength = [8,24,24] then
                  G:= 23;
               else
                  Print("Error in deg8 !!!\n\n");
                  G:= false;
               fi;  
            else
               Print("Error in deg8 !!!\n\n");
               G:= false;
            fi;
         elif groupspossible = [26,35,40,44,47,50] then
            fold:=f;
            t:= GaloisMSetPol(f,2);  
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [4,8,16] then
               orbitlength_special6:= special6(factors,f,fold,16);
               if orbitlength_special6 = [16,16] then
                  G:= 26;
               elif orbitlength_special6 = [32] then
                  G:= 35;
               fi;   
            else
               Print("Error in deg8 !!!\n\n");
               G:= false;
            fi;
         elif groupspossible = [47,50] or 
           groupspossible = [35,44,47,50] then
            t:= GaloisMSetPol(f,2);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [4,8,16] then   
               G:= 35;
            elif orbitlength = [4,24] then 
               G:= 44;
            elif orbitlength = [12,16] then   
               G:= 47;
            elif orbitlength = [28] then   
               G:= 50;   
            else
               Print("Error in deg8 !!!\n\n");
            fi;
         elif groupspossible = [46,47,50] then  
            fold:= f;
            t:= GaloisMSetPol(f,2);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [12,16] then 
               orbitlength_special6:= special6(factors,f,fold,12);
               if orbitlength_special6 = [12,12] then
                  G:= 46;
               else
                  G:= 47;   
               fi;   
            elif orbitlength = [28] then   
               G:= 50;   
            else
               Print("Error in deg8 !!!\n\n");
            fi;
         elif groupspossible = [44,50] then  
            t:= GaloisMSetPol(f,2);   
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,2);
            if orbitlength = [4,24] then   
               G:= 44;
            elif orbitlength = [28] then   
               G:= 50;   
            else
               Print("Error in deg8 !!!\n\n");
            fi;
         elif groupspossible = [43,50] then  
            t:= GaloisMSetPol(f,4);   
            orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,4);
            if orbitlength = [28,42] then   
               G:= 43;
            elif orbitlength = [70] then   
               G:= 50;   
            else
               Print("Error in deg8 !!!\n\n");
            fi;
         elif groupspossible = [40,44,47,50] then   
            fold:= f;
            t:= GaloisMSetPol(f,2); 
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [4,24] then 
               orbitlength_special6:= special8(factors, fold);
               if orbitlength_special6 = [12,12] then    
                  G:= 40;
               elif orbitlength_special6 = [24] then
                  G:= 44;
               else
                  Print("Error in deg8 !!!\n\n");
               fi; 
            elif orbitlength = [12,16] then   
               G:= 47;   
            elif orbitlength = [28] then   
               G:= 50;   
            else
               Print("Error in deg8 !!!\n\n");
            fi; 
         elif groupspossible = [38,44,50] then   
            t:= GaloisMSetPol(f,2);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [4,24] then 
               factor_deg4:= Filtered(factors, g->PolyDeg(g[1]) = 4)[1][1];
               factor_deg4:= IntegralSplit(factor_deg4);
               o:= Field(factor_deg4);
               GaloisModulo(o,50);
               group:= GaloisGroupsPossible(o);
               if group[1] = 4 then
                  G:= 38;
               elif group[1] = 5 then
                  G:= 44;
               else
                  Print("Error in deg8 !!!\n\n");  
               fi;
            elif orbitlength = [28] then   
               G:= 50;   
            elif groupspossible = [23,40,44,47,50] then   
               t:= GaloisMSetPol(f,3);  
               orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);               
            else
               Print("Error in deg8 !!!\n\n");
            fi;    
         elif groupspossible = [27,35,44,47,50] or 
           groupspossible = [26,27,35,40,44,47,50] then    
            t:= GaloisTwoSequencePol(f);
            orbitlength:= Orbitlength(o,ox,f,t,GaloisTwoSequencePol,0);
            if orbitlength = [8,16,16,16] then  
               G:= 27;
            elif orbitlength = [8,16,32] then
               G:= 35;
            elif orbitlength = [8,48] then
               G:= 44;
            elif orbitlength = [24,32] then
               G:= 47;   
            elif orbitlength = [56] then
               G:= 50;
            fi;
         elif groupspossible = [27,31,35,38,44,47,50] then   
            t:= GaloisMSetPol(f,2);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [4,8,8,8] then   
               G:= 31;
            elif orbitlength = [4,8,16] then
               t:= GaloisMSetPol(f,3);  
               orbitlength:= Orbitlength(o,ox,f,t,GaloisMSetPol,3);     
               if orbitlength = [8,8,8,32] then   
                  G:= 27;
               elif orbitlength = [8,16,32] then
                  G:= 35;
               fi;
            elif orbitlength = [4,24] then
               factor_deg4:= Filtered(factors, g->PolyDeg(g[1]) = 4)[1][1];
               o:= Order(PolyMove(factor_deg4, Z));
               GaloisModulo(o,50);
               group:= GaloisGroupsPossible(o);
               if group[1] = 4 then
                  G:= 38;
               elif group[1] = 5 then
                  G:= 44;
               else
                  Print("Error in deg8 !!!\n\n");  
               fi;
            elif orbitlength = [12,16] then   
               G:= 47;
            elif orbitlength = [28] then
               G:= 50;
            else
               Print("Error in deg8 !!!\n\n");
            fi;    
         elif groupspossible = [26,30,35,40,44,47,50] then     
            fold:= f;
            t:= GaloisMSetPol(f,2);
            factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,2);
            orbitlength:= List(factors, g->PolyDeg(g[1]));
            Sort(orbitlength);
            if orbitlength = [4,8,16] then
               orbitlength_special6:= special6(factors,f,fold,16);
               if orbitlength_special6 = [32] then   
                  t:= GaloisMSetPol(f,3);
                  factors:= Orbitlength_factors(o,ox,f,t,GaloisMSetPol,3);
                  orbitlength_special6:= special6(factors,f,fold,16);
                  if orbitlength_special6 = [16,16] then 
                     G:= 30;
                  elif orbitlength_special6 = [32] then 
                     G:= 35;
                  fi;
               elif orbitlength_special6 = [16,16] then
                  G:= 26;
               fi;   
            elif orbitlength = [4,24] then
               orbitlength_special6:= special8(factors, fold);
               if orbitlength_special6 = [12,12] then    
                  G:= 40;
               elif orbitlength_special6 = [24] then
                  G:= 44;
               else
                  Print("Error in deg8 !!!\n\n");
               fi;    
            elif orbitlength = [12,16] then
               G:= 47;
            elif orbitlength = [28] then
               G:= 50;
            fi;
         fi;   
      fi;
   fi;
   return G;   
end;



GaloisSymbT := function(F)
   
   local G, deg, f, groupspossible, ox, Fnew;

   PRINTLEVEL_GALOISSYMB := PRINTLEVEL("GALOIS");
   
   if IsField1(F) then
      Fnew:= F;
      if not GaloisGroupKnown(F) = false then
         return GaloisGroupKnown(F);
      fi;
      if IsOrder(Fnew) then
         Fnew := OrderEquationOrder(Fnew);
         f := OrderPoly(Fnew);
         Fnew := Field( IntegralSplit(f) );
      fi;
   elif IsPoly(F) then
      if PolyDeg(F) > 1 then
         Fnew := Field( IntegralSplit(F) );
      else
         return 1;
      fi;
   else
      return Error("usage: GaloisSymbT( <field> | <poly> )");
   fi;
   
   f := FieldPoly(Fnew);
   f := f / PolyToList(f)[1];
   
   deg := PolyDeg(f);
   
   if deg > 8 then
      return Error("only supported up to degree 8");
   fi;
   
   ox := PolyAlg(f);
   
   if not PolyIsIrreducible(f) then
      return Error("polynomial is not irreducible");
   fi;
   
   if deg = 1 then
      G := 1;
   elif deg = 2 then   
      G := 1;
   elif deg = 3 then
      G := galoisgroup_deg3(f);
   else
      if not IsOrder(Fnew) then
         groupspossible := GaloisModulo(Fnew, 100);
         GalPrintSymb(9, "groupspossible:", groupspossible, "\n");   
      else
         groupspossible := GaloisModulo(Fnew, 200);
         GalPrintSymb(9, "groupspossible:", groupspossible, "\n");  
      fi;
      G := galoisgroup(deg, Fnew, ox, f, groupspossible);
   fi;
   #GalPrintSymb(1, "The Galois group is ", G, "\n");  
   if IsField1(F) then
      GaloisGroupSet(F, G);
   fi;
   return G;
end;


GaloisSymb := function(F)
   local G, deg;
   
   G := GaloisSymbT(F);
   
   if IsField1(F) then
      deg := FieldDeg(F);  
   else
      deg := PolyDeg(F);
   fi;
   
   return GaloisNumberToName(deg, G);
end;








