#############################################################################
##
#F  GaloisBlocks( <order> ) 
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       GaloisBlocks
#SHORTDOC  Excluding unpossible Galois groups by blocks.
#CALLING   GaloisBlocks(o);\\
#          GaloisBlocks(o, true);
#LONGDOC   This function computes the lengths of the blocks of the Galois group
#          of the order
#          using subfields. Transitive groups with other block systems 
#          are excluded from the list of possible Galois groups. Is the 
#          second parameter used, this function computes the Galois groups 
#          of the subfields and excludes transitive groups with other 
#          Galois groups of the subfields from the list of possible Galois 
#          groups.
#ARGUS     o @ order
#          
#XREF      GaloisGroupsPossible, GaloisModulo, Galois.
#PACKAGE   Galois
#KASH      @
#          o := Order(x^15+5*x^14+8*x^13+7*x^12+8*x^11-3*x^9+7*x^8-2*x^7+7*x^6-2*x^5-3*x^4+5*x^3-2*x+1);
#          GaloisGroupsPossible(o);
#          GaloisBlocks(o);
#          GaloisBlocks(o, true);     
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##



_blockliste := 
  [ [ [ [  ], [ 1 ] ] ], [ [ [  ], [ 1 ] ] ], [ [ [  ], [ 1, 2 ] ] ], 
  [ [ [  ], [ 4, 5 ] ], [ [ 2 ], [ 1, 3 ] ], [ [ 2, 2, 2 ], [ 2 ] ] ], 
  [ [ [  ], [ 1, 2, 3, 4, 5 ] ] ], 
  [ [ [  ], [ 12, 14, 15, 16 ] ], [ [ 2 ], [ 4, 6, 7, 8, 11 ] ], 
      [ [ 2, 2, 2, 3 ], [ 2 ] ], [ [ 3 ], [ 5, 9, 10, 13 ] ], 
      [ [ 2, 3 ], [ 1, 3 ] ] ], [ [ [  ], [ 1, 2, 3, 4, 5, 6, 7 ] ] ], 
  [ [ [  ], [ 25, 36, 37, 43, 48, 49, 50 ] ], 
      [ [ 2 ], [ 12, 23, 32, 38, 39, 40, 44 ] ], 
      [ [ 2, 2, 2, 4 ], [ 10, 18 ] ], [ [ 2, 2, 2, 4, 4, 4 ], [ 2, 9 ] ], 
      [ [ 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 4, 4 ], [ 3 ] ], 
      [ [ 4 ], [ 33, 34, 41, 42, 45, 46, 47 ] ], 
      [ [ 2, 4 ], [ 1, 6, 7, 8, 13, 14, 15, 16, 17, 19, 20, 24, 26, 27, 28, 
              29, 30, 35 ] ], [ [ 2, 2, 2, 2, 2, 4, 4, 4 ], [ 4 ] ], 
      [ [ 2, 4, 4, 4 ], [ 5, 11, 21, 22, 31 ] ] ], 
  [ [ [  ], [ 9, 14, 15, 16, 19, 23, 26, 27, 32, 33, 34 ] ], 
      [ [ 3 ], [ 1, 3, 6, 7, 10, 11, 12, 13, 17, 18, 20, 21, 22, 24, 25, 28, 
              29, 30, 31 ] ], [ [ 3, 3 ], [ 4, 8 ] ], 
      [ [ 3, 3, 3, 3 ], [ 2, 5 ] ] ], 
  [ [ [  ], [ 7, 13, 26, 30, 31, 32, 35, 44, 45 ] ], 
      [ [ 2 ], [ 8, 14, 15, 16, 23, 24, 25, 29, 34, 36, 37, 38, 39 ] ], 
      [ [ 2, 2, 2, 2, 2, 5 ], [ 2 ] ], 
      [ [ 5 ], [ 6, 9, 10, 17, 18, 19, 20, 21, 27, 28, 33, 40, 41, 42, 43 ] ],
      [ [ 2, 5 ], [ 1, 3, 4, 5, 11, 12, 22 ] ] ], 
  [ [ [  ], [ 1, 2, 3, 4, 5, 6, 7, 8 ] ] ], 
  [ [ [  ], [ 179, 218, 272, 295, 300, 301 ] ], 
      [ [ 2 ], [ 33, 76, 124, 230, 255, 256, 257, 270, 277, 285, 286, 287, 
              293 ] ], [ [ 2, 2, 2, 2, 2, 2, 2, 3, 4, 4, 4, 6, 6, 6 ], [ 3 ] ]
        , [ [ 2, 2, 2, 3, 3, 3, 3, 4 ], [ 4 ] ], 
      [ [ 2, 2, 2, 3, 4, 6, 6, 6 ], [ 2, 10 ] ], 
      [ [ 2, 2, 2, 3, 6 ], [ 17, 35 ] ], [ [ 2, 2, 2, 3, 6, 6, 6 ], [ 16 ] ], 
      [ [ 2, 2, 2, 4 ], [ 6, 26, 32, 56, 67, 68, 90, 101, 103, 139 ] ], 
      [ [ 2, 2, 2, 4, 4, 4, 6 ], [ 9, 21 ] ], 
      [ [ 2, 2, 2, 4, 6 ], [ 7, 23, 24, 25, 48 ] ], [ [ 2, 3, 3, 4 ], [ 8 ] ],
      [ [ 2, 3, 4, 4, 4, 6 ], [ 5, 15 ] ], 
      [ [ 2, 3, 4, 6 ], [ 1, 11, 12, 13, 14, 28 ] ], 
      [ [ 2, 3, 6 ], [ 19, 36, 38, 39, 41, 42, 78, 79, 80, 81, 82, 125 ] ], 
      [ [ 2, 3, 6, 6, 6 ], [ 18, 34, 37, 40, 77 ] ], 
      [ [ 2, 4 ], [ 22, 27, 31, 49, 55, 57, 59, 60, 61, 62, 63, 64, 65, 66, 
              88, 89, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 102, 104, 110, 
              111, 112, 113, 114, 115, 137, 138, 140, 141, 142, 143, 144, 
              146, 147, 148, 149, 150, 151, 152, 153, 184, 185, 186, 187, 
              188, 189, 190, 191, 192, 221, 222, 223, 224, 225, 226, 227, 250 
             ] ], [ [ 2, 4, 4, 4, 6 ], [ 30, 50, 69, 106, 107, 135 ] ], 
      [ [ 2, 4, 6 ], 
          [ 29, 51, 52, 53, 54, 58, 86, 87, 105, 108, 109, 134, 136, 145, 
              154, 155, 193 ] ], 
      [ [ 2, 6 ], [ 74, 75, 123, 126, 158, 159, 160, 161, 162, 163, 180, 183, 
              195, 196, 197, 198, 199, 200, 201, 202, 203, 208, 219, 235, 
              236, 237, 238, 240, 241, 260 ] ], 
      [ [ 3 ], [ 122, 132, 133, 157, 175, 176, 177, 178, 194, 213, 231, 232, 
              233, 234, 258, 259, 271, 280, 281, 282, 289 ] ], 
      [ [ 3, 4 ], [ 20, 43, 44, 45, 83 ] ], 
      [ [ 3, 6 ], [ 46, 72, 73, 84, 116, 118, 119, 120, 121, 131, 156, 167, 
              169, 170, 173, 209, 211, 212, 215, 216, 217, 243, 244, 245, 
              247, 248, 249, 262, 263, 264, 266, 267, 274 ] ], 
      [ [ 3, 6, 6, 6 ], 
          [ 47, 70, 71, 117, 130, 168, 171, 172, 174, 210, 214, 242, 246, 261 
             ] ], 
      [ [ 4 ], [ 85, 127, 128, 129, 164, 165, 166, 204, 205, 206, 207, 228, 
              229, 239, 251, 252, 253, 254, 265, 268, 273, 275, 276, 283, 
              284, 290, 291, 292, 294 ] ], 
    [ [ 6 ], [ 181, 182, 220, 269, 278, 279, 288, 296, 297, 298, 299 ] ] ], 
    [ [ [  ], [ 1, 2, 3, 4, 5, 6, 7, 8, 9 ] ] ],
    [ [ [  ], [ 30, 39, 62, 63 ] ], 
    [ [ 2 ], [ 6, 9, 10, 11, 17, 18, 21, 27, 28, 29, 33, 34, 35, 38, 40, 41, 
          42, 43, 44, 48, 50, 51, 53, 54, 55, 56, 57 ] ], 
    [ [ 2, 2, 2, 2, 2, 2, 2, 7 ], [ 2 ] ], 
    [ [ 2, 7 ], [ 1, 3, 4, 5, 7, 19, 46, 47, 49 ] ], 
    [ [ 7 ], [ 8, 12, 13, 14, 15, 16, 20, 22, 23, 24, 25, 26, 31, 32, 36, 37, 
            45, 52, 58, 59, 60, 61 ] ] ],
    [ [ [  ], [ 20, 28, 47, 72, 103, 104 ] ], 
    [ [ 3 ], [ 5, 10, 15, 21, 26, 33, 34, 35, 36, 41, 42, 43, 44, 45, 46, 52, 
          53, 54, 55, 56, 61, 62, 63, 64, 69, 70, 71, 76, 77, 78, 79, 80, 81, 
          83, 84, 85, 86, 87, 88, 89, 90, 91, 93 ] ], 
    [ [ 3, 5 ], [ 1, 2, 3, 4, 6, 7, 8, 11, 16, 22, 23, 24, 29 ] ], 
    [ [ 5 ], [ 9, 12, 13, 14, 17, 18, 19, 25, 27, 30, 31, 32, 37, 38, 39, 40, 
          48, 49, 50, 51, 57, 58, 59, 60, 65, 66, 67, 68, 73, 74, 75, 82, 92, 
          94, 95, 96, 97, 98, 99, 100, 101, 102 ] ] ]];

_galoisgroups_of_subfields:=
[ [ [ [  ], [ 1 ] ] ], [ [ [  ], [ 1 ] ] ], [ [ [  ], [ 1, 2 ] ] ], 
  [ [ [  ], [ 4, 5 ] ], [ [ [ 2, 1 ] ], [ 1, 3 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ] ], [ 2 ] ] ], 
  [ [ [  ], [ 1, 2, 3, 4, 5 ] ] ], 
  [ [ [  ], [ 12, 14, 15, 16 ] ], [ [ [ 2, 1 ] ], [ 5, 9, 10, 13 ] ], 
      [ [ [ 2, 1 ], [ 3, 1 ] ], [ 1 ] ], [ [ [ 2, 1 ], [ 3, 2 ] ], [ 3 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 3, 2 ], [ 3, 2 ] ], [ 2 ] ], 
      [ [ [ 3, 1 ] ], [ 4, 6 ] ], [ [ [ 3, 2 ] ], [ 7, 8, 11 ] ] ], 
  [ [ [  ], [ 1, 2, 3, 4, 5, 6, 7 ] ] ], 
  [ [ [  ], [ 25, 36, 37, 43, 48, 49, 50 ] ], 
      [ [ [ 2, 1 ] ], [ 33, 34, 41, 42, 45, 46, 47 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], 
              [ 2, 1 ], [ 4, 2 ], [ 4, 2 ], [ 4, 2 ], [ 4, 2 ], [ 4, 2 ], 
              [ 4, 2 ], [ 4, 2 ] ], [ 3 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 4, 1 ], [ 4, 1 ], [ 4, 2 ] ], [ 2 ] 
         ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 4, 2 ] ], [ 5, 11, 21, 22, 31 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 4, 2 ], [ 4, 3 ], [ 4, 3 ] ], [ 9 ] 
         ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 4, 2 ], [ 4, 3 ], [ 4, 3 ], [ 4, 3 ]
                , [ 4, 3 ] ], [ 4 ] ], 
      [ [ [ 2, 1 ], [ 4, 1 ] ], [ 1, 7, 16, 20, 27 ] ], 
      [ [ [ 2, 1 ], [ 4, 1 ], [ 4, 3 ], [ 4, 3 ] ], [ 10 ] ], 
      [ [ [ 2, 1 ], [ 4, 3 ] ], [ 6, 8, 15, 17, 19, 26, 28, 29, 30, 35 ] ], 
      [ [ [ 2, 1 ], [ 4, 3 ], [ 4, 3 ], [ 4, 3 ] ], [ 18 ] ], 
      [ [ [ 2, 1 ], [ 4, 4 ] ], [ 13 ] ], 
      [ [ [ 2, 1 ], [ 4, 5 ] ], [ 14, 24 ] ], [ [ [ 4, 4 ] ], [ 12, 32, 38 ] ]
        , [ [ [ 4, 5 ] ], [ 23, 39, 40, 44 ] ] ], 
  [ [ [  ], [ 9, 14, 15, 16, 19, 23, 26, 27, 32, 33, 34 ] ], 
      [ [ [ 3, 1 ] ], [ 1, 6, 7, 13, 17, 22, 25, 28 ] ], 
      [ [ [ 3, 1 ], [ 3, 1 ], [ 3, 1 ], [ 3, 1 ] ], [ 2 ] ], 
      [ [ [ 3, 1 ], [ 3, 2 ] ], [ 4 ] ], 
      [ [ [ 3, 2 ] ], [ 3, 10, 11, 12, 18, 20, 21, 24, 29, 30, 31 ] ], 
      [ [ [ 3, 2 ], [ 3, 2 ] ], [ 8 ] ], 
      [ [ [ 3, 2 ], [ 3, 2 ], [ 3, 2 ], [ 3, 2 ] ], [ 5 ] ] ], 
  [ [ [  ], [ 7, 13, 26, 30, 31, 32, 35, 44, 45 ] ], 
      [ [ [ 2, 1 ] ], [ 6, 9, 10, 17, 18, 19, 20, 21, 27, 28, 33, 40, 41, 42, 
              43 ] ], [ [ [ 2, 1 ], [ 5, 1 ] ], [ 1 ] ], 
      [ [ [ 2, 1 ], [ 5, 2 ] ], [ 3 ] ], 
      [ [ [ 2, 1 ], [ 5, 2 ], [ 5, 2 ], [ 5, 2 ], [ 5, 2 ], [ 5, 2 ] ], [ 2 ] 
         ], [ [ [ 2, 1 ], [ 5, 3 ] ], [ 4, 5 ] ], 
      [ [ [ 2, 1 ], [ 5, 4 ] ], [ 11 ] ], 
      [ [ [ 2, 1 ], [ 5, 5 ] ], [ 12, 22 ] ], [ [ [ 5, 1 ] ], [ 8, 14 ] ], 
      [ [ [ 5, 2 ] ], [ 15, 16, 23 ] ], [ [ [ 5, 3 ] ], [ 24, 25, 29 ] ], 
      [ [ [ 5, 4 ] ], [ 34, 36 ] ], [ [ [ 5, 5 ] ], [ 37, 38, 39 ] ] ], 
  [ [ [  ], [ 1, 2, 3, 4, 5, 6, 7, 8 ] ] ], 
  [ [ [  ], [ 179, 218, 272, 295, 300, 301 ] ], 
      [ [ [ 2, 1 ] ], [ 181, 182, 220, 269, 278, 279, 288, 296, 297, 298, 299 
             ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 3, 1 ], [ 4, 2 ], [ 6, 1 ], 
              [ 6, 1 ], [ 6, 1 ] ], [ 2 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 3, 2 ], [ 3, 2 ], [ 3, 2 ], 
              [ 4, 2 ], [ 6, 2 ], [ 6, 3 ], [ 6, 3 ], [ 6, 3 ], [ 6, 3 ], 
              [ 6, 3 ], [ 6, 3 ] ], [ 3 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 3, 2 ], [ 4, 2 ], [ 6, 3 ], 
              [ 6, 3 ], [ 6, 3 ] ], [ 10 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 4, 2 ] ], 
          [ 47, 70, 71, 117, 130, 168, 171, 172, 174, 210, 214, 242, 246, 261 
             ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 4, 2 ], [ 6, 5 ] ], [ 18 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 4, 2 ], [ 6, 9 ] ], [ 37 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 4, 2 ], [ 6, 9 ], [ 6, 9 ], 
              [ 6, 9 ] ], [ 16 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 4, 2 ], [ 6, 10 ] ], [ 40 ] ], 
      [ [ [ 2, 1 ], [ 2, 1 ], [ 2, 1 ], [ 4, 2 ], [ 6, 13 ] ], [ 34, 77 ] ], 
      [ [ [ 2, 1 ], [ 3, 1 ], [ 4, 1 ], [ 6, 1 ] ], [ 1 ] ], 
      [ [ [ 2, 1 ], [ 3, 1 ], [ 4, 3 ], [ 6, 1 ] ], [ 14 ] ], 
      [ [ [ 2, 1 ], [ 3, 1 ], [ 6, 1 ] ], [ 29, 51, 58, 87, 105, 134 ] ], 
      [ [ [ 2, 1 ], [ 3, 1 ], [ 6, 1 ], [ 6, 4 ], [ 6, 6 ] ], [ 7 ] ], 
      [ [ [ 2, 1 ], [ 3, 1 ], [ 6, 1 ], [ 6, 6 ], [ 6, 6 ] ], [ 25 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 3, 2 ], [ 3, 2 ], [ 4, 1 ], [ 6, 2 ] ], [ 5 ] 
         ], [ [ [ 2, 1 ], [ 3, 2 ], [ 3, 2 ], [ 3, 2 ], [ 4, 3 ], [ 6, 2 ] ], 
          [ 15 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 3, 2 ], [ 3, 2 ], [ 6, 2 ] ], [ 30, 50, 69, 
              106, 107, 135 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 3, 2 ], [ 3, 2 ], [ 6, 2 ], [ 6, 7 ], 
              [ 6, 8 ] ], [ 9 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 3, 2 ], [ 3, 2 ], [ 6, 2 ], [ 6, 11 ], 
              [ 6, 11 ] ], [ 21 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 4, 1 ], [ 6, 3 ] ], [ 11 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 4, 3 ], [ 6, 3 ] ], [ 12, 13, 28 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 6, 3 ] ], 
          [ 52, 53, 54, 86, 108, 109, 136, 145, 154, 155, 193 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 6, 3 ], [ 6, 7 ], [ 6, 11 ] ], [ 23 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 6, 3 ], [ 6, 8 ], [ 6, 11 ] ], [ 24 ] ], 
      [ [ [ 2, 1 ], [ 3, 2 ], [ 6, 3 ], [ 6, 11 ], [ 6, 11 ] ], [ 48 ] ], 
      [ [ [ 2, 1 ], [ 4, 1 ] ], 
          [ 46, 72, 73, 119, 131, 170, 173, 211, 215, 244, 245, 264 ] ], 
      [ [ [ 2, 1 ], [ 4, 1 ], [ 6, 5 ] ], [ 19 ] ], 
      [ [ [ 2, 1 ], [ 4, 1 ], [ 6, 9 ] ], [ 39 ] ], 
      [ [ [ 2, 1 ], [ 4, 1 ], [ 6, 10 ] ], [ 41 ] ], 
      [ [ [ 2, 1 ], [ 4, 1 ], [ 6, 10 ], [ 6, 10 ], [ 6, 10 ] ], [ 17 ] ], 
      [ [ [ 2, 1 ], [ 4, 1 ], [ 6, 13 ] ], [ 79 ] ], 
      [ [ [ 2, 1 ], [ 4, 3 ] ], 
          [ 84, 116, 118, 120, 121, 156, 167, 169, 209, 212, 216, 217, 243, 
              247, 248, 249, 262, 263, 266, 267, 274 ] ], 
      [ [ [ 2, 1 ], [ 4, 3 ], [ 6, 5 ] ], [ 42 ] ], 
      [ [ [ 2, 1 ], [ 4, 3 ], [ 6, 9 ] ], [ 38, 81 ] ], 
      [ [ [ 2, 1 ], [ 4, 3 ], [ 6, 10 ] ], [ 82 ] ], 
      [ [ [ 2, 1 ], [ 4, 3 ], [ 6, 13 ] ], [ 36, 78, 80, 125 ] ], 
      [ [ [ 2, 1 ], [ 4, 3 ], [ 6, 13 ], [ 6, 13 ], [ 6, 13 ] ], [ 35 ] ], 
      [ [ [ 2, 1 ], [ 6, 5 ] ], [ 126, 158, 159, 208 ] ], 
      [ [ [ 2, 1 ], [ 6, 9 ] ], [ 161, 163, 195, 196, 197, 240 ] ], 
      [ [ [ 2, 1 ], [ 6, 10 ] ], [ 160, 162, 198, 199, 241 ] ], 
      [ [ [ 2, 1 ], [ 6, 12 ] ], [ 75 ] ], 
      [ [ [ 2, 1 ], [ 6, 13 ] ], [ 200, 201, 202, 203, 235, 236, 237, 238, 
              260 ] ], [ [ [ 2, 1 ], [ 6, 14 ] ], [ 74, 123 ] ], 
      [ [ [ 2, 1 ], [ 6, 15 ] ], [ 180 ] ], 
      [ [ [ 2, 1 ], [ 6, 16 ] ], [ 183, 219 ] ], 
      [ [ [ 3, 1 ] ], [ 85, 129, 164, 166, 205, 228, 229, 253, 265, 273, 284, 
              292 ] ], [ [ [ 3, 1 ], [ 4, 4 ] ], [ 20 ] ], 
      [ [ [ 3, 1 ], [ 4, 4 ], [ 4, 4 ], [ 4, 4 ], [ 4, 4 ], [ 6, 4 ], 
              [ 6, 4 ], [ 6, 4 ] ], [ 4 ] ], 
      [ [ [ 3, 1 ], [ 4, 5 ] ], [ 45 ] ], 
      [ [ [ 3, 1 ], [ 6, 4 ] ], [ 31, 55, 57, 59, 61, 88, 92, 93, 144, 188 ] ]
        , [ [ [ 3, 1 ], [ 6, 4 ], [ 6, 4 ], [ 6, 4 ] ], [ 32 ] ], 
      [ [ [ 3, 1 ], [ 6, 4 ], [ 6, 6 ], [ 6, 6 ] ], [ 6, 56 ] ], 
      [ [ [ 3, 1 ], [ 6, 6 ] ], 
          [ 60, 89, 91, 94, 99, 104, 141, 142, 143, 187, 189, 222 ] ], 
      [ [ [ 3, 1 ], [ 6, 6 ], [ 6, 6 ], [ 6, 6 ] ], [ 26, 90 ] ], 
      [ [ [ 3, 2 ] ], 
          [ 127, 128, 165, 204, 206, 207, 239, 251, 252, 254, 268, 275, 276, 
              283, 290, 291, 294 ] ], [ [ [ 3, 2 ], [ 4, 4 ] ], [ 43 ] ], 
      [ [ [ 3, 2 ], [ 4, 5 ] ], [ 44, 83 ] ], 
      [ [ [ 3, 2 ], [ 4, 5 ], [ 4, 5 ], [ 6, 7 ] ], [ 8 ] ], 
      [ [ [ 3, 2 ], [ 6, 7 ] ], 
          [ 22, 62, 63, 64, 66, 95, 96, 100, 110, 111, 114, 115, 137, 140, 
              184, 190, 227 ] ], 
      [ [ [ 3, 2 ], [ 6, 7 ], [ 6, 7 ], [ 6, 7 ] ], [ 67 ] ], 
      [ [ [ 3, 2 ], [ 6, 7 ], [ 6, 8 ], [ 6, 8 ] ], [ 68 ] ], 
      [ [ [ 3, 2 ], [ 6, 7 ], [ 6, 11 ], [ 6, 11 ] ], [ 101 ] ], 
      [ [ [ 3, 2 ], [ 6, 8 ] ], 
          [ 27, 65, 97, 98, 102, 146, 149, 191, 192, 224 ] ], 
      [ [ [ 3, 2 ], [ 6, 8 ], [ 6, 11 ], [ 6, 11 ] ], [ 103 ] ], 
      [ [ [ 3, 2 ], [ 6, 11 ] ], 
          [ 49, 112, 113, 138, 147, 148, 150, 151, 152, 153, 185, 186, 221, 
              223, 225, 226, 250 ] ], 
      [ [ [ 3, 2 ], [ 6, 11 ], [ 6, 11 ], [ 6, 11 ] ], [ 139 ] ], 
      [ [ [ 4, 4 ] ], [ 122, 132, 133, 176, 194, 232, 234, 271, 280 ] ], 
      [ [ [ 4, 5 ] ], 
          [ 157, 175, 177, 178, 213, 231, 233, 258, 259, 281, 282, 289 ] ], 
      [ [ [ 6, 12 ] ], [ 33, 76, 230, 255 ] ], 
      [ [ [ 6, 14 ] ], [ 124, 256, 257, 270 ] ], 
      [ [ [ 6, 15 ] ], [ 277, 286 ] ], [ [ [ 6, 16 ] ], [ 285, 287, 293 ] ] ],
  [ [ [  ], [ 1, 2, 3, 4, 5, 6, 7, 8, 9 ] ] ], 
  [ [ [  ], [ 30, 39, 62, 63 ] ], 
      [ [ [ 2, 1 ] ], [ 8, 12, 13, 14, 15, 16, 20, 22, 23, 24, 25, 26, 31, 
              32, 36, 37, 45, 52, 58, 59, 60, 61 ] ], 
      [ [ [ 2, 1 ], [ 7, 1 ] ], [ 1 ] ], [ [ [ 2, 1 ], [ 7, 2 ] ], [ 3 ] ], 
      [ [ [ 2, 1 ], [ 7, 2 ], [ 7, 2 ], [ 7, 2 ], [ 7, 2 ], [ 7, 2 ], 
              [ 7, 2 ], [ 7, 2 ] ], [ 2 ] ], [ [ [ 2, 1 ], [ 7, 3 ] ], [ 5 ] ]
        , [ [ [ 2, 1 ], [ 7, 4 ] ], [ 4, 7 ] ], 
      [ [ [ 2, 1 ], [ 7, 5 ] ], [ 19 ] ], [ [ [ 2, 1 ], [ 7, 6 ] ], [ 47 ] ], 
      [ [ [ 2, 1 ], [ 7, 7 ] ], [ 46, 49 ] ], 
      [ [ [ 7, 1 ] ], [ 6, 9, 21, 29 ] ], [ [ [ 7, 2 ] ], [ 27, 28, 38 ] ], 
      [ [ [ 7, 3 ] ], [ 11, 18, 35, 44 ] ], [ [ [ 7, 4 ] ], [ 40, 41, 48 ] ], 
      [ [ [ 7, 5 ] ], [ 10, 17, 33, 34, 42, 43, 50, 51 ] ], 
      [ [ [ 7, 6 ] ], [ 53, 56 ] ], [ [ [ 7, 7 ] ], [ 54, 55, 57 ] ] ], 
  [ [ [  ], [ 20, 28, 47, 72, 103, 104 ] ], 
      [ [ [ 3, 1 ] ], [ 9, 12, 19, 25, 30, 38, 39, 50, 57, 59, 67, 75, 92, 
              95, 98, 101 ] ], [ [ [ 3, 1 ], [ 5, 1 ] ], [ 1 ] ], 
      [ [ [ 3, 1 ], [ 5, 2 ] ], [ 3 ] ], [ [ [ 3, 1 ], [ 5, 3 ] ], [ 8 ] ], 
      [ [ [ 3, 1 ], [ 5, 4 ] ], [ 16 ] ], [ [ [ 3, 1 ], [ 5, 5 ] ], [ 24 ] ], 
      [ [ [ 3, 2 ] ], 
          [ 13, 14, 17, 18, 27, 31, 32, 37, 40, 48, 49, 51, 58, 60, 65, 66, 
              68, 73, 74, 82, 94, 96, 97, 99, 100, 102 ] ], 
      [ [ [ 3, 2 ], [ 5, 1 ] ], [ 4 ] ], [ [ [ 3, 2 ], [ 5, 2 ] ], [ 2, 7 ] ],
      [ [ [ 3, 2 ], [ 5, 3 ] ], [ 6, 11 ] ], 
      [ [ [ 3, 2 ], [ 5, 4 ] ], [ 23 ] ], 
      [ [ [ 3, 2 ], [ 5, 5 ] ], [ 22, 29 ] ], 
      [ [ [ 5, 1 ] ], [ 26, 33, 36, 44, 71, 81 ] ], 
      [ [ [ 5, 2 ] ], [ 34, 35, 43, 45, 46, 55, 79, 80, 86 ] ], 
      [ [ [ 5, 3 ] ], [ 41, 42, 52, 54, 56, 64, 84, 85, 87 ] ], 
      [ [ [ 5, 4 ] ], [ 5, 15, 53, 61, 69, 76, 88, 90 ] ], 
      [ [ [ 5, 5 ] ], [ 10, 21, 62, 63, 70, 77, 78, 83, 89, 91, 93 ] ] ] ];

NumberOfTrG := function(deg)
   local l;
   l:= [1,1,2,5,5,16,7,50,34,45,8,301,9,63,104];
   return l[deg];
end;

GaloisBlocks:= function(arg)
	
   local sub, deg, L, f, l, sublist, o, gg, ff, fff;	
   
   o:= arg[1];
   if not IsOrder(o) then
      Error("using: GaloisBlocks( <order [, true]> );\n");
   fi;
   if Length(arg) = 2 then
      if not arg[2] = true then
         Error("using: GaloisBlocks( <order [, true]> );\n");
      fi;
   elif Length(arg) > 2 then
      Error("using: GaloisBlocks( <order [, true]> );\n");
   fi;
      
   deg := OrderDeg(o);
   if deg > 15 then
      Error("degree must be <= 15\n");
   fi;   
   
   sub := OrderSubfield(o);


   if Length(sub) > 0 then
      L:=List(sub, g->deg/OrderDeg(g));
      Sort(L);
      f := Filtered(_blockliste[deg], g->g[1]=L);	
   else
      f := Filtered(_blockliste[deg], g->g[1]=[]);
   fi;

   l:= List([1..NumberOfTrG(deg)], g->g);
   sublist:= Filtered(l, g->not g in f[1][2]);
   GaloisGroupsPossible(o, sublist, false);

   if Length(arg) > 1 and Length(GaloisGroupsPossible(o)) > 1 and 
      arg[2] = true and Length(sub) > 0 then
      gg:= List(sub, g->[OrderDeg(g),GaloisSymbT(g)]);
      Sort(gg);
      ff:= Filtered(_galoisgroups_of_subfields[deg], g->g[1] = gg)[1][2];
      fff:= Set(GaloisGroupsPossible(o));
      SetIntersect(fff,Set(ff));
      sublist:= Filtered(l, g->not g in fff);
      GaloisGroupsPossible(o, sublist, false);   
      return GaloisGroupsPossible(o);  
   else
      return GaloisGroupsPossible(o);
      fi;   
end;	






