
GetIdealsEpsHilbert := function(ord, prim)
# Determines possible ideals idealsEps[1] and idealsEps[2] (prime to 6*prim)
# for the calculation of the eta-quotient used as generator for the hilbert
# classfield;
# therefor two primitiv ideals with small norm congruent 1 modulo 4
# and 1 modulo 6 (if possible!) are used;
# the reason why looking for ideals with small norm is to get later numerically
# good conditioned values for the computation of the infinite series of the
# dedekind eta-function
local j, k, idList, primeList, temp, foundMod4, foundMod6, idealEps1, idealEps2;

    if OrderClassGroup(ord)[1]=1 then
      idealEps1 := 1*ord; idealEps2 := 1*ord;
    else
      idList := [];
      primeList := [[],[]];
      j := 5;
      while Length(idList)<20 do
        if IntGcd(j,6*prim)<>1 then
          j := j + 2;
        else
          temp := Factor(j);
          for k in temp do
            if not (k[1] in primeList[1]) then
              Add(primeList[1],k[1]);
              Add(primeList[2],Factor(k[1]*ord)[1][1]);
            fi;
          od;
          temp := Product(temp,x->primeList[2][Position(primeList[1],x[1])]^(x[2]));
          if Abs(MatElt(IdealBasisUpperHNF(temp)[2],2,2))<>1 or
             IdealIsPrincipal(temp)<>false then
            j := j + 2;
          else
            Add(idList,temp);
          fi;
        fi;
      od;
      foundMod4 := false; foundMod6 := false;
      idealEps1 := idList[1]; idealEps2 := idList[1];
      for j in [1..Length(idList)] do
        if foundMod4=false and ((IdealNorm(idList[j])-1) mod 4 = 0) then
          foundMod4 := true; idealEps1 := idList[j];
        fi;
        if foundMod6=false and ((IdealNorm(idList[j])-1) mod 6 = 0) then
          foundMod6 := true; idealEps2 := idList[j];
        fi;
      od;
    fi;

    return [idealEps1,idealEps2];
end;




HilbertEtaGenerator := function(classRepr, idealsEps)
# Calculates a generating element for the hilbert classfield over an imaginary
# quadratic field and its conjugates (the generating element is a quotient
# of eta-values) by use of the defining ideals idealsEps[1], idealsEps[2] and
# the given representants classRepr of the ideal classes
local j, reprA, eps1A, eps2A, hilfid, hilfidBasis, power, epsQuotient;

  epsQuotient := [];
  power := 6 / IntGcd(6,(IdealNorm(idealsEps[1])-1)*(IdealNorm(idealsEps[2])-1)/4);
  eps1A := IdealMin(idealsEps[1]);
  eps2A := IdealMin(idealsEps[2]);
  for j in [1..Length(classRepr)] do
    reprA := IdealMin(classRepr[j]);
    hilfid := idealsEps[1]*idealsEps[2]*IdealNorm(classRepr[j])/classRepr[j];
    hilfidBasis := EltCon(Elt(IdealOrder(hilfid),MatToColList(IdealBasisUpperHNF(hilfid)[2])[2]),1);
    if Im(hilfidBasis)<0 then hilfidBasis := -hilfidBasis; fi;
    epsQuotient[j] := ( DedekindEta(hilfidBasis/(reprA*eps1A))*
                        DedekindEta(hilfidBasis/(reprA*eps2A))/
                        ( DedekindEta(hilfidBasis/(reprA*eps1A*eps2A))*
                          DedekindEta(hilfidBasis/reprA) ) )^power;
  od;

  return epsQuotient;
end;




ImQuadHilbert := function(arg)
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#USERDEF
#FXN       ImQuadHilbert
#SHORTDOC  Determines the Hilbert class field of an imaginary
#          quadratic field.
#LONGDOC   Let $K$ be an imaginary quadratic field and $K|{(1)}$ its Hilbert
#          class field.
#          Possible generators for $K|{(1)}$ over $K$ are for example
#          quotients of the Dedekind $\eta$--function of the form
#          \[ \left(\frac{\eta(a)\,\eta(b)}{\eta(a\,b)\,\eta(\O|K)}\right)^n,\quad n\in\N, \]
#          where the integral $\O|K$--ideals $a$ and $b$ have
#          to be suitably normalized (this is implicitly contained 
#          in \cite{Scherz})
#          A defining equation for $\O|{K|{(1)}}$ is calculated 
#          using complex
#          approximations one gets by evaluating the defining infinite
#          series of the Dedekind $\eta$--function.
#          
#          If {\tt "roots"} is specified, also the roots of the minimal polynomial
#          in the ordering given by the ordering of the used ideal class
#          representants (that correspond to the Galois automorphisms
#          by use of the Artin map) are returned.
#
#CALLING   O := ImQuadHilbert(o [,repr] [,"roots"] );\\
#ARGUS     o @ order @ imaginary quadratic field
#          repr @ list @ representants for the ideal classes of o
#          O @ order @ Hilbert class field
#DEBUG
#XREF      OrderHilbert, ImQuadRayField, RayClassField.
#PACKAGE   lib/ImQuad
#KASH      @
#          o := OrderMaximal(Z, 2, -47);
#          oK1 := ImQuadHilbert(o);
#          @
#          Test for correctness (in other examples the coefficients of
#          the generating polynomial one gets by using the routine
#          {\tt ImQuadHilbert} are usually quite small):@
#          O := OrderMaximal(OrderAbs(oK1));
#          Factor(Abs(OrderDisc(O)));
#
#
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
#
#**C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***/
# Determines the minimal polynomial over the imaginary quadratic field o
# for a generating integral element of the hilbert classfield.
# Numerical approximations for a generating element and its conjugates are
# calculated using quotients of dedekind eta-functions.
# The theoretical background is (implicitly) contained in two articles of Schertz,
# published in
#   J. Number Theory 34 (1) (1990), 41-53
#   Seminaire de Theorie des Nombres, Bordeaux 4 (1992), 239-262
#
local l, prec, oprec, precOld, printRoots, ord, idealsEps, hilbertRepr, prim,
      generator, genPol, oK1, debug;

    ord := Copy(arg[1]);
    printRoots := false;
    debug := false;
    if Length(arg)>1 and arg[2]<>"roots" and arg[2]<>"debug" then
      hilbertRepr := Copy(arg[2]);
    else
      hilbertRepr := IdealClassGroupRepr (ord);
    fi;
    if (Length(arg)>=2 and arg[2]="roots") or (Length(arg)>=3 and arg[3]="roots") then
      printRoots := true;
    fi;
    if arg[Length(arg)]="debug" then
      debug := true;
    fi;
    if OrderDegAbs(ord)<>2 or OrderDisc(ord)>0 then
      Error("ImQuadHilbert: Field must be imaginary quadratic!\n");
    fi;
    if not OrderIsMaximal(ord) then
      Error("ImQuadHilbert: Only maximal orders are supported!\n");
    fi;

#    if Length(hilbertRepr)=1 then
#      if printRoots then
#        return [ord,[Comp(1.0,0)]];
#      else
#        return ord;
#      fi;
#    fi;

    prim := FactorProd (OrderDisc(ord) * Product(hilbertRepr,x->IdealNorm(x)));
    idealsEps := GetIdealsEpsHilbert (ord, prim);
    if debug then
      Print("idealsEps = ",idealsEps,"\n          = ",List(idealsEps,x->Factor(x)),"\n");
      for l in [1..Length(idealsEps)] do
        Print("IdealIsPrincipal(idealsEps[",l,"]) = ",IdealIsPrincipal(idealsEps[l]),"\n");
      od;
    fi;
    l := 1;
    precOld := [Prec(),OrderPrec(ord)];
    prec := Round(Maximum(Prec(),20+Length(hilbertRepr))/2);
    oprec := Maximum(prec,Round(OrderPrec(ord)[1]/2));

    repeat
      l := l+1;
      prec := Prec(2*prec); oprec := OrderPrec(ord,2*oprec)[1];
      Print("used real precision is ",prec," places\n");
      generator := HilbertEtaGenerator (hilbertRepr, idealsEps);
      if debug then
        Print("generator = ",generator,"\n");
      fi;
      genPol := PolyFromConjugatesOverImQuad (ord, generator, 1);
      if genPol<>false then
        if Maximum(List(PolyToList(genPol),x->Round(Log(Abs(EltCon(x,1))+1)/Log(10))))>prec/2 then
          genPol := false;
        fi;
      fi;
    until genPol<>false or l>3;
    Prec(precOld[1]); OrderPrec(ord,precOld[2][1],precOld[2][2]);

    if genPol<>false then
      if PolyDeg(genPol)=1 then
        oK1 := ord;
      else
        oK1 := Order (genPol);
      fi;
      if printRoots then
        return [oK1,generator];
      else
        return oK1;
      fi;
    else
      Print("real precision of ",prec," places was not sufficient for calculation!\n\n");
      return false;
    fi;
end;



