

CompToImQuad := function(ord, z)
# Tries to convert a complex number to an integral element of the imaginary
# quadratic field given by order ord; if the resulting element isn't integral
# or precision is not sufficient, false is returned.
# Attention: ord is assumed to be maximal (but this will not be tested)
local zvalue, temp, result;

    zvalue := [];
    temp := EltCon(OrderBasis(ord)[2],1);
    zvalue[2] := Im(z)/Im(temp);
    zvalue[1] := Re(z-zvalue[2]*temp);
    if Abs(zvalue[1]-Round(zvalue[1])) > 0.001 or
       Abs(zvalue[2]-Round(zvalue[2])) > 0.001 then
      result := false;
    else
      result := Elt(ord,[Round(zvalue[1]),Round(zvalue[2])]);
    fi;

    return result;
end;




PolyFromConjugatesOverImQuad := function(ord, conj, denom)
# calculates the polynomial in denom^(-1)*ord[x] with roots conj (order ord
# must be maximal);
# if precision is not sufficient, false is returned
local j, pol, y, c_coeffs, coeffs, result;

    if OrderDegAbs(ord)<>2 or OrderDisc(ord)>0 then
      Error("PolyFromConjugatesOverImQuad: Field must be imaginary quadratic!\n");
    fi;
    if not OrderIsMaximal(ord) then
      Error("PolyFromConjugatesOverImQuad: Order must be maximal!\n");
    fi;
    pol := 1;
    y := Poly (PolyAlg(C), [1,0]);
    for j in [1..Length(conj)] do
      pol := pol * (y-conj[j]);
    od;

    c_coeffs := PolyToList(pol);
    result := true;
    coeffs := [];
    for j in [1..Length(c_coeffs)] do
      coeffs[j] := CompToImQuad(ord,c_coeffs[j]*denom);
      if coeffs[j]=false then
        result := false;
      else
        coeffs[j] := coeffs[j]/denom;
      fi;
    od;

    if result=false then
      return false;
    else
      return Poly(PolyAlg(ord),coeffs);
    fi;
end;




oK1EltFromConjugates := function(oK1, oK1gen, eltConj)
# Tries to convert a complex valued conjugate vector into an element of the
# field belonging to order oK1; if this is not possible or
# if precision is not sufficient, false is returned.
# Attention: the order of the imaginary quadratic subfield is assumed to
#            be maximal (but this will not be tested)
local j, k, genMatInv, id, index, coeffs, result;

    if OrderDegAbs(oK1)=2 then
      return CompToImQuad(oK1,eltConj[1]);
    fi;

    if IsMat(oK1gen) then
      genMatInv := Copy(oK1gen);
    else
      genMatInv := [];
      for j in [1..Length(oK1gen)] do
        coeffs := [Comp(1,0)];
        for k in [2..Length(oK1gen)] do
          coeffs[k] := coeffs[k-1] * oK1gen[j];
        od;
        genMatInv[j] := Copy(coeffs);
      od;
      genMatInv := MatInv (Mat(C,genMatInv));
    fi;

    index := Copy(OrderDisc(oK1));
    id := Product(Factor(index),x->x[1]^(x[2]/2));
    if IdealIsPrincipal(id)<>false then
      index := IdealIsPrincipal (id);
    else
      index := IdealIsPrincipal (index);
    fi;

    coeffs := MatToColList (genMatInv * Mat(C,List(eltConj,x->[x])))[1];
    result := true;
    for j in [1..Length(coeffs)] do
      coeffs[j] := CompToImQuad(OrderCoefOrder(oK1),coeffs[j]*EltCon(index,1));
      if coeffs[j]=false then
        result := false;
      else
        coeffs[j] := coeffs[j]/index;
      fi;
    od;

    if result=false then
      return false;
    else
      return Elt(oK1,coeffs);
    fi;
end;




PolyFromConjugatesOverHilbert := function(oK1, oK1gen, conj, denom)
# calculates the polynomial in denom^(-1)*oK1[x] with rootconjugates conj;
# here oK1 is the equation order given by a generator of the hilbert classfield
# over the maximal order of its imaginary quadratic subfield;
# the ordering of the conjugates in conj and oK1gen (the conjugates of a
# generator of the hilbert classfield) must be the same, i.e. the ordering
# conj[1]..conj[h] (conj[j] containes the j-th conjugates of the roots of the
# searched polynomial) must be the same as for the values oK1gen[1]..oK1gen[h];
# if precision is not sufficient, false is returned
local j, k, genMatInv, polConj, y, temp, result;

    if ( OrderDegAbs(oK1)=2 and (OrderDisc(oK1)>0 or not OrderIsMaximal(oK1)) )
        or
       ( OrderDegAbs(oK1)<>2 and (OrderDegAbs(OrderCoefOrder(oK1))<>2 or
         OrderDisc(OrderCoefOrder(oK1))>0 or not OrderIsMaximal(OrderCoefOrder(oK1))) )
        then
      Error("PolyFromConjugatesOverHilbert: Field must be above imaginary quadratic!\n");
    fi;
    polConj := [];
    y := Poly (PolyAlg(C), [1,0]);
    for j in [1..Length(conj)] do
      temp := 1;
      for k in [1..Length(conj[1])] do
        temp := temp * (y-conj[j][k]);
      od;
      polConj[j] := PolyToList(temp);
    od;

    genMatInv := [];
    for j in [1..Length(oK1gen)] do
      temp := [Comp(1,0)];
      for k in [2..Length(oK1gen)] do
        temp[k] := temp[k-1] * oK1gen[j];
      od;
      genMatInv[j] := Copy(temp);
    od;
    genMatInv := MatInv (Mat(C,genMatInv));

    temp := [];
    result := true;
    for j in [1..Length(polConj[1])] do
      temp[j] := oK1EltFromConjugates (oK1, genMatInv, List([1..Length(polConj)],x->denom*polConj[x][j]));
      if temp[j]=false then
        result := false;
      else
        temp[j] := temp[j]/denom;
      fi;
    od;

    if result=false then
      Print("PolyFromConjugatesOverHilbert: precision of ",Prec()," places not sufficient!\n\n");
      return false;
    else
      return Poly(PolyAlg(oK1),temp);
    fi;
end;





