if not IsBound(Info) then Info := Ignore; fi;

#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       RayClassFieldAuto  
#
#SHORTDOC  Given the output of {\tt RayClassField}, this function computes
#          a primitive element for the Ray Class Field togther with the
#          automorphisms.
#
#LONGDOC   Starting with the data computed by {\tt RayClassField}, this function
#          first computes a primitive element for {\tt O/o}. Next all the
#          automorphisms of {\tt O/o} are computed, together with information
#          allowing one to use the Artin isomorphism.
#
#          If automorphisms of the base field ({\tt o}) are known
#          (prior to the call of {\tt RayClassField}), and if
#          they extend to {\tt O/Q} they to are computed.
#
#CALLING   L := RayClassFieldAuto(c);
#
#ARGUS     L @ list @ containing {\tt O} and a list of automorphisms 
#          c @ list @ output of {\tt RayClassField}
#
#CSRC      
#
#DEBUG     Info := Print
#
#XREF      RayClassField
#
#PACKAGE   lib/RCLF
#
#KASH      We will consider various extensions of $\Q(\sqrt{10})$ to demonstrate
#          some effects and possibilities:@
#          o := OrderMaximal(Z, 2, 10);
#KASH      The ray class group of modul $(9)$ is isomorphic to $C_2\times C_3$,
#          therefore we get two generating polynomials, one of degree $2$ and
#          the other of degree $3$.@
#          c := RayClassField(9*o);
#KASH      Now we'll combine them to get a primitive element and to initialise
#          the Artin map@
#          L := RayClassFieldAuto(c);
#KASH      Since {\tt L[2]} has length 6, we know all $o$ automorphisms.@
#          O := L[1];
#          p := Factor(191*o)[1][1];
#          aut := RayClassFieldArtin(p, O);
#          aut * OrderBasis(O, O)[2];
#          OrderAutomorphisms(o);
#          d := RayClassField(9*o);
#          LL := RayClassFieldAuto(d);
#          OO := LL[1];
#          c := LL[2][Position(LL[2], "c")];
#          a := LL[2][Position(LL[2], "a")];
#          c * OrderBasis(O, O)[2];
#          a * OrderBasis(o, o)[2];
#          c * OrderBasis(o, o)[2];
#          i := 37*o;
#          c := RayClassField(i, 3);
#          L := RayClassFieldAuto(c);
#          O := L[1];
#          a := L[2][Position(L[2], "a")];
#          b := L[2][Position(L[2], "b")];
#          c := L[2][Position(L[2], "c")];
#          l := RayClassGroupCyclicFactors(i);
#          RayClassFieldArtin(l[3][1], O);
#          c^-1 * RayClassFieldArtin(c*l[3][1], O) * c;
#          
#
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY**HISTORY***HISTORY***HISTORY***HISTORY
#
#**C***C***|***C***C***C***C***C***C**C***C***C***C***C***C***C***C***C***C***/
RayClassFieldAuto := function(arg)
  local O, d, g, map, artin_map, rec_print, data, i, j, k, cf, sb, sbi,
        main, ClassFieldAuto1, ClassFieldAuto2, AllAutos, EltAbsAutomorphism,
        aslist, get_index, mult_mui, rep_mat, to_dist, mult_gen, get_powers,
        mult_fast, compute_auto;

  cf := arg[1];

  ClassFieldAuto1 := function(O, cf)
    local i, j, k, l, P, o, a, res, c, r, p, Oe, o,
          test_disc, artin_map, rec_print, m, n, s, t, c, MatToFp, MatFromFp;

    c := cf;

    MatToFp := function(m, p)
      local i, j;

      if TYPE(m)="KANT matrix" then
        m := MatToRowList(m);
      fi;

      for i in [1..Length(m)] do
        for j in [1..Length(m[i])] do
          m[i][j] := XEltToFFE(m[i][j], p);
        od;
      od;
      return Mat(XIdealResidueField(p), m);
    end;

    MatFromFp := function(m, p)
      local i, j;

      if TYPE(m)="KANT matrix" then
        m := MatToRowList(m);
      fi;

      for i in [1..Length(m)] do
        for j in [1..Length(m[i])] do
          m[i][j] := XFFEToElt(m[i][j], p);
        od;
      od;
      return Mat(XIdealOrder(p), m);
    end;


    artin_map := function(arg)
      local i, j, k, l, data, I, O, res; 

      I := arg[1];
      O := arg[2];
      

      data := O.data;
      #data [1][1..l]  the auto's attached to each generator of rcg
      #     [2][1..Deg(O)] the list of automorphisms sorted as Id, s, s2, ...
      i := IdealRayClassRep(I, O.rg, O.inf); 
      res := 0;
      for j in [1..Length(data[1])] do
        res := res + i[1][j] * data[1][j];
      od;
      res := res mod XOrderDeg(O);
      if Length(arg)>2 then
        return res;
      else
        return data[2][res+1];
      fi;
    end;

    rec_print := function(i)
      Print(i.base);
    end;

    test_disc := function(p)
      local f, i, j, lp;
      
      lp := XIdealFactor(XIdeal(o, p));
      i := 0;
      repeat
        i := i+1;
        f := PolyToList(OrderPoly(O));
        Apply(f, j->XEltToFFE(j, lp[i][1]));
        f := XDisc(Poly(PolyAlg(XIdealResidueField(lp[i][1])), f));
      until i>=Length(lp) or f = XEltToFFE(0, lp[i][1]);

      GASMAN("g");
      return not f = XEltToFFE(0, lp[i][1]);
    end;

    o := XOrderCoefOrder(O);
    l := List(c.cf[2], i->-PolyToList(i)[2]);
   
    Oe := O;

    k := Lcm([
         XOrderDeg(XEltOrder(l[1])), 
         XIdealMin(c.cf[3]), # should be a multiple of Norm()
         XOrderDeg(O)]
         );
    if XOrderCoefOrder(O)=Z then
      k := Lcm([k, Abs(XDisc(OrderEquationOrder(O)))]);
    else
      k := Lcm([k, Abs(XDisc(OrderEquationOrder(XOrderCoefOrder(O))))]);
    fi;
    p := 1000; #in ClassGroup we'll use primes up to 1000
    repeat
      p := NextPrime(p);
    until Gcd(p, k)=1 and test_disc(p);

    l := l{[1..2]};
   
    if XEltOrder(l[2]) = O.base then
      res := l;
    else
      a := [l[1]^0, l[1] mod p];
      for i in [3..XOrderDeg(O)] do
        a[i] := (a[i-1]*a[2]) mod p;
      od;

      l := [l[2]];
      Apply(l, EltToList);
      Apply(l, i->Flat(List(i, EltToList)));

      Apply(a, EltToList);
      Apply(a, i->Flat(List(i, EltToList)));

      o := XEltOrder(l[1][1]);
      P := XIdealFactor(XIdeal(o, p));

      res := [];
      for j in l do
        r := [];
        for i in P do
          c := Concatenation(Copy(a), -Copy([j]));
          c := MatToFp(c, i[1]);
          c := MatKernel(c);
          if MatRows(c)<1 then
            Error("kernel to small");
          fi;
          k := 1;
          while c[k][MatCols(c)]=0*c[k][MatCols(c)] do
            k:= k+1;
          od;

          c := c/c[k][MatCols(c)];
          c := MatFromFp(c, i[1]);
          c := MatToRowList(c)[k]{[1..XOrderDeg(O)]};
          if r=[] then
            r := [i[1], c];
          else
            for k in [1..XOrderDeg(O)] do
              r[2][k] := IdealChineseRemainder(r[1], i[1], r[2][k], c[k]) mod p;
            od;
            r[1] := r[1] * i[1];
          fi;
        od;
        c := Elt(Oe, r[2]) mod p;
        Add(res, c);
      od;
      Info("mod ", p, ": ", res, "\n"); # troubles if res=0....
      res := [OrderBasis(Oe, Oe)[2], res[1]];
      res[2] := EltNewtonLift(Oe, res[2], OrderPoly(O), p, -1, false);
      Info("Newton step OK\n");
    fi;

    if IsRec(Oe) then
      OrderAutomorphisms(Oe.base, res);
    else
      OrderAutomorphisms(Oe, res);
    fi;

    for i in [2..XOrderDeg(O)-1] do
      Add(res, EltAutomorphism(res[i], 2));
    od;
    Info("Iteration done\n");

    if IsRec(Oe) then
      OrderAutomorphisms(Oe.base, res);
    else
      OrderAutomorphisms(Oe, res);
    fi;

    if IsRec(Oe) then
      Oe.data := [[],res, p];
      Oe.operations := rec(Print := rec_print);
      Oe.map := artin_map;
      Oe.rg  := cf.cf[5].rg;
      Oe.inf := cf.cf[5].inf;
      if not IsBound(cf.cf[4]) then
        Error();
      else
        m := MatTrans(cf.cf[4][2]);
      fi;
      n := MatId(Z, Length(RayClassGroup(Oe.rg, Oe.inf)[2]));
      n := MatToRowList(n);
      t := List(cf.cf[4][1], i->i[2]);
      for i in n do
        s := MatSolve(m, MatTrans(Z, [i]));
        s := s[1];
        s := MatToColList(s)[1];
        Add(Oe.data[1], s*t mod XOrderDeg(Oe));
      od;
      return artin_map;
    fi;

    return true;
  end;


  EltAbsAutomorphism := function(arg) 
    local i, j, k, l, O, Oa, Oi, data, base, ext, elt, md, p;
    
    elt := arg[1];

    if IsInt(arg[Length(arg)]) then 
      md := true;
      p := arg[Length(arg)];
      Unbind(arg[Length(arg)]);
    else  
      md := false;
    fi;

    if Length(arg)>2 then
      base := arg[2];
      ext := arg[3];
      O := XEltOrder(ext);
    else
     data := arg[2];
     base := data[1];
     ext := data[2];
     if IsElt(ext) then   # change to rep mat!!
       data[2] := [ext^0];
       for i in [2..XOrderDeg(XEltOrder(ext))] do
         data[2][i] := data[2][i-1]*ext;
        od;
      fi;
      ext := false;
    fi;
    if base = 1 then
      if IsElt(ext) then
        return EltAutomorphism(elt, ext);
      else
        if IsInt(elt) then return elt; fi;
        elt := data[2]*EltToList(elt);
        return elt;
      fi;
    fi;
    if XOrderCoefOrder(XEltOrder(elt)) = Z then
      if md then
        return EltAutomorphism(elt, base) mod p;
      else
        return EltAutomorphism(elt, base);
      fi;
    fi;
    Oi := XEltOrder(elt);
    if md then
      elt := elt mod p;
    fi;
    elt := XEltMove(elt, OrderEquationOrder(Oi));
    elt := EltToList(elt);
    O := XEltOrder(elt[1]);
    Oa := OrderAbs(O);
    Apply(elt, i->XEltMove(i, Oa));
    Apply(elt, i->EltAutomorphism(i, base));
    Apply(elt, i->XEltMove(i, O));
    if IsElt(ext) then
      elt := Poly(PolyAlg(O), Reversed(elt));
      if md then
        ext := ext mod p;
        elt := EvalMod(elt, ext, p);
      else
        elt := Eval(elt, ext);
      fi;
    else
      if md then
        i := Copy(data[2]);
        Apply(i, j->j mod p);
        elt := i*elt;
      else
        elt := data[2]*elt;
      fi;
    fi;
    GASMAN("g");
    return elt; #######
    if md then
      return XEltMove(elt, Oi) mod p;
    else
      return XEltMove(elt, Oi);
    fi;
  end;

  aslist := function(l)
    if ForAll([1..Length(l)], x->IsBound(l[x])) then
      return;
    else
      Error("wrong list");
    fi;
  end;
  get_index := function(mu, O)
    local i,k;
    k :=0;
    for i in Reversed([1..Length(mu)]) do
      k := k*O.deg[i]+mu[i];
    od;
    return k;
  end;

  mult_mui := function(x, i, O)
    local e, i, j, r, d, k, n, l;
    d := List(O.deg, i->i-1);
    l := MultiCounterInit(Length(d));
    r := List(l, i->0);
    r[i] := 1;
    e := [];
    while MultiCounterInc(l, d) do
      if l[i]<d[i] then
        e[get_index(l+r, O)+1] := x[get_index(l, O)+1];
      else
        k := x[get_index(l, O)+1];
        l[i] := 0;
        e[get_index(l, O)+1] := XEltMove(O.mu[i], OrderAbs(O.oc))*k;
        l[i] := d[i];
      fi;
    od;
    aslist(e);
    return e;
  end;

  rep_mat := function(x, O)
    local i, j, k, l, m, d;
    m := [];
    d := List(O.deg, i->i-1);
    i := MultiCounterInit(Length(O.Oi));
    x := Copy(x);
    while MultiCounterInc(i, d) do
      k := x;
      for j in [1..Length(i)] do
        for l in [1..i[j]] do
          k := mult_mui(k, j, O);
        od;
      od;    
      Add(m, k);
    od;
    IsMat(m);
    m := MatTrans(m);
    return m;
  end;

  to_dist := function(x, i, O)
    local j,k,l,m;
    m := List([1..Product(O.deg)], j->0);
    l := List(O.Oi, j->0);
    x := EltToList(x);
    Apply(x, j->XEltMove(j, OrderAbs(O.oc)));
    for j in [1..O.deg[i]] do
      l[i] := j-1;
      m[get_index(l, O)+1] := x[j];
    od;
    
    aslist(m);
    return m;
  end;

  mult_gen := function(a,b, O)
    local m,i;
    m := rep_mat(a, O);
    IsMat(b);
    i := m * MatTrans(b);
    b := MatToRowList(b)[1];
    i := MatToColList(i)[1];
    return i;
  end;

  get_powers := function(a,d, O)
    local m, i, j;
    m := [Copy(a)];
    IsMat(m);
    m := [MatTrans(m)];
    j := rep_mat(a, O);
    for i in [2..d] do
      m[i] := j*m[i-1];
    od;
    Apply(m, i->MatToColList(i)[1]);
    m := Concatenation([List(a, i->0)], m);
    m[1][1] :=1;

    return m;
  end;

  mult_fast := function(a,b,O)
    local i,j,k,l,m, d;

    d := List(O.deg, i->-1);
    i := MultiCounterInit(Length(O.Oi));
    m := List([1..XOrderDeg(O)], k->0);
    while MultiCounterInc(i, l) do
      if a[get_index(i, O)+1] <> 0 then
        j := MultiCounterInit(Length(O.Oi));
        while MultiCounterInc(j, d) do
          if b[get_index(j, O)+1]<>0 then
            m[get_index(i+j, O)+1] := a[get_index(i, O)+1] *
                                      b[get_index(j, O)+1];
          fi;
        od;
      fi;
    od;
    aslist(m);
    return m;
  end;

  compute_auto := function(a, base, ext, O)
    local i, j, k, l,o,oe, d;
#Error();
    d := List(O.deg, l->l-1);
    ext := List([1..Length(ext)], i->get_powers(ext[i], d[i], O));
    i := MultiCounterInit(Length(O.Oi));
    l := [];
    while MultiCounterInc(i, d) do
      l[get_index(i, O)+1] := ext[1][i[1]+1];
      for j in [2..Length(O.Oi)] do
        l[get_index(i, O)+1] := mult_gen(l[get_index(i, O)+1], 
                                                        ext[j][i[j]+1], O);
      od;
    od;
    ext := l;
    a := Copy(a);
    for i in [1..Length(a)] do
      if not IsInt(a[i]) then
        o := XEltOrder(a[i]);
        oe := OrderEquationOrder(o);
        a[i] := EltAutomorphism(a[i], base);
      fi;
    od;
    l := ext[1]*a[1];
    for i in [2..Length(a)] do
      l := l+ext[i]*a[i];
    od;
    return l;
  end;

  ClassFieldAuto2 := function(Ob, c)
    local E, oc, oca, mu, alpha, i, j, k, l, smu, i0, im, pe, o, mui, imu, jj,
          ii, kk, ll, M, S, SS, O, jjj, oce, deg, p;

    if not IsBound(Ob.Oi) then
      O := [Ob];
    else
      O := Ob.Oi;
    fi;

    deg := Maximum(List(c, i->i.cf[5].deg));
    p := Factor(deg)[1][1];
    j := Position(List(c, i->i.cf[5].deg), deg);

    o := XOrderCoefOrder(O[1]);
    E := XEltOrder(PolyToList(c[j].cf[2][1])[2]);
    oc := XOrderCoefOrder(E);
    Ob.oc := oc;
    if oc<>Z then
      oca := OrderAbs(oc);
      oce := OrderEquationOrder(oca);
    else
      oca := oc;
      oce := oc;
    fi;


    if o=Z or Length(OrderAutomorphisms(o, []))=1 then
      return  false;     ###### TODO  ######
    fi;

    for i in c do
      S := XOrderCoefOrder(XEltOrder(PolyToList(i.cf[2][1])[2]));
      if S<>oc and S<>o then
        OrderInstallHom(S, OrderEquationOrder(oc), 
                      OrderBasis(OrderEquationOrder(oc))[2]^(deg/i.cf[5].deg));
      fi;
    od;

    # 1st find generators for the automorphism group of o
    # 2nd find extension of those in oca

    # 1st:

    i := GetGens(o); 

    Info("I seem to need ", i, " as gens\n");
    
    # 2nd
    i := Concatenation([1], i);
    smu := OrderAutomorphisms(o, []);
    smu := smu{i};
    
    Apply(smu, j->XEltMove(j, oca));
    j := List(OrderAutomorphisms(oca, []), k->EltAutomorphism(smu[1], k));
    k := [];
    for l in [2..Length(smu)] do
      Add(k, [i[l], Position(j, smu[l])]); 
    od;

    i0 := k;

    # smu should be a list of non trivial automorphism of the base field but
    # given as an automorphism of the Kummer base field

    # next one should compute the correct i before caling EltRootTest!
    # done! I simply need to understand my algorithm...
    # the order should be computed too (and passed to the next function)

    Info("Starting main loop\n");

    for ii in [1..Length(i0)] do
      S := [];
      SS := [];
      for jj in [1..Length(O)] do
        j := RayClassGroupCyclicFactors(Ob.rg, Ob.inf);
        Add(SS, List(j, kk->O[jj].map(kk[1], O[jj], 1)));
        Add( S, List(j, kk->O[jj].map(IdealAutomorphism(kk[1], i0[ii][1]),
                                             O[jj], 1)));
      od;
      if o<>oc then
        mui := OrderBasis(OrderEquationOrder(oc), oca)[2];
      else
        mui := XOrderTorsionUnit(o)^(XOrderTorsionUnitRank(o)/XOrderDeg(O[1]));
      fi;
      j := 1;
      imu := EltAutomorphism(mui, i0[ii][2]);
      k := mui; 
      while k<>imu and j < 100 do j := j+1; k := k*mui; od;
      if j=100 then
        Error("kill the automorphisms!");
      fi;
      SS := SS*j;
      jjj := [];
      for jj in [1..Length(O)] do
        M := Concatenation(S, -[SS[jj]]);
        IsMat(M);
        k := MatKernel(M, Ob.deg[jj]);
        if Length(k)>1 then 
          Error("wrong k, field probably not normal");
        fi;
        k := MatToRowList(k);
        k := k[1];
        k := k/k[Length(k)];
        Unbind(k[Length(k)]);
    
        j := 1;
        for kk in [1..Length(k)] do
          E := XEltOrder(PolyToList(c[kk].cf[2][1])[2]);
          mu := -PolyToList(OrderPoly(E))[XOrderDeg(E)+1];
          j := j*(1/mu)^k[kk];
        od;
        E := XEltOrder(PolyToList(c[jj].cf[2][1])[2]);
        mu := -PolyToList(OrderPoly(E))[XOrderDeg(E)+1];
        i0[ii][3] := XEltMove(EltAutomorphism(XEltMove(mu, oca), i0[ii][2]), 
                                   oc);
        j := j* i0[ii][3];
        j := EltRoot2(XEltMove(j, oca), XOrderDeg(E), c[jj].cf[3]);
        mu := List([1..Product(Ob.deg)], x->0);
        for kk in [1..Length(k)] do
          if Ob.deg[jj]<=Ob.deg[kk] then
            k[kk] := k[kk]*Ob.deg[kk] /Ob.deg[jj];
          else
            k[kk] := k[kk] / Ob.deg[kk];
            if not IsInt(k[kk]) then
              Error("yet another bug");
            fi;
          fi;
        od;
        mu[get_index(k, Ob)+1] := j; 
        pe := to_dist(-PolyToList(c[jj].cf[2][1])[2],jj, Ob);
        Add(jjj, [mu, pe, k]);
      od; 
#      Error();
      pe := Ob.m*List(jjj, j->j[2]);
      im := compute_auto(pe, i0[ii][2], List(jjj, j->j[1]), Ob);
      Info("Automorphism done\n");
      j := get_powers(pe, OrderDeg(Ob)-1, Ob);
      Info("powers done\n");
      Info("mat move\n");
      pe := j;
      Add(pe, im);
      if oc<>oca then
        Apply(pe, i->Flat(List(i, j->EltToList(XEltMove(j, oc)))));
      fi;
      Info("mat move done\n");

      IsMat(pe);
      if IsInt(pe[1][1]) or IsRat(pe[1][1]) then
        pe := MatMove(pe, Q);
        pe := MatKernel(pe);
      else
        pe := ModKernel(pe);
      fi;
      pe := MatToRowList(pe)[1]{[1..XOrderDeg(Ob)]}/pe[1][XOrderDeg(Ob)+1];
      pe := -Elt(Ob, pe);

      im := OrderAutomorphisms(o, [])[i0[ii][1]];
      i0[ii] := [im, pe];
    od;
    return i0;
  end;

  AllAutos := function(O, d)
    local auto, aut_mult, aut_eq, l, o, g, i, j, k, ii, rel, op, aut_print, b,
          aut_pow, cl, om, h, t, H, aut_gt, aut_lt, e;

    aut_print := function(i)
      Print(i.r);
    end;

  # use dimino and compare mod p first!!!!!
    aut_mult := function(li) #first check mod p
      local i, c, d, e, a,b, s, p;
      a := li[1]; b := li[2];
      if a.r = "e" then
        return b;
      elif IsRec(b) and b.r = "e" then
        return a;
      fi;
      if IsElt(b) then
        if not IsBound(a.data) then
          a.data := [a.im, a.pe];
        fi;
        return EltAbsAutomorphism(b, a.data);
      elif XIsIdeal(b) then
        if not IsBound(a.data) then
          a.data := [a.im, a.pe];
        fi;
        return XIdeal(EltAbsAutomorphism(IdealGen(b, 1), a.data),
                     EltAbsAutomorphism(IdealGen(b, 2), a.data));
      else
        s := Concatenation(a.r, "*", b.r);
        c := Position(List(a.g, i->i.r), s);
        if c<>false then
          return a.g[c];
        fi;
        if not IsBound(b.data) then
          b.data := [b.im, b.pe];
        fi;
        c := EltAbsAutomorphism(a.im, b.data);
        d := EltAbsAutomorphism(a.pe, b.data);
        c := rec(im := c, pe := d, r:= s, operations := a.operations, g := a.g);
        d := Position(a.g, c);
        if d <> false then
          return a.g[d];
        else
          return c;
        fi;
      fi;
    end;

    aut_eq := function(a,b)
      local s;
      if IsElt(b) and IsRec(a) then
        if XEltOrder(a.im) = XEltOrder(b) then
          if a.im = b and 
             Position(OrderAutomorphisms(XEltOrder(a.pe), []), a.pe) = 1 then
            return true;
          else
            return false;
          fi;
        elif XEltOrder(a.pe) = XEltOrder(b) then
          if a.pe = b and 
             ( a.im = 1 or 
             Position(OrderAutomorphisms(XEltOrder(a.im), []), a.im) = 1) then
            return true;
          else
            return false;
          fi;
        fi;
      elif IsString(b) and IsRec(a) then
        return b=a.r;
      elif IsElt(a) and IsRec(b) then
        if XEltOrder(b.im) = XEltOrder(a) then
          if b.im = a and 
             Position(OrderAutomorphisms(XEltOrder(b.pe), []), b.pe) = 1 then
            return true;
          else
            return false;
          fi;
        elif XEltOrder(b.pe) = XEltOrder(a) then
          if b.pe = a and 
             (b.im = 1 or
             Position(OrderAutomorphisms(XEltOrder(b.im), []), b.im) = 1) then
            return true;
          else
            return false;
          fi;
        fi;
      elif IsString(a) and IsRec(b) then
        return a=b.r;
      elif a.im = b.im and a.pe = b.pe then
        if a.r = b.r then
          Info("trivial relation\n");
        else
          if a.r="e" then
            s := b.r;
          elif b.r = "e" then
            s := a.r;
          else
            s := Concatenation("(", a.r, ")^-1*", b.r);
            IsString(s);
          fi;
          Info("relation: ", a.r, " = ", b.r, "\n");
          if Position(rel, s)=false then
            Add(rel, s);
          fi;
        fi;
        return true;
      else
        return false;
      fi;
    end;

    aut_pow := function(q, n)
      local i, j, k, t, p, e;

      if n=0 then
        return q.g[1];
      fi;

      if n<0 then 
        n := -n;
        i := 1;
        e := q.g[Position(q.g, "e")];
        while i<= Length(q.g) do  # should be done in a better way
          if q.g[i]*q=e then
            q := q.g[i];
            i := Length(q.g)+10;
          fi;
          i := i+1;
        od;
      fi;

      p := q;
      for i in [2..n] do
        p := p*q;
      od;

      return p;
    end;

    aut_lt := function(a,b)
      return a.r<b.r;
    end;

    aut_gt := function(a,b)
      return a.r>b.r;
    end;

    op := rec(Print := aut_print, 
      \^ := aut_pow,
      \* := function(q,w) return aut_mult([q,w]); end,
      \/ := function(q,w) return q*w^(-1); end,
      \= := aut_eq,
      \> := aut_gt,
      \< := aut_lt);
   
    b := "abcdfghijklmnopqrstuvwxyz";
    o := XOrderCoefOrder(O);
    if d<> false then
      for i in [1..Length(d)] do
        if IsInt(d[i][1]) then 
          d[i][1] := OrderAutomorphisms(o, [])[d[i][1]]; fi; 
      od;
    fi;

    l := [];
    if o=Z then
      e := rec(im := 1, pe := OrderAutomorphisms(O, [])[1],
                 r := "e", operations := op, g := l);
    else
      e := rec(im := OrderAutomorphisms(o, [])[1], 
             pe := OrderAutomorphisms(O, [])[1],
                    r := "e",  operations := op, g := l);
    fi;
    Add(l, e);
    if IsRec(O) and IsBound(O.gens) then
      i := O.gens;
    else
      i := GetGens(O); 
    fi;
    g := [];
    for j in [1..Length(i)] do
      if o=Z then
        Add(g, rec(im := 1,
              pe := OrderAutomorphisms(O, [])[i[j]],
                        r := [b[j]], g := l, operations := op));
      else
        Add(g, rec(im := OrderAutomorphisms(o, [])[1],
              pe := OrderAutomorphisms(O, [])[i[j]],
                        r := [b[j]], g := l, operations := op));
      fi;
    od;
    if d<>false then
      Append(g, List([1..Length(d)],
               j->rec(im := d[j][1], pe := d[j][2], r := [b[j+Length(i)]],
               operations := op, g := l)));
    fi;

    
    rel := [];
    i := 1;
    while Length(l)<OrderDegAbs(O) and i<= Length(g) do
      H := ShallowCopy(l);
      cl := Set([e]);
      repeat
        t := cl[1];
        SetRemove(cl, t);
        for j in [1..i] do
          om := t*g[j];
          if Position(l, om)=false then
            for k in H do
              SetAdd(l, k*om);
            od;
            Info("Length now: ", Length(l), "\n");
            SetAdd(cl, om);
          fi;
        od;
      until Length(cl)=0;
      i := i+1;
    od;
    List(rel, IsString);
    return [l, rel];
  end;  

 
  artin_map := function(arg)
    local i, j, k, l, data, I, O, res; 

    I := arg[1];
    O := arg[2];
    

    data := O.data;
    #data [1][1+i+deg*j+...] automorphism corresponding to the i'th auto
    #                        in O.Oi[1], the j'th in O.Oi[2],...#
    #                         i, j, ... start at 0
    i := IdealRayClassRep(I, O.rg, O.inf); 
    k := 0;
    for l in [1..Length(O.Oi)] do
    #for l in Reversed([1..Length(O.Oi)]) do
      res := 0;
      for j in [1..Length(O.Oi[l].data[1])] do
        res := res + i[1][j] * O.Oi[l].data[1][j];
      od;
      res := res mod XOrderDeg(O.Oi[l]);
      k := k*XOrderDeg(O.Oi[l])+res; 
    od;
    if Length(arg)>2 and arg[3] = "raw" then
      return k;
    elif Length(arg)>2 and arg[3] = "auto" then
      return OrderAutomorphisms(O, [])[data[1][k+1]];
    else
      return O.g[1][Position(O.g[1], OrderAutomorphisms(O, [])[data[1][k+1]])];
    fi;
  end;
  main := function(l)
    local i, j, k, l, cf, O, Ot, Os, deg;
    cf := l[1];
    Sort(cf, function(a,b)
               return Factor(a.cf[5].deg)[1][1]< Factor(b.cf[5].deg)[1][1];
             end);
    i := 1;
    deg := Factor(cf[i].cf[5].deg)[1][1];
    for j in [1..Length(cf)+1] do
      if j>Length(cf) or  Factor(cf[j].cf[5].deg)[1][1] <> deg then
        Ot := RayClassFieldAuto(cf{[i..j-1]}, "raw");
        if not IsBound(O) then
          O := Ot;
          i := j;
          if i<=Length(cf) then
            deg := Factor(cf[i].cf[5].deg)[1][1];
          fi;
        else
          #merge all relevant data

          # 1st: the fields:
          d := RelOrderMergeWithAutomorphisms(O, Ot, "with s", "fields");
          Os := rec(base := d[1], operations := Ot.operations);

          # 2nd: sort the auto's => map will work!
          Os.Oi := Concatenation(O.Oi, Ot.Oi);
          data := [];
          for k in [1..Length(O.data[1])] do
            k := XEltMove(OrderAutomorphisms(O.base, [])[O.data[1][k]], d[1]);
            for l in [1..Length(Ot.data[1])] do
              l := Ot.data[1][l];
              if IsInt(l) then
                l := OrderAutomorphisms(Ot, [])[l];
              fi;
              Add(data, Position(OrderAutomorphisms(d[1], []),
                   k+d[2]*XEltMove(l, d[1])));
            od;
          od;
          if ForAny(data, x->x=false) then
            Error("Position was false");
          fi;
          Os.map := artin_map;
          Os.data := [data];
          Info(Ot.gens, "\t", O.gens, "\t", XOrderDeg(O), "\n");
          Os.gens := Concatenation(O.gens, 
                                      List(Ot.gens, l->(l-1)*XOrderDeg(O)+1));
          #3rd sort the O?.d's. They should be in the same order!
          if O.d <> false then
            Os.d := List([1..Length(O.d)],
                          k->[O.d[k][1], XEltMove(O.d[k][2], Os)+
                                              d[2]*XEltMove(Ot.d[k][2], Os)]);
          else
            Os.d := false;
          fi;
          Os.rg := O.rg;
          Os.inf := O.inf;
          if j<=Length(cf) then
            deg := Factor(cf[j].cf[5].deg)[1][1];
          fi;
          O := Os;
          i := j;
        fi;
      fi;
    od;
    g := AllAutos(O, O.d);
    Unbind(O.d);
    O.g := g;
    return [O, O.g[1]];
  end;
  if Length(arg) = 1 or arg[2]<> "raw" then
    return main(arg);
  fi;

  rec_print := function(i)
    Print(i.base);
  end;


  O := rec(Oi := [], mu := [], gens:= [], deg := [],
           operations := rec(Print := rec_print));
  sb := []; 
  for i in [1..Length(cf)] do
    if sb<>cf[i].cf[5] then
      sbi := 1;
      sb := cf[i].cf[5];
    else
      sbi := sbi+1;
    fi;
    if PolyDeg(cf[i])=2 or  ### TODO find a better criterion then IsIdeal
      IsIdeal(cf[i].cf[5].rg) and XOrderCoefOrder(cf[i].cf[5].Or) = Z then
      O.Oi[i] := XEltOrder(PolyToList(cf[i].cf[2][1])[2]);
    else
      d := KASHLEVEL("factor");
      KASHLEVEL("factor", 0);
      O.Oi[i] := Order(cf[i]);
      KASHLEVEL("factor", d);
    fi;
    O.Oi[i] := rec(base := O.Oi[i], offset := sbi);
    O.mu[i] := cf[i].cf[5].unramified[cf[i].cf[7]][1];
    O.deg[i] := cf[i].cf[5].unramified[cf[i].cf[7]][2];
    ClassFieldAuto1(O.Oi[i], cf[i]);
    if not IsBound(O.base) then
      O.base := O.Oi[1];
      O.data := [];
      for k in O.Oi[1].data[2] do
        Add(O.data, Position(OrderAutomorphisms(O.Oi[i], []), k));
      od;
      O.data := [O.data];
      O.map := artin_map;
      O.m := [1];
      O.gens := [2];
    else
      d := RelOrderMergeWithAutomorphisms(O.base, O.Oi[i], "fields", "with s");
      data := [];
      for k in [1..Length(O.data[1])] do
        k := XEltMove(OrderAutomorphisms(O.base, [])[O.data[1][k]], d[1]);
        for j in [1..Length(O.Oi[i].data[2])] do
          j := O.Oi[i].data[2][j];
          Add(data, Position(OrderAutomorphisms(d[1], []),
               k+d[2]*XEltMove(j, d[1])));
        od;
      od;
      O.m := Concatenation(O.m, [d[2]]);
      O.base := d[1];
      O.data := [data];
    fi;
  od;
  O.gens := [];
  j :=1;
  #for i in [1..Length(cf)] do
  for i in Reversed([1..Length(cf)]) do
    Add(O.gens, O.data[1][1+j]);
    j := j*PolyDeg(cf[i]);
  od;
  O.rg := O.Oi[1].rg;
  O.inf := O.Oi[1].inf;
  d := ClassFieldAuto2(O, cf);
  Unbind(O.oc);
  Unbind(O.mu);
  Unbind(O.m);
  O.d := d;
  return O;
end; 

#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       RayClassFieldArtin 
#
#SHORTDOC  Given an ideal, this function computes the corresponding automorphism. 
#
#LONGDOC   This function essentially provides $(\a, O/o)\in\text{Gal}(O/o)$
#          for given unramified ideals. Note, that this function won't check
#          if the conductor of the abelian extension is known. Therefore
#          it is only possible to compute automorphisms for ideals coprime
#          to the defining module.\par
#          Note, that this function won't work on an arbitraily defined abelian
#          extensions {\tt O} of {\tt o}. It is necessary to compute {\tt O}
#          using {\tt RayClassFieldAuto}.
#
#CALLING   aut := RayClassFieldArtin(id, O);
#
#ARGUS     aut @ an automorphism
#          id @ ideal @ coprime to the defining module
#          O  @ order @ output of {\tt RayClassFieldAuto}
#
#CSRC      
#
#DEBUG     Info := Print;
#
#XREF      RayClassFieldAuto, RayClassField
#
#PACKAGE   lib/RCLF
#
#KASH      @o := OrderMaximal(Z, 2, 10);
#          i := 9*o;
#          O := RayClassFieldAuto(RayClassField(i))[1];
#          l := RayClassGroupCyclicFactors(i);
#          ap := RayClassFieldArtin(l[1][1], O);
#          ap2 := RayClassFieldArtin(l[1][1]^2, O);
#          ap^2 = ap2;
#          OrderAutomorphisms(o);
#          RayClassFieldArtin(l[2][1], O);
#          RayClassFieldArtin(IdealAutomorphism(l[2][1], 2), O);
#
#KASH      
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY**HISTORY***HISTORY***HISTORY***HISTORY
#
#**C***C***|***C***C***C***C***C***C**C***C***C***C***C***C***C***C***C***C***/
RayClassFieldArtin := function(a,b)
  if IsRec(b) and IsBound(b.map) then
    return b.map(a, b);
  else
    Error("usage: RayClassFieldArtin(ideal, Order) and Order must come from ClassFieldAuto!");
  fi;
end;



