#RandomSeed(1);

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#
#FXN       RandomElt
#
#SHORTDOC  Computes a ``random'' element of a given order.
#
#LONGDOC   By default a ``random'' element with coefficients in $[-20,20]$
#          will be returned. Of course, it is not possible to do a truly
#          random element from an infinite set.
#
#CALLING   alpha := RandomElt(R [, l | b ]);
#
#ARGUS     alpha @ element @ of {\tt o}
#          R @ ring @ may be an order, an ideal, $\Z$, a module or an function field order. 
#          l @ list @ of integers
#          b @ integer @ equivalent to $l := [-b..b]$
#       
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      @o := Order(Z, 5, 3);
#          RandomElt(o);
#          RandomElt(o, 100);
#          RandomElt(o, [-1,1]);
#
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
RandomElt := function(arg)
  # random element of a KANT order, ideal, module, ring, or Z (first argument)
  # optional second argument: range for integral coefficients or maximal
  #   absolute value of integer coefficients
  local n, o, l, M, I, O;

  if IsList(arg[1]) then arg := arg[1]; fi;
  o := arg[1];

  if Length(arg)>1 then 
    if IsList(arg[2]) then 
      l := arg[2]; 
    elif IsInt(arg[2]) then
      l:= [-arg[2]..arg[2]];
    else l := [-20..20]; fi;
  else l := [-20..20]; fi;

  if o=Z or TYPE(o)="KANT ring" then
    return RandomList(l);
  elif TYPE(o)="KANT ideal" then
    return Sum(IdealGenerators(o),g->g*RandomElt(IdealOrder(o), l));
  elif TYPE(o)="KANT module" then
    M:=MatToColList(ModuleMatrix(o));
    I:=ModuleIdeals(o);
    return MatTrans(ModuleOrder(o),
                   [Sum([1..Length(I)],i->M[i]*RandomElt(I[i],l))]);
  elif TYPE(o)="KANT order" then
    n := OrderDeg(o);
    return Elt(o, List([1..n], i->RandomElt(OrderCoefOrder(o), l)));
  elif TYPE(o)="KANT function field order" then
    n := AlffDeg(AlffOrderAlff(o));
    return AlffElt(o, List([1..n], i->RandomList(l)));
  elif IsFF(o) then
    I := FFSize(o);
    return Eval(Poly(Zx, List([1..I[2]], i->RandomList([1..I[1]]))),
                FFGenerator(o));
  elif IsPolyAlg(o) then
    return RandomPoly(PolyAlgCoef(o), [0..20]);
  else Error("Sorry, not supported\n");
  fi;
end;


#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#
#FXN       RandomMatrix
#
#SHORTDOC  Using {\tt RandomElt} a square matrix with ``random'' entries is returned.
#
#LONGDOC   
#
#CALLING  m := RandomMatrix(R, n, l); 
#
#ARGUS     m @ matrix @ in ${\tt R}^{{\tt n} \times {\tt n}}$
#          R @ ring @ may be an order, an ideal, $\Z$, a module or an function field order. 
#          n @ integer
#          l @ list @ of possible (coeffiecents of the) entries.
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      @m := RandomMatrix(Z, 5);
#          m := RandomMatrix(Z, 5);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
RandomMatrix := function(arg)
  local m, i, j, mm, o, n, l, ar;

  ar := Copy(arg);
  o := ar[1];
  n := ar[2];
  if Length(ar)>2 then
    ar[2] := ar[3];
    Unbind(ar[3]);
  else
    Unbind(ar[2]);
  fi;

  m := [];
  for i in [1..n] do
    mm := [];
    for j in [1..n] do
      Add(mm, RandomElt(ar));
    od;
    Add(m, mm);
  od;
  m := Mat(o, m);

  return m;
end;


#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#
#FXN       RandomPoly
#
#SHORTDOC  Produces a monic random polynomial of given degree.
#
#LONGDOC   
#
#CALLING   f := RandomPoly(R, d [, l]);
#
#ARGUS     f @ polynomial @ in ${\tt R}[x]$
#          R @ ring @ may be an order, an ideal, $\Z$, a module or an function field order. 
#          l @ list @ of possible (coeffiecents of the) coefficients.
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      @f := RandomPoly(Z, 3);
#          o := Order(f);
#          g := RandomPoly(o, 4);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
RandomPoly := function(arg)
local f,eq_o,List,i, o, deg, l, ox, ar;

#Zufaellig erzeugtes Polynom

if IsList(arg[1]) then ar := Copy(arg[1]); else ar := Copy(arg); fi;

o := ar[1];
ox := PolyAlg(o);

deg := ar[2];
if Length(ar)>2 then
  ar[2] := ar[3];
  Unbind(ar[3]);
else
  Unbind(ar[2]);
fi;

if TYPE(o) = "KANT function field order" then
  List := [AlffElt(o, 1)];
else
  List := [1];
fi;
for i in [2..deg+1] do
  Add (List, RandomElt(ar));
od;
f := Poly(ox,List);

return f;

end;


#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#FXN       RandomOrder
#
#SHORTDOC  Using {\tt RandomPoly} a random order is computed.
#
#LONGDOC   
#
#CALLING   o := RandomOrder(R, n [, l]);
#
#ARGUS     o @ order
#          R @ ring @ may be $\Z$, an order or an functions field order.
#          n @ integer @ degree
#          l @ list
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      @o := RandomOrder(Z, 4);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
RandomOrder := function(arg)
local f,i, o, deg, l, ox, ar;

#Zufaellig erzeugte GleichungsOrdnung, mit Polynom vom Grad deg
#und Koeffizienten, die durch maxcoef beschraenkt sind.

ar := Copy(arg);

repeat
  f := RandomPoly(ar);
until PolyIsIrreducible(f);

return Order(f);

end;
