if not IsBound(OrderCyclotomicAdd) then
  ReadLib("RCLF/RayField");
fi;

_RecPrint := function(a)
  Print("Record of type ", a.Type, "\n");
end;

MatHermiteRowUpper := x->MatTrans(MatHermiteColLower(MatTrans(x)));
MatHermiteRowUpperTrans := function(m)
  m := MatHermiteColLowerTrans(MatTrans(m));
  return [m[1], MatTrans(m[2]), MatTrans(m[3])];
end;

MatHermiteRowReduce := function(m)
  local i;
  m := MatHermiteRowUpper(m);
  m := MatToRowList(m);
  i := Filtered(m, i->i<>0*i);
  if i=[] then
    i := [m[1]];
  fi;
  m := Mat(Z, i);
  return m;
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       IdealPrimeCountInit
#
#SHORTDOC  Initializes an environment for enumerating prime ideals
#
#LONGDOC   
#
#CALLING   s := IdealPrimeCountInit(o[, m);
#
#ARGUS     s @ record
#          o @ order
#          m @ integer @ only primes coprime to {\tt m} are used.
#
#CSRC      
#
#DEBUG    
#
#XREF      IdealPrimeCountNext 
#
#PACKAGE   
#
#EXAMPLE   See {\tt IdealPrimeCountNext} for exmaples.         
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
IdealPrimeCountInit := function(arg)
  local i, j, k, s;
  s := rec(Type := "IdealPrimeCount",
           o := arg[1], 
           operations := rec(Print := _RecPrint));
  if Length(arg)>1 then
    s.m := arg[2];
  else
    s.m := 1;
  fi;
  s.p := 1;
  s.c := 0;
  s.lp := [];

  return s;
end;


#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       IdealPrimeCountNext
#
#SHORTDOC  Enumerates the next prime ideal.
#
#LONGDOC   Be careful with relative extensions: index divisors won't work here!
#
#CALLING   p := IdealPrimeCountNext(s [, 1]);
#
#ARGUS     p @ prime ideal
#          s @ record @ from IdealPrimeCountInit
#          1 @ if present, p will be a pair $[p, f]$ with p a prime of the coefficient order and {\tt f} the degree of {\tt p}
#
#CSRC      
#
#DEBUG    
#
#XREF      IdealPrimeCountInit
#
#PACKAGE   
#
#KASH      We'll work in the field $\Q(\sqrt 2)$:@
#          o := OrderMaximal(Z, 2, 2);
#          s := IdealPrimeCountInit(o);
#          repeat
#            p := IdealPrimeCountNext(s);
#            Print(p, "\n");
#          until IdealMin(p)>20;
#
#          @Next, only the factorization shape of ideals coprime to $2$ is 
#          investigated:@
#          s := IdealPrimeCountInit(o, 2);
#          repeat
#            fs := IdealPrimeCountNext(s, 1);
#            Print(fs, "\n");
#          until fs[1]>20;
#
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
IdealPrimeCountNext := function(arg)
  local i, j, k, l, p, fp, i, j, s, norm, P;
  
  s := arg[1];
  if Length(arg)>1 then norm := 3; else norm := 1; fi;
  s.c := s.c+1;

  if s.c<=Length(s.lp) then
    return s.lp[s.c][norm];
  fi;

  repeat
    s.p := NextPrime(s.p);
  until IntGcd(s.p, s.m) = 1;
  
  if s.o=Z then
    s.lp := [[ZIdealCreate(s.p), 1, [s.p, 1]]];
  elif OrderCoefOrder(s.o) = Z then
    s.lp := Factor(s.p*s.o);
    if norm = 3 then
      s.lp := [[, s.lp[1][2], [s.p, IdealDegree(s.lp[1][1])]]];
    else
      Apply(s.lp, i->[i[1], i[2], Norm(i[1])]);
    fi;
  else
    if not IsBound(s.rel) then
      s.rel := IdealPrimeCountInit(OrderCoefOrder(s.o), s.m);
    fi;
    p := IdealPrimeCountNext(s.rel);
    fp := IdealResidueField(p);
    s.lp := PolyToList(OrderPoly(s.o));
    Apply(s.lp, i->EltToFFE(i, p));
    s.lp := Factor(Poly(PolyAlg(fp), s.lp));
    if norm=3 then 
      s.lp := [[, s.lp[1][2], [p, PolyDeg(s.lp[1][1])]]];
    else
      Apply(s.lp, i->[List(PolyToList(i[1]), j->FFEToElt(j, p)), i[2]]);
      P := IdealMove(p, s.o);
      j := OrderBasis(OrderEquationOrder(s.o))[2];
      j := EltMove(j, s.o);
      Apply(s.lp, i->[P+Eval(Poly(PolyAlg(s.o), i[1]), j)*s.o, i[2], 
                          p^(Length(i[1])-1)]);
       

    fi;
  fi;
  s.c := 1;
  return s.lp[s.c][norm];
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       MatIndex
#
#SHORTDOC  Computes the determinant of a row normal form of an integral matrix.
#
#LONGDOC   Suppose we have $\Z$ modules $M|1\subset M|2$ given via a basis of $M|2$
#          and a set of elements generating $M|1$. Furthermore let $m$ be the
#          matrix which rows contain the aforementioned generators. {\tt MatIndex}$(m)$
#          will compute the index of $M|1$ in $M|2$. If the index is not finite, a $0$ 
#          will be returned.
#
#CALLING   n := MatIndex(m); 
#
#ARGUS     n @ integer @ index
#          m @ matrix @ over $\Z$
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      @
#          m := RandomMatrix(Z, 3);
#          MatIndex(m);
#          m := m{[1..2]};IsMat(m);
#          MatIndex(m);
#          m := Concatenation(m, RandomMatrix(Z, 3));IsMat(m);
#          MatIndex(m);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
MatIndex := function(m)
  local i;
  IsMat(m);
  if MatRows(m)<MatCols(m) then
    return 0;
  fi;
  m := MatHermiteRowUpper(m);
  return Abs(Product(List([1..MatCols(m)], i->m[i][i])));
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       AbelianFieldToRCF
#
#SHORTDOC  Computes the data neccessary to get an abelian field as an Ray Class Field.
#
#LONGDOC   
#
#CALLING   rc := AbelianFieldToRCF(o [, I]);
#
#ARGUS     rc @ list @ $[$ ideal, inf, relations $]$
#          o @ order @ of an abelian field with known automorphisms
#          I @ ideal @ of the coef. order of {\tt o}. Must be a multiple of\
#                          the conductor.
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      Suppose we know that the field generated by a root of $x^4-4x^2+1$
#          is abelian, and we'd like to work with this field:@
#          o := OrderMaximal(x^4-4*x^2+1);
#          OrderAutomorphisms(o);
#          rc1 := AbelianFieldToRCF(o, Disc(o));
#          f := RayConductor(rc1);
#          rc2 := AbelianFieldToRCF(o, f[1]);
#          rcf := RayClassField(rc2);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
AbelianFieldToRCF := function(arg)
  local i, j, k, l, o, s, p, m, d, dZ, inf, IsZ, pc, O, ok;

  O := arg[1];
  o := OrderCoefOrder(O);
  if o=Z then IsZ := true; else IsZ := false; fi;

  d := Disc(O);
  if Length(arg)>1 then d := arg[2]; fi;
  if IsZ and IsRecType(d, "ZIdeal") then
    d := Min(d);
  fi;
  # reduce d ...

  if IsZ then
    dZ := Abs(d);
    d := ZIdealCreate(d);
  else
    dZ := Norm(d)*Norm(Disc(O));
  fi;

  if IsZ then
    inf := OrderSig(O);
    if inf[1]=0 then inf := [1];
                else inf := [];
    fi;
  else
    inf := List([1..OrderSig(o)[1]], x->x);
  fi;

  pc := 0;
  s := IdealPrimeCountInit(O, dZ);
  m := MatDiag(Z, RayClassGroup(d, inf)[2]);
  ok := 20;
  repeat
    p := IdealPrimeCountNext(s, 1);
    pc := pc+1;
    if IsZ then
      Add(m,  IdealRayClassRep(ZIdealCreate(p[1]), d, inf)*p[2]);
    else
      Add(m , IdealRayClassRep(p[1], d, inf)*p[2]);
    fi;
    IsMat(m);
    m := MatHermiteRowUpper(m);
    if MatCols(m)<MatRows(m) then
      m := m{[1..MatCols(m)]};
      IsMat(m);
    fi;
    if pc>200 then
      i := RayClassGroupCyclicFactors(d, inf);
      for k in i do
        l := Factor(O*k[1].Generator);
        for j in l do
          Add(m, IdealRayClassRep(ZIdealCreate(Norm(j[1])), d, inf));
        od;
      od;
      IsMat(m);
    fi;
    i := MatIndex(m)-OrderDeg(O);
    if i=0 then
      ok := ok-1;
    elif i<0 then
      Error("field not abelian or wrong conductor");
    fi;
      
  until ok=0;

  return RayClassGroupToAbelianGroup(d, inf, m);
end;


#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       AbelianGroupEnumInit
#
#SHORTDOC  Generates a environment for the enumeration of all elements of a finite abelian group $G$.
#
#LONGDOC   
#
#CALLING   s := AbelianGroupEnumInit(G [, "gen"]);
#
#ARGUS     s @ record
#          G @ AbelianGroup
#          gen @ arbitray @ if present, only a set of generators will be enumerated.
#
#CSRC      
#
#DEBUG    
#
#XREF      AbelianGroupEnumNext 
#
#PACKAGE   
#
#EXAMPLE   See {\tt AbelianGroupEnumNext} for an example. 
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
AbelianGroupEnumInit := function(arg)
  local i, j, k, g, gen, G;

  G := arg[1];
  if Length(arg)>1 then
    gen := true;
  else
    gen := false;
  fi;
  
  g := AbelianGroupSmithCreate(G, false);
  i := rec(Type := "AbelianGroupEnum",
           G := G,
           g := g, 
           vec := MultiCounterInit(Length(g.Relations)),
           bound := List([1..Length(g.Relations)], j->g.Relations[j][j]),
           operations := rec(Print := _RecPrint)
           );
  if gen then
    Apply(i.bound, j->Minimum(j-1, 1));
  else
    Apply(i.bound, j->j-1);
  fi;
  i.elt := AbelianGroupEltMove(AbelianGroupEltCreate(g, Copy(i.vec)), G);
  return i;
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       AbelianGroupEnumNext
#
#SHORTDOC  Computes ``the next'' element of an abelian group.
#
#LONGDOC   
#
#CALLING   flag := AbelianGroupEnumNext(s);
#
#ARGUS     flag @ boolean @ {\tt true} iff there is a valid element in {\tt s}
#          s @ record @ generated by {\tt AbelianGroupEnumInit}
#          s.elt @ AbelianGroupElt
#
#CSRC      
#
#DEBUG    
#
#XREF      AbelianGroupEnumInit
#
#PACKAGE   
#
#KASH      We'll produce an ideal in every ray modulo $9$ in $\Q(\sqrt {10})$:@
#          o := OrderMaximal(Z, 2, 10);
#          G := RayClassGroupToAbelianGroup(9*o);
#          s := AbelianGroupEnumInit(G);
#          while AbelianGroupEnumNext(s) do
#            Print(s.elt, "\t", AbelianGroupDiscreteExp(s.elt), "\n");
#          od;
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
AbelianGroupEnumNext := function(s)
  local i, j;

  if MultiCounterInc(s.vec, s.bound) then
    s.elt := AbelianGroupEltMove(AbelianGroupEltCreate(s.g, Copy(s.vec)), s.G);
    return true;
  else
    return false;
  fi;
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       AbelianGroupEltRandom
#
#SHORTDOC  Generates a random element of an abelian group.
#
#LONGDOC   
#
#CALLING   e := AbelianGroupEltRandom(G);
#
#ARGUS     e @ AbelianGroupElt
#          G @ AbelianGroup
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      We'll produce some representatives for random  rays modulo $9$ in $\Q(\sqrt {10})$:@
#          o := OrderMaximal(Z, 2, 10);
#          G := RayClassGroupToAbelianGroup(9*o);
#          for i in [1..10] do
#            a := AbelianGroupEltRandom(G);
#            Print(a, "\t", AbelianGroupDiscreteExp(a), "\n");
#          od;
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
AbelianGroupEltRandom := function(g)
  local s, t, smith;
  if not IsRecType(g, "AbelianGroup") then
    return Error("usage: AbelianGroupEltRandom( group );");
  fi;

  smith := AbelianGroupSmithCreate(g, false);
  t := [];
  for s in [1..Length(smith.Relations[1])] do
    Add(t, RandomList([1..smith.Information.Orders[s]]));
  od;

  return AbelianGroupEltMove(AbelianGroupEltCreate(smith, t), g);
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       AbelianRayClassGroupAutoCreate
#
#SHORTDOC  Given an field automorphism, this function generates the 
#          corresponding automorphsim of abelian groups.
#
#LONGDOC   
#
#CALLING   aut := AbelianRayClassGroupAutoCreate(G, sigma);
#
#ARGUS     aut @ AbelianGroupHom
#          G   @ AbelianGroup @ must be RayClassGroupToAbelianGroup
#          sigma @ KASH function @ acting on ideals
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      We'll demonstrate this in $\Q(\sqrt{10})$:@
#          o := OrderMaximal(Z, 2, 10);
#          G := RayClassGroupToAbelianGroup(27*o);
#          OrderAutomorphisms(o);
#          aut := AbelianRayClassGroupAutoCreate(G, x->IdealAutomorphism(x, 2));
#      
#          @This should be equivalent to (but faster for larger groups):@
#          l := AbelianGroupGenerators(G);
#          Apply(l, AbelianGroupDiscreteExp);
#          Apply(l, x->IdealAutomorphism(x, 2));
#          Apply(l, x->AbelianGroupDiscreteLog(G, x));
#          Apply(l, x->x.expvec);
#          IsMat(l);
#          aut2 := AbelianGroupHomCreate(G, G, l);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
AbelianRayClassGroupAutoCreate := function(g, sigma)
  local o, i, j, k, l, m, n, s, lp, m1, m2, dLog;

  if IsBound(g.Information.Module) and
     IsRecType(g.Information.Module, "ZIdeal") then
    o := Z;
  else
    o := IdealOrder(g.Information.RCG.m0);
  fi;
  n := Norm(g.Information.RCG.m0);

  dLog := AbelianGroupDiscreteLog;
   

  if not IsBound(g.Information.PrimeGens) then
    s := IdealPrimeCountInit(o, n);
    AbelianGroupOrder(g);
    l := [];
    m1 := Copy(g.Relations);
    i := IdealPrimeCountNext(s);
    lp := rec(Primes := [], Logs := []);
    m2 := 0;
    repeat
      k := i;
      j := dLog(g, k);
#      Print(j,"\n");
      Add(lp.Primes, k);
      Add(lp.Logs, j.expvec);
      Add(m1, Copy(j.expvec));
      i := IdealPrimeCountNext(s);
      if Min(i)<>Min(k) then
        m := MatIndex(m1);
#        Print(m, "\t", MatIndex(m1), "\t", m2, "\n");
        if m<m2 or m2=0 or m2=AbelianGroupOrder(g) then
          Add(l, lp);
          m2 := m;
        else
#          Print("SHT\n");
          m1 := MatToRowList(m1);
          m1 := m1{[1..Length(m1)-Length(lp.Primes)]};
        fi;
        lp := rec(Primes := [], Logs := []);
      fi;
    until Min(i)<>Min(k) and m2=1;
    g.Information.PrimeGens := l;
#    Print(m1, "\n");
    IsMat(m1);
    # we assume the hermite normal form to be of the shape Trans(Id|0)!!
    m1 := MatHermiteRowUpperTrans(m1);
#    Print(m1, "\n");
    m1 := m1[3];
    m1 := MatToRowList(m1);
    m1 := m1{[1..Length(g.Relations[1])]};
    Apply(m1, x->x{[Length(g.Relations)+1..Length(x)]});
    
    IsMat(m1);
    g.Information.PrimeGensTrans := m1;
  fi;

  m1 := [];
  for i in g.Information.PrimeGens do
    for j in i.Primes do
      Add(m1, Copy(i.Logs[Position(i.Primes, sigma(j))]));
    od;
  od;
  IsMat(m1);
  m1 := g.Information.PrimeGensTrans*m1;
  m1 := MatToRowList(m1);
  Apply(m1, x->AbelianGroupEltCreate(g, x));
  Apply(m1, AbelianGroupEltReduce);
  Apply(m1, x->x.expvec);
  IsMat(m1);
  return AbelianGroupHomCreate(g, g, m1);
end;

AbelianGroupHomCreateId := function(g)
  return AbelianGroupHomCreate(g, g, MatId(Z, MatCols(g.Relations)));
end;


#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#
#FXN       FindMaximalCentralField
#
#SHORTDOC  Finds the maximal factorgroup where a list of automorphisms
#          acts trivial.
#
#LONGDOC   
#
#CALLING   sg := FindMaximalCentralField(o | I [, inf][, aut]); \\
#          sg := FindMaximalCentralField(G [, aut]);
#
#ARGUS     sg @ Matrix @ describing the aditional relations
#          o @ order
#          I @ integral ideal
#          inf @ list @ of infinite places
#          G @ AbelianGroup @ must be from RayClassGroupToAbelianGroup
#          aut @ list @ of automorphisms of {\tt o}, if omitted \
#                        {\tt OrderAutomorphisms(o, [])} is used.
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      We'll investigate the ray class field mod $m := (6)\p|1^\infty\p|2^\infty$ in $k := \Q(\sqrt{10})$:@
#          o := OrderMaximal(Z, 2, 10);
#          OrderAutomorphisms(o);
#          G := RayClassGroupToAbelianGroup(6*o, [1, 2]);
#          rcg := FindMaximalCentralField(G);
#
#          @Since {\tt rcg} corresponds not to the whole group, we can construct two fields
#          that are normal over $\Q$: the ray class field modulo $m$ and the maximal central
#          extension of $k$:@
#          cf1 := RayClassField(6*o, [1, 2], rcg);
#          cf2 := RayClassField(6*o, [1, 2]);
#
#          @Finally we'll compute defining equations and the automorphisms:@
#          O1 := RayClassFieldAuto(cf1);
#          O2 := RayClassFieldAuto(cf2);
#
#          @\ldots and see that {\tt O1} is abelian over $\Q$ whereas {\tt O2} is non-abelian. Since
#          $k$ is cyclic over $\Q$, central extensions coincide with absolute abelian ones.@
#          a := O1[2][Position(O1[2], "a")];
#          b := O1[2][Position(O1[2], "b")];;
#          c := O1[2][Position(O1[2], "c")];;
#          d := O1[2][Position(O1[2], "d")];;
#          a*b = b*a; a*c = c*a; a*d = d*a;
#          b*c = c*b; b*d = d*b; c*d = d*c;
#          a := O2[2][Position(O2[2], "a")];;
#          b := O2[2][Position(O2[2], "b")];;
#          c := O2[2][Position(O2[2], "c")];;
#          d := O2[2][Position(O2[2], "d")];;
#          a*b = b*a; a*c = c*a; a*d = d*a;
#          b*c = c*b; b*d = d*b; c*d = d*c;
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
FindMaximalCentralField := function(arg)
  local g, o, m0, inf, hom, argc, i, j;

  if IsOrder(arg[1]) then
    o := arg[1];
    m0 := 1*o;
    inf := [];
    g := RayClassGroupToAbelianGroup(m0, inf);
    argc := 1;
  elif IsIdeal(arg[1]) then
    m0 := arg[1];
    o := IdealOrder(m0);
    if Length(arg)>1 and TYPE(arg[2])="list" then
      inf := arg[2];
      argc := 2;
    else 
      inf := [];
      argc := 1;
    fi;
    g := RayClassGroupToAbelianGroup(m0, inf);
  elif IsRecType(arg[1], "ZIdeal") then
    m0 := arg[1];
    o := Z;
    if Length(arg)>1 and TYPE(arg[2])="list" then
      inf := arg[2];
      argc := 2;
    else 
      inf := [];
      argc := 1;
    fi;
    g := RayClassGroupToAbelianGroup(m0, inf);
  elif IsRecType(arg[1], "AbelianGroup") then
    g := arg[1];
    argc := 1;
    m0 := g.Information.RCG.m0;
    inf := g.Information.RCG.minf;
    if IsIdeal(m0) then
      o := IdealOrder(m0);
    else
      o := Z;
    fi;
  else
    Error("usage: FindMaximalCentralField( [ ord | ideal [, inf ] ] | rcg)");
  fi;

  if o=Z then
    return g;
  fi;

  if Length(arg)>argc then
    hom := arg[argc+1];
  else
    hom := GetGens(o);
  fi;
     
  j := AbelianGroupHomCreateId(g);   
  hom := List(hom, x->-j + AbelianRayClassGroupAutoCreate(g,
                            i->IdealAutomorphism(i, x)));
  Apply(hom, AbelianGroupHomImage);
  Apply(hom, AbelianGroupGenerators);
  hom := Concatenation(hom);
  hom := AbelianSubGroup(hom);
  hom := AbelianQuotientGroup(g, hom);
  return MatHermiteRowUpper(hom.Relations);
end;


_PermEnumInit := function(n)
  local s;
  s := rec(Type := "_PermEnumType",
           pi := [1..n],
           n := n,
           first := true,
           operations := rec(Print := _RecPrint)
           );
  return s;
end;

_PermEnumReInit := function(s)
  s.first := true;
  s.pi := [1..s.n];
end;

_PermEnumRestriction := function(s, x, y)
  local i, j, k, l;
  l := [];

  for i in [1..s.n] do
    l[i] := Set(Filtered([1..s.n], j->x[j]>=y[i]));
  od;
  s.y := y;
  s.rest := l;
  s.first := true;
  s.pi := [1..s.n];
end;

_PermEnumNext := function(s)
  local i, j, k, l;
   
  if s.first then
    s.first := false;
    return true;
  fi;
  if s.n<1 then 
    return false;
  fi;
  i := s.n;
  j := Set([]);
  repeat
    if j=[] then
      SetAdd(j, s.pi[i]);
      i := i-1;
    else
      k := Filtered(j, x->x>s.pi[i]);
      if IsBound(s.rest) then
        k := Set(k);
        SetIntersect(k, s.rest[i]);
        if i>1 and s.y[i]=s.y[i-1] then
          SetIntersect(k, [s.pi[i-1]+1..s.n]);
        fi;
      fi;
      if k =[] then
        SetAdd(j, s.pi[i]);
        i := i-1;
      else
        SetAdd(j, s.pi[i]);
        s.pi[i] := Minimum(k);
        SetRemove(j, s.pi[i]);
        k := i;
        while j<>[] and i>0 do
          i := i+1;
          s.pi[i] := Minimum(j);
#         Print("#:", s.pi, "\t", j, "\n");
          if not IsBound(s.rest) or ( s.pi[i] in s.rest[i] and
            (i=1 or (i>1 and (s.y[i]<>s.y[i-1] or s.pi[i]>s.pi[i-1])))) then
            SetRemove(j, s.pi[i]);
          else
            i := -i;
          fi;
        od;
        if i>0 then 
          return true;
        else
          i := -i;
        fi;
      fi;
    fi;
  until i=0;
  return false;
end;

_ListPSubgroupsEnumInit := function(p,x,y)
  local s, sp, i,j,k,l,m,n,o, sigma, tau;
 
  j := Length(y);
  if Length(y)<Length(x) then
    y := Concatenation(y, 0*[1..Length(x)-Length(y)]);
  fi;

  sp := _PermEnumInit(Length(x));
  _PermEnumRestriction(sp, x , y);
 
  s := rec(Type := "_PSubgroupsEnum",
           sp := sp,
           p := p,
           x := x,
           y := y,
           t := j,
           first := true,
           operations := rec(Print := _RecPrint)
          );
  return s;
end;

_ListPSubgroupsEnumReInit := function(s)
  local i,j;
  _PermEnumReInit(s.sp);
  s.first := true;
  for i in s.c do
    Apply(i[1], x->0);
  od;
end;

_ListPSubgroupsEnumNext := function(s)
  local s, sp, i,j,k,l,m,n,o, sigma, tau, init, expand;

  expand := function(s)
    local i, j, m, r;
    if IsBound(s.elt) then
      m := s.elt;
    else
      m := List([1..s.t], x->[]);
    fi;
    for i in [1..s.sp.n] do
      r := [];
      for j in [1..Length(s.c[i][1])] do
        m[j][i] := (s.c[i][1][j]+1)*s.c[i][3][j];
      od;
      j := Length(s.c[i][1])+1;
      if j<=s.t then
        m[j][i] := s.p^(s.x[i]-s.y[j]);
      fi;
      for j in [Length(s.c[i][1])+2..s.t] do
        m[j][i] := s.p^s.x[i];
      od;
    od;
    s.elt := m;
    return true;
  end;

  init := function(s)
    local i, j, k, l, sigma, tau, c, b;
    sigma := PermList(s.sp.pi);
    tau := sigma^-1;
    c := [];
    for i in [1..s.sp.n] do
      b := [];
      k := [];
      for j in [1..Minimum(i^tau-1, s.t)] do
        if i<j^sigma then
          Add(b, s.p^(s.y[j]-s.y[i^tau])-1);
          Add(k, s.p^(s.x[i]-s.y[j]));
        elif s.x[i]<s.y[j] then
          Add(b, s.p^(s.x[i]-s.y[i^tau])-1); 
          Add(k, 1);
        else
          Add(b, s.p^(s.y[j]-s.y[i^tau]-1)-1);
          Add(k, s.p^(s.x[i]-s.y[j]+1));
        fi;
      od;
      Add(c, [0*b, b, k]);
    od;
    s.c :=c;
    return expand(s); 
  end;
  
  if s.first then
    s.first := false;
    if s.x<s.y then
      return false;
    fi;
    if _PermEnumNext(s.sp) then
      return init(s);
   else
      return false;
    fi;
  fi;

  i := Length(s.c);
  while i>0 do
    if MultiCounterInc(s.c[i][1], s.c[i][2]) then
      return expand(s);
    else
      Apply(s.c[i][1], x->0);
      i := i-1;
    fi;
  od;
  if _PermEnumNext(s.sp) then
    return init(s);
  fi;  
  return false;
end;


#be careful!! this will give an isomorphic quotient, but not
#neccessarily the one you are looking for!
_SubgroupToQuotient := function(g, m)
  local h, i, j, gs;

  if not IsBound(g.Information.SubToQuo) then
    g.Information.SubToQuo := AbelianGroupCreate(Mat(Z, 
                     [[AbelianGroupExponent(g)]]));
  fi;

  j := MatDiag(Z, List(g.Information.Orders, x->AbelianGroupExponent(g)/x));

  if IsRecType(m, "AbelianGroupElt") then
    m := MatToRowList(m.expvec);
  fi;
  m := List(m, x->Mat(Z, [x])*j);
  IsMat(m);
  h := MatKernel(MatTrans(m), AbelianGroupExponent(g));
  h := Concatenation(h, g.Relations);
  IsMat(h);
  return MatHermiteRowUpper(h);
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       FindQuotientOfShapeEnumInit
#
#SHORTDOC  Initializes an environment to enumerate certain subgroups
#
#LONGDOC   
#
#CALLING   s := FindQuotientOfShapeEnumInit(G, L);
#
#ARGUS     s @ record
#          G @ AbelianGroup
#          L @ list @ of integers describing the shape of the subgroup
#
#CSRC      
#
#DEBUG    
#
#XREF      FindQuotientOfShapeEnumNext
#
#PACKAGE   
#
#KASH      Let $G := C^2|8\times C|6^2\times C|2$. We'll enumerate all quotients $Q=G/U$ of $G$
#          such that $Q\cong C|4\times C|2$:@
#          G := AbelianGroupCreate(MatDiag(Z, [8, 8, 6, 6, 2]));
#          s := FindQuotientOfShapeEnumInit(G, [4, 2]);
#          while FindQuotientOfShapeEnumNext(s) and s.no <3 do
#            Print("Number ", s.no, "\n", s.elt, "\n");
#            l := AbelianQuotientGroup(G, AbelianSubGroup(G, s.elt));
#            gs := AbelianGroupEnumInit(l);
#            l := [];
#            while AbelianGroupEnumNext(gs) do Add(l, gs.elt); od;
#            Apply(l, x->AbelianGroupEltMove(x, G));
#            Print("Containing: ", l, "\n");
#          od;
#          while FindQuotientOfShapeEnumNext(s) do
#            i := 1;
#          od;
#          Print("Total: ", s.no, "\n");
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
FindQuotientOfShapeEnumInit := function(g, shape)
  local o,i,j,k,l,m,n, gs, qs, x, y, s, lp, p, d, S;

  gs := AbelianGroupSmithCreate(g, false);
  qs := AbelianGroupCreate(MatDiag(Z, shape));
  qs := AbelianGroupSmithCreate(qs);
  if Length(qs.Relations)>Length(gs.Relations) or
     AbelianGroupOrder(g)<AbelianGroupOrder(qs) then
    return [];
  fi;

  S := rec(Type := "FindQuotientOfShapeEnum",
           g := g,
           gs :=gs,
           qs := qs,
           first := true,
           operations := rec(Print := _RecPrint)
          );

  o := qs.Information.Exponent;
  lp := Factor(o);
  k := [];
  for m in [1..Length(lp)] do
    p := lp[m][1];

    x := Reversed(List([1..Length(gs.Relations)],
                     i->Valuation(p,gs.Relations[i][i])));
   
    y := Reversed(List([1..Length(qs.Relations)],
                     i->Valuation(p,qs.Relations[i][i])));
    d := MatDiag(Z, List(List(gs.Information.Orders, i->i/p^Valuation(p,i))));
    s := _ListPSubgroupsEnumInit(p, x, y);
    Add(k, [s, d]);
  od;
  S.main := k;
  S.no := 0;
  return S;
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       FindQuotientOfShapeEnumNext
#
#SHORTDOC  Steps through the subgroups.
#
#LONGDOC   
#
#CALLING   flag :=  FindQuotientOfShapeEnumNext(s [, 1]);
#
#ARGUS     flag @ boolean
#          s @ record @ generated by {\tt FindQuotientOfShapeEnumInit}
#          1 @ @ if present find subgroups instead of quotients.
#
#CSRC      
#
#DEBUG    
#
#XREF      FindQuotientOfShapeEnumInit 
#
#PACKAGE   
#
#EXAMPLE   See {\tt FindQuotientOfShapeEnumInit} for an example. 
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
FindQuotientOfShapeEnumNext := function(arg)
  local i, j, k, l, m, n, d, s;

  s := arg[1];
  if s=[] then
    return false;
  fi;

  if s.first then
    s.first := false;
    k := [];
    for i in s.main do
      if not _ListPSubgroupsEnumNext(i[1]) then
        return false;
      fi;
      j := i[1].elt;
      Apply(j, Reversed);
      Add(k, Mat(Z, j)*i[2]);
    od;
    k := Concatenation(k);
    if Length(arg)=1 then
      s.elt := _SubgroupToQuotient(s.gs, k);
    else
      s.elt := k;
    fi;
    s.elt := MatToRowList(s.elt);
    Apply(s.elt, i->AbelianGroupEltMove(AbelianGroupEltCreate(s.gs, i), s.g));
    Apply(s.elt, i->i.expvec);
    IsMat(s.elt);

    s.no := s.no+1;
    return true;
  fi;
  
  l:= Length(s.main);
  while l>0 do
    if _ListPSubgroupsEnumNext(s.main[l][1]) then
      k := [];
      for i in s.main do
        j := i[1].elt;
        Apply(j, Reversed);
        Add(k, Mat(Z, j)*i[2]);
      od;
      k := Concatenation(k);
      if Length(arg)=1 then
        s.elt := _SubgroupToQuotient(s.gs, k);
      else
        s.elt := k;
      fi;
      s.elt := MatToRowList(s.elt);
      Apply(s.elt, i->AbelianGroupEltMove(AbelianGroupEltCreate(s.gs, i), s.g));
      Apply(s.elt, i->i.expvec);
      IsMat(s.elt);

      s.no := s.no+1;
      return true;
    else
      _ListPSubgroupsEnumReInit(s.main[l][1]);
      l := l-1;
    fi;
  od;
  return false;
end;
  
RayClassFieldSplittingField := Ignore;  
_RayClassFieldIsNormalAbel := function(Arg)
  local i, j, k, l, m, inf, nrels, arels, hom, SG, hard_case, G, l, abel;
  
  G := Arg[1];
  abel := Arg[Length(Arg)];
  Unbind(Arg[Length(Arg)]);
  if Length(Arg)=2 then
    l := Arg[2];
  else
    l := OrderAutomorphisms(IdealOrder(G.Information.RCG.m0), []);
    l := l{[2..Length(l)]};
  fi;

  hard_case := function()
    local i;
    i := RayClassFieldSplittingField(G, l);
    if AbelianGroupOrder(i)<>AbelianGroupOrder(G) then
      return false;
    fi;
    if not abel then
      return true;
    else
      return _RayClassFieldIsNormalAbel(G, l, abel);
    fi;
  end;
    
  m := G.Information.RCG.m0;
  inf := G.Information.RCG.minf;
  if IsBound(G.Information.RCG.Sub) then
    SG := G.Information.RCG.Sub.Supergroups[1].g2;
    nrels := G.Information.RCG.Sub.Supergroups[1].Mat12;
  else
    SG := G;
    nrels := 0*G.Relations;
  fi;
  
  nrels := MatHermiteRowMod(AbelianGroupExponent(G), nrels);
  arels := MatIndex(nrels);
  if TYPE(l)<>"list" then
    l := [l];
  fi;
 
  if inf<>[] and Length(inf)<>XOrderSig(XIdealOrder(m))[1] then
    return hard_case();
  fi;

  for i in l do
    if IsRecType(i, "AbelianGroupHom") then
      hom := i;
    elif IsInt(i) or IsElt(i) then 
      if m<>IdealAutomorphism(m,i) then
        return hard_case();
      fi;
      hom := AbelianRayClassGroupAutoCreate(SG, j->IdealAutomorphism(j,i));
    else
      if m<>i(m) then
        return hard_case();
      fi;
      hom := AbelianRayClassGroupAutoCreate(SG, i);
    fi;
    j := MatHermiteRowMod(AbelianGroupExponent(G), nrels*hom.Mat12);
    
    if j<>nrels then
      return false;
    fi;

    if abel then
      j := Concatenation(nrels, hom.Mat12-MatId(Z, Length(hom.Mat12)));
      IsMat(j);
      j := MatIndex(j);
      if j<>arels then
        return false;
      fi;
    fi;
  od;

  return true;
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       RayClassFieldIsCentral
#
#SHORTDOC  Tests whether the Ray Class Field defined by the given data 
#          will be a central extension.
#
#LONGDOC   
#
#CALLING   flag := RayClassFieldIsCentral(G [, l]);
#
#ARGUS     flag @ boolean
#          G @ AbelianGroup @ a quotient from RayClassGroupToAbelianGroup
#          l @ list @ of automorphisms, if not present all known automorphisms are used.
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      We'll investigate the ray class field mod $m := (6)\p|1^\infty\p|2^\infty$ in $k := \Q(\sqrt{10})$:@
#          o := OrderMaximal(Z, 2, 10);
#          OrderAutomorphisms(o);
#          G1 := RayClassGroupToAbelianGroup(6*o, [1, 2]);
#          rcg := FindMaximalCentralField(G1);
#          G2 := AbelianQuotientGroup(G1, AbelianSubGroup(G1, rcg));
#          RayClassFieldIsCentral(G1);
#          RayClassFieldIsCentral(G2);
#
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
RayClassFieldIsCentral := function(arg)
  Add(arg, true);
  return _RayClassFieldIsNormalAbel(arg);
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       RayClassFieldIsNormal
#
#SHORTDOC  Tests whether the Ray Class Field defined by the given data 
#          will be a normal extension.
#
#LONGDOC   
#
#CALLING   flag := RayClassFieldIsNormal(G [, l]);
#
#ARGUS     flag @ boolean
#          G @ AbelianGroup @ a quotient from RayClassGroupToAbelianGroup
#          l @ list @ of automorphisms, if not present all known automorphisms are used.
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#EXAMPLE   See {\tt RayClassFieldSplittingField} for an example.
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
RayClassFieldIsNormal := function(arg)
  Add(arg, false);
  return _RayClassFieldIsNormalAbel(arg);
end;

# l must contain ALL automorphisms, not just generators
# you may skip the identity.

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       RayClassFieldSplittingField
#
#SHORTDOC  Computes data to get the splitting field of the Ray Class Field.
#
#LONGDOC   
#
#CALLING   sg := RayClassFieldSplittingField(G, l);
#
#ARGUS     sg @ AbelianGroup
#          G @ AbelianGroup @ quotient of RayClassGroupToAbelianGroup
#          l @ list @ of automorphisms. Must be complete, not just generators. 
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      @
#          o := OrderMaximal(Z, 2, 10);
#          OrderAutomorphisms(o);
#          p3 := Factor(3*o)[2][1];
#          G1 := RayClassGroupToAbelianGroup(p3, [2]);
#          G2 := RayClassFieldSplittingField(G1);
#          RayClassFieldIsNormal(G1);
#          RayClassFieldIsNormal(G2);
#          O1 := RayClassField(G1);
#          Galois(OrderAbs(Order(O1[1])));
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
RayClassFieldSplittingField := function(arg)
  local i, j, k, l, m, inf, nrels, arels, hom, SG, mn, infn, G, l;
 
  if Length(arg)=[] then
    return Error("usage: RayClassFieldSplittingField( G [, list of aut] );");
  fi;
  G := arg[1];
  m := G.Information.RCG.m0;
  if Length(arg)>1 then
    l := arg[2];
  elif IsIdeal(m) then
    l := OrderAutomorphisms(IdealOrder(m), []);
    l := l{[2..Length(l)]};
    if Length(l)=0 then 
      Error("no automorphisms present");
    fi;
  fi;  
  inf := G.Information.RCG.minf;
  if IsBound(G.Information.RCG.Sub) then
    SG := G.Information.RCG.Sub.Supergroups[1].g2;
    nrels := G.Information.RCG.Sub.Supergroups[1].Mat12;
  else
    SG := G;
    nrels := 0*G.Relations;
  fi;
  
  if TYPE(l)<>"list" then
    l := [l];
  fi;

  mn := m;
  for i in l do
    if IsInt(i) or IsElt(i) then
      mn := IdealLcm(m, IdealAutomorphism(m, i));
    elif TYPE(i) ="function" then
      mn := IdealLcm(mn, i(mn));
    else
      Error("wrong argument in list");
    fi;
  od;

  infn := List([1..OrderSig(IdealOrder(mn))[1]], i->i);
  SG := RayClassGroupToAbelianGroup(mn, infn);
  i := AbelianGroupGenerators(SG);
  Apply(i, j->AbelianGroupDiscreteLog(G, AbelianGroupDiscreteExp(j)));
  Apply(i, j->j.expvec);
  IsMat(i);
  i := AbelianGroupHomCreate(SG, G, i);
  i := AbelianGroupHomKernel(i);
  G := AbelianQuotientGroup(SG, i);
  nrels := G.Information.RCG.Sub.Supergroups[1].Mat12;

  arels := Copy(nrels);
  for i in l do
    if IsRecType(i, "AbelianGroupHom") then
      hom := i;
    elif IsInt(i) or IsElt(i) then 
      hom := AbelianRayClassGroupAutoCreate(SG, j->IdealAutomorphism(j,i));
    else
      hom := AbelianRayClassGroupAutoCreate(SG, i);
    fi;

    j := Concatenation(arels, nrels*hom.Mat12);
    IsMat(j);
    j := MatHermiteRowUpperTrans(j);
    j := j[3]{[j[1]+1..Length(j[3])]};
    j := MatToRowList(j);
    Apply(j, k->k{[1..Length(arels)]});
    IsMat(j);
    arels := MatHermiteRowMod(AbelianGroupExponent(G),j*arels);
    arels := arels{[1..Length(arels[1])]};
    IsMat(arels);
  od;  

  return AbelianQuotientGroup(SG, AbelianSubGroup(SG, arels));
end;

#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX*
#
#USERDEF   
#
#FXN       RayClassFieldIsAbelian
#
#SHORTDOC  Tests if a class field is also abelian over $\Q$.  
#
#LONGDOC   Let $\O$ be the ring of integers of an abelian extention over $\Q$.
#          ${\goth m}$ is a multiple of its conductor ${\goth f} \in \Z$. 
#          G is a class group in $\O$, K the abelian extension over $\O$ defined
#          by G. The function determines, if K is also abelian over $\Q$.
#
#CALLING   a:=RayClassFieldIsAbelian(G [,m]);
#
#ARGUS     G @ AbelianGroup @ defining a class group
#          a @ boolean
#          m @ integer 
#
#CSRC      
#
#DEBUG    
#
#XREF      
#
#PACKAGE   
#
#KASH      @
#          o:=OrderMaximal(x^2-10);        
#          G:=RayClassGroupToAbelianGroup(9*o);
#          m:=Disc(o);
#          RayClassFieldIsAbelian(G,m);
#          @Let's check this. First we'll compute the {\tt RayClassField}@ 
#          L:=RayClassField(G); 
#          @The only critical part is the cubic polynomial, therfore we'll show
#          that this polynomial defines a $C|3$ extension over $\Q$.@
#          g:=PolyMove(L[2], Z);
#          OrderAutomorphismsAbel(Order(g));
#          o:=OrderMaximal(x^4-2);
#          G:=RayClassGroupToAbelianGroup(9*o);
#          RayClassFieldIsAbelian(G);
#           
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
RayClassFieldIsAbelian:=function(arg)
  local ff, m, subgr, gen, G, H, n, N, a, f, rg, o, var;

  if Length(arg)=0 then 
     return Error("usage: RayClassFieldIsAbelian(AbelianGroup [, integer])");
  fi;

  if Length(arg)>1 then var:=arg[2]; elif Length(arg)=1 then var:=1; fi;

  if IsRecType(arg[1],"AbelianGroup") and 
      IsBound(arg[1].Information.RCG) and IsInt(var) then
    
    a:=arg[1];

    o:=IdealOrder(a.Information.RCG.m0);
    N:=OrderDeg(o);

    if Length(arg)>2 then 
      N:=N/OrderDeg(arg[3]);  
    fi;
    if Length(arg)=2 and IsInt(arg[2]) then
      f:=arg[2]; 
    else 
      f:=Disc(o);
    fi;

    N:=AbelianGroupOrder(a)*N;

    ff:=RayConductor(a);
    m:=IdealMin(ff[1])*f;
    rg:=RayClassGroupToAbelianGroup(m*o);
     
    subgr:=AbelianRayGroupImbed(rg,a);
    gen:=List(AbelianGroupGenerators(subgr),z->
                               IdealNorm(AbelianGroupDiscreteExp(z)));
    if IsInt(m) then
      m:=ZIdealCreate(m);
      gen:=List(gen,z->ZIdealCreate(z));
    fi;

    G:=RayClassGroupToAbelianGroup(m);
    H:=AbelianSubGroup(G,List(gen,z->AbelianGroupDiscreteLog(G,z)));
    n:=AbelianGroupOrder(AbelianQuotientGroup(G,H));
     
    return N=n;
  else 
    return Error("usage: RayClassFieldIsAbelian(Abelian Group [, integer])");
  fi;
end;

