#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       OrderAutomorphisms
#
#SHORTDOC  Computes or stores automorphisms of the given extension.
#
#LONGDOC   This function computes or stores the automorphisms of the 
#          given extension. The automorphisms are represented by algebraic
#          numbers which are zeros of the generating polynomials of the
#          given extension. They can be applied to algebraic numbers
#          with the function {\tt EltAutomorphism}.
#
#          In the case that some automorphisms are
#          known in the list {\tt L}, they can be stored with 
#          {\tt OrderAutomorphisms(o, L);}.
#
#          In the other cases the automorphisms will be computed. The option 
#          {\tt "normal"} can be omitted. With the option {\tt "abel"} you
#          can turn on the more efficient algorithm for abelian extensions.
#          The computation of automorphisms is only possible for absolute
#          normal extensions. In the case that an extension is not normal
#          the function will return {\tt false}. The algorithms
#          are described in \cite{Kl2,AcKl1}.
#
#CALLING   aut := OrderAutomorphisms(o);\\
#          aut := OrderAutomorphisms(o, L);\\
#          aut := OrderAutomorphisms(o, "normal");\\
#          aut := OrderAutomorphisms(o, "abel");
#
#ARGUS     aut @ list  @ list of automorphisms
#          o   @ order @ the given order
#          L   @ list  @ list of some known automorphisms
#
#XREF      EltAutomorphism, OrderAutomorphismsAbel, OrderAutomorphismsNormal.
#
#PACKAGE   Order
#
#KASH      Store an automorphism:@
#          o := Order(x^4-4*x^2+1);;
#          a := Elt(o, [0,-1,0,0]);;
#          OrderAutomorphisms(o, [a]);
#          @
#          Compute the automorphisms:@
#          o := Order(x^4-4*x^2+1);;
#          OrderAutomorphisms(o);
#          @
#          Compute the automorphisms in the abelian case (faster):@
#          o := Order(x^4-4*x^2+1);;
#          OrderAutomorphisms(o, "abel");
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY**HISTORY***HISTORY***HISTORY***HISTORY
#
#**C***C***|***C***C***C***C***C***C**C***C***C***C***C***C***C***C***C***C***/



_OrderAutomorphisms := OrderAutomorphisms;

OrderAutomorphisms := function(arg)
   local L;

   if Length(arg)=1 and IsOrder(arg[1]) then
      L:=_OrderAutomorphisms(arg[1],[]);
      if Length(L) = OrderDeg(arg[1]) then
         return L;
      fi;
      if OrderAutomorphismsNormal(arg[1]) then
        L:=_OrderAutomorphisms(arg[1],[]);
        Apply(L,x->EltMove(x, arg[1]));
        return L;
      else
        return false;
      fi;
   fi;
   
   if Length(arg)=2 and IsOrder(arg[1]) and IsString(arg[2]) then
      L:=_OrderAutomorphisms(arg[1],[]);
      if Length(L) = OrderDeg(arg[1]) then
         return L;
      fi;
 
      if arg[2]="abel" then
         if OrderAutomorphismsAbel(arg[1]) then
           L:=_OrderAutomorphisms(arg[1],[]);
           Apply(L,x->EltMove(x, arg[1]));
           return L;
         else
           return false;
         fi;
      elif arg[2]="normal" then
         if OrderAutomorphismsNormal(arg[1]) then
           L:=_OrderAutomorphisms(arg[1],[]);
           Apply(L,x->EltMove(x, arg[1]));
           return L;
         else
           return false;
         fi;
      fi;
      
   fi;
   
   if Length(arg)=2 and IsOrder(arg[1]) and IsList(arg[2]) then
      return _OrderAutomorphisms(arg[1], arg[2]);
   fi;
  
   Error("usage: OrderAutomorphisms( <order> [, <list> ] ); OrderAutomorphism( <order>, \"abel\" );  OrderAutomorphism( <order>, \"normal\" ); ");
end;      
