#############################################################################
##
#A  kantdiv.g                    KANT library              Mario Daberkow
##
#A  @(#)$Id: kantdiv.g 1.00 1995/02/09 Daberkow $
##
#Y  Copyright 1995,  Prof. Dr. M. Pohst, TU - Berlin,  Germany
##
##  This file contains the support (diverse) KANT functions .
##
#H  $Log: kantdiv.g $
#H  Revision 1.00  1995/03/12  Daberkow
#H  added MultiCounterInit MD
#H  added MultiCounterInc  MD
##

#############################################################################
##
#F  MultiCounterInit ( < integer > ) . . .  Initialization of multi counter.
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       MultiCounterInit
#CALLING   L := MultiCounterInit(n);
#SHORTDOC  Initializes a multi counter.
#LONGDOC   Initializes a list {\tt L} of length {\tt n} to be used by
#          MultiCounterInc.
#ARGUS     L @ list
#          n @ integer
#
#PACKAGE   lib
#
#XREF      MultiCounterInc
#KASH      A counter with 2 items:@
#          L := MultiCounterInit(2);;
#          while MultiCounterInc(L, 2) do
#            Print (L, " ");
#          od; Print("\n");
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##



MultiCounterInit := function (len)
   local   L, i;

   L := [];
   for i in [1..len] do  Add (L,0);  od;
   if len>0 then L[1] := -1; fi;

   return L;
end;

#############################################################################
##
#F  MultiCounterInc ( <List> , <Bound> ) . . .  Increment a multi counter.
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       MultiCounterInc
#CALLING   b := MultiCounterInc(L, n);
#SHORTDOC  Increments a multi counter subject to a bound.
#LONGDOC   Increments a multi counter {\tt L}. Incrementation is done if
#          {\tt L[i] < n} for some {\tt i}, if {\tt n} is an integer or
#          {\tt L[i] < n[i]} for some i, if {\tt n} is a list. In case
#          the counter was incremented, ``true'' is returned.
#          Otherwise ``false'' is returned.
#ARGUS     b @ boolean
#          L @ list
#          n @ integer | list
#
#PACKAGE   lib
#
#XREF      MultiCounterInit
#KASH      A counter with 2 items:@
#          L := MultiCounterInit(2);;
#          while MultiCounterInc(L, 2) do
#            Print(L, " ");
#          od; Print("\n");
#
#KASH      counter with 2 items:@
#          L := MultiCounterInit(2);;
#          while MultiCounterInc (L, [2,3]) do
#            Print(L, " ");
#          od; Print("\n");
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


MultiCounterInc := function (L, Bound)
   local   testcoeff,
           len, i, ende;

   testcoeff := function (i)
     if (IsList (Bound)) then  return (L[i] <= Bound[i]);
     else
       return (L[i] <= Bound);
     fi;
   end;

  i   := 1;
  len := Length (L);
  ende := false;

  while (not ende) and (i <= len) do
    L[i] := L[i]+1;

    if not testcoeff (i) then
      L[i] := 0;
      i    := i+1;
    else
      ende := true;
    fi;
  od;

  return (i <= len);
end;


MultiCounterInitHalf := function (len, bound)
   local   L, i;

  if IsList(bound) then
    L := [-bound[1]-1];
    for i in [2..len] do  Add (L,-bound[i]);  od;
  else
    L := [-bound-1];
    for i in [2..len] do  Add (L,-bound);  od;
  fi;

  return L;
end;



MultiCounterIncHalf := function (L, Bound)
   local   testcoeff,
           len, i, ende, x;

   testcoeff := function (i)
     if ((IsList (Bound)) and (L[i] <= Bound[i])) or (L[i] <= Bound) then
       return true;
     else
       return false;
     fi;
   end;

  i   := 1;
  len := Length (L);
  ende := false;

  while (not ende) and (i <= len) do
    L[i] := L[i]+1;

    if not testcoeff (i) and ForAny(L, x->x<>0)then
      if IsList(Bound) then
        L[i] := -Bound[i];
      else
        L[i] := -Bound;
      fi;
      i    := i+1;
    else
      ende := true;
    fi;
  od;

  return (i <= len) and ForAny(L, x->x<>0);
end;

#############################################################################
##
#F  Binomial( <n>, <k> )  . . . . . . . . .  binomial coefficient of integers
##
Binomial := function ( n, k )
    local   bin, i, j;
    if   k < 0  then
        bin := 0;
    elif k = 0  then
        bin := 1;
    elif n < 0  then
        bin := (-1)^k * Binomial( -n+k-1, k );
    elif n < k  then
        bin := 0;
    elif n = k  then
        bin := 1;
    elif n-k < k  then
        bin := Binomial( n, n-k );
    else
        bin := 1;  j := 1;
        for i  in [n-k+1..n]  do
            bin := bin * i;
            while j <= k  and bin mod j = 0  do
                bin := bin / j;
                j := j + 1;
            od;
        od;
    fi;
    return bin;
end;

#############################################################################
##
#F  Factorial( <n> )  . . . . . . . . . . . . . . . . factorial of an integer
##

Factorial := function ( n )
    if n < 0  then Error("n must be non negative");  fi;
    return Product( [1..n] );
end;





