#############################################################################
##
#A  kantideal.g                    KANT library              Mario Daberkow
##
#A  @(#)$Id: kantideal.g 1.00 1995/02/09 Daberkow $
##
#Y  Copyright 1995,  Prof. Dr. M. Pohst, TU - Berlin,  Germany
##
##  This file contains the Ideal KANT functions .
##
#H  $Log: kantideal.g $
#H  Revision 1.00  1995/02/09  Daberkow
#H  added 'IsIdeal'
#H  added IdealMove CF
#H  added Valuation
#H  added Ideal2EltAssure and IdealGen (which are no longer internal) AH
#H  used IdealImprove for IdealMove 
##


#############################################################################
##
#F  IsIdeal( <object> )  . . . . . . . . . .  checks if argument is an ideal
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       IsIdeal
#CALLING   b := IsIdeal(x);
#SHORTDOC  Returns true iff the argument is an ideal.
#ARGUS     b @ boolean
#          x
#
#PACKAGE   lib
#
#KASH      We have for example:@
#          o := Order(Z, 2, 3);
#          IsIdeal(o);
#          IsIdeal(1*o);
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


IsIdeal := function ( I )

  if (TYPE (I) = "KANT ideal") then
    return true;
  else
    return false;
  fi;
end;

#############################################################################
##
#F  IdealGen( <ideal>, <small integer> ) . . Ideal Generators
##
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       IdealGen
#CALLING   g := IdealGen(I, i);
#LONGDOC   Any ideal {\tt I} of a maximal order $\o$ may be generated using
#          two elements $g_1$ and $g_2$ (i.e. {\tt I} $= g_1\o + g_2\o$).
#          This functions returns $g_i$ ($i\in\{1,2\}$) after computing
#          suitable generators if necessary.
#
#          The restriction to maximal orders is only necessary to compute the
#          generators. Note that the generators are not unique! In fact
#          since the algorithm is probabilistic, they may change
#          between two sessions.
#ARGUS     g @ algebraic element
#          I @ ideal
#          i @ small integer @ {$\in\{1,2\}$}
#XREF      IdealGenerators, IdealBasis
#
#PACKAGE   lib
#
#KASH      @
#          O := Order(Poly(Zx, [1, 6, 6, 6]));;
#          alpha := Elt(O, [0, 1, 0]);;
#          I := Ideal(6, alpha);
#          IdealGen(I, 1);
#          IdealGen(I, 2);
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

IdealGen:=function(I,n)
  if (TYPE (I) = "KANT ideal") and (TYPE (n) = "Small integer") then
    if n=1 then return IdealGenerators(I)[1]; fi;
    if n=2 then return IdealGenerators(I)[2]; fi;
    Error("usage: IdealGen( <ideal>, !<small integer 1 or 2>! )");
  else
    Error("usage: IdealGen( <ideal>, <small integer> )");
  fi;
end;

#############################################################################
##
#F  IdealMove( <ideal>, <ord>)  . . . . . . . . . . . . Moves ideal to ord
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       IdealMove
#
#SHORTDOC  Embeds one or more ideals into a given order.
#
#LONGDOC   If {\tt I1} is an ideal, it is moved into {\tt o}. This is done
#          via a 2-element representation of the ideal. We assume that KASH
#          already knows an embedding of the coeficient order of {\tt I1}
#          into {\tt o}.
#
#          If {\tt I1} is a (factor) list of ideals all ideals contained are
#          moved.
#
#CALLING   I2 := IdealMove(I1, o);
#
#ARGUS     I2 @ same type as I1
#          I1 @ ideal $|$ list of ideals
#          o @ order
#
#CSRC
#
#XREF      EltMove.
#
#PACKAGE   lib
#
#KASH      We will create an order $\O$, search (and find) an order $\o$
#          created by a better polynomial (smaller index or coefficients),
#          factor 2 in $\o$ where it is no index divisor and move it to $\O$.@
#          O := OrderMaximal(Order(x^6 - 9*x^4 - 4*x^3 + 27*x^2 - 36*x - 23));;
#          o := OrderShort(O);;
#          IdealMove(2*o, O);
#          IdealMove(Factor(2*o), O);
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
#
#**C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***/
#
##


IdealMove := function(arg)
  local a, o, I, move, newl;

  move := function(I)
    local a;
    Ideal2EltAssure(I);
    a := Ideal(EltMove(IdealGen(I,1),o), EltMove(IdealGen(I, 2),o));
    if OrderCoefOrder(IdealOrder(a))= Z then
      return IdealImprove(a);
    else
      return IdealImprove(a);
    fi;
  end;

  if Length(arg) <> 2 then
    Print("usage: IdealMove(Ideal, ord) or IdealMove(listOfIdeals, ord)\n");
    return;
  fi;
  o := arg[2];
  if not IsOrder(o) then
    Print("2nd Argument is not an Order\n");
    return;
  fi;

  if IsRecType(arg[1], "ZIdeal") then
    return arg[1].Generator*arg[2];
  fi;
  
  if IsList(arg[1]) then                 # we move faclsts and lists too
     newl := [];
     for a in arg[1] do
       if IsList(a) then                 # now we should have a faclst
         if not IsIdeal(a[1]) then
            Print(a, " is no Ideal\n");
            return;
         fi;
         Add(newl, [move(a[1]), a[2]]);
       else
         if not IsIdeal(a) then
            Print(a, " is no Ideal\n");
            return;
         fi;
         Add(newl, move(a));             # a simple list
       fi;
     od;
     return newl;
  else
     if IsIdeal(arg[1]) then
       return move(arg[1]);                # a single Ideal
     else
       Print("No ideal\n");
       return;
     fi;
  fi;
end;


#############################################################################
##
#F  Valuation ( < obj1 > , < obj2 > ) . . . computes the valuation of obj2.
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       Valuation
#CALLING   d := Valuation(P, a);
#SHOTDOC   Computes the valuation of the argument at a prime (if possible).
#LONGDOC   Returns the valuation of an integer, an algebraic element, 
#          an (fractional) ideal at a prime ideal, a rational prime or an
#          alff order ideal at a prime ideal.
#          If the first argument is an integer then the second argument
#          must also be an integer.
#          For alff order ideals they have to be defined in the same order
#          which must be maximal.\\
#          This function calls EltValuation, IdealValuation, IntValuation,
#          or AlffIdealValuation.
#ARGUS     d @ integer
#          P @ ideal $|$ integer $|$ alff order ideal @ must be prime.
#          a @ ideal $|$ algebraic element $|$ integer  @ to valuate.
#          a @ alff order ideal @ to valuate.
#CSRC      IdealValuation, EltValuation, IntValuation, AlffIdealValuation.
#
#PACKAGE   lib
#
#KASH      Computation of the valuation of an algebraic number. @ 
#          O := OrderMaximal(Order(Z, 2, 3));
#          L := Factor(2*O);
#          P := L[1][1];
#          Valuation(P, Elt(O, 2));
#          Valuation(P, Elt(O, [1, 1]));
#
#KASH      Computation of the valuation of a fractional ideal. @
#          O := OrderMaximal(Order(Poly(Zx, [1, 6, 6, 6])));
#          P := Factor(5*O)[1][1];
#          Valuation(P, Elt(O, [5, 5, 5])*O);
#          P := Factor(2*O)[1][1];
#          Valuation(P, Elt(O, [8, 8, 8])*O);
#          Valuation(P, 2*O);
#KASH      Valuation of integers at prime ideals of maximal orders. @
#          O := OrderMaximal(Order(Poly(Zx, [1, 6, 6, 6])));
#          P := Factor(2*O)[1][1];
#          Valuation(P, 3);
#          Valuation(P, 2);
#KASH      Valuation of integers at rational primes. @
#          Valuation(5, 5^21);
#KASH      An example for alff order ideals:@
#          AlffInit(FF(3,1));
#          AlffOrders(y^3+T^3*y+T); 
#          alpha := AlffElt(o, [1, T, 1]);
#          I := (T+2*w)*o+alpha*o;
#          P := AlffIdealFactor(I)[1][1];
#          Valuation(P, I^(-2));
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

infinity := rec(Type := "infinity",
                operations := rec(
                  Print := function(a) Print("infinity"); end,
                  \+ := function(a,b)
                    if IsRecType(a, "infinity") then
                      return a;
                    elif IsRecType(b, "infinity") then
                      return b;
                    fi;
                  end,
                  \> := function(a,b)
                    if IsRecType(a, "infinity") then
                      return true;
                    else
                      return false;
                    fi;
                  end));

Valuation := function ( P, a )
  if (a = 0) then 
    return infinity;
  elif IsAlffIdeal(P) and IsAlffIdeal(a) then
    return AlffIdealValuation(P, a);
  elif IsIdeal(P)  and IsIdeal(a) then
    return IdealValuation(P, a);
  elif IsIdeal(P)  and (IsElt(a) or IsInt(a)) then
    return EltValuation(P, a);
  elif IsPrime(P) and IsRat(a) and not IsInt(a) then
    return IntValuation(P, Numerator(a)) - IntValuation(P, Denominator(a));
  elif IsPrime(P) and IsInt(a) then
    return IntValuation(P, a);
  else
    return Error("Usage: Valuation( prime, ideal | elt | int | rat )");
  fi;
end;

#hb:
#############################################################################
##
#F  IdealWithNorm( <object> )  . . . . . . . . . .  checks if argument is an ideal
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       IdealWithNorm
#CALLING   L := IdealWithNorm(n,o);
#SHORTDOC  Computed all ideals of {\tt o} of a given norm {\tt n}.
#ARGUS     L @ List
#          n @ integer @ $>1$
#          o  @ MaximalOrder
#
#PACKAGE   lib
#
#KASH      We have for example:@
#          o := OrderMaximal(Z, 2, 3);
#          IdealWithNorm(2,o);
#          IdealWithNorm(11*13,o);
#
#          
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
IdealWithNorm:=function(N,O)
  local  l, l1,l2,A, b, L, L1, L2, F, x, y, z;

  F:=function(z)
    local s, ll;
    ll:=[];
    s:=SimplexInit(z[1],z[2]); 
    while SimplexNext(s) do
      Add(ll, SimplexElt(s));
    od;
    return ll;
  end;

  if IsInt(N) and N >1 and IsOrder(O) then
    if OrderIsMaximal(O) then
      l:=IntFactor(N); 
      l1:=List(l, 
            x->List(IdealFactor(x[1]*O),y->[IdealDegree(y[1]),y[1],x[1]])); 
      l2:=List(l1,x->List(x,y->y[1])); 
      l1:=List(l1,x->List(x,y->y[2]));
      A:=List(l2,
         x->Mat(R,Concatenation(MatToRowList
                      (MatDiag(R,List([1..Length(x)],x->-1))),[x],-[x]))); 
      b:=List([1..Length(l)],x->
                 MatTrans(R,[Concatenation(
                      List([1..Length(l1[x])],z->0),[l[x][2]],[-l[x][2]])])); 
      L:=List([1..Length(l)],x->[A[x],b[x]]);
    
      Apply(L, F);
      L1:=List([1..Length(l)],x->
                List(L[x],y->Product(List([1..Length(y)],z->l1[x][z]^y[z]))));
      L2:= Cartesian(L1);
      return List(L2, Product);
    else 
      return "Order is not maximal";
    fi;
  else
    Error("usage: IdealWithNorm( <integer > 1>, <maximal order> );");
  fi;
end;

     
