#############################################################################
##
#F  IsMat( <object> )  . . . . . . . . . . . . checks if argument is a matrix
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       IsMat
#CALLING   b := IsMat(x);
#SHORTDOC  Returns true iff the argument is of type "KANT matrix".
#ARGUS     b @ boolean
#          x
#
#PACKAGE   lib
#
#KASH      @
#          M := MatSym(Z, [[1,2,3,4], [5,6,7], [8,9], [10]] );
#          IsMat(M);
#          M := [ [1,2], [3,4] ];
#          IsMat(M);
#          M;
#          M[1];
#          M[1][1];
#          IsMat(false=IsMat(IsMat(IsMat(IsMat))));
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


#############################################################################
##
#F  MatDiag( <ring>, <list> ) . . . . . . .  Creates a diagonal matrix
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       MatDiag
#CALLING   M := MatDiag(S, L);
#LONGDOC   Creates the diagonal matrix {\tt M} over the coefficient ring {\tt S}.
#          {\tt L} is a list containing the diagonal entries.
#ARGUS     M @ matrix
#          S @ ring
#          L @ list @ diagonal entries
#
#PACKAGE   lib
#
#KASH      Create the diagonal matrix
#          \[\left(\begin{array}{ccccc} 1 & 0 & 0 & 0 & 0 \\ 0 & 2 & 0 & 0 & 0 \\
#          0 & 0 & 3 & 0 & 0 \\ 0 & 0 & 0 & 4 & 0 \\ 0 & 0 & 0 & 0 & 5 \end{array}\right)
#          \in {\Bbb{Z}}^{5 \times 5}.\] @
#          MatDiag(Z, [1, 2, 3, 4, 5]);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

MatDiag := function (ring,l)
  local n, m, ll, zero;

  if not ( (TYPE(ring) = "KANT ring" or
           (TYPE(ring) = "KANT order") or
           (TYPE(ring) = "KANT finite field") )
       and IsList(l)) then
    Print("Usage: MatDiag( <ring>, <list> );\n");
    return;
  fi;

  zero := 0*l[1];
  ll:=[];
  for n in [1 .. Length(l)] do
    ll[n]:=[];
    for m in [1 .. Length(l)] do
      if n=m then
        ll[n][m]:=l[n];
      else
        ll[n][m]:=zero;
      fi;
    od;
  od;

  return Mat(ring,ll);
end;

#############################################################################
##
#F  MatSym( <ring>, <list> ) . . . . . . .  Creates a symmetric matrix
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       MatSym
#CALLING   M := MatSym(S, L);
#LONGDOC   Creates the symmetric matrix {\tt M} over the coefficient ring {\tt S}.
#ARGUS     M @ matrix
#          S @ ring
#          L @ list
#
#PACKAGE   lib
#
#KASH      Create the symmetric matrix
#          \[\left(\begin{array}{cccc} 1 & 2 & 3 & 4 \\ 2 & 5 & 6 & 7 \\
#          3 & 6 & 8 & 9 \\ 4 & 7 & 9 & 10 \end{array}\right)
#          \in {\Bbb{Z}}^{4 \times 4}.\] @
#          MatSym(Z, [[1,2,3,4], [5,6,7], [8,9], [10]]);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

MatSym := function (ring,l)
  local n, m, ll;

  if not ( (TYPE(ring) = "KANT ring" or
           (TYPE(ring) = "KANT order") or
           (TYPE(ring) = "KANT finite field") )
         and IsList(l)) then
    Print("Usage: MatSym( <ring>, <list> );\n");
    return;
  fi;

  ll:=[];
  for n in [1 .. Length(l)] do
    ll[n]:=[];
  od;
  for n in [1 .. Length(l)] do
    for m in [1 .. Length(l)-n+1] do
      ll[n][m+n-1]:=l[n][m];
      if not n=m+n-1 then
        ll[m+n-1][n]:=l[n][m];
      fi;
    od;
  od;

  return Mat(ring,ll);
end;

#############################################################################
##
#F  MatToRowList( <mat>) . . . . . . .  returns a list of the rows
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       MatToRowList
#CALLING   L := MatToRowList(M);
#SHORTDOC  Returns a list of the rows.
#ARGUS     L @ list
#          M @ matrix
#
#PACKAGE   lib
#
#KASH      @
#          M := Mat(Z, [[1,2,3], [4,5,6], [7,8,9], [10,11,12]]);
#          L := MatToRowList(M);
#          @
#          The rows can also be accessed through:@
#          M[1];
#          IsList(M);
#
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

MatToRowList := function(M)
local i,j,row,col,L,L1;

if not  IsMat(M)  then
    Print("Usage: MatToList( <mat> );\n");
    return;
fi;

row := MatRows(M);
col :=MatCols(M);
L:=[];
L[row] := 1;
for i in [1..row] do
  L1:=[];
  L1[col]:=1;
  for j in [1..col] do
    L1[j] := MatElt(M,i,j);
  od;
  L[i] := L1;
od;
return (L);
end;

#############################################################################
##
#F  MatToColList( <mat>) . . . . . . .  returns a list of the cols
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       MatToColList
#CALLING   L := MatToColList(M);
#SHORTDOC  Returns a list of the columns.
#ARGUS     L @ list
#          M @ matrix
#
#PACKAGE   lib
#
#KASH      @
#          M := Mat(Z, [[1,2,3], [4,5,6], [7,8,9], [10,11,12]]);
#          L := MatToColList(M);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

MatToColList := function(M)
local i,j,row,col,L,L1;

if not  IsMat(M)  then
    Print("Usage: MatToColList( <mat> );\n");
    return;
fi;

row := MatRows(M);
col :=MatCols(M);
L:=[];
L[col]:=1;
for i in [1..col] do
  L1:=[];
  L1[row]:=1;
  for j in [1..row] do
    L1[j]:=MatElt(M,j,i);
  od;
  L[i]:=L1;
od;
return (L);
end;



