#############################################################################
##
#A  kantmod.g                    KANT library              Andreas Hoppe
##
#A  @(#)$Id: kantideal.g 1.00 1996/08/13 Hoppe $
##
#Y  Copyright 1996,  Prof. Dr. M. Pohst, TU - Berlin,  Germany
##
##  This file contains more KANT module functions .
##
#H  $Log: kantmod.g $
##


#############################################################################
##
#F  IsModule( <object> )  . . . . . . . . . .  checks if argument is an ideal
#/*TEX**TEX***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       IsModule
#CALLING   b := IsModule(m);
#SHORTDOC  Returns true iff the argument is a module.
#ARGUS     b @ boolean
#          m @ object
#
#PACKAGE   lib
#
#KASH      @ 
#          o := OrderMaximal(Z, 3, 3);;
#          IsModule(o);
#          IsModule(ModuleId(o, 4));
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##
IsModule := function ( I )

  if (TYPE (I) = "KANT module") then
    return true;
  else
    return false;
  fi;
end;

#############################################################################
##
#F  ModuleId( <order>, <natural number> )  . . . . . . . returns a 1-module
#/*TEX**TEX***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       ModuleId
#CALLING   m := ModuleId(o, n);
#SHORTDOC  Returns the trivial module over an order with a certain degree.
#LONGDOC   Creates the trivial module $\o^n$.
#ARGUS     m @ Module
#          o @ order
#          n @ positive integer @ the degree
#
#PACKAGE   lib
#
#KASH      @
#          o := OrderMaximal(Poly(Zx, [1, 1, 10]));;
#          ModuleId(o, 4);
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

ModuleId:=function(O,n)
  local one, i;
  if not IsInt(n) or n<1 or not IsOrder(O) then
    return Error("usage: ModuleId( <Order>, <n> );");
  fi;
  one:=1*O;
  return Module(List([1..n],i->one), MatId(O,n));
end;


#############################################################################
##
#F  ModuleMove( <module>, <ord>)  . . . . Moves module to a module over ord
#/*TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#
#FXN       ModuleMove
#
#SHORTDOC  Changes the coefficient order of the module.
#
#LONGDOC   Suppose {\tt M1} is an $\o_1$-module and there is an
#          order $\o_2$ such that there is an homomorphism from
#          $\phi:\o_1\to\o_2$ this function will compute $\phi({\tt M1})$.
#          This is done simply by applying $\phi$ to all the
#          matrix elements and to the coefficient ideals of {\tt M1}.
#          $\phi$ must be known to the system before using this
#          function.
#
#          If {\tt M1} is a list of modules all modules contained are
#          moved.
#
#CALLING   M2 := ModuleMove(M1, o);
#
#ARGUS     M2 @ same type as M1
#          M1 @ module $|$ list of modules
#          o @ order
#
#CSRC
#
#XREF      EltMove, IdealMove.
#
#PACKAGE   lib
#
#KASH      @
#          O:=Order(Poly(Zx,[1,1,-4,-14,3,1]));;
#          o:=OrderMaximal(O);;
#          M:=Module([Ideal(2,Elt(o,[0, 0, 0, 1, 1])),1*o],
#            Mat(o,[[1,Elt(o, [2, 2, 0, 2, 0])],[0,1]]));
#          M1:=ModuleMove(M, O);
#          M2:=ModuleMove(M1, o);
#          M=M2;
#**HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY
#
#      20. 1.98   AH   written
#
#**C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***C***/
#
##


ModuleMove := function(arg)
  local a, o, I, move, newl;

  move := function(M)
    return Module(IdealMove(ModuleIdeals(M), o),
                  MatMove(ModuleMatrix(M), o));
  end;

  if Length(arg) <> 2 then
    return
      Error("usage: ModuleMove(<module>, <ord>) or ModuleMove(<list of modules>, <ord>)\n");
  fi;
  o := arg[2];
  if not IsOrder(o) then
    return Error("2nd Argument is not an Order\n");
  fi;

  if IsList(arg[1]) then                 # we move faclsts and lists too
     newl := [];
     for a in arg[1] do
       if not IsModule(a) then
         return Error(a, " is no Module\n");
       fi;
       Add(newl, move(a));
     od;
     return newl;
   else
     if IsModule(arg[1]) then
       return move(arg[1]);                # a single Module
     else
       return Error("No module\n");
     fi;
  fi;
end;


# undocumented definition for compatibility
ModuleCHNF:=ModuleNF;

