#############################################################################
##
#A  kantord.g                    KANT library              Mario Daberkow
##
#A  @(#)$Id: kantord.g 1.00 1995/02/09 Daberkow $
##
#Y  Copyright 1995,  Prof. Dr. M. Pohst, TU - Berlin,  Germany
##
##  This file contains the Polynomial KANT functions .
##
#H  $Log: kantord.g $
#H  Revision 1.00  1995/02/09  Daberkow
#H  added 'IsPoly'
#H
##


#############################################################################
##
#F  IsOrder( <object> )  . . . . . . . . .  checks if argument is an order
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       IsOrder
#CALLING   b := IsOrder(x);
#SHORTDOC  Returns true iff the argument is an order.
#          b @ boolean
#          x
#
#PACKAGE   lib
#
#KASH      @
#          IsOrder(Z);
#          IsOrder(Order(Z, 2, 3));
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##


IsOrder := function ( ord )

  if (TYPE (ord) = "KANT order") then
    return true;
  else
    return false;
  fi;
end;



#############################################################################
##
#F  OrderBasisIsRel( <ord> ) . .  checks if order is given by transformation
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       OrderBasisIsRel
#CALLING   b := OrderBasisIsRel(o);
#SHORTDOC  Returns true iff the basis of an order is given by a transformation
# matrix.
#ARGUS     b @ boolean
#          o @ order
#XREF      OrderBasisIsPower, OrderEquationOrder
#
#
#PACKAGE   lib
#
#KASH      Checks, whether a certain order is an equation order or given
#          by a transformation matrix. @
#          O:= Order (Poly (Zx,[1,0,-186,0,13097,0,-412704,0,4946176]));
#          OrderBasisIsRel(OrderEquationOrder(O));  
#          OrderBasisIsRel(O);                       
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

OrderBasisIsRel := function ( ord )

   if  (IsOrder (ord)) then
      return not OrderBasisIsPower ( ord );
   fi;

   return Error("usage: OrderBasisIsRel( <order> )");
end;


#############################################################################
##
#F  OrderSUnitsPositive( <list> ) .. computes positive S-units
#/*TEX**TEX|***TEX***TEX***TEX***TEX**TEX***TEX***TEX***TEX***TEX***TEX***TEX**
#FXN       OrderSUnitsPositive
#CALLING   P := OrderSUnitsPositive(L);
#SHORTDOC  Returns a list of positive S-units
#LONGDOC   Given a list of independent S-units this function
#          returns a list consisting of a list of positive S-units
#          and the index in the original S-units.      
#ARGUS     P @ list
#          L @ list
#XREF      OrderSUnits, OrderUnitsFund, EltMinkowski
#
#PACKAGE   lib
#
#KASH      @
#          o := OrderMaximal(x^3 + x^2 - 6*x - 1);
#          OrderSig(o);
#          L := OrderUnitsFund(o);
#          List(L, EltMinkowski);
#          P := OrderSUnitsPositive(L);
#          List(P[1], EltMinkowski);
#
#**HISTORY*|*HISTORY***HISTORY***HISTORY***HISTORY***HISTORY***HISTORY**HISTORY
#
#  980318  FH installed.
#
#**KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH***KASH*/
##

OrderSUnitsPositive := function(L)
   
   local IsZero, First, Signum, MatEchelonTrans, 
         o, r1, LL, M, H, P, i;
   
   IsZero := x->x=0;
   
   First := function ( list, func )
      local  l;
      for l  in list  do
         if func( l )  then
            return l;
         fi;
      od;
      return false;
   end;

   Signum := function(x)
      return (1-Round(x/Abs(x)))/2;
   end;
   
   MatEchelonTrans := function(M)
   #
   #  M is list of rows, only one or zero entries
   #  (the trafo matrix must lift to an unimodular matrix)
   #   
      local S, T, rows, cols, i0, j0, i, m, ipiv;
      M := Copy(M);
      rows := Length(M);
      cols := Length(M[1]);
      T := MatToRowList(MatId(Z, rows));
      for m in M do
         if not Length(m) = cols then
            return Error("wrong row");
         fi;
      od;
      S := [];
      for i in [1..rows] do
         if not IsZero(M[i][1]) then
            Apply(M[i], a->(a+1) mod 2);
            Add(S, i);
         fi;
      od;
      i0 := 1;
      j0 := 1;
      while j0 <= cols and i0 <= rows do
         ipiv := First([i0..rows], i->not IsZero(M[i][j0]));
         if not ipiv = false then
            m := T[i0];
            T[i0] := T[ipiv];
            T[ipiv] := m;
            m := M[i0];
            M[i0] := M[ipiv];
            M[ipiv] := m;
            for i in [ipiv+1..rows] do
               T[i] := T[i] - M[i][j0]/M[i0][j0] * T[i0];
               Apply(T[i], x->x mod 2);
               M[i] := M[i] - M[i][j0]/M[i0][j0] * M[i0];
               Apply(M[i], x->x mod 2);
            od;
            i0 := i0 + 1;
         fi;
         j0 := j0 + 1;   
      od;
      return [M, T, S];
   end;
   
   if Length(L) = 0 then
      return [];
   fi;
   if not ForAll(L, IsElt) then
      return Error("list contains wrong element");
   fi;
   o := EltOrder(L[1]);
   r1 := OrderSig(o)[1];
   if r1 = 0 then
      return Concatenation(L, [ 1 ]);
   fi;
   L := Copy(L);
   
   # make matrix of entries 1 and 0
   LL := List(L, a->EltToList(EltMinkowski(a)){[1..r1]});
   Apply(LL, a->List(a, x->Signum(x)));
   
   # row echelonize this matrix mod 2 
   H := MatEchelonTrans(LL);
   
   # get the positions of non zero rows
   P := Filtered([1..Length(LL)], i->
                not ForAll([1..r1], j->IsZero(H[1][i][j])));
   
   # make a change of basis for L
   for i in H[3] do
      L[i] := -L[i];
   od;
   L := List([1..Length(LL)], i->Product([1..Length(LL)], j->L[j]^H[2][i][j]));
        
   # go to the positive basis
   for i in P do
      L[i] := L[i]^2;
   od;
   
   return [L, 2^Length(P)];
end;

